#     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
#     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
#     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
#     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
#     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
#     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
#     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
#     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
#     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
#     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
#     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
#     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#     FOR A PARTICULAR PURPOSE.
#     
#     (c) Copyright 2004 Xilinx, Inc.
#     All rights reserved.

# Globals
lappend drvlist
set ltypes [file join $env(XILINX_EDK) "sw/lib/bsp/linux_v2_00_a/data/Ltypes"]
set xilinx_pic [file join $env(XILINX_EDK) "sw/lib/bsp/linux_v2_00_a/data/xilinx_pic.c"]
set did_pci false

proc linux_drc {lib_handle} {
    puts "\#--------------------------------------"
    puts "\# Linux BSP DRC..."
    puts "\#--------------------------------------"
}

proc generate {libname} {
    puts "\#--------------------------------------"
    puts "\# Linux BSP generate..."
    puts "\#--------------------------------------"    
}

proc xget_corefreq {} {
    set processor [xget_processor] 
    set name [xget_value $processor "NAME"]
    #puts "procname : $name"
    set processor_driver [xget_swhandle [xget_value $processor "NAME"]]
    #puts "procdrv : $processor_driver"
    if {[string compare -nocase $processor_driver ""] != 0} {
	set arg "CORE_CLOCK_FREQ_HZ"
	#set retval [xget_value $processor_driver "PARAMETER" $arg]
	set retval [xget_dname [xget_value $processor_driver "NAME"] $arg]
	return $retval
    }
}

# procedure that adds # defines to xparameters.h as XPAR_argument
proc xredefine_params {handle file_name args} {
	
    #puts "xredefine ..."
    # Open include file
    set file_handle [xopen_include_file $file_name]
    #puts "args : $args"

    foreach arg $args {
	if {[string compare -nocase $arg "CORE_CLOCK_FREQ_HZ"] == 0} {
	    set value [xget_corefreq]
	    #puts "corefreq : $value"
	} else {
	    set value [xget_value $handle "PARAMETER" $arg]
	    #puts "value : $value"
	}
	
	if {$value != ""} {
	    set value [xformat_addr_string $value $arg]
	    if {[string compare -nocase $arg "MEM_SIZE"] == 0} {
		set name "DDR_0_SIZE"
	    } elseif {[string compare -nocase $arg "IIC_PERSISTENT_BASEADDR"] == 0} {
		set name "PERSISTENT_0_IIC_0_BASEADDR"
	    } elseif {[string compare -nocase $arg "IIC_PERSISTENT_HIGHADDR"] == 0} {
		set name "PERSISTENT_0_IIC_0_HIGHADDR"
	    } elseif {[string compare -nocase $arg "IIC_PERSISTENT_EEPROMADDR"] == 0} {
		set name "PERSISTENT_0_IIC_0_EEPROMADDR"
	    } elseif {[string compare -nocase $arg "POWERDOWN_BASEADDR"] == 0} {
		set name "POWER_0_POWERDOWN_BASEADDR"
	    } elseif {[string compare -nocase $arg "POWERDOWN_HIGHADDR"] == 0} {
		set name "POWER_0_POWERDOWN_HIGHADDR"
	    } elseif {[string compare -nocase $arg "POWERDOWN_VALUE"] == 0} {
		set name "POWER_0_POWERDOWN_VALUE"
	    } else {
		set name [string toupper $arg]
	    }
	    set name [format "XPAR_%s" $name]
	    puts $file_handle "#define $name $value"
	}
    }

    puts $file_handle "\n/******************************************************************/\n"
    close $file_handle
}

# procedure that adds static #defines to xparameters.h
proc xstatic_defines {handle file_name args} {
    
    variable did_pci

    #puts "static defines ..."
    #puts "args : $args"

    if {! $did_pci} {
	# Open include file
	set file_handle [xopen_include_file $file_name]
	puts $file_handle "#define XPAR_PCI_0_CLOCK_FREQ_HZ    0"

	puts $file_handle "\n/******************************************************************/\n"
	close $file_handle
    }
}

# uart redefines...
proc xredefine_uartns550 {drvhandle file_name} {
    
    xredefine_include_file $drvhandle $file_name "uartns550" "C_BASEADDR" "C_HIGHADDR" "CLOCK_HZ" "DEVICE_ID"
    
}

proc xredefine_intc {drvhandle file_name} {
    
    xredefine_include_file $drvhandle $file_name "intc" "C_BASEADDR" "C_HIGHADDR" "KIND_OF_INTR" "DEVICE_ID"
    
    # Open include file
    set config_inc [xopen_include_file $file_name]
    
    set periphs [xget_periphs $drvhandle] 
    set device_id 0
    set periph_name [string toupper "intc"]
    
    foreach periph $periphs {
	
	# Get the edk based name of peripheral for printing redefines
	set edk_periph_name [xget_value $periph "NAME"]

	# Get ports that are driving the interrupt
	set source_ports [xget_interrupt_sources $periph]
	set i 0
	foreach source_port $source_ports {
	    set source_port_name($i) [xget_value $source_port "NAME"]
	    set source_periph($i) [xget_handle $source_port "PARENT"]
	    set source_name($i) [xget_value $source_periph($i) "NAME"]
	    incr i
	}
	set num_intr_inputs [xget_value $periph "PARAMETER" "C_NUM_INTR_INPUTS" ]
	for {set i 0} {$i < $num_intr_inputs} {incr i} {
	    set drv [xget_swhandle $source_name($i)]
	    set iptype [xget_value $source_periph($i) "OPTION" "IPTYPE"]
	    if {[llength $source_name($i)] != 0 && [llength $drv] != 0 && [string compare -nocase $iptype "PERIPHERAL"] == 0} {
		
		set instance [xfind_instance $drv $source_name($i)]
		
		set drvname [xget_value $drv "NAME"]
		if {[string compare -nocase $drvname "touchscreen_ref"] == 0} {
		    set drvname "touchscreen"
		} elseif {[string compare -nocase $drvname "pci_ref"] == 0} {
		    set drvname "pci"
		} elseif {[string compare -nocase $drvname "ps2_ref"] == 0} {
		    set drvname "ps2"
		    set instance [expr $instance*2]
		    if {[string match -nocase  "*SYS_INTR2" $source_port_name($i)] == 1} {
			incr instance
		    }
		}
		
		set drvname [string toupper $drvname]
		set first_part [format "#define XPAR_%s_%s_%s_%s_VEC_ID" $periph_name $device_id $drvname $instance]
		set second_part [format "XPAR_%s_%s_%s_INTR" [string toupper $edk_periph_name] [string toupper $source_name($i)] [string toupper $source_port_name($i)] ]
		# write the redefine if the driver is not "generic"
		if {[string compare -nocase $drvname "pci"] == 0} {
		    puts $config_inc "${first_part}_A $second_part"
		    puts $config_inc "${first_part}_B $second_part"
		    puts $config_inc "${first_part}_C $second_part"
		    puts $config_inc "${first_part}_D $second_part"
		} elseif {[string compare -nocase $drvname "generic"] != 0} {
		    puts $config_inc "$first_part $second_part"
		}
	    }
	}
	incr device_id
    }
    puts $config_inc "\n/******************************************************************/\n"
    close $config_inc
}

proc xfind_instance {drvhandle instname} {
    
    set instlist [xget_periphs $drvhandle]
    set i 0
    foreach inst $instlist {
	set name [xget_value $inst "NAME"] 
	if {[string compare -nocase $instname $name] == 0} {
	    return $i
	}
	incr i
    }	
    set i 0
    return $i
}

proc xredefine_iic {drvhandle file_name} {
    
    xredefine_include_file $drvhandle $file_name "iic" "C_BASEADDR" "C_HIGHADDR" "C_TEN_BIT_ADR" "DEVICE_ID"
    
}


proc xredefine_emac {drvhandle file_name} {
    
    xredefine_include_file $drvhandle $file_name "emac" "C_BASEADDR" "C_HIGHADDR" "C_DMA_PRESENT" "C_MII_EXIST" "C_ERR_COUNT_EXIST" "DEVICE_ID"
    
}

proc xredefine_gemac {drvhandle file_name} {
    
    xredefine_include_file $drvhandle $file_name "gemac" "C_BASEADDR" "C_HIGHADDR" "C_DMA_PRESENT" "C_MII_EXIST" "C_ERR_COUNT_EXIST" "DEVICE_ID"
    
}

proc xredefine_gpio {drvhandle file_name} {
    
    xredefine_include_file $drvhandle $file_name "gpio" "C_BASEADDR" "C_HIGHADDR" "DEVICE_ID"
    
}

proc xredefine_sysace {drvhandle file_name} {
    
    xredefine_include_file $drvhandle $file_name "sysace" "C_BASEADDR" "C_HIGHADDR" "DEVICE_ID"
    
}

proc xredefine_ps2 {drvhandle file_name} {
    
    xredefine_include_file $drvhandle $file_name "ps2" "C_BASEADDR" "C_HIGHADDR" "DEVICE_ID"
    
}

proc xredefine_uartlite {drvhandle file_name} {
    
    xredefine_include_file $drvhandle $file_name "uartlite" "C_BASEADDR" "C_HIGHADDR" "DEVICE_ID"
    
}

proc xredefine_spi {drvhandle file_name} {
    
    xredefine_include_file $drvhandle $file_name "spi" "C_BASEADDR" "C_HIGHADDR" "DEVICE_ID"
}

proc xredefine_touchscreen {drvhandle file_name} {
    
    xredefine_include_file $drvhandle $file_name "touchscreen" "C_BASEADDR" "C_HIGHADDR" "DEVICE_ID"
}

proc xredefine_tft {drvhandle file_name} {
    
    xredefine_include_file $drvhandle $file_name "tft" "C_BASEADDR"
}

proc xredefine_pci {drvhandle file_name} {
    
    variable did_pci

    xredefine_include_file $drvhandle $file_name "pci" "C_BASEADDR" "C_HIGHADDR" "CONFIG_ADDR" "CONFIG_DATA" "LCONFIG_ADDR" "MEM_BASEADDR" "MEM_HIGHADDR" "IO_BASEADDR" "IO_HIGHADDR" "CLOCK_HZ" "DEVICE_ID"
    set did_pci true
}

#######################

proc xredefine_include_file {drv_handle file_name drv_string args} {
    
    # Open include file
    set file_handle [xopen_include_file $file_name]
    
    # Get all peripherals connected to this driver
    set periphs [xget_periphs $drv_handle] 
    
    set pname [format "XPAR_%s_" [string toupper $drv_string]]
    
    # Print all parameters for all peripherals
    set device_id 0
    foreach periph $periphs {
	if {[xget_value $periph "PARAMETER" "C_IS_DUAL"] == 1 || [string compare -nocase $drv_string "ps2"] == 0} {
	    set sub_periphs 2
	} else {
	    set sub_periphs 1
	}
	#puts "$periph : $drv_string : $sub_periphs"

	for {set i 0} {$i < $sub_periphs} {incr i} {
	    foreach arg $args {
		set name "${pname}${device_id}_"
	    
		if {[string compare -nocase "CLOCK_HZ" $arg] == 0} {
		    set xdrv_string [format "%s%s" "X" $drv_string]
		    set value [xget_dname $xdrv_string $arg]
		    set name "${name}CLOCK_FREQ_HZ"
		} else {
		    if {[string match C_* $arg]} {
			set name [format "%s%s" $name [string range $arg 2 end]]
		    } else {
			set name "${name}${arg}"
		    }
		    if {[string compare -nocase $drv_string "tft"] == 0} {
			set value [xget_name $periph C_DCR_BASEADDR]
		    } else {
			set value [xget_name $periph $arg]
		    }
		}

		if {$sub_periphs > 1} {
		    set value "${value}_$i"
		}

		if {[string compare -nocase "uartns550" $drv_string] == 0} {
		    if {[string compare -nocase "C_BASEADDR" $arg] == 0} {
			set value [format "(%s%s%s)" $value "+" "0x1000"]
		    }
		}

		puts $file_handle "#define $name $value"
		if {[string compare -nocase "DEVICE_ID" $arg] == 0} {
		    incr device_id
		}
	    }
	}
    }		
    puts $file_handle "\n/******************************************************************/\n"
    close $file_handle
}

##################################################
# procedure post_generate
# This generates the drivers directory for linux
# and runs the mklinux script
##################################################

proc post_generate {lib_handle} {
    
    variable drvlist
    variable xilinx_pic

    ###################################### previously in generate #######################################

    # Get list of peripherals connected to linux
    set conn_periphs [xget_handle $lib_handle "ARRAY" "connected_periphs"]
    #lappend drvlist
    if {[string compare -nocase $conn_periphs ""] != 0} {
	set conn_periphs_elems [xget_handle $conn_periphs "ELEMENTS" "*"]
	# For each periph
	foreach periph_elem $conn_periphs_elems {
	    set periph [xget_value $periph_elem "PARAMETER" "periph_name"]
	    # 1. Get driver
	    set drv [xget_swhandle $periph]
	    set posn [lsearch -exact $drvlist $drv]
	    if {$posn == -1} {
		lappend drvlist $drv
	    }
	}

	set file_handle [xopen_include_file "xparameters.h"]
	puts $file_handle "\n/******************************************************************/\n"
	puts $file_handle "/* Linux Redefines */"
	puts $file_handle "\n/******************************************************************/\n"
	close $file_handle

	foreach drv $drvlist {

	    set drvname [xget_value $drv "NAME"]
	    
	    #Redefines xparameters.h

	    if {[string compare -nocase $drvname "tft_ref"] == 0} {
		    xredefine_tft $drv "xparameters.h"
	    }
	    
	    # redefines for 16550
	    if {[string compare -nocase $drvname "uartns550"] == 0} {
		xredefine_uartns550 $drv "xparameters.h"
	    } elseif {[string compare -nocase $drvname "intc"] == 0} {
		set drvver [xget_value $drv "PARAMETER" "DRIVER_VER"]
		set level [xget_value $drv "PARAMETER" "LEVEL"]
		if {[string compare -nocase $drvver "1.00.b"] == 0} {
		    if {$level == 1} {
			xredefine_intc $drv "xparameters.h"
		    }
		} else {
		    xredefine_intc $drv "xparameters.h"
		}
	    } elseif {[string compare -nocase $drvname "iic"] == 0} {
		xredefine_iic $drv "xparameters.h"
	    }  elseif {[string compare -nocase $drvname "emac"] == 0} {
		xredefine_emac $drv "xparameters.h"
	    }  elseif {[string compare -nocase $drvname "gemac"] == 0} {
		xredefine_gemac $drv "xparameters.h"
	    } elseif {[string compare -nocase $drvname "gpio"] == 0} {
		xredefine_gpio $drv "xparameters.h"
	    } elseif {[string compare -nocase $drvname "sysace"] == 0} {
		xredefine_sysace $drv "xparameters.h"
	    } elseif {[string compare -nocase $drvname "ps2_ref"] == 0} {
		xredefine_ps2 $drv "xparameters.h"
	    } elseif {[string compare -nocase $drvname "uartlite"] == 0} {
		xredefine_uartlite $drv "xparameters.h"
	    } elseif {[string compare -nocase $drvname "spi"] == 0} {
		xredefine_spi $drv "xparameters.h"
	    } elseif {[string compare -nocase $drvname "touchscreen_ref"] == 0} {
		xredefine_touchscreen $drv "xparameters.h"
	    } elseif {[string compare -nocase $drvname "pci_ref"] == 0} {
		xredefine_pci $drv "xparameters.h"
	    } 
	}
    }
    
    # define core_clock, plb_clock, mem_size
    xredefine_params $lib_handle "xparameters.h" "PLB_CLOCK_FREQ_HZ" "CORE_CLOCK_FREQ_HZ" "MEM_SIZE"

    # define the values for the persistent storage in IIC
    xredefine_params $lib_handle "xparameters.h" "IIC_PERSISTENT_BASEADDR" "IIC_PERSISTENT_HIGHADDR" "IIC_PERSISTENT_EEPROMADDR"

    # define the powerdown values if POWERDOWN_BASEADDR exists
    if {[xget_value $lib_handle "PARAMETER" POWERDOWN_BASEADDR] != ""} {
	xredefine_params $lib_handle "xparameters.h" "POWERDOWN_BASEADDR" "POWERDOWN_HIGHADDR" "POWERDOWN_VALUE"
    }

    xstatic_defines $lib_handle "xparameters.h"

    ###################################### previously in generate #######################################
    
    # Create Linux tree structure
    set pwd [pwd]
    set simple "linux/arch/ppc/boot/simple"
    set xilinx_ocp "linux/arch/ppc/platforms/xilinx_ocp"
    set block "linux/drivers/block"
    set char "linux/drivers/char"
    set i2c "linux/drivers/i2c"
    set net "linux/drivers/net"
    set kernel "linux/arch/ppc/kernel"
    
    exec bash -c "mkdir -p $simple;mkdir -p $xilinx_ocp;mkdir -p $kernel"
    exec bash -c "mkdir -p $block;mkdir -p $char;mkdir -p $i2c;mkdir -p $net"
    
    # Copy files for xilinx_ocp
    xcopy_ocpfiles

    foreach drv $drvlist {
	set drvname [xget_value $drv "NAME"]
	set ver [xget_value $drv "PARAMETER" "DRIVER_VER"]
	set ver [string map {. _} $ver]
	set dirname [format "%s_v%s" $drvname $ver]
	
	if {[string compare -nocase $drvname "iic"] == 0} {
	    xcopy_iic $drv $dirname
	}  elseif {[string compare -nocase $drvname "emac"] == 0} {
	    xcopy_emac $drv $dirname
	}  elseif {[string compare -nocase $drvname "gemac"] == 0} {
	    xcopy_gemac $drv $dirname
	} elseif {[string compare -nocase $drvname "gpio"] == 0} {
	    xcopy_gpio $drv $dirname
	} elseif {[string compare -nocase $drvname "sysace"] == 0} {
	    xcopy_sysace $drv $dirname
	} elseif {[string compare -nocase $drvname "ps2_ref"] == 0} {
	    xcopy_ps2 $drv $dirname
	} elseif {[string compare -nocase $drvname "uartlite"] == 0} {
	    xcopy_uartlite $drv $dirname
	} elseif {[string compare -nocase $drvname "spi"] == 0} {
	    xcopy_spi $drv $dirname
	} elseif {[string compare -nocase $drvname "touchscreen_ref"] == 0} {
	    xcopy_touchscreen $drv $dirname
	}
    }
    
    # Call Ltypes Script here
    set linux "linux"
    xltype_file $linux

    # Patch the interrupt controller driver
    exec bash -c "cp $xilinx_pic $kernel"

    # Move xparameters.h around
    exec bash -c "cp ../../include/xparameters.h $xilinx_ocp/xparameters_ml300.h"
    exec bash -c "rm -f $xilinx_ocp/xparameters.h"

    # copy the whole Linux BSP to its final destination
    set value [xget_value $lib_handle "PARAMETER" TARGET_DIR]
    puts "TARGET_DIR : $value"

    if {$value != ""} {
        if {[file isdirectory $value] == 0} {
            exec bash -c "mkdir -p $value"
        }
	set cur_dir [pwd]
	exec bash -c "cd ../../../"
        exec bash -c "cp -Rfp $cur_dir/linux/* $value"
    }
}

proc xcopy_ocpfiles {} {

    variable drvlist

    set xilinx_ocp "linux/arch/ppc/platforms/xilinx_ocp"
    
    foreach drv $drvlist {
	set depends [xget_value $drv "OPTION" "DEPENDS"]
	foreach dep $depends {
	    #puts "dep : $dep"
	    if {[file isdirectory "../$dep"] == 1} {
		exec bash -c "cp -f ../$dep/src/*.c $xilinx_ocp"
		exec bash -c "cp -f ../$dep/src/*.h $xilinx_ocp"
	    }
	}
    }
    
}


proc xcopy_iic {drv_handle dirname } {
    set i2c "drivers/i2c/xilinx_iic"
    set simple "linux/arch/ppc/boot/simple"
    
    xcopy_dir $dirname $i2c
    
    if {[file isdirectory "../$dirname"] == 1} {
	# Copy files for simple
	exec bash -c "cp -f ../$dirname/src/xiic_l.c $simple"
    } else {
	puts "$dirname does not exist ..."
    }
    
}

proc xcopy_gpio {drv_handle dirname} {
    set gpio "drivers/char/xilinx_gpio"
    xcopy_dir $dirname $gpio
}

proc xcopy_sysace {drv_handle dirname} {
    set sysace "drivers/block/xilinx_sysace"
    xcopy_dir $dirname $sysace
}

proc xcopy_emac {drv_handle dirname} {
    set emac "drivers/net/xilinx_enet"
    xcopy_dir $dirname $emac
}

proc xcopy_gemac {drv_handle dirname} {
    set gemac "drivers/net/xilinx_enet"
    xcopy_dir $dirname $gemac
}

proc xcopy_ps2 {drv_handle dirname} {
    set ps2 "drivers/char/xilinx_keyb"
    xcopy_dir $dirname $ps2
}

proc xcopy_uartlite {drv_handle dirname} {
    set uartlite "drivers/char/xilinx_uartlite"
    xcopy_dir $dirname $uartlite
}

proc xcopy_spi {drv_handle dirname} {
    set spi "drivers/char/xilinx_spi"
    xcopy_dir $dirname $spi
}

proc xcopy_touchscreen {drv_handle dirname} {
    set touchscreen "drivers/char/xilinx_ts"
    xcopy_dir $dirname $touchscreen
}

proc xcopy_dir {srcdir dstdir} {
    
    set dstdirname [format "%s%s" "linux/" $dstdir]
    if {[file isdirectory "../$srcdir"] == 1} {
	# Copy files from src to dst
	exec bash -c "mkdir -p $dstdirname"
	exec bash -c "cp -f ../$srcdir/src/*.c $dstdirname"
	exec bash -c "cp -f ../$srcdir/src/*.h $dstdirname"
    } else {
	puts "$srcdir does not exist ..."
    }
}


proc xltype_file {filename} {

    variable ltypes

    puts $filename

    if {[file isdirectory $filename]} {
	foreach entry [glob -nocomplain [file join $filename *]] {
	    xltype_file $entry
	}
    } else {
	exec bash -c "$ltypes $filename"
    }
    
}
