#
#     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
#     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
#     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
#     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
#     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
#     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
#     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
#     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
#     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
#     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
#     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
#     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#     FOR A PARTICULAR PURPOSE.
#     
#     (c) Copyright 2004 Xilinx, Inc.
#     All rights reserved.
# 

# Invoke this .do file with arguments
# do testbench.do [bfm]
#   bfm - will use PPC405 BFM instead of SWIFT model

# Check to see whether doing behavioral or structural simulation
# based upon the current directory

set current_dir [pwd]

if { [ regexp {simulation/behavioral$} $current_dir ] } {
    echo "Performing a behavioral simulation."
    set BEHAVIORAL_SIM 1
} elseif { [ regexp {simulation/structural$} $current_dir ] } {
    echo "Performing a structural simulation."
    set BEHAVIORAL_SIM 0
} else {
    echo "Current directory is not simulation/behavioral nor simulation/structural"
    error "Testbench supports only behavioral and structural simulation...quitting"
}

# Check whether doing BFM or SWIFT simulation
# Set PPC_MODEL_BFM to 1 if using BFM version of PPC405
# Set PPC_MODEL_BFM to 0 if using SWIFT MODEL of PPC405 (default)
set PPC_MODEL_BFM 0
if {$argc == 1} {
    switch -glob $1 {
        [bB][fF][mM] {
            set PPC_MODEL_BFM 1
        }
    }
} else {
    set PPC_MODEL_BFM 0
}

if {$PPC_MODEL_BFM == 1} {
    echo "Using Bus-Functional Model (BFM) of PPC405"
} else {
    echo "Using SWIFT Model of PPC405"
}

# ccundefined is 0 if CORECONNECT environment variable is defined
set ccundefined [string compare [ array names env CORECONNECT ] "CORECONNECT"]

if {$PPC_MODEL_BFM == 1 && $ccundefined != 0} { 
    error "A BFM simulation requires the IBM(R) CoreConnect(TM) Toolkit and CORECONNECT set to the installation location. Refer to the documentation for more information"
}

# Generate .mem files for DDR SDRAM models using data2bram
exec sh ../../data/gen_memfiles.sh

# Alter system_init.v file to use the testbench hierarchy
if { [file readable "system_init.v"] } {
    echo
    echo "Modifying system_init.v to use the top-level instance name"
    echo
    exec sh ../../data/init_patch.sh
}

# Compile Design Files
do system.do

# Compile Missing Simulation Files
# this is a bug in the current EDK tools
do ../../data/edn_patch.do

# Compile Testbench Models
vlog -incr -work work ../../data/uart_rcvr.v ../../data/mt46v32m8.v ../../data/AT24CXXX.v ../../data/pci_targ32.v

# Compile testbench files and CoreConnect bus monitors. Different file sets are
# used depending on if the CoreConnect Toolkit is installed

if {$PPC_MODEL_BFM == 1} {
    set testbench_defines "+define+PPC_MODEL_BFM"
} else {
    set testbench_defines "+define+PPC_MODEL_SWIFT"
}

if {$BEHAVIORAL_SIM == 1} {
    set testbench_defines "$testbench_defines+BEHAVIORAL_SIM"
} else {
    set testbench_defines "$testbench_defines+STRUCTURAL_SIM"
}

if {$ccundefined == 0} {
    vlog $testbench_defines -vlog95compat -incr -work work -f ../../data/compile_corecon.f ../../data/testbench.v 
} else {
    vlog $testbench_defines -vlog95compat -incr -work work -f ../../data/compile_corecon_dummy.f ../../data/testbench.v
}

# Conditionally compile in PPC405 BFM files
if {$PPC_MODEL_BFM == 1} {
    do ../../data/use_ppc405_bfm.do
    vsim -Lf bfm_lib -Lf unisims_ver -Lf simprims_ver -Lf XilinxCoreLib_ver -t 1ps -c work.testbench work.glbl work.test_system
} else {
    vlog -incr -work work ../../data/bfl/test_system_place_holder.v
    vsim -Lf unisims_ver -Lf simprims_ver -Lf XilinxCoreLib_ver -t 1ps -c work.testbench work.glbl work.test_system
}

# Start UART Terminals
if { ! [string compare $tcl_platform(platform) "windows"] } then {
    exec ../../data/start_terminal.bat &
} else {
    exec sh ../../data/start_terminal.sh &
}

# Open default waveform of system
do ../../data/wave.do

echo "Type \"run -all\" to Start the Simulation"
echo "Press <ESC> in the Uart Terminal to Stop the Simulation"
