//-------------------------------------------------------------------------
// TestBench - Module
//-------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//-------------------------------------------------------------------------
// Filename: testbench.v
// 
// Description:    
//   Test Bench Top Level wrapper
//
// Design Notes:
//   Contains behavioral code to connect the device under test
//   (Virtex-II Pro FPGA) to external devices for testing.
//
//-------------------------------------------------------------------------
// Author:      
// History:
//
//-------------------------------------------------------------------------
// Naming Conventions:
//      active low signals:                   "*_n"
//      clock signals:                        "clk", "clk_div#", "clk_#x" 
//      reset signals:                        "rst", "rst_n" 
//      generics/parameters:                  "C_*" 
//      user defined types:                   "*_TYPE" 
//      state machine next state:             "*_ns" 
//      state machine current state:          "*_cs" 
//      combinatorial signals:                "*_com" 
//      pipelined or register delay signals:  "*_d#" 
//      counter signals:                      "*cnt*"
//      clock enable signals:                 "*_ce" 
//      internal version of output port       "*_i"
//      device pins:                          "*_pin" 
//      ports:                                - Names begin with Uppercase 
//      processes:                            "*_PROCESS" 
//      component instantiations:             "<ENTITY_>I_<#|FUNC>
//-------------------------------------------------------------------------

`timescale  1 ns / 1 ps

module testbench ();

///////////////////////////////////////////////////////////////////////////
// Signal Declarations
///////////////////////////////////////////////////////////////////////////

  // Internal Signals
  reg         clk;
  reg         sysace_clk;
  reg         gsr_rst;
  reg         external_rst_n;

  // UART 1
  wire        uart1_ctsn;
  wire        uart1_rtsn;
  reg         uart1_sin;
  wire        uart1_sout;
  // UART 2
  wire        uart2_ctsn;
  wire        uart2_rtsn;
  reg         uart2_sin;
  wire        uart2_sout;
  // GPIO
  tri1 [0:31] gpio;
  wire        case_light;
  wire [0:15] fpga_led;
  wire        fpga_pwr_sig_n;
  tri1 [0:15] gp_button;
  wire        gp_button_xcvr_dir;
  wire        gp_button_xcvr_sba; 
  wire        gp_button_xcvr_oe;
  wire        fpga_led_xcvr_sab;
  wire        opb_error;
  wire        plb_error;
  // Trace
  wire        trc_clk;
  wire        trc_ts1o;
  wire        trc_ts2o;
  wire        trc_ts1e;
  wire        trc_ts2e;
  wire        trc_ts3;
  wire        trc_ts4;
  wire        trc_ts5;
  wire        trc_ts6;
  // BDM
  tri1        cpu_tck;
  tri1        cpu_tdi;
  tri1        cpu_tdo;
  tri1        cpu_halt;
  tri1        cpu_tms;
  tri1        cpu_trst;
  // DDR SDRAM
  wire          ddr_clk_fb;
  wire   [12:0] ddr_ad;
  wire   [12:0] ddr_ad_o;
  reg    [12:0] ddr_ad_o_d1;
  wire   [1:0]  ddr_ba;
  wire   [1:0]  ddr_ba_o;
  reg    [1:0]  ddr_ba_o_d1;
  wire          ddr_casb;
  wire          ddr_casb_o;
  reg           ddr_casb_o_d1;
  wire          ddr_cke;
  wire          ddr_cke_o;
  reg           ddr_cke_o_d1;
  wire          ddr_clk;
  wire          ddr_clkb;
  wire          ddr_csb;
  wire          ddr_csb_o;
  reg           ddr_csb_o_d1;
  wire   [3:0]  ddr_dm;
  wire   [3:0]  ddr_dqs;
  wire          ddr_rasb;
  wire          ddr_rasb_o;
  reg           ddr_rasb_o_d1;
  wire          ddr_web;
  wire          ddr_web_o;
  reg           ddr_web_o_d1;
  wire   [31:0] ddr_dq;
  // System Ace MPU
  wire   [6:0]  sysace_mpa;
  wire          sysace_mpce;
  wire   [7:0]  sysace_mpd;
  wire          sysace_mpirq;
  wire          sysace_mpoe;
  wire          sysace_mpwe;
  reg    [7:0]  sysace_data_reg[6:0];
  // IIC
  tri1          iic_scl;
  tri1          iic_sda;
  tri1          iic_irq_n;
  tri1          iic_temp_crit_n;
  wire          iic_eeprom_wp;
  // Ethernet
  wire          phy_col;
  wire          phy_crs;
  wire          phy_dv;
  wire          phy_rx_clk;
  wire   [3:0]  phy_rx_data;
  wire          phy_rx_er;
  wire          phy_tx_clk;
  wire          phy_mii_clk;
  wire          phy_mii_int_n;
  wire          phy_rst_n;
  wire   [3:0]  phy_tx_data;
  wire          phy_tx_en;
  wire          phy_tx_er;
  wire          phy_mii_ata;
  wire          phy_slew1;
  wire          phy_slew2;

  // SPI
  wire          spi_miso;
  wire          spi_mosi;
  wire          spi_sck;
  wire          spi_ss;

  // PS2
  wire ps2_clk_rx_1;
  wire ps2_clk_rx_2;
  wire ps2_clk_tx_1;
  wire ps2_clk_tx_2;
  wire ps2_d_rx_1;
  wire ps2_d_rx_2;
  wire ps2_d_tx_1;
  wire ps2_d_tx_2;

  // TFT
  wire [5:0] tft_lcd_b;
  wire tft_lcd_clk;
  wire tft_lcd_de;
  wire tft_lcd_dps;
  wire [5:0] tft_lcd_g;
  wire tft_lcd_hsync;
  wire [5:0] tft_lcd_r;
  wire tft_lcd_vsync;

  // TSD
  wire tsd_busy;
  wire tsd_cs;
  wire tsd_dclk;
  wire tsd_din;
  wire tsd_dout;
  wire tsd_penirq;

  // AC97
  wire AC97_Reset_n;
  wire ac97_SData_In;
  wire ac97_SData_Out;
  wire ac97_Sync;
  reg  ac97_bit_clk;

  // Parallel Port
  wire [7:0] PP_DATA;
  wire PP_DIR;
  wire PP_HD;
  wire PP_INTR;
  wire PP_USER1;
  wire PP_USER2;
  wire PP_USER3;
  wire PP_nASTRB;
  wire PP_nDSTRB;
  wire PP_nINIT;
  wire PP_nWAIT;
  wire PP_nWRITE;

  // PCI
  tri1          PCI_INTA;
  tri1          PCI_INTB;
  tri1          PCI_INTC;
  tri1          PCI_INTD;
  tri1          PCI_ACK64_N;
  tri1          PCI_DEVSEL_N;
  tri1          PCI_FRAME_N;
  tri1          PCI_IRDY_N;
  tri1          PCI_PERR_N;
  tri1          PCI_REQ64_N;
  tri1          PCI_SERR_N;
  tri1          PCI_STOP_N;
  tri1          PCI_TRDY_N;
  wire          PCI_IDSEL;
  wire          PCI_PAR64;
  wire          PCI_PAR;
  wire          PCI_CB_CLK;
  wire          PCI_PMC_CLK;
  wire          PCI_PMC_CLK_FB;
  wire          PCI_CB_REQ;
  wire          PCI_CB_GNT;
  wire          PCI_PMC_REQ;
  wire          PCI_PMC_GNT;
  wire          PCI_GRST_N;
  wire          PCI_RST_N;
  wire   [63:0] PCI_AD;
  wire   [7:0]  PCI_CBE;
   
  // Display Testbench Messages on/off Flags
  wire          uart1_MsgOn;
  wire          uart2_MsgOn;
  wire          intr_MsgOn;             
  wire          reg_MsgOn;
  // UART 1 Simulation Terminal Signals
  reg    [15:0] char1_out;
  integer       FP1;
  integer       loop1;
  integer       loop_out1;
  reg           serial1_reset;
  reg           serial1_baudclock;
  reg    [15:0] uart1_char_in;
  wire   [7:0]  uart1_char_out;
  reg    [15:0] uart1_mem_in[0:1023];
  reg    [15:0] uart1_mem_in_ptr;
  reg    [7:0]  uart1_mem_out[0:1023];
  reg    [15:0] uart1_mem_out_ptr;
  wire          uart1_rcvr_valid;
  // UART 2 Simulation Terminal Signals
  reg    [15:0] char2_out;
  integer       FP2;
  integer       loop2;
  integer       loop_out2;
  reg           serial2_reset;
  reg           serial2_baudclock;
  reg    [15:0] uart2_char_in;
  wire   [7:0]  uart2_char_out;
  reg    [15:0] uart2_mem_in[0:1023];
  reg    [15:0] uart2_mem_in_ptr;
  reg    [7:0]  uart2_mem_out[0:1023];
  reg    [15:0] uart2_mem_out_ptr;
  wire          uart2_rcvr_valid;
  // Bus Monitor
  `include "opb_dcl.inc"
  wire   [0:31] synch_in;
  wire   [0:31] plb_monitor_synch_out;
  wire          plb_monitor_error;
  wire   [0:31] opb_monitor_synch_out;
  wire   [0:31] dcr_monitor0_synch_out;
  wire   [0:31] dcr_monitor1_synch_out;
  wire          monitor_bfm_error;
  reg           already_stopped;
  wire   [0:(`max_opb_devices-1)] mx_request;
  wire   [0:(`max_opb_devices-1)] mx_select;
  wire   [0:(`max_opb_devices-1)] opb_mxgrant;
  wire   [0:7]  opb_be;
  wire   [0:(`max_opb_devices-1)] sl_xferack;

  // Parameters for Simulation
  `include "../../data/sim_params.v"

///////////////////////////////////////////////////////////////////////////
// FPGA Instantiation as Device Under Test
///////////////////////////////////////////////////////////////////////////

   // BRAM initialization
   system_conf system_conf_0();

  system system_0
    (
     // System Signals
     .sys_clk            (clk),                // I
     .sysace_clk_in      (sysace_clk),         // I
     .sys_rst_in         (external_rst_n),     // I
     // GPIOs
     .gpio               (gpio),               // IO
     // UART 1
     .uart1_ctsn         (uart1_ctsn),         // I
     .uart1_rtsn         (uart1_rtsn),         // O
     .uart1_sin          (uart1_sin),          // I
     .uart1_sout         (uart1_sout),         // O
     // UART 2
     .uart2_ctsN         (uart2_ctsn),         // I
     .uart2_rtsN         (uart2_rtsn),         // O
     .uart2_sin          (uart2_sin),          // I
     .uart2_sout         (uart2_sout),         // O
     // Misc Buttons, LEDs, Control Signals
     .case_light         (case_light),         // O
     .fpga_led           (fpga_led),           // O [0:15]
     .fpga_pwr_sig_n     (fpga_pwr_sig_n),     // O
     .gp_button          (gp_button),          // I [0:15]
     .iic_eeprom_wp      (iic_eeprom_wp),      // O
     .gp_button_xcvr_dir (gp_button_xcvr_dir), // O
     .gp_button_xcvr_sba (gp_button_xcvr_sba), // O
     .gp_button_xcvr_oe  (gp_button_xcvr_oe),  // O
     .fpga_led_xcvr_sab  (fpga_led_xcvr_sab),  // O
     .opb_error          (opb_error),          // O
     .plb_error          (plb_error),          // O
     // Trace Signals
     .trc_clk            (trc_clk),            // O
     .trc_ts1o           (trc_ts1o),           // O
     .trc_ts2o           (trc_ts2o),           // O
     .trc_ts1e           (trc_ts1e),           // O
     .trc_ts2e           (trc_ts2e),           // O
     .trc_ts3            (trc_ts3),            // O
     .trc_ts4            (trc_ts4),            // O
     .trc_ts5            (trc_ts5),            // O
     .trc_ts6            (trc_ts6),            // O
     // JTAG Signals
     .cpu_tck            (cpu_tck),            // I with Pullup
     .cpu_tdi            (cpu_tdi),            // I with Pullup
     .cpu_tdo            (cpu_tdo),            // O with Pullup
     .cpu_halt           (cpu_halt),           // I with Pullup
     .cpu_tms            (cpu_tms),            // I with Pullup
     .cpu_trst           (cpu_trst),           // I with Pullup
     // DDR SDRAM
     .ddr_ad             (ddr_ad_o),           // O  [0:12]
     .ddr_ba             (ddr_ba_o),           // O  [0:1] 
     .ddr_casb           (ddr_casb_o),         // O
     .ddr_cke            (ddr_cke_o),          // O
     .ddr_clk            (ddr_clk),            // O
     .ddr_clk_fb         (ddr_clk_fb),         // I
     .ddr_clkb           (ddr_clkb),           // O
     .ddr_csb            (ddr_csb_o),          // O  
     .ddr_dm             (ddr_dm),             // O  [3:0]
     .ddr_dq             (ddr_dq),             // IO [31:0]
     .ddr_dqs            (ddr_dqs),            // O  [3:0]
     .ddr_rasb           (ddr_rasb_o),         // O
     .ddr_web            (ddr_web_o),          // O
     // System ACE
     .sysace_clk_oe_n    (),                   // O
     .sysace_mpa         (sysace_mpa),         // O  [6:0]
     .sysace_mpce        (sysace_mpce),        // O
     .sysace_mpd         (sysace_mpd),         // IO [7:0]
     .sysace_mpirq       (sysace_mpirq),       // I
     .sysace_mpoe        (sysace_mpoe),        // O
     .sysace_mpwe        (sysace_mpwe),        // O
     // IIC Signals
     .iic_scl            (iic_scl),            // IO
     .iic_sda            (iic_sda),            // IO
     .iic_irq_n          (iic_irq_n),          // I
     .iic_temp_crit_n    (iic_temp_crit_n),    // I
     // SPI
     .spi_miso           (spi_miso),           // IO
     .spi_mosi           (spi_mosi),           // IO
     .spi_sck            (spi_sck),            // IO
     .spi_ss             (spi_ss),             // IO
     // PS2
     .ps2_clk_rx_1       (ps2_clk_rx_1),       // I
     .ps2_clk_rx_2       (ps2_clk_rx_2),       // I
     .ps2_clk_tx_1       (ps2_clk_tx_2),       // O
     .ps2_clk_tx_2       (ps2_clk_tx_2),       // O
     .ps2_d_rx_1         (ps2_d_rx_1),         // I
     .ps2_d_rx_2         (ps2_d_rx_2),         // I
     .ps2_d_tx_1         (ps2_d_tx_1),         // O
     .ps2_d_tx_2         (ps2_d_tx_2),         // O
     // TFT
     .tft_lcd_b          (tft_lcd_b),          // O
     .tft_lcd_clk        (tft_lcd_clk),        // O
     .tft_lcd_de         (tft_lcd_de),         // O
     .tft_lcd_dps        (tft_lcd_dps),        // O
     .tft_lcd_g          (tft_lcd_g),          // O [5:0]
     .tft_lcd_hsync      (tft_lcd_hsync),      // O
     .tft_lcd_r          (tft_lcd_r),          // O [5:0]
     .tft_lcd_vsync      (tft_lcd_vsync),      // O
     // TSD
     .tsd_busy           (tsd_busy),           // I
     .tsd_cs             (tsd_cs),             // O
     .tsd_dclk           (tsd_dclk),           // O
     .tsd_din            (tsd_din),            // O
     .tsd_dout           (tsd_dout),           // I
     .tsd_penirq         (tsd_penirq),         // I
     // AC97
     .ac97_SData_In      (ac97_SData_In),      // I
     .ac97_SData_Out     (ac97_SData_Out),     // O
     .ac97_Sync          (ac97_Sync),          // O
     .ac97_bit_clk       (ac97_bit_clk),       // I
     .AC97_Reset_n       (AC97_Reset_n),       // O
     // Parallel Port
     .PP_DATA            (PP_DATA),            // O [7:0]
     .PP_DIR             (PP_DIR),             // O
     .PP_HD              (PP_HD),              // O
     .PP_INTR            (PP_INTR),            // I
     .PP_USER1           (PP_USER1),           // I
     .PP_USER2           (PP_USER2),           // I
     .PP_USER3           (PP_USER3),           // I
     .PP_nASTRB          (PP_nASTRB),          // O
     .PP_nDSTRB          (PP_nDSTRB),          // O
     .PP_nINIT           (PP_nINIT),           // O
     .PP_nWAIT           (PP_nWAIT),           // I
     .PP_nWRITE          (PP_nWRITE),          // O
     .PCI_RST_N          (PCI_RST_N),          // O
     .PCI_ACK64_N        (PCI_ACK64_N),        // IO
     .PCI_AD             (PCI_AD[31:0]),       // IO [31:0]
     .PCI_CBE            (PCI_CBE[3:0]),       // IO [3:0]
     .PCI_DEVSEL_N       (PCI_DEVSEL_N),       // IO
     .PCI_FRAME_N        (PCI_FRAME_N),        // IO
     .PCI_IDSEL          (PCI_IDSEL),          // IO
     .PCI_IRDY_N         (PCI_IRDY_N),         // IO
     .PCI_PAR            (PCI_PAR),            // IO
     .PCI_PERR_N         (PCI_PERR_N),         // IO
     .PCI_REQ64_N        (PCI_REQ64_N),        // IO
     .PCI_SERR_N         (PCI_SERR_N),         // IO
     .PCI_STOP_N         (PCI_STOP_N),         // IO
     .PCI_TRDY_N         (PCI_TRDY_N),         // IO
     .PCI_CB_CLK         (PCI_CB_CLK),         // O
     .PCI_PMC_CLK        (PCI_PMC_CLK),        // O
     .PCI_PMC_CLK_FB     (PCI_PMC_CLK_FB),     // I
     .PCI_CB_REQ         (PCI_CB_REQ),         // I
     .PCI_CB_GNT         (PCI_CB_GNT),         // O
     .PCI_PMC_REQ        (PCI_PMC_REQ),        // I
     .PCI_PMC_GNT        (PCI_PMC_GNT),        // O
     .PCI_GRST_N         (PCI_GRST_N),         // O
     .PCI_INTA           (PCI_INTA),           // I
     .PCI_INTB           (PCI_INTB),           // I
     .PCI_INTC           (PCI_INTC),           // I
     .PCI_INTD           (PCI_INTD),           // I
     // Ethernet Signals
     .phy_col            (phy_col),            // I
     .phy_crs            (phy_crs),            // I
     .phy_dv             (phy_dv),             // I
     .phy_rx_clk         (phy_rx_clk),         // I
     .phy_rx_data        (phy_rx_data),        // I [3:0]
     .phy_rx_er          (phy_rx_er),          // I
     .phy_tx_clk         (phy_tx_clk),         // I
     .phy_mii_clk        (phy_mii_clk),        // IO
     .phy_mii_int_n      (phy_mii_int_n),      // I
     .phy_rst_n          (phy_rst_n),          // O
     .phy_tx_data        (phy_tx_data),        // O [3:0]
     .phy_tx_en          (phy_tx_en),          // O
     .phy_tx_er          (phy_tx_er),          // O
     .phy_mii_data       (phy_mii_data),       // IO
     .phy_slew1          (phy_slew1),          // O
     .phy_slew2          (phy_slew2)           // O
     );
  
///////////////////////////////////////////////////////////////////////////
// Generate Clock and Resets for Testbench
///////////////////////////////////////////////////////////////////////////

  // Initialize clock and resets 
  initial begin
    clk = 0;
    sysace_clk = 0;
    external_rst_n = 1; // Represents Reset Button on the Board
    serial1_baudclock = 0;
    serial1_reset = 1;
    serial2_baudclock = 0;
    serial2_reset = 1;
    gsr_rst = 1;
    #(`SIM_GSR_DELAY) gsr_rst = 0;
    $display ("\nGSR Deasserted at time = %t", $realtime);
    serial1_reset = 0;
    serial2_reset = 0;
  end

  assign glbl.GSR = gsr_rst; // assert reset to Xilinx sim primitives

   assign #3 PCI_PMC_CLK_FB = PCI_PMC_CLK;


  always #(`SIM_REF_CLOCK_PERIOD*0.5) clk = ~clk; // Main clock
  always #(`SIM_SYSTEMACE_CLOCK_PERIOD*0.5) sysace_clk = ~sysace_clk; //SystemACE
  always #(`SIM_SERIAL1_Clock_Period*0.5) // Testbench Uart I/F clock
    serial1_baudclock = ~serial1_baudclock;
  always #(`SIM_SERIAL2_Clock_Period*0.5) // Testbench Uart I/F clock
    serial2_baudclock = ~serial2_baudclock;

  // stop simulation if max sim time (specified in sim_params.v) is exceeded
  initial begin
    if (`SIM_MAX_SIM_TIME > 0) begin // Set max time to 0 to bypass
      #(`SIM_MAX_SIM_TIME);
      $stop;
    end
  end

///////////////////////////////////////////////////////////////////////////
// Uart 1 Testbench Logic
///////////////////////////////////////////////////////////////////////////

assign uart1_ctsn = 1'b1;

// Instantiate UART receiver to capture UART outputs
uart_rcvr uart1_rcvr(
  .reset        (serial1_reset),     // I
  .serial_clock (serial1_baudclock), // I
  .serial_in    (uart1_sout),        // I
  .char_out     (uart1_char_out),    // O [7:0]
  .char_valid   (uart1_rcvr_valid)   // O
  );

// Open file to write out chars sent out by UART inside FPGA
initial begin
  uart1_mem_out_ptr = 16'b0;
  FP1 = $fopen("Uart1.output");
  if (FP1 == 0) begin
    $display ("Could Not Open \"Uart1.output\" For Writing");
    $stop;
  end
  $fclose(FP1);
end

// Print to diplay and write to file the char recevied by UART1
always @ (posedge serial1_baudclock)
  if (uart1_rcvr_valid) begin
    // store char received from UART output into array
    uart1_mem_out[uart1_mem_out_ptr] = uart1_char_out;
    uart1_mem_out_ptr = uart1_mem_out_ptr + 1;
    // open file output to write out char to file for terminal program
    FP1 = $fopen("Uart1.output");
    if (FP1 == 0) begin
      $display ("Could Not Open \"Uart1.output\" For Writing");
      $stop;
    end
    // write out all chars received from UART output to File
    $fdisplay (FP1, "%d", uart1_mem_out_ptr);
    for (loop_out1=0; loop_out1<uart1_mem_out_ptr; loop_out1=loop_out1+1)
    begin 
      char1_out = uart1_mem_out[loop_out1];
      $fdisplay (FP1, "%h", char1_out[7:0]);
    end
    $fclose(FP1);
    FP1 = 0;
    // write out latest char received from UART output to display
    if (uart1_MsgOn)
      begin
        if (uart1_char_out > 8'd31 & uart1_char_out < 8'd127)
          $display ("UART 1 Transmitted Out Char = 0x%h = \"%c\" @ Time = %t",
                    uart1_char_out, uart1_char_out, $realtime);
        else
          $display ("UART 1 Transmitted Out Char = 0x%h = <special char> @ Time = %t",
                    uart1_char_out, $realtime);
      end
  end

// Loop Routine to read input chars from a file and send them out to UART1
initial begin
  uart1_mem_in_ptr = 10'b0;
  uart1_sin = 1'b1;
  uart1_char_in = 16'b0;
  while (!((uart1_char_in[7:0] == 8'd27)
        && (uart1_mem_in[0] == uart1_mem_in_ptr))) begin
    $readmemh("Uart1.input", uart1_mem_in);
    if (uart1_mem_in[0] > uart1_mem_in_ptr) begin
      uart1_mem_in_ptr = uart1_mem_in_ptr + 1;
      uart1_char_in = uart1_mem_in[uart1_mem_in_ptr];
      uart1_sin = 1'b0;
      for (loop1=0; loop1<16; loop1=loop1+1) @(posedge serial1_baudclock);
      uart1_sin = uart1_char_in[0];
      for (loop1=0; loop1<16; loop1=loop1+1) @(posedge serial1_baudclock);
      uart1_sin = uart1_char_in[1];
      for (loop1=0; loop1<16; loop1=loop1+1) @(posedge serial1_baudclock);
      uart1_sin = uart1_char_in[2];
      for (loop1=0; loop1<16; loop1=loop1+1) @(posedge serial1_baudclock);
      uart1_sin = uart1_char_in[3];
      for (loop1=0; loop1<16; loop1=loop1+1) @(posedge serial1_baudclock);
      uart1_sin = uart1_char_in[4];
      for (loop1=0; loop1<16; loop1=loop1+1) @(posedge serial1_baudclock);
      uart1_sin = uart1_char_in[5];
      for (loop1=0; loop1<16; loop1=loop1+1) @(posedge serial1_baudclock);
      uart1_sin = uart1_char_in[6];
      for (loop1=0; loop1<16; loop1=loop1+1) @(posedge serial1_baudclock);
      uart1_sin = uart1_char_in[7];
      for (loop1=0; loop1<16; loop1=loop1+1) @(posedge serial1_baudclock);
      uart1_sin = 1'b1;
      for (loop1=0; loop1<16; loop1=loop1+1) @(posedge serial1_baudclock); 
      if (uart1_MsgOn)
      begin
        if (uart1_char_in[7:0] > 8'd31 & uart1_char_in[7:0] < 8'd127)
          $display ("Char Sent to UART 1 = 0x%h = \"%c\" @ Time = %t",
                    uart1_char_in[7:0], uart1_char_in[7:0], $realtime);
        else
          $display ("Char Sent to UART 1 = 0x%h = <special char> @ Time = %t",
                    uart1_char_in[7:0], $realtime);
      end
    end
    else begin // Sit out for 1 serial bit time if no char to be sent
      for (loop1=0; loop1<16; loop1=loop1+1) @(posedge serial1_baudclock);
    end
  end
  $display ("<ESC> Char received from UART 1. Simulation Halted.");
  $stop;
end

///////////////////////////////////////////////////////////////////////////
// UART 2 Terminal Interface Behavioral Logic
///////////////////////////////////////////////////////////////////////////

assign uart2_ctsn = 1'b1;

// Instantiate UART receiver to capture UART outputs
uart_rcvr uart2_rcvr(
  .reset(serial_reset),
  .serial_clock(serial2_baudclock),
  .serial_in(uart2_sout),
  .char_out(uart2_char_out),
  .char_valid(uart2_rcvr_valid)
  );
// Print to diplay and write to file the char recevied by UART2
initial begin
  uart2_mem_out_ptr = 16'b0;
  FP2 = $fopen("Uart2.output");
  if (FP2 == 0) begin
    $display ("Could Not Open \"Uart2.output\" For Writing");
    $stop;
  end
  $fclose(FP2);
end

always @ (posedge serial2_baudclock)
  if (uart2_rcvr_valid) begin
    // store char received from UART output into array
    uart2_mem_out[uart2_mem_out_ptr] = uart2_char_out;
    uart2_mem_out_ptr = uart2_mem_out_ptr + 1;
    // open file output to write out char to file for terminal program
    FP2 = $fopen("Uart2.output");
    if (FP2 == 0) begin
      $display ("Could Not Open \"Uart2.output\" For Writing");
      $stop;
    end
    // write out all chars received from UART output to File
    $fdisplay (FP2, "%d", uart2_mem_out_ptr);
    for (loop_out2=0; loop_out2<uart2_mem_out_ptr; loop_out2=loop_out2+1)
    begin 
      char2_out = uart2_mem_out[loop_out2];
      $fdisplay (FP2, "%h", char2_out[7:0]);
    end
    $fclose(FP2);
    FP2 = 0;
    // write out latest char received from UART output to display
    if (uart2_MsgOn)
      begin
        if (uart2_char_out > 8'd31 & uart2_char_out < 8'd127)
          $display ("UART 2 Transmitted Out Char = 0x%h = \"%c\" @ Time = %t",
                    uart2_char_out, uart2_char_out, $realtime);
        else
          $display ("UART 2 Transmitted Out Char = 0x%h = <special char> @ Time = %t",
                    uart2_char_out, $realtime);
      end
  end

// Loop Routine to read input chars from a file and send them out to UART2
initial begin
  uart2_mem_in_ptr = 10'b0;
  uart2_sin = 1'b1;
  uart2_char_in = 16'b0;
  while (!((uart2_char_in[7:0] == 8'd27)
        && (uart2_mem_in[0] == uart2_mem_in_ptr))) begin
    $readmemh("Uart2.input", uart2_mem_in);
    if (uart2_mem_in[0] > uart2_mem_in_ptr) begin
      uart2_mem_in_ptr = uart2_mem_in_ptr + 1;
      uart2_char_in = uart2_mem_in[uart2_mem_in_ptr];
      uart2_sin = 1'b0;
      for (loop2=0; loop2<16; loop2=loop2+1) @(posedge serial2_baudclock);
      uart2_sin = uart2_char_in[0];
      for (loop2=0; loop2<16; loop2=loop2+1) @(posedge serial2_baudclock);
      uart2_sin = uart2_char_in[1];
      for (loop2=0; loop2<16; loop2=loop2+1) @(posedge serial2_baudclock);
      uart2_sin = uart2_char_in[2];
      for (loop2=0; loop2<16; loop2=loop2+1) @(posedge serial2_baudclock);
      uart2_sin = uart2_char_in[3];
      for (loop2=0; loop2<16; loop2=loop2+1) @(posedge serial2_baudclock);
      uart2_sin = uart2_char_in[4];
      for (loop2=0; loop2<16; loop2=loop2+1) @(posedge serial2_baudclock);
      uart2_sin = uart2_char_in[5];
      for (loop2=0; loop2<16; loop2=loop2+1) @(posedge serial2_baudclock);
      uart2_sin = uart2_char_in[6];
      for (loop2=0; loop2<16; loop2=loop2+1) @(posedge serial2_baudclock);
      uart2_sin = uart2_char_in[7];
      for (loop2=0; loop2<16; loop2=loop2+1) @(posedge serial2_baudclock);
      uart2_sin = 1'b1;
      for (loop2=0; loop2<16; loop2=loop2+1) @(posedge serial2_baudclock); 
      if (uart2_MsgOn)
      begin
        if (uart2_char_in[7:0] > 8'd31 & uart2_char_in[7:0] < 8'd127)
          $display ("Char Sent to UART 2 = 0x%h = \"%c\" @ Time = %t",
                    uart2_char_in[7:0], uart2_char_in[7:0], $realtime);
        else
          $display ("Char Sent to UART 2 = 0x%h = <special char> @ Time = %t",
                    uart2_char_in[7:0], $realtime);
      end
    end
    else begin // Sit out for 1 serial bit time if no char to be sent
      for (loop2=0; loop2<16; loop2=loop2+1) @(posedge serial2_baudclock);
    end
  end
  $display ("<ESC> Char received from UART 2. Simulation Halted.");
  $stop;
end

///////////////////////////////////////////////////////////////////////////
//  Misc Logic
///////////////////////////////////////////////////////////////////////////

assign gp_button = 16'b0;

///////////////////////////////////////////////////////////////////////////
// Instantiate, Initialize and Connect DDR SDRAM Memory Model
///////////////////////////////////////////////////////////////////////////
  
// Simulate Registered DIMM
  always @(posedge ddr_clk)
  begin
    ddr_ad_o_d1   <= #1 ddr_ad_o;
    ddr_ba_o_d1   <= #1 ddr_ba_o;
    ddr_casb_o_d1 <= #1 ddr_casb_o;
    ddr_cke_o_d1  <= #1 ddr_cke_o;
    ddr_csb_o_d1  <= #1 ddr_csb_o; 
    ddr_rasb_o_d1 <= #1 ddr_rasb_o;
    ddr_web_o_d1  <= #1 ddr_web_o;
  end
 
  assign ddr_ad   = ddr_ad_o_d1;
  assign ddr_ba   = ddr_ba_o_d1;
  assign ddr_casb = ddr_casb_o_d1;
  assign ddr_cke  = ddr_cke_o_d1;
  assign ddr_csb  = ddr_csb_o_d1;
  assign ddr_rasb = ddr_rasb_o_d1;
  assign ddr_web  = ddr_web_o_d1;

assign  ddr_clk_fb = ddr_clk; // feed DDR clock back to FPGA

mt46v32m8 ddr0 (
  .Addr  (ddr_ad[12:0]),  // I  [12:0]
  .Ba    (ddr_ba),        // I  [1:0]
  .Cas_n (ddr_casb),      // I
  .Cke   (ddr_cke),       // I
  .Clk   (ddr_clk),       // I
  .Clk_n (ddr_clkb),      // I
  .Cs_n  (ddr_csb),       // I
  .Dm    (ddr_dm[0]),     // I
  .Dq    (ddr_dq[7:0]),   // IO [7:0]
  .Dqs   (ddr_dqs[0]),    // I
  .Ras_n (ddr_rasb),      // I
  .We_n  (ddr_web)        // I
  );

mt46v32m8 ddr1 (
  .Addr  (ddr_ad[12:0]),  // I  [12:0]
  .Ba    (ddr_ba),        // I  [1:0]
  .Cas_n (ddr_casb),      // I
  .Cke   (ddr_cke),       // I
  .Clk   (ddr_clk),       // I
  .Clk_n (ddr_clkb),      // I
  .Cs_n  (ddr_csb),       // I
  .Dm    (ddr_dm[1]),     // I
  .Dq    (ddr_dq[15:8]),  // IO [7:0]
  .Dqs   (ddr_dqs[1]),    // I
  .Ras_n (ddr_rasb),      // I
  .We_n  (ddr_web)        // I
  );

mt46v32m8 ddr2 (
  .Addr  (ddr_ad[12:0]),  // I  [12:0]
  .Ba    (ddr_ba),        // I  [1:0]
  .Cas_n (ddr_casb),      // I
  .Cke   (ddr_cke),       // I
  .Clk   (ddr_clk),       // I
  .Clk_n (ddr_clkb),      // I
  .Cs_n  (ddr_csb),       // I
  .Dm    (ddr_dm[2]),     // I
  .Dq    (ddr_dq[23:16]), // IO [7:0]
  .Dqs   (ddr_dqs[2]),    // I
  .Ras_n (ddr_rasb),      // I
  .We_n  (ddr_web)        // I
  );

mt46v32m8 ddr3 (
  .Addr  (ddr_ad[12:0]),  // I  [12:0]
  .Ba    (ddr_ba),        // I  [1:0]
  .Cas_n (ddr_casb),      // I
  .Cke   (ddr_cke),       // I
  .Clk   (ddr_clk),       // I
  .Clk_n (ddr_clkb),      // I
  .Cs_n  (ddr_csb),       // I
  .Dm    (ddr_dm[3]),     // I
  .Dq    (ddr_dq[31:24]), // IO [7:0]
  .Dqs   (ddr_dqs[3]),    // I
  .Ras_n (ddr_rasb),      // I
  .We_n  (ddr_web)        // I
  );

// Initialize DDR SDRAM Memory Model

  // DATA Content: 4 DDR devices of 8 bit data width for 32 total data bits
  //   ddr0 has bytes 0 ; ddr1 has bytes 1, etc
  // ADDRESS Content: Each DDR device has 4 banks of [Row,Col] address
  //   Size of each bank is 32M entries (of 8 bits of data)
  //   In 32 bit data space organization (4 bytes)
  //    Bank 0 = Base Addr DDR + 0x00000000,Bank 1 = Base Addr DDR + 0x02000000
  //    Bank 2 = Base Addr DDR + 0x04000000,Bank 3 = Base Addr DDR + 0x06000000
  //   Memory range of both ddr devices will thus be:
  //     (DDR Base Address + 0x00000000) to (DDR Base Address + 0x07FFFFFF)
  //   Graphical Data Format:  ddr3    ddr2     ddr1     ddr0
  //               Data bits   31:24   23:16    15:8      7:0
  //               Byte Addr   Byte3   Byte2    Byte1   Byte0
  initial begin
    $display ("Loading DDR devices from .../sys/mem/ddrN_bankB.mem files");
    $readmemh("../../data/mem/ddr0_bank0.mem",testbench.ddr0.Bank0);
    $readmemh("../../data/mem/ddr1_bank0.mem",testbench.ddr1.Bank0);
    $readmemh("../../data/mem/ddr2_bank0.mem",testbench.ddr2.Bank0);
    $readmemh("../../data/mem/ddr3_bank0.mem",testbench.ddr3.Bank0);

    $readmemh("../../data/mem/ddr0_bank1.mem",testbench.ddr0.Bank1);
    $readmemh("../../data/mem/ddr1_bank1.mem",testbench.ddr1.Bank1);
    $readmemh("../../data/mem/ddr2_bank1.mem",testbench.ddr2.Bank1);
    $readmemh("../../data/mem/ddr3_bank1.mem",testbench.ddr3.Bank1);

    $readmemh("../../data/mem/ddr0_bank2.mem",testbench.ddr0.Bank2);
    $readmemh("../../data/mem/ddr1_bank2.mem",testbench.ddr1.Bank2);
    $readmemh("../../data/mem/ddr2_bank2.mem",testbench.ddr2.Bank2);
    $readmemh("../../data/mem/ddr3_bank2.mem",testbench.ddr3.Bank2);

    $readmemh("../../data/mem/ddr0_bank3.mem",testbench.ddr0.Bank3);
    $readmemh("../../data/mem/ddr1_bank3.mem",testbench.ddr1.Bank3);
    $readmemh("../../data/mem/ddr2_bank3.mem",testbench.ddr2.Bank3);
    $readmemh("../../data/mem/ddr3_bank3.mem",testbench.ddr3.Bank3);
    $display ("  Completed Loading DDR devices");
  end

///////////////////////////////////////////////////////////////////////////
//  System Ace MPU Interface
///////////////////////////////////////////////////////////////////////////

// Implements a array of data registers to emulate system ace MPU interface
// Data Registers can be written and read back.
// Does not model actual behavior or timing of system ace chip, only
// its read/write interface
assign sysace_mpirq = 1'b0;

always @(posedge sysace_clk)
  if (~sysace_mpwe & ~sysace_mpce & sysace_mpoe)
    sysace_data_reg[sysace_mpa] <= sysace_mpd;

wire sysace_read;
reg  sysace_read_d1;

assign sysace_read = ~sysace_mpoe & sysace_mpwe & ~sysace_mpce;

assign #1 sysace_mpd = (sysace_read)?
                       sysace_data_reg[sysace_mpa] : 8'bz;

//reg [7:0] debug_cntr;
//initial debug_cntr = 0;
//always @(posedge clk)
//  debug_cntr <= debug_cntr + 1;
//assign #1 sysace_mpd = debug_cntr;

///////////////////////////////////////////////////////////////////////////
// Instantiate and Connect PCI 32 bit Target
///////////////////////////////////////////////////////////////////////////


pci_targ32 pci_targ32 (
  .AD       (PCI_AD[31:0]),  // IO [31:0]
  .CBE      (PCI_CBE[3:0]),  // IO [3:0]
  .PAR      (PCI_PAR),       // IO
  .FRAME_N  (PCI_FRAME_N),   // IO
  .TRDY_N   (PCI_TRDY_N),    // IO
  .IRDY_N   (PCI_IRDY_N),    // IO
  .IDSEL    (PCI_AD[17]),    // I
  .STOP_N   (PCI_STOP_N),    // IO
  .DEVSEL_N (PCI_DEVSEL_N),  // IO
  .RST_N    (PCI_RST_N),     // I
  .CLK      (PCI_PMC_CLK & PCI_CB_CLK) // I (Make sure both clks running)
);

assign PCI_IDSEL = PCI_AD[16];
assign PCI_CB_REQ = 1'b1;
assign PCI_PMC_REQ = 1'b1;

//PCI_INTA
//PCI_INTB
//PCI_INTC
//PCI_INTD


///////////////////////////////////////////////////////////////////////////
// PS/2 Model
///////////////////////////////////////////////////////////////////////////

assign ps2_clk_rx_1 = 1'b1;
//ps2_clk_tx_1
assign ps2_d_rx_1   = 1'b1;
//ps2_d_tx_1
assign ps2_clk_rx_2 = 1'b1;
//ps2_clk_tx_2
assign ps2_d_rx_2   = 1'b1;
//ps2_d_tx_2


///////////////////////////////////////////////////////////////////////////
// Touch Screen Digitizer Model
///////////////////////////////////////////////////////////////////////////

reg tsd_dout_reg;

assign tsd_penirq = 1'b1;
assign tsd_busy   = tsd_dclk;
assign tsd_cs     = 1'b0;

// generate a toggling signal for the touch screen Data Out so 0xAAAA will
// be read out.
initial tsd_dout_reg = 1'b0;
always @(posedge tsd_dclk)
  tsd_dout_reg <= ~tsd_dout_reg & ~tsd_cs;
assign tsd_dout   = tsd_dout_reg;

//tsd_din

///////////////////////////////////////////////////////////////////////////
// Parallel Port Model
///////////////////////////////////////////////////////////////////////////
       
assign PP_INTR  = 1'b1; // SPP : nAck
assign PP_nWAIT = 1'b0; // SPP : Busy
assign PP_USER1 = 1'b0; // SPP : Out of Paper or PError
assign PP_USER3 = 1'b1; // SPP : Select
//PP_DATA[7:0]
//PP_DIR
//PP_HD
//PP_nASTRB
//PP_nDSTRB
//PP_nINIT
//PP_nWRITE
assign PP_USER2 = 1'b1;

///////////////////////////////////////////////////////////////////////////
// AC97 Audio Controller Model
///////////////////////////////////////////////////////////////////////////

reg [4:0] slot0;
reg [4:0] slot1;
reg [4:0] slot2;
reg [4:0] slot3;
reg [4:0] slot4;
reg [4:0] slot5;
reg [4:0] slot6;
reg [4:0] slot7;
reg [4:0] slot8;
reg [4:0] slot9;
reg [4:0] slot10;
reg [4:0] slot11;
reg [4:0] slot12;
reg [7:0] bit_num;
reg AC97_Sync_d1;

initial begin
  bit_num = 0;
  #5000;
  while (AC97_Reset_n != 1'b1) #100;
  #1000;
  while (1) begin
     #(40.69) ac97_bit_clk = ~ac97_bit_clk;
  end
end

always @(posedge ac97_bit_clk)
begin
  AC97_Sync_d1 <= ac97_Sync;
  if (ac97_Sync & ~AC97_Sync_d1)
    bit_num <= 8'd255;
  else
    bit_num <= bit_num-1;
end

assign AC97_SData_In = (bit_num == 255) | (bit_num == 252) | (bit_num == 251);

always @(bit_num)
begin
  slot0 <= 'hx;
  slot1 <= 'hx;
  slot2 <= 'hx;
  slot3 <= 'hx;
  slot4 <= 'hx;
  slot5 <= 'hx;
  slot6 <= 'hx;
  slot7 <= 'hx;
  slot8 <= 'hx;
  slot9 <= 'hx;
  slot10 <= 'hx;
  slot11 <= 'hx;
  slot12 <= 'hx;
  if (bit_num > 239)
    slot0  <= bit_num - 240;
  else if (bit_num > 219)
    slot1  <= bit_num - 220;
  else if (bit_num > 199)
    slot2  <= bit_num - 200;
  else if (bit_num > 179)
    slot3  <= bit_num - 180;
  else if (bit_num > 159)
    slot4  <= bit_num - 160;
  else if (bit_num > 139)
    slot5  <= bit_num - 140;
  else if (bit_num > 119)
    slot6  <= bit_num - 120;
  else if (bit_num > 99)
    slot7  <= bit_num - 100;
  else if (bit_num > 79)
    slot8  <= bit_num - 80;
  else if (bit_num > 59)
    slot9  <= bit_num - 60;
  else if (bit_num > 39)
    slot10 <= bit_num - 40;
  else if (bit_num > 19)
    slot11 <= bit_num - 20;
  else if (bit_num > 00)
    slot12 <= bit_num - 00;
end
// AC97_Reset_n
// AC97_SData_Out
// AC97_Sync

///////////////////////////////////////////////////////////////////////////
// Instantiate and Connect IIC slave Devices
///////////////////////////////////////////////////////////////////////////

defparam AT24C64.InitialMemFile = "";
AT24CXXX AT24C64 (
  .WP  (iic_eeprom_wp),
  .A2  (1'b0), 
  .A1  (1'b0),
  .A0  (1'b0),
  .GND (1'b0),
  .VCC (1'b1),
  .SCL (iic_scl),
  .SDA (iic_sda)
  );


///////////////////////////////////////////////////////////////////////////
// CoreConnect Toolkit PLB Bus Monitor
///////////////////////////////////////////////////////////////////////////

// Work Around EDK Issue with plb_smbusy/plb_smerr
wire [0:7] plb_smbusy;
wire [0:7] plb_smerr;

   `define SYSTEM_PATH system_0
assign plb_smbusy = {`SYSTEM_PATH.plb_v34_0_Sl_MBusy[0:3]
                   | `SYSTEM_PATH.plb_v34_0_Sl_MBusy[4:7]
                   | `SYSTEM_PATH.plb_v34_0_Sl_MBusy[8:11], 4'b0};

assign plb_smerr =  {`SYSTEM_PATH.plb_v34_0_Sl_MErr[0:3]
                   | `SYSTEM_PATH.plb_v34_0_Sl_MErr[4:7]
                   | `SYSTEM_PATH.plb_v34_0_Sl_MErr[8:11], 4'b0};

plb_monitor3x
#(4'b0000, // monitor instace number
  // PLB Slave 0-7 addr ranges
  32'b0,                32'b0,
  32'b0,                32'b0,
  32'b0,                32'b0,
  32'b0,                32'b0,
  32'b0,                32'b0,
  32'b0,                32'b0,
  32'b0,                32'b0,
  32'b0,                32'b0,

  32'b0,                32'b0,
  32'b0,                32'b0,
  32'b0,                32'b0,
  32'b0,                32'b0,
  32'b0,                32'b0,
  32'b0,                32'b0,
  32'b0,                32'b0,
  32'b0,                32'b0)
plb_monitor
(
  .synch_in          (synch_in),                                         // i [0:31]
  .plb_clk           (`SYSTEM_PATH.plb_clk),             // i
  .plb_reset         (`SYSTEM_PATH.sys_rst[0]),             // i
  .m_abort_0         (`SYSTEM_PATH.plb_v34_0_M_abort[0]),          // i
  // Gate off DCU abort under core reset to prevent too many unnecessary PLB Monitor "Notes"
  .m_abort_1         (`SYSTEM_PATH.plb_v34_0_M_abort[1] & ~`SYSTEM_PATH.resetcore_dly), // i
  .m_abort_2         (`SYSTEM_PATH.plb_v34_0_M_abort[2]),          // i
  .m_abort_3         (`SYSTEM_PATH.plb_v34_0_M_abort[3]),          // i
  .m_abort_4         (1'b0),                                             // i
  .m_abort_5         (1'b0),                                             // i
  .m_abort_6         (1'b0),                                             // i
  .m_abort_7         (1'b0),                                             // i
  .m_abus_0          (`SYSTEM_PATH.plb_v34_0_M_ABus[0:31]),        // i [0:31]
  .m_abus_1          (`SYSTEM_PATH.plb_v34_0_M_ABus[32:63]),       // i [0:31]
  .m_abus_2          (`SYSTEM_PATH.plb_v34_0_M_ABus[64:95]),       // i [0:31]
  .m_abus_3          (`SYSTEM_PATH.plb_v34_0_M_ABus[96:127]),      // i [0:31]
  .m_abus_4          (32'b0),                                            // i [0:31]
  .m_abus_5          (32'b0),                                            // i [0:31]
  .m_abus_6          (32'b0),                                            // i [0:31]
  .m_abus_7          (32'b0),                                            // i [0:31]
  .m_be_0            (`SYSTEM_PATH.plb_v34_0_M_BE[0:7]),           // i [0:7]
  .m_be_1            (`SYSTEM_PATH.plb_v34_0_M_BE[8:15]),          // i [0:7]
  .m_be_2            (`SYSTEM_PATH.plb_v34_0_M_BE[16:23]),         // i [0:7]
  .m_be_3            (`SYSTEM_PATH.plb_v34_0_M_BE[24:31]),         // i [0:7]
  .m_be_4            (8'b0),                                             // i [0:7]
  .m_be_5            (8'b0),                                             // i [0:7]
  .m_be_6            (8'b0),                                             // i [0:7]
  .m_be_7            (8'b0),                                             // i [0:7]
  .m_buslock_0       (`SYSTEM_PATH.plb_v34_0_M_busLock[0]),        // i
  .m_buslock_1       (`SYSTEM_PATH.plb_v34_0_M_busLock[1]),        // i
  .m_buslock_2       (`SYSTEM_PATH.plb_v34_0_M_busLock[2]),        // i
  .m_buslock_3       (`SYSTEM_PATH.plb_v34_0_M_busLock[3]),        // i
  .m_buslock_4       (1'b0),                                             // i
  .m_buslock_5       (1'b0),                                             // i
  .m_buslock_6       (1'b0),                                             // i
  .m_buslock_7       (1'b0),                                             // i
  .m_compress_0      (`SYSTEM_PATH.plb_v34_0_M_compress[0]),       // i
  .m_compress_1      (`SYSTEM_PATH.plb_v34_0_M_compress[1]),       // i
  .m_compress_2      (`SYSTEM_PATH.plb_v34_0_M_compress[2]),       // i
  .m_compress_3      (`SYSTEM_PATH.plb_v34_0_M_compress[3]),       // i
  .m_compress_4      (1'b0),                                             // i
  .m_compress_5      (1'b0),                                             // i
  .m_compress_6      (1'b0),                                             // i
  .m_compress_7      (1'b0),                                             // i
  .m_guarded_0       (`SYSTEM_PATH.plb_v34_0_M_guarded[0]),        // i
  .m_guarded_1       (`SYSTEM_PATH.plb_v34_0_M_guarded[1]),        // i
  .m_guarded_2       (`SYSTEM_PATH.plb_v34_0_M_guarded[2]),        // i
  .m_guarded_3       (`SYSTEM_PATH.plb_v34_0_M_guarded[3]),        // i
  .m_guarded_4       (1'b0),                                             // i
  .m_guarded_5       (1'b0),                                             // i
  .m_guarded_6       (1'b0),                                             // i
  .m_guarded_7       (1'b0),                                             // i
  .m_lockerr_0       (`SYSTEM_PATH.plb_v34_0_M_lockErr[0]),        // i
  .m_lockerr_1       (`SYSTEM_PATH.plb_v34_0_M_lockErr[1]),        // i
  .m_lockerr_2       (`SYSTEM_PATH.plb_v34_0_M_lockErr[2]),        // i
  .m_lockerr_3       (`SYSTEM_PATH.plb_v34_0_M_lockErr[3]),        // i
  .m_lockerr_4       (1'b0),                                             // i
  .m_lockerr_5       (1'b0),                                             // i
  .m_lockerr_6       (1'b0),                                             // i
  .m_lockerr_7       (1'b0),                                             // i
  .m_msize_0         (`SYSTEM_PATH.plb_v34_0_M_MSize[0:1]),        // i [0:1]
  .m_msize_1         (`SYSTEM_PATH.plb_v34_0_M_MSize[2:3]),        // i [0:1]
  .m_msize_2         (`SYSTEM_PATH.plb_v34_0_M_MSize[4:5]),        // i [0:1]
  .m_msize_3         (`SYSTEM_PATH.plb_v34_0_M_MSize[6:7]),        // i [0:1]
  .m_msize_4         (2'b0),                                             // i [0:1]
  .m_msize_5         (2'b0),                                             // i [0:1]
  .m_msize_6         (2'b0),                                             // i [0:1]
  .m_msize_7         (2'b0),                                             // i [0:1]
  .m_ordered_0       (`SYSTEM_PATH.plb_v34_0_M_ordered[0]),        // i
  .m_ordered_1       (`SYSTEM_PATH.plb_v34_0_M_ordered[1]),        // i
  .m_ordered_2       (`SYSTEM_PATH.plb_v34_0_M_ordered[2]),        // i
  .m_ordered_3       (`SYSTEM_PATH.plb_v34_0_M_ordered[3]),        // i
  .m_ordered_4       (1'b0),                                             // i
  .m_ordered_5       (1'b0),                                             // i
  .m_ordered_6       (1'b0),                                             // i
  .m_ordered_7       (1'b0),                                             // i
  .m_priority_0      (`SYSTEM_PATH.plb_v34_0_M_priority[0:1]),     // i [0:1]
  .m_priority_1      (`SYSTEM_PATH.plb_v34_0_M_priority[2:3]),     // i [0:1]
  .m_priority_2      (`SYSTEM_PATH.plb_v34_0_M_priority[4:5]),     // i [0:1]
  .m_priority_3      (`SYSTEM_PATH.plb_v34_0_M_priority[6:7]),     // i [0:1]
  .m_priority_4      (2'b0),                                             // i [0:1]
  .m_priority_5      (2'b0),                                             // i [0:1]
  .m_priority_6      (2'b0),                                             // i [0:1]
  .m_priority_7      (2'b0),                                             // i [0:1]
  .m_rdburst_0       (`SYSTEM_PATH.plb_v34_0_M_rdBurst[0]),        // i
  .m_rdburst_1       (`SYSTEM_PATH.plb_v34_0_M_rdBurst[1]),        // i
  .m_rdburst_2       (`SYSTEM_PATH.plb_v34_0_M_rdBurst[2]),        // i
  .m_rdburst_3       (`SYSTEM_PATH.plb_v34_0_M_rdBurst[3]),        // i
  .m_rdburst_4       (1'b0),                                             // i
  .m_rdburst_5       (1'b0),                                             // i
  .m_rdburst_6       (1'b0),                                             // i
  .m_rdburst_7       (1'b0),                                             // i
  .m_request_0       (`SYSTEM_PATH.plb_v34_0_M_request[0]),        // i
  .m_request_1       (`SYSTEM_PATH.plb_v34_0_M_request[1]),        // i
  .m_request_2       (`SYSTEM_PATH.plb_v34_0_M_request[2]),        // i
  .m_request_3       (`SYSTEM_PATH.plb_v34_0_M_request[3]),        // i
  .m_request_4       (1'b0),                                             // i
  .m_request_5       (1'b0),                                             // i
  .m_request_6       (1'b0),                                             // i
  .m_request_7       (1'b0),                                             // i
  .m_rnw_0           (`SYSTEM_PATH.plb_v34_0_M_RNW[0]),            // i
  .m_rnw_1           (`SYSTEM_PATH.plb_v34_0_M_RNW[1]),            // i
  .m_rnw_2           (`SYSTEM_PATH.plb_v34_0_M_RNW[2]),            // i
  .m_rnw_3           (`SYSTEM_PATH.plb_v34_0_M_RNW[3]),            // i
  .m_rnw_4           (1'b0),                                             // i
  .m_rnw_5           (1'b0),                                             // i
  .m_rnw_6           (1'b0),                                             // i
  .m_rnw_7           (1'b0),                                             // i
  .m_size_0          (`SYSTEM_PATH.plb_v34_0_M_size[0:3]),         // i [0:3]
  .m_size_1          (`SYSTEM_PATH.plb_v34_0_M_size[4:7]),         // i [0:3]
  .m_size_2          (`SYSTEM_PATH.plb_v34_0_M_size[8:11]),        // i [0:3]
  .m_size_3          (`SYSTEM_PATH.plb_v34_0_M_size[12:15]),       // i [0:3]
  .m_size_4          (4'b0),                                             // i [0:3]
  .m_size_5          (4'b0),                                             // i [0:3]
  .m_size_6          (4'b0),                                             // i [0:3]
  .m_size_7          (4'b0),                                             // i [0:3]
  .m_type_0          (`SYSTEM_PATH.plb_v34_0_M_type[0:2]),         // i [0:2]
  .m_type_1          (`SYSTEM_PATH.plb_v34_0_M_type[3:5]),         // i [0:2]
  .m_type_2          (`SYSTEM_PATH.plb_v34_0_M_type[6:8]),         // i [0:2]
  .m_type_3          (`SYSTEM_PATH.plb_v34_0_M_type[9:11]),        // i [0:2]
  .m_type_4          (3'b0),                                             // i [0:2]
  .m_type_5          (3'b0),                                             // i [0:2]
  .m_type_6          (3'b0),                                             // i [0:2]
  .m_type_7          (3'b0),                                             // i [0:2]
  .m_uabus_0         (32'h00000000),                                     // i [0:31]
  .m_uabus_1         (32'h00000000),                                     // i [0:31]
  .m_uabus_2         (32'h00000000),                                     // i [0:31]
  .m_uabus_3         (32'h00000000),                                     // i [0:31]
  .m_uabus_4         (32'h00000000),                                     // i [0:31]
  .m_uabus_5         (32'h00000000),                                     // i [0:31]
  .m_uabus_6         (32'h00000000),                                     // i [0:31]
  .m_uabus_7         (32'h00000000),                                     // i [0:31]
  .m_wrburst_0       (`SYSTEM_PATH.plb_v34_0_M_wrBurst[0]),        // i
  .m_wrburst_1       (`SYSTEM_PATH.plb_v34_0_M_wrBurst[1]),        // i
  .m_wrburst_2       (`SYSTEM_PATH.plb_v34_0_M_wrBurst[2]),        // i
  .m_wrburst_3       (`SYSTEM_PATH.plb_v34_0_M_wrBurst[3]),        // i
  .m_wrburst_4       (1'b0),                                             // i
  .m_wrburst_5       (1'b0),                                             // i
  .m_wrburst_6       (1'b0),                                             // i
  .m_wrburst_7       (1'b0),                                             // i
  .m_wrdbus_0        (`SYSTEM_PATH.plb_v34_0_M_wrDBus[0:63]),      // i [0:63]
  .m_wrdbus_1        (`SYSTEM_PATH.plb_v34_0_M_wrDBus[64:127]),    // i [0:63]
  .m_wrdbus_2        (`SYSTEM_PATH.plb_v34_0_M_wrDBus[128:191]),   // i [0:63]
  .m_wrdbus_3        (`SYSTEM_PATH.plb_v34_0_M_wrDBus[192:255]),   // i [0:63]
  .m_wrdbus_4        (64'b0),                                            // i [0:63]
  .m_wrdbus_5        (64'b0),                                            // i [0:63]
  .m_wrdbus_6        (64'b0),                                            // i [0:63]
  .m_wrdbus_7        (64'b0),                                            // i [0:63]
  .plb_maddrack_0    (`SYSTEM_PATH.plb_v34_0_PLB_MAddrAck[0]),     // i
  .plb_maddrack_1    (`SYSTEM_PATH.plb_v34_0_PLB_MAddrAck[1]),     // i
  .plb_maddrack_2    (`SYSTEM_PATH.plb_v34_0_PLB_MAddrAck[2]),     // i
  .plb_maddrack_3    (`SYSTEM_PATH.plb_v34_0_PLB_MAddrAck[3]),     // i
  .plb_maddrack_4    (1'b0),                                             // i
  .plb_maddrack_5    (1'b0),                                             // i
  .plb_maddrack_6    (1'b0),                                             // i
  .plb_maddrack_7    (1'b0),                                             // i
  .plb_mbusy_0       (`SYSTEM_PATH.plb_v34_0_PLB_MBusy[0]),        // i
  .plb_mbusy_1       (`SYSTEM_PATH.plb_v34_0_PLB_MBusy[1]),        // i
  .plb_mbusy_2       (`SYSTEM_PATH.plb_v34_0_PLB_MBusy[2]),        // i
  .plb_mbusy_3       (`SYSTEM_PATH.plb_v34_0_PLB_MBusy[3]),        // i
  .plb_mbusy_4       (1'b0),                                             // i
  .plb_mbusy_5       (1'b0),                                             // i
  .plb_mbusy_6       (1'b0),                                             // i
  .plb_mbusy_7       (1'b0),                                             // i
  .plb_merr_0        (`SYSTEM_PATH.plb_v34_0_PLB_MErr[0]),         // i
  .plb_merr_1        (`SYSTEM_PATH.plb_v34_0_PLB_MErr[1]),         // i
  .plb_merr_2        (`SYSTEM_PATH.plb_v34_0_PLB_MErr[2]),         // i
  .plb_merr_3        (`SYSTEM_PATH.plb_v34_0_PLB_MErr[3]),         // i
  .plb_merr_4        (1'b0),                                             // i
  .plb_merr_5        (1'b0),                                             // i
  .plb_merr_6        (1'b0),                                             // i
  .plb_merr_7        (1'b0),                                             // i
  .plb_mrdbterm_0    (`SYSTEM_PATH.plb_v34_0_PLB_MRdBTerm[0]),     // i
  .plb_mrdbterm_1    (`SYSTEM_PATH.plb_v34_0_PLB_MRdBTerm[1]),     // i
  .plb_mrdbterm_2    (`SYSTEM_PATH.plb_v34_0_PLB_MRdBTerm[2]),     // i
  .plb_mrdbterm_3    (`SYSTEM_PATH.plb_v34_0_PLB_MRdBTerm[3]),     // i
  .plb_mrdbterm_4    (1'b0),                                             // i
  .plb_mrdbterm_5    (1'b0),                                             // i
  .plb_mrdbterm_6    (1'b0),                                             // i
  .plb_mrdbterm_7    (1'b0),                                             // i
  .plb_mrddack_0     (`SYSTEM_PATH.plb_v34_0_PLB_MRdDAck[0]),      // i
  .plb_mrddack_1     (`SYSTEM_PATH.plb_v34_0_PLB_MRdDAck[1]),      // i
  .plb_mrddack_2     (`SYSTEM_PATH.plb_v34_0_PLB_MRdDAck[2]),      // i
  .plb_mrddack_3     (`SYSTEM_PATH.plb_v34_0_PLB_MRdDAck[3]),      // i
  .plb_mrddack_4     (1'b0),                                             // i
  .plb_mrddack_5     (1'b0),                                             // i
  .plb_mrddack_6     (1'b0),                                             // i
  .plb_mrddack_7     (1'b0),                                             // i
  .plb_mrddbus_0     (`SYSTEM_PATH.plb_v34_0_PLB_MRdDBus[0:63]),   // i [63:0]
  .plb_mrddbus_1     (`SYSTEM_PATH.plb_v34_0_PLB_MRdDBus[64:127]), // i [63:0]
  .plb_mrddbus_2     (`SYSTEM_PATH.plb_v34_0_PLB_MRdDBus[128:191]),  // i [63:0]
  .plb_mrddbus_3     (`SYSTEM_PATH.plb_v34_0_PLB_MRdDBus[192:255]),  // i [63:0]
  .plb_mrddbus_4     (64'b0),                                            // i [63:0]
  .plb_mrddbus_5     (64'b0),                                            // i [63:0]
  .plb_mrddbus_6     (64'b0),                                            // i [63:0]
  .plb_mrddbus_7     (64'b0),                                            // i [63:0]
  .plb_mrdwdaddr_0   (`SYSTEM_PATH.plb_v34_0_PLB_MRdWdAddr[0:3]),  // i [0:3]
  .plb_mrdwdaddr_1   (`SYSTEM_PATH.plb_v34_0_PLB_MRdWdAddr[4:7]),  // i [0:3]
  .plb_mrdwdaddr_2   (`SYSTEM_PATH.plb_v34_0_PLB_MRdWdAddr[8:11]),  // i [0:3]
  .plb_mrdwdaddr_3   (`SYSTEM_PATH.plb_v34_0_PLB_MRdWdAddr[12:15]),  // i [0:3]
  .plb_mrdwdaddr_4   (4'b0),                                             // i [0:3]
  .plb_mrdwdaddr_5   (4'b0),                                             // i [0:3]
  .plb_mrdwdaddr_6   (4'b0),                                             // i [0:3]
  .plb_mrdwdaddr_7   (4'b0),                                             // i [0:3]
  .plb_mrearbitrate_0(`SYSTEM_PATH.plb_v34_0_PLB_MRearbitrate[0]), // i
  .plb_mrearbitrate_1(`SYSTEM_PATH.plb_v34_0_PLB_MRearbitrate[1]), // i
  .plb_mrearbitrate_2(`SYSTEM_PATH.plb_v34_0_PLB_MRearbitrate[2]), // i
  .plb_mrearbitrate_3(`SYSTEM_PATH.plb_v34_0_PLB_MRearbitrate[3]), // i
  .plb_mrearbitrate_4(1'b0),                                             // i
  .plb_mrearbitrate_5(1'b0),                                             // i
  .plb_mrearbitrate_6(1'b0),                                             // i
  .plb_mrearbitrate_7(1'b0),                                             // i
  .plb_mssize_0      (`SYSTEM_PATH.plb_v34_0_PLB_MSSize[0:1]),     // i [0:1]
  .plb_mssize_1      (`SYSTEM_PATH.plb_v34_0_PLB_MSSize[2:3]),     // i [0:1]
  .plb_mssize_2      (`SYSTEM_PATH.plb_v34_0_PLB_MSSize[4:5]),     // i [0:1]
  .plb_mssize_3      (`SYSTEM_PATH.plb_v34_0_PLB_MSSize[6:7]),     // i [0:1]
  .plb_mssize_4      (2'b0),                                             // i [0:1]
  .plb_mssize_5      (2'b0),                                             // i [0:1]
  .plb_mssize_6      (2'b0),                                             // i [0:1]
  .plb_mssize_7      (2'b0),                                             // i [0:1]
  .plb_mwrbterm_0    (`SYSTEM_PATH.plb_v34_0_PLB_MWrBTerm[0]),     // i
  .plb_mwrbterm_1    (`SYSTEM_PATH.plb_v34_0_PLB_MWrBTerm[1]),     // i
  .plb_mwrbterm_2    (`SYSTEM_PATH.plb_v34_0_PLB_MWrBTerm[2]),     // i
  .plb_mwrbterm_3    (`SYSTEM_PATH.plb_v34_0_PLB_MWrBTerm[3]),     // i
  .plb_mwrbterm_4    (1'b0),                                             // i
  .plb_mwrbterm_5    (1'b0),                                             // i
  .plb_mwrbterm_6    (1'b0),                                             // i
  .plb_mwrbterm_7    (1'b0),                                             // i
  .plb_mwrdack_0     (`SYSTEM_PATH.plb_v34_0_PLB_MWrDAck[0]),      // i
  .plb_mwrdack_1     (`SYSTEM_PATH.plb_v34_0_PLB_MWrDAck[1]),      // i
  .plb_mwrdack_2     (`SYSTEM_PATH.plb_v34_0_PLB_MWrDAck[2]),      // i
  .plb_mwrdack_3     (`SYSTEM_PATH.plb_v34_0_PLB_MWrDAck[3]),      // i
  .plb_mwrdack_4     (1'b0),                                             // i
  .plb_mwrdack_5     (1'b0),                                             // i
  .plb_mwrdack_6     (1'b0),                                             // i
  .plb_mwrdack_7     (1'b0),                                             // i
  .plb_pavalid       (`SYSTEM_PATH.plb_v34_0_PLB_PAValid),         // i
  .plb_savalid       (`SYSTEM_PATH.plb_v34_0_PLB_SAValid),         // i
  .plb_rdprim        (`SYSTEM_PATH.plb_v34_0_PLB_rdPrim),          // i
  .plb_wrprim        (`SYSTEM_PATH.plb_v34_0_PLB_wrPrim),          // i
  .plb_masterid      ({1'b0,`SYSTEM_PATH.plb_v34_0_PLB_masterID}), // i [0:2]
  .plb_abort         (`SYSTEM_PATH.plb_v34_0_PLB_abort),           // i
  .plb_buslock       (`SYSTEM_PATH.plb_v34_0_PLB_busLock),         // i
  .plb_rnw           (`SYSTEM_PATH.plb_v34_0_PLB_RNW),             // i
  .plb_be            (`SYSTEM_PATH.plb_v34_0_PLB_BE),              // i [0:7]
  .plb_msize         (`SYSTEM_PATH.plb_v34_0_PLB_MSize),           // i [0:1]
  .plb_size          (`SYSTEM_PATH.plb_v34_0_PLB_size),            // i [0:3]
  .plb_type          (`SYSTEM_PATH.plb_v34_0_PLB_type),            // i [0:2]
  .plb_compress      (`SYSTEM_PATH.plb_v34_0_PLB_compress),        // i
  .plb_guarded       (`SYSTEM_PATH.plb_v34_0_PLB_guarded),         // i
  .plb_ordered       (`SYSTEM_PATH.plb_v34_0_PLB_ordered),         // i
  .plb_lockerr       (`SYSTEM_PATH.plb_v34_0_PLB_lockErr),         // i
  .plb_uabus         (32'h00000000),                                     // i [0:31]
  .plb_abus          (`SYSTEM_PATH.plb_v34_0_PLB_ABus),            // i [0:31]
  .plb_wrdbus        (`SYSTEM_PATH.plb_v34_0_PLB_wrDBus),          // i [0:63]
  .plb_wrburst       (`SYSTEM_PATH.plb_v34_0_PLB_wrBurst),         // i
  .plb_rdburst       (`SYSTEM_PATH.plb_v34_0_PLB_rdBurst),         // i
  .plb_pendreq       (`SYSTEM_PATH.plb_v34_0_PLB_pendReq),         // i
  .plb_pendpri       (`SYSTEM_PATH.plb_v34_0_PLB_pendPri),         // i [0:1]
  .plb_reqpri        (`SYSTEM_PATH.plb_v34_0_PLB_reqPri),          // i [0:1]

  .sl_addrack_0      (`SYSTEM_PATH.plb_v34_0_Sl_addrAck[0]),       // i
  .sl_addrack_1      (`SYSTEM_PATH.plb_v34_0_Sl_addrAck[1]),       // i
  .sl_addrack_2      (`SYSTEM_PATH.plb_v34_0_Sl_addrAck[2]),       // i
  .sl_addrack_3      (1'b0),                                             // i
  .sl_addrack_4      (1'b0),                                             // i
  .sl_addrack_5      (1'b0),                                             // i
  .sl_addrack_6      (1'b0),                                             // i
  .sl_addrack_7      (1'b0),                                             // i
  .sl_mbusy_0        ({`SYSTEM_PATH.plb_v34_0_Sl_MBusy[0:3],4'b0}),// i [0:7]
  .sl_mbusy_1        ({`SYSTEM_PATH.plb_v34_0_Sl_MBusy[4:7],4'b0}),// i [0:7]
  .sl_mbusy_2        ({`SYSTEM_PATH.plb_v34_0_Sl_MBusy[8:11],4'b0}),// i [0:7]
  .sl_mbusy_3        (8'b0),                                             // i [0:7]
  .sl_mbusy_4        (8'b0),                                             // i [0:7]
  .sl_mbusy_5        (8'b0),                                             // i [0:7]
  .sl_mbusy_6        (8'b0),                                             // i [0:7]
  .sl_mbusy_7        (8'b0),                                             // i [0:7]
  .sl_merr_0         ({`SYSTEM_PATH.plb_v34_0_Sl_MErr[0:3],4'b0}), // i [0:7]
  .sl_merr_1         ({`SYSTEM_PATH.plb_v34_0_Sl_MErr[4:7],4'b0}), // i [0:7]
  .sl_merr_2         ({`SYSTEM_PATH.plb_v34_0_Sl_MErr[8:11],4'b0}), // i [0:7]
  .sl_merr_3         (8'b0),                                             // i [0:7]
  .sl_merr_4         (8'b0),                                             // i [0:7]
  .sl_merr_5         (8'b0),                                             // i [0:7]
  .sl_merr_6         (8'b0),                                             // i [0:7]
  .sl_merr_7         (8'b0),                                             // i [0:7]
  .sl_rdbterm_0      (`SYSTEM_PATH.plb_v34_0_Sl_rdBTerm[0]),       // i
  .sl_rdbterm_1      (`SYSTEM_PATH.plb_v34_0_Sl_rdBTerm[1]),       // i
  .sl_rdbterm_2      (`SYSTEM_PATH.plb_v34_0_Sl_rdBTerm[2]),       // i
  .sl_rdbterm_3      (1'b0),                                             // i
  .sl_rdbterm_4      (1'b0),                                             // i
  .sl_rdbterm_5      (1'b0),                                             // i
  .sl_rdbterm_6      (1'b0),                                             // i
  .sl_rdbterm_7      (1'b0),                                             // i
  .sl_rdcomp_0       (`SYSTEM_PATH.plb_v34_0_Sl_rdComp[0]),        // i
  .sl_rdcomp_1       (`SYSTEM_PATH.plb_v34_0_Sl_rdComp[1]),        // i
  .sl_rdcomp_2       (`SYSTEM_PATH.plb_v34_0_Sl_rdComp[2]),        // i
  .sl_rdcomp_3       (1'b0),                                             // i
  .sl_rdcomp_4       (1'b0),                                             // i
  .sl_rdcomp_5       (1'b0),                                             // i
  .sl_rdcomp_6       (1'b0),                                             // i
  .sl_rdcomp_7       (1'b0),                                             // i
  .sl_rddack_0       (`SYSTEM_PATH.plb_v34_0_Sl_rdDAck[0]),        // i
  .sl_rddack_1       (`SYSTEM_PATH.plb_v34_0_Sl_rdDAck[1]),        // i
  .sl_rddack_2       (`SYSTEM_PATH.plb_v34_0_Sl_rdDAck[2]),        // i
  .sl_rddack_3       (1'b0),                                             // i
  .sl_rddack_4       (1'b0),                                             // i
  .sl_rddack_5       (1'b0),                                             // i
  .sl_rddack_6       (1'b0),                                             // i
  .sl_rddack_7       (1'b0),                                             // i
  .sl_rddbus_0       (`SYSTEM_PATH.plb_v34_0_Sl_rdDBus[0:63]),     // i [0:63]
  .sl_rddbus_1       (`SYSTEM_PATH.plb_v34_0_Sl_rdDBus[64:127]),   // i [0:63]
  .sl_rddbus_2       (`SYSTEM_PATH.plb_v34_0_Sl_rdDBus[128:191]),  // i [0:63]
  .sl_rddbus_3       (64'b0),                                            // i [0:63]
  .sl_rddbus_4       (64'b0),                                            // i [0:63]
  .sl_rddbus_5       (64'b0),                                            // i [0:63]
  .sl_rddbus_6       (64'b0),                                            // i [0:63]
  .sl_rddbus_7       (64'b0),                                            // i [0:63]
  .sl_rdwdaddr_0     (`SYSTEM_PATH.plb_v34_0_Sl_rdWdAddr[0:3]),    // i [0:3]
  .sl_rdwdaddr_1     (`SYSTEM_PATH.plb_v34_0_Sl_rdWdAddr[4:7]),    // i [0:3]
  .sl_rdwdaddr_2     (`SYSTEM_PATH.plb_v34_0_Sl_rdWdAddr[8:11]),   // i [0:3]
  .sl_rdwdaddr_3     (4'b0),                                             // i [0:3]
  .sl_rdwdaddr_4     (4'b0),                                             // i [0:3]
  .sl_rdwdaddr_5     (4'b0),                                             // i [0:3]
  .sl_rdwdaddr_6     (4'b0),                                             // i [0:3]
  .sl_rdwdaddr_7     (4'b0),                                             // i [0:3]
  .sl_rearbitrate_0  (`SYSTEM_PATH.plb_v34_0_Sl_rearbitrate[0]),   // i
  .sl_rearbitrate_1  (`SYSTEM_PATH.plb_v34_0_Sl_rearbitrate[1]),   // i
  .sl_rearbitrate_2  (`SYSTEM_PATH.plb_v34_0_Sl_rearbitrate[2]),   // i
  .sl_rearbitrate_3  (1'b0),                                             // i
  .sl_rearbitrate_4  (1'b0),                                             // i
  .sl_rearbitrate_5  (1'b0),                                             // i
  .sl_rearbitrate_6  (1'b0),                                             // i
  .sl_rearbitrate_7  (1'b0),                                             // i
  .sl_ssize_0        (`SYSTEM_PATH.plb_v34_0_Sl_SSize[0:1]),       // i [0:1]
  .sl_ssize_1        (`SYSTEM_PATH.plb_v34_0_Sl_SSize[2:3]),       // i [0:1]
  .sl_ssize_2        (`SYSTEM_PATH.plb_v34_0_Sl_SSize[4:5]),       // i [0:1]
  .sl_ssize_3        (2'b0),                                             // i [0:1]
  .sl_ssize_4        (2'b0),                                             // i [0:1]
  .sl_ssize_5        (2'b0),                                             // i [0:1]
  .sl_ssize_6        (2'b0),                                             // i [0:1]
  .sl_ssize_7        (2'b0),                                             // i [0:1]
  .sl_wait_0         (`SYSTEM_PATH.plb_v34_0_Sl_wait[0]),          // i
  .sl_wait_1         (`SYSTEM_PATH.plb_v34_0_Sl_wait[1]),          // i
  .sl_wait_2         (`SYSTEM_PATH.plb_v34_0_Sl_wait[2]),          // i
  .sl_wait_3         (1'b0),                                             // i
  .sl_wait_4         (1'b0),                                             // i
  .sl_wait_5         (1'b0),                                             // i
  .sl_wait_6         (1'b0),                                             // i
  .sl_wait_7         (1'b0),                                             // i
  .sl_wrbterm_0      (`SYSTEM_PATH.plb_v34_0_Sl_wrBTerm[0]),       // i
  .sl_wrbterm_1      (`SYSTEM_PATH.plb_v34_0_Sl_wrBTerm[1]),       // i
  .sl_wrbterm_2      (`SYSTEM_PATH.plb_v34_0_Sl_wrBTerm[2]),       // i
  .sl_wrbterm_3      (1'b0),                                             // i
  .sl_wrbterm_4      (1'b0),                                             // i
  .sl_wrbterm_5      (1'b0),                                             // i
  .sl_wrbterm_6      (1'b0),                                             // i
  .sl_wrbterm_7      (1'b0),                                             // i
  .sl_wrcomp_0       (`SYSTEM_PATH.plb_v34_0_Sl_wrComp[0]),        // i
  .sl_wrcomp_1       (`SYSTEM_PATH.plb_v34_0_Sl_wrComp[1]),        // i
  .sl_wrcomp_2       (`SYSTEM_PATH.plb_v34_0_Sl_wrComp[2]),        // i
  .sl_wrcomp_3       (1'b0),                                             // i
  .sl_wrcomp_4       (1'b0),                                             // i
  .sl_wrcomp_5       (1'b0),                                             // i
  .sl_wrcomp_6       (1'b0),                                             // i
  .sl_wrcomp_7       (1'b0),                                             // i
  .sl_wrdack_0       (`SYSTEM_PATH.plb_v34_0_Sl_wrDAck[0]),        // i
  .sl_wrdack_1       (`SYSTEM_PATH.plb_v34_0_Sl_wrDAck[1]),        // i
  .sl_wrdack_2       (`SYSTEM_PATH.plb_v34_0_Sl_wrDAck[2]),        // i
  .sl_wrdack_3       (1'b0),                                             // i
  .sl_wrdack_4       (1'b0),                                             // i
  .sl_wrdack_5       (1'b0),                                             // i
  .sl_wrdack_6       (1'b0),                                             // i
  .sl_wrdack_7       (1'b0),                                             // i
  .plb_saddrack      (`SYSTEM_PATH.plb_v34_0_PLB_SaddrAck),        // i
  .plb_swait         (`SYSTEM_PATH.plb_v34_0_PLB_Swait),           // i
  .plb_srearbitrate  (`SYSTEM_PATH.plb_v34_0_PLB_Srearbitrate),    // i
  .plb_swrdack       (`SYSTEM_PATH.plb_v34_0_PLB_SwrDAck),         // i
  .plb_swrcomp       (`SYSTEM_PATH.plb_v34_0_PLB_SwrComp),         // i
  .plb_swrbterm      (`SYSTEM_PATH.plb_v34_0_PLB_SwrBTerm),        // i
  .plb_srddbus       (`SYSTEM_PATH.plb_v34_0_PLB_SrdDBus),         // i [0:63]
  .plb_srdwdaddr     (`SYSTEM_PATH.plb_v34_0_PLB_SrdWdAddr),       // i [0:3]
  .plb_srddack       (`SYSTEM_PATH.plb_v34_0_PLB_SrdDAck),         // i
  .plb_srdcomp       (`SYSTEM_PATH.plb_v34_0_PLB_SrdComp),         // i
  .plb_srdbterm      (`SYSTEM_PATH.plb_v34_0_PLB_SrdBTerm),        // i
  .plb_smbusy        (plb_smbusy),                    // i [0:7]
  .plb_smerr         (plb_smerr),                     // i [0:7]
//  .plb_smbusy        ({`SYSTEM_PATH.plb_v34_0_PLB_SMBusy,4'b0}),   // i [0:7]
//  .plb_smerr         ({`SYSTEM_PATH.plb_v34_0_PLB_SMErr,4'b0}),    // i [0:7]
  .plb_sssize        (`SYSTEM_PATH.plb_v34_0_PLB_Sssize),          // i [0:1]
  .plb_monitor_error (plb_monitor_error),                                // o
  .synch_out         (plb_monitor_synch_out)                             // o [0:31]
  );

   
///////////////////////////////////////////////////////////////////////////
// CoreConnect Toolkit OPB Bus Monitor
///////////////////////////////////////////////////////////////////////////

// OPB Monitor is designed to check 64 bit OPB systems. Therefore, some 
// small amount of glue logic is need to connect the 64 bit OPB bus monitor
// to the 32 bit OPB.

// There are 2 Masters, so connect up OPB master
// #0 and #1 and zero out rest of the master signals

assign mx_request[0:1] = `SYSTEM_PATH.opb_v20_0_M_request[0:1];
assign mx_request[2:(`max_opb_devices-1)] = {(`max_opb_devices-2){1'b0}};

assign mx_select[0:1] = `SYSTEM_PATH.opb_v20_0_M_select[0:1];
assign mx_select[2:(`max_opb_devices-1)] = {(`max_opb_devices-2){1'b0}};

assign opb_mxgrant[0:1] = `SYSTEM_PATH.opb_v20_0_OPB_MGrant[0:1];
assign opb_mxgrant[2:(`max_opb_devices-1)] = {(`max_opb_devices-2){1'b0}};

// Convert 32 bit byte enables to 64 bit equivalents, 32 bit BEs are set
// to 1111 to prevent the OPB monitor from generating errors due
// to the fact that the two LSB address bits are always 00. Since only
// 32 bit BE transfers occur over OPB, Abus[30:31] is always 00.
assign opb_be[0:3] = ~{4{`SYSTEM_PATH.opb_v20_0_OPB_ABus[29]}};
assign opb_be[4:7] =  {4{`SYSTEM_PATH.opb_v20_0_OPB_ABus[29]}};

assign sl_xferack  = {`SYSTEM_PATH.opb_v20_0_Sl_xferAck[0:13],
                      {(`max_opb_devices-14){1'b0}}};

opb_monitor 
  #("opb_checks")
  opb_mon (
     // OPB DMA protocol not used so set DMA signals to zero
    .dma_slack       ({(`max_opb_devices){1'b0}}),
    // Masters internally gate off their M_DBus outputs instead of relying
    // on the opb bus logic to do it. Therefore, M_DBusEn can be thought
    // of as always asserted.
    .m_dbus_en       ({(`max_opb_devices){1'b1}}),
    // only have 32 bit devices, upper half of 64 bit data bus disabled
    .m_dbus_en32_63  ({(`max_opb_devices){1'b0}}),
    .mx_request      (mx_request),
    .mx_select       (mx_select),
    .opb_abus        ({`SYSTEM_PATH.opb_v20_0_OPB_ABus[0:29], 2'b0}),
    .opb_be          (opb_be),
    // OPB system only uses byte enable transfers so beAck is implicitly
    // asserted with xferAck
    .opb_beack       (`SYSTEM_PATH.opb_v20_0_OPB_beAck),
    // all masters will only generate 32 bit BE transfers. beXfer is implicitly
    // asserted with select
    .opb_bexfer      (`SYSTEM_PATH.opb_v20_0_OPB_select),
    .opb_buslock     (`SYSTEM_PATH.opb_v20_0_OPB_busLock),
    .opb_clk         (`SYSTEM_PATH.opb_clk),
    .opb_dbus        ({2{`SYSTEM_PATH.opb_v20_0_OPB_DBus}}),
    .opb_dwack       (1'b0), // slaves only respond to 32 bit BE transfers
    .opb_dwxfer      (1'b0), // masters only generate 32 bit BE transfers
    .opb_errack      (`SYSTEM_PATH.opb_v20_0_OPB_errAck),
    // slave always responds as a 32 bit byte enable capable device. Therefore
    // fwAck is implicit with xferAck
    .opb_fwack       (`SYSTEM_PATH.opb_v20_0_OPB_xferAck),
    // pretend it is a fullword (fw=1,hw=1), BE transfer to the OPB monitor so
    // it calculates seqAddr bursts correctly
    .opb_fwxfer      (`SYSTEM_PATH.opb_v20_0_OPB_select),
    .opb_hwack       (1'b0), // slaves only responds to 32 bit BE transfers
    // pretend it is a fullword (fw=1,hw=1), BE transfer to the OPB monitor so
    // it calculates seqAddr bursts correctly
    .opb_hwxfer      (`SYSTEM_PATH.opb_v20_0_OPB_select),
    .opb_mxgrant     (opb_mxgrant),
    .opb_reset       (`SYSTEM_PATH.opb_v20_0_OPB_Rst),
    .opb_retry       (`SYSTEM_PATH.opb_v20_0_OPB_retry),
    .opb_rnw         (`SYSTEM_PATH.opb_v20_0_OPB_RNW),
    .opb_seqaddr     (`SYSTEM_PATH.opb_v20_0_OPB_seqAddr),
    .opb_timeout     (`SYSTEM_PATH.opb_v20_0_OPB_timeout),
    .opb_toutsup     (`SYSTEM_PATH.opb_v20_0_OPB_toutSup),
    .opb_uabus       (32'b0),
    .opb_xferack     (`SYSTEM_PATH.opb_v20_0_OPB_xferAck),
    .sl_beack        (sl_xferack),
    // Slaves internally gate off their Sl_DBus outputs instead of relying
    // on the opb bus logic to do it. Therefore, Sl_DBusEn can be thought
    // of as always asserted.
    .sl_dbus_en      ({(`max_opb_devices){1'b1}}),
    // only have 32 bit devices, upper half of 64 bit data bus disabled
    .sl_dbus_en32_63 ({(`max_opb_devices){1'b0}}),
     // OPB DMA protocol not used so set DMA signals to zero
    .sl_dmareq       ({(`max_opb_devices){1'b0}}),
    .sl_xferack      (sl_xferack),
    .synch_in        (synch_in),
    .synch_out       (opb_monitor_synch_out)
    );

///////////////////////////////////////////////////////////////////////////
// CoreConnect Toolkit DCR Bus Monitor on the CPU
///////////////////////////////////////////////////////////////////////////

// Currently nothing is connected to CPU's DCR bus so this DCR bus monitor
// is commented out

// uncomment to include DCR monitor for CPU's DCR port

  //dcr_monitor #(4'h0) dcr_monitor0 (
  //    .synch_in       (synch_in),
  //    .synch_out      (dcr_monitor0_synch_out),
  //    .dcr_clk        (`PPC405_PATH.CPMC405CLOCK),
  //    .dcr_reset      (`PPC405_PATH.RSTC405RESETCHIP
  //                   | `PPC405_PATH.RSTC405RESETCORE
  //                   | `PPC405_PATH.RSTC405RESETSYS),
  //    .dcr_cpuack     (`PPC405_PATH.DCRC405ACK),
  //    .dcr_cpudbusin  (`PPC405_PATH.DCRC405DBUSIN),
  //    .cpu_dcrwrite   (`PPC405_PATH.C405DCRWRITE),  
  //    .cpu_dcrread    (`PPC405_PATH.C405DCRREAD),
  //    .cpu_dcrabus    (`PPC405_PATH.C405DCRABUS),
  //    .cpu_dcrdbusout (`PPC405_PATH.C405DCRDBUSOUT)
  //  );

// comment out to include DCR monitor for CPU's DCR port
assign dcr_monitor0_synch_out = 32'b0;

///////////////////////////////////////////////////////////////////////////
// CoreConnect Toolkit DCR Bus Monitor on the OPB-to-DCR Bridge's DCR Bus
///////////////////////////////////////////////////////////////////////////

dcr_monitor #(4'h1) dcr_monitor1 (
    .synch_in       (synch_in),
    .synch_out      (dcr_monitor1_synch_out),
    .dcr_clk        (`SYSTEM_PATH.opb_clk),
    .dcr_reset      (`SYSTEM_PATH.opb_v20_0_OPB_Rst),
    .dcr_cpuack     (`SYSTEM_PATH.dcr_v29_0_DCR_Ack),
    .dcr_cpudbusin  (`SYSTEM_PATH.dcr_v29_0_DCR_M_DBus),
    .cpu_dcrwrite   (`SYSTEM_PATH.dcr_v29_0_M_dcrWrite),
    .cpu_dcrread    (`SYSTEM_PATH.dcr_v29_0_M_dcrRead),
    .cpu_dcrabus    (`SYSTEM_PATH.dcr_v29_0_M_dcrABus),
    .cpu_dcrdbusout (`SYSTEM_PATH.dcr_v29_0_M_dcrDBus)
  );


///////////////////////////////////////////////////////////////////////////
// Glue Logic for Bus Monitors
///////////////////////////////////////////////////////////////////////////

`ifdef PPC_MODEL_BFM
  assign synch_in = plb_monitor_synch_out
                    | dcr_monitor0_synch_out
                    | dcr_monitor1_synch_out
                    | opb_monitor_synch_out
                    | `PPC405_PATH.synch_out;
  assign `PPC405_PATH.synch_in = synch_in;
  assign monitor_bfm_error = plb_monitor_error
                           | `PPC405_PATH.bfm_error_detect;
`else
  assign synch_in = plb_monitor_synch_out  | opb_monitor_synch_out  |
                    dcr_monitor0_synch_out | dcr_monitor1_synch_out ;
  assign monitor_bfm_error = plb_monitor_error;
`endif

  // look for synch[31] to know when to end a simulation.
  // simulations are also ended when an error is detected
  initial already_stopped = 1'b0;
  always @(posedge `SYSTEM_PATH.plb_clk) begin
     if (synch_in[31])
       $display("Synch 31 Received...Simulation Complete");
     if (~already_stopped & (synch_in[31] | monitor_bfm_error)) begin
       $display("Simulation Ended @ time = %t.", $realtime);
       $stop;
       already_stopped <= 1'b1;
     end
  end

///////////////////////////////////////////////////////////////////////////
// Swift Model Simulation Related Commands
///////////////////////////////////////////////////////////////////////////

`ifdef PPC_MODEL_SWIFT
`ifdef STRUCTURAL_SIM
   initial $lm_monitor_enable(`PPC405_PATH.IPPC405_SWIFT.I1);
`endif
   // VIEW_REG: display register updates in CPU swift model
   `include "../../data/view_reg.v"
`endif

endmodule // testbench
