/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2002 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/******************************************************************************
*
* FILENAME:
*
* xgpio_example.c
*
* DESCRIPTION:
*
* This file contains a design example using the GPIO driver (XGpio) and hardware
* device.  It does not require any other hardware devices.
*
* - Setup some discretes for input some for output
* - Set some output discretes
* - Clear some output discretes
* - Read input discretes.
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a rmm  03/13/02 First release
* 1.00a rpm  08/04/03 Removed second example and invalid macro calls
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "xbasic_types.h"
#include "xstatus.h"
#include "xparameters.h"
#include "xgpio.h"

/************************** Constant Definitions *****************************/

/*
 * Assume an abritrary system has two output discretes and one input discrete.
 * Their bitmasks are defined below. 
 */
#define GPIO_INPUT_SWITCH1     0x00000001  /* Input discrete comes from a switch */
#define GPIO_OUTPUT_LED_FAULT  0x00010000  /* LED. Illuminated if discrete set to 1 */
#define GPIO_OUTPUT_LED_POWER  0x00020000  /* LED. Illuminated if discrete set to 1 */


/**************************** Type Definitions *******************************/


/***************** Macros (Inline Functions) Definitions *********************/


/************************** Function Prototypes ******************************/


/************************** Variable Definitions *****************************/

/* Declared global so it is zeroed during boot */
static XGpio Gpio;

/************************** Function Prototypes ******************************/


/************************** Variable Definitions *****************************/

/****************************************************************************
*
* FUNCTION:
*
* GpioExample
*
* DESCRIPTION:
*
* Perform duties as described in file header using a XGpio component instance
*
* ARGUMENTS:
*
* None.
*
* RETURN VALUE:
*
* None
*
* NOTES:
*
* None
*
****************************************************************************/
void GpioExample(void)
{
    Xuint32 Data;
    XStatus Status;

    /*
     * Initialize the GPIO component
     */
    Status = XGpio_Initialize(&Gpio, XPAR_GPIO_0_DEVICE_ID);
    if (Status != XST_SUCCESS)
    {
        /* Handle error */
    }

    /*
     * Setup output enable register (only the switch is set for input,
     * the rest are output)
     *
     * Turn on power led, turn off fault led
     */
    XGpio_SetDataDirection(&Gpio, GPIO_INPUT_SWITCH1);
    XGpio_DiscreteSet(&Gpio, GPIO_OUTPUT_LED_POWER);
    XGpio_DiscreteClear(&Gpio, GPIO_OUTPUT_LED_FAULT);

    /*
     * Read state of switch
     */
    Data = XGpio_DiscreteRead(&Gpio);

    if (Data & GPIO_INPUT_SWITCH1)
    {
        /* Handle switch closed */
    }
    else
    {
        /* Handle switch open */
    }
}
