/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2002 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/******************************************************************************
*
* FILENAME:
*
* xgpio_low_level_example.c
*
* DESCRIPTION:
*
* This file contains a design example using the General Purpose I/O (GPIO) low
* level driver and hardware device.
*
* NOTES:
*
* None.
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00b jhl  04/29/02 First release
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "xbasic_types.h"
#include "xparameters.h"
#include "xgpio_l.h"

/************************** Constant Definitions *****************************/

#define LED 0x01            /* assumes bit 0 of GPIO is connected to an LED */

/**************************** Type Definitions *******************************/


/***************** Macros (Inline Functions) Definitions *********************/


/************************** Function Prototypes ******************************/


/************************** Variable Definitions *****************************/


/******************************************************************************
*
* FUNCTION:
*
* GpioLowLevelExample
*
* DESCRIPTION:
*
* The purpose of this function is to illustrate how to use the GPIO low level
* driver.
*
* ARGUMENTS:
*
* None.
*
* RETURN VALUE:
*
* None.
*
* NOTES:
*
* None.
*
******************************************************************************/
void GpioLowLevelExample()
{
    Xuint32 Data;

    /* Set the direction for all signals to be inputs except the LED output */

    XGpio_mSetDataDirection(XPAR_GPIO_0_REG_BASE, ~LED);

    /* Loop forever blinking the LED, this loop will execute so fast that the
     * blinking is not visible to the eye without a delay
     */
    while (1)
    {
        /* Read the state of the data so that only the LED state can be
         * modified
         */
        Data = XGpio_mGetDataReg(XPAR_GPIO_0_REG_BASE);

        /* Set the LED to the opposite state such that it blinks */

        if (Data & LED)
        {
            XGpio_mSetDataReg(XPAR_GPIO_0_REG_BASE, Data & ~LED);
        }
        else
        {
            XGpio_mSetDataReg(XPAR_GPIO_0_REG_BASE, Data | LED);
        }
    }
}
