//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//

module clocks
  (
   sys_clk,       // I
   sysace_clk,    // O
   sysace_clk_in, // I
   tft_clk,       // O
   dcm_rst_n,     // I
   cpu_clk,       // O
   plb_clk,       // O
   plb_clk90,     // O
   opb_clk,       // O
   dcr_clk,       // O
   ddr_fb_clk,    // I
   ddr_fb_clk90,  // O
   dcm_locked,    // O
   pci_clk,       // O
   PCI_PMC_CLK,   // O
   PCI_CB_CLK,    // O
   PCI_PMC_CLK_FB // I
   );
  input  sys_clk;
  output sysace_clk;
  input  sysace_clk_in;
  output tft_clk;
  input  dcm_rst_n;
  output cpu_clk;
  output plb_clk;
  output plb_clk90;
  output opb_clk;
  output dcr_clk;
  input  ddr_fb_clk;
  output ddr_fb_clk90;
  output dcm_locked;
  output pci_clk;
  output PCI_PMC_CLK;
  output PCI_CB_CLK;
  input  PCI_PMC_CLK_FB;

  wire    [3:0] por_ff;

  wire         dcm_rst = ~dcm_rst_n;
  wire         dcm1_rst;
  wire         dcm2_rst;
  wire         dcm3_rst;
  wire         clk_i;
  wire         clk;         // clk_i after BUFG
  wire         clk90_i;
  wire         clk90;       // clk90_i after BUFG
  wire         cpuclk_i;
  wire         cpuclk;      // cpuclk_i after BUFG
  wire         ddrclk0_i;
  wire         ddrclk0;
  wire         ddrclk90_i;
  wire         ddrclk90;
  wire         dcm1_locked;
  wire         dcm2_locked;
  wire         dcm3_locked;
  wire         dcm4_locked;
  wire         tft_clk_i;
  wire         clkpci_i;
  wire         clkpci_fb_i;
  wire         clkpci_fb;
  wire         PCI_PMC_CLK_FB_i;   
  wire         sys_clk_i;

  /////////////////////////////////////////////////////////////////////////////
  // DCM 1 Instantiation
  /////////////////////////////////////////////////////////////////////////////

  DCM dcm1
    (
     .CLK0     (clk_i),       // O
     .CLK180   (),            // O
     .CLK270   (),            // O
     .CLK2X    (),            // O
     .CLK2X180 (),            // O
     .CLK90    (clk90_i),     // O
     .CLKDV    (clkpci_i),    // O
     .CLKFB    (clk),         // I
     .CLKFX    (clkcpu_i),    // O
     .CLKFX180 (),            // O
     .CLKIN    (sys_clk_i),   // I
     .DSSEN    (1'b0),        // I
     .LOCKED   (dcm1_locked), // O
     .PSCLK    (1'b0),        // I
     .PSDONE   (),            // O
     .PSEN     (1'b0),        // I
     .PSINCDEC (1'b0),        // I
     .RST      (dcm1_rst),    // I
     .STATUS   ()             // O
     ) /* synthesis xc_props = "CLKDV_DIVIDE=3, CLKFX_MULTIPLY=3, DLL_FREQUENCY_MODE=LOW,DFS_FREQUENCY_MODE=HIGH,CLKIN_PERIOD=10" */;
  // synthesis attribute CLKDV_DIVIDE   of dcm1 is "3"
  // synthesis attribute CLKFX_MULTIPLY of dcm1 is "3"
  // synthesis attribute DLL_FREQUENCY_MODE of dcm1 is "LOW"
  // synthesis attribute DFS_FREQUENCY_MODE of dcm1 is "HIGH"
  // synthesis attribute CLKIN_PERIOD of dcm1 is "10"

/* synthesis translate_off */
// Set the clock multiplier ratio for the CPU
   defparam    dcm1.CLKDV_DIVIDE = 3;
   defparam    dcm1.CLKFX_MULTIPLY = 3;
   defparam    dcm1.DLL_FREQUENCY_MODE = "LOW";
   defparam    dcm1.DFS_FREQUENCY_MODE = "HIGH";
   defparam    dcm1.CLKIN_PERIOD = 10;
/* synthesis translate_on */
  
  /////////////////////////////////////////////////////////////////////////////
  // DCM 2 Instantiation
  /////////////////////////////////////////////////////////////////////////////

  DCM dcm2
    (
     .CLK0     (ddrclk0_i),   // O
     .CLK180   (),            // O
     .CLK270   (),            // O
     .CLK2X    (),            // O
     .CLK2X180 (),            // O
     .CLK90    (ddrclk90_i),  // O
     .CLKDV    (tft_clk_i),   // O
     .CLKFB    (ddrclk0),     // I
     .CLKFX    (),            // O
     .CLKFX180 (),            // O
     .CLKIN    (ddr_fb_clk),  // I
     .DSSEN    (1'b0),        // I
     .LOCKED   (dcm2_locked), // O
     .PSCLK    (1'b0),        // I
     .PSDONE   (),            // O
     .PSEN     (1'b0),        // I
     .PSINCDEC (1'b0),        // I
     .RST      (dcm2_rst),    // I
     .STATUS   ()             // O
     );

  // synthesis attribute DLL_FREQUENCY_MODE of dcm2 is "LOW"
  // synthesis attribute CLKOUT_PHASE_SHIFT of dcm2 is "FIXED"
  // synthesis attribute PHASE_SHIFT of dcm2 is "19"
  // synthesis attribute CLKIN_PERIOD of dcm2 is "10"
  // synthesis attribute CLKDV_DIVIDE of dcm2 is "8"

  /* synthesis translate_off */
   defparam    dcm2.DLL_FREQUENCY_MODE = "LOW";
   defparam    dcm2.CLKOUT_PHASE_SHIFT = "FIXED";
   defparam    dcm2.PHASE_SHIFT = 19;
   defparam    dcm2.CLKIN_PERIOD = 10;
   defparam    dcm2.CLKDV_DIVIDE = 8;
  /* synthesis translate_on */

  /////////////////////////////////////////////////////////////////////////////
  // DCM 3 Instantiation
  /////////////////////////////////////////////////////////////////////////////

  DCM dcm3
    (
     .CLK0     (PCI_PMC_CLK),      // O
     .CLK180   (),                 // O
     .CLK270   (),                 // O
     .CLK2X    (),                 // O
     .CLK2X180 (),                 // O
     .CLK90    (),                 // O
     .CLKDV    (),                 // O
     .CLKFB    (PCI_PMC_CLK_FB_i), // I
     .CLKFX    (),                 // O
     .CLKFX180 (),                 // O
     .CLKIN    (pci_clk),          // I
     .DSSEN    (1'b0),             // I
     .LOCKED   (dcm3_locked),      // O
     .PSCLK    (1'b0),             // I
     .PSDONE   (),                 // O
     .PSEN     (1'b0),             // I
     .PSINCDEC (1'b0),             // I
     .RST      (dcm3_rst),         // I
     .STATUS   ()                  // O
     ) /* synthesis xc_props = "DLL_FREQUENCY_MODE=LOW,CLKIN_PERIOD=30 */;

  // synthesis attribute DLL_FREQUENCY_MODE of dcm3 is "LOW"
  // synthesis attribute CLKIN_PERIOD of dcm3 is "30"

  /* synthesis translate_off */
   defparam    dcm3.DLL_FREQUENCY_MODE = "LOW";
   defparam    dcm3.CLKIN_PERIOD = 30;
  /* synthesis translate_on */

  /////////////////////////////////////////////////////////////////////////////
  // Global Buffers to drive clock outputs from DCM
  /////////////////////////////////////////////////////////////////////////////
  BUFG clk_bufg        (.I(clk_i),         .O(clk));
  BUFG clk90_bufg      (.I(clk90_i),       .O(clk90));
  BUFG clkcpu_bufg     (.I(clkcpu_i),      .O(clkcpu));
  BUFG ddrclk0_bufg    (.I(ddrclk0_i),     .O(ddrclk0));
  BUFG ddrclk90_bufg   (.I(ddrclk90_i),    .O(ddrclk90));
  BUFG tftclk_bufg     (.I(tft_clk_i),     .O(tft_clk));
  BUFG clkpci_bufg     (.I(clkpci_i),      .O(pci_clk));

  IBUFG pci_pmc_clk_fb_ibufg (.I(PCI_PMC_CLK_FB), .O(PCI_PMC_CLK_FB_i));
  IBUFG sysclk_ibufg         (.I(sys_clk),        .O(sys_clk_i));

  /////////////////////////////////////////////////////////////////////////////
  // Reset Logic
  /////////////////////////////////////////////////////////////////////////////

  // Use Free Running (Non-DCM controlled) clock to control logic to
  // generate a power-on reset pulse to the DCMs. This ensures a clean
  // startup reset pulse on the DCMs.
   FDP por_ff3 (.C(sysace_clk), .D(1'b0), .Q(por_ff[3]), .PRE(dcm_rst));
   FDP por_ff2 (.C(sysace_clk), .D(por_ff[3]), .Q(por_ff[2]), .PRE(dcm_rst));
   FDP por_ff1 (.C(sysace_clk), .D(por_ff[2]), .Q(por_ff[1]), .PRE(dcm_rst));
   FDP por_ff0 (.C(sysace_clk), .D(por_ff[1]), .Q(por_ff[0]), .PRE(dcm_rst));

   // synthesis attribute INIT of por_ff3 is "1"
   // synthesis attribute INIT of por_ff2 is "1"
   // synthesis attribute INIT of por_ff1 is "1"
   // synthesis attribute INIT of por_ff0 is "1"
   
   // synthesis translate_off
   defparam    por_ff3.INIT = 1'b1;
   defparam    por_ff2.INIT = 1'b1;
   defparam    por_ff1.INIT = 1'b1;
   defparam    por_ff0.INIT = 1'b1;
   // synthesis translate_on
   
  assign   dcm1_rst = por_ff[0];

  assign   dcm2_rst = por_ff[0] | ~dcm1_locked;

  // Ensure DCM3 reset is high for at least 3 input clock cycles.
  // This avoids a simulation timing violation.  Using sysace_clk since
  // it is roughly same period as PCI clock and is free running.

  // synthesis translate_off
  defparam dcm3_rst_ffs.INIT = 16'hFFFF;
  // synthesis translate_on
  SRL16 dcm3_rst_ffs
    (.D   (dcm2_rst),
     .CLK (sysace_clk),
     .A3  (1'b0),
     .A2  (1'b0),
     .A1  (1'b1),
     .A0  (1'b1),
     .Q   (dcm3_rst)
     );

  // synthesis attribute INIT of dcm3_rst_ffs is "FFFF"
   
  /////////////////////////////////////////////////////////////////////////////
  // Outputs
  /////////////////////////////////////////////////////////////////////////////

  assign   dcm_locked = dcm1_locked & dcm2_locked & dcm3_locked;

  assign   cpu_clk = clkcpu;
  assign   plb_clk = clk;
  assign   plb_clk90 = clk90;
  assign   opb_clk = clk;
  assign   dcr_clk = clk;
  assign   ddr_fb_clk90 = ddrclk90;
  assign   sysace_clk = sysace_clk_in;

  assign   PCI_CB_CLK = PCI_PMC_CLK;

endmodule // clocks
