/////////////////////////////////////////////////////////////////////////
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
/////////////////////////////////////////////////////////////////////////

  module misc_logic
    (
     // Signals for GPIO #1 buttons, LEDs, misc ML300 reg bits
     clk,            // I
     rst,            // I
     led_button_i,   // O [0:31]
     led_button_o,   // I [0:31]
     gp_output_o,    // I [0:31]
     plb_arb_error,  // I
     plb2opb_error,  // I
     opb2plb_error,  // I
     iic_eeprom_wp,  // O
     fpga_pwr_sig_n, // O
     opb_error,      // O
     plb_error,      // O
     case_light,     // O
     fpga_led,       // O  [0:15]
     gp_button,      // I  [0:15]
     // Signals to delay core reset
     core_reset,     // I
     core_reset_delayed, // O
     // Signals to handle external interrupts
     iic_irq_n,      // I
     iic_irq,        // O
     iic_temp_crit_n, // I
     iic_temp_crit,  // O
     phy_mii_int_n,  // I
     phy_mii_int,    // O
     phy_rst_n,      // O
     phy_rst_n_int,  // I
     resetsys,       // I
     AC97_Reset_n,   // O
     GPIO2_IO_I,     // I
     GPIO2_IO_O,     // O
     GPIO2_IO_T,     // O
     GPIO2_d_out, // I
     GPIO2_t_out, // I
     GPIO2_in    // O
     );

   // Signals for GPIO #1 buttons, LEDs, misc ML300 reg bits
   input         clk;
   input         rst;
   output [0:31] led_button_i;
   input [0:31]  led_button_o;
   input [0:31]  gp_output_o;
   input         plb_arb_error;
   input         plb2opb_error;
   input         opb2plb_error;
   output        iic_eeprom_wp;
   output        fpga_pwr_sig_n;
   output        opb_error;
   output        plb_error;
   output        case_light;
   output [0:15] fpga_led;
   input [0:15]  gp_button;
   // Signals to delay core reset
   input         core_reset;
   output        core_reset_delayed;
   // Signals to handle external interrupts
   input         iic_irq_n;
   output        iic_irq;
   input         iic_temp_crit_n;
   output        iic_temp_crit;
   input         phy_mii_int_n;
   output        phy_mii_int;
   // Ethernet reset logic
   output        phy_rst_n;
   input         phy_rst_n_int;
   input         resetsys;
   output        AC97_Reset_n;
   input  [0:31] GPIO2_IO_I;     // I
   output [0:31] GPIO2_IO_O;     // O
   output [0:31] GPIO2_IO_T;     // O
   input  [0:31] GPIO2_d_out; // I
   input  [0:31] GPIO2_t_out; // I
   output  [0:31] GPIO2_in;    // O
   
   reg        opb_error;
   reg        plb_error;
   reg        core_reset_delayed;
  
  // assign buttons/leds to second GPIO
  assign led_button_i[0:15]  = gp_button;
  assign led_button_i[16:31] = led_button_o[16:31];
  assign fpga_led = led_button_o[16:31];

  // ML300 misc regs, IIC Write Protect
  assign iic_eeprom_wp = gp_output_o[2];

  // Key/lock register, writing 0x0FF as in "off" to turn off board 
  assign fpga_pwr_sig_n = (gp_output_o[20:31] == 12'h0FF);

  // turn on case light when not under reset or if manually turned off
  assign case_light = ~rst & gp_output_o[3];

  // latch OPB error condition since error signal is just a pulse  
  always @(posedge clk)
  if (rst | gp_output_o[4])
    opb_error <= 1'b0;
  else if (plb2opb_error)
    opb_error <= 1'b1;

  // latch PLB error condition since error signal is just a pulse  
  always @(posedge clk)
  if (rst | gp_output_o[0])
    plb_error <= 1'b0;
  else if (plb_arb_error | opb2plb_error)
    plb_error <= 1'b1;

  // Don't delay core_reset
  always @(core_reset)
      core_reset_delayed <= core_reset;

  // External interrupts
  assign iic_irq = ~iic_irq_n;  
  assign iic_temp_crit = ~iic_temp_crit_n;
  assign phy_mii_int = ~phy_mii_int_n;

  // Ethernet Reset - make sure reset with system reset
  assign phy_rst_n = phy_rst_n_int & ~resetsys;

   assign AC97_Reset_n = ~core_reset;

   //invert ouput enable for SEG test code
   assign GPIO2_in = GPIO2_IO_I;
   assign GPIO2_IO_O = GPIO2_d_out;
   assign GPIO2_IO_T = GPIO2_t_out;
 
endmodule // misc_logic

