//----------------------------------------------------------------------------
// OPB to PLB Bridge In - Top Level Module
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
// 
//----------------------------------------------------------------------------
// Filename:     opb2plb_bgi.v
// 
// Description:    
//   This module is the top level file for the OPB Slave to PLB Master
//   Bridge.
//
// Design Notes:
//   - PLB Master interface supports the full 64 bit PLB protocol.
//   - OPB Slave interface supports only the 32 Bit OPB Byte Enable transfer
//     protocol
//
//
//-----------------------------------------------------------------------------
// Structure:   
// 
//            -- opb2plb_bgi.v
//
//-----------------------------------------------------------------------------
// Author:      
// History:
//   KD        7/30/01 -- EA 3 Release
//
//-----------------------------------------------------------------------------
// Naming Conventions:
//      active low signals:                     "*_n"
//      clock signals:                          "clk", "clk_div#", "clk_#x" 
//      reset signals:                          "rst", "rst_n" 
//      generics/parameters:                    "C_*" 
//      user defined types:                     "*_TYPE" 
//      state machine next state:               "*_ns" 
//      state machine current state:            "*_cs" 
//      combinatorial signals:                  "*_com" 
//      pipelined or register delay signals:    "*_d#" 
//      counter signals:                        "*cnt*"
//      clock enable signals:                   "*_ce" 
//      internal version of output port         "*_i"
//      device pins:                            "*_pin" 
//      ports:                                  - Names begin with Uppercase 
//      processes:                              "*_PROCESS" 
//      component instantiations:               "<ENTITY_>I_<#|FUNC>
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
// Module Declaration
///////////////////////////////////////////////////////////////////////////////

module opb2plb_bridge_ref (
  PLB_Clk,           // I
  PLB_Rst,           // I
  OPB_Clk,           // I
  OPB_Rst,           // I
  // Bus Error Detected - Interrupt Request
  Bus_Error_Det,     // O
  // Retry Request - Prevents deadlock between bridges
  BGI_Trans_Abort,         // I 
  // OPB Slave Signals
  OPB_ABus,          // I [0:31]
  OPB_BE,            // I [0:3]
  OPB_DBus,          // I [0:31]
  OPB_RNW,           // I
  OPB_select,        // I
  OPB_seqAddr,       // I
  Sl_DBus,           // O [0:31]
  Sl_errAck,         // O
  Sl_retry,          // O
  Sl_toutSup,        // O
  Sl_xferAck,        // O
  // PLB Master Signals
  PLB_MnAddrAck,     // I
  PLB_MnBusy,        // I
  PLB_MnErr,         // I
  PLB_MnRdBTerm,     // I
  PLB_MnRdDAck,      // I
  PLB_MnRdDBus,      // I [0:63]
  PLB_MnRdWdAddr,    // I [0:3]
  PLB_MnRearbitrate, // I
  PLB_Mnssize,       // I [0:1]
  PLB_MnWrBTerm,     // I
  PLB_MnWrDAck,      // I
  PLB_pendPri,       // I [0:1]
  PLB_pendReq,       // I
  PLB_reqPri,        // I [0:1]
  Mn_abort,          // O
  Mn_ABus,           // O [0:31]
  Mn_BE,             // O [0:7]
  Mn_busLock,        // O
  Mn_compress,       // O
  Mn_guarded,        // O
  Mn_lockErr,        // O
  Mn_msize,          // O [0:1]
  Mn_ordered,        // O
  Mn_priority,       // O [0:1]
  Mn_rdBurst,        // O
  Mn_request,        // O
  Mn_RNW,            // O
  Mn_size,           // O [0:3]
  Mn_type,           // O [0:2]
  Mn_wrBurst,        // O
  Mn_wrDBus,         // O [0:63]
  // DCR Slave Interface (For Error Status Register)
  DCR_ABus,          // I [0:9]
  DCR_DBusIn,        // I [0:31]
  DCR_Read,          // I
  DCR_Write,         // I
  DCR_Ack,           // O
  DCR_DBusOut        // O [0:31]
  );


///////////////////////////////////////////////////////////////////////////////
// Port Declarations
///////////////////////////////////////////////////////////////////////////////

  input         PLB_Clk;
  input         PLB_Rst;
  input         OPB_Clk;
  input         OPB_Rst;
  // Bus Error Detected - Interrupt Request
  output        Bus_Error_Det;
  // Retry Request - Prevents deadlock between bridges
  input         BGI_Trans_Abort;
  // OPB Slave Signals
  input  [0:31] OPB_ABus;
  input  [0:3]  OPB_BE;
  input  [0:31] OPB_DBus;
  input         OPB_RNW;
  input         OPB_select;
  input         OPB_seqAddr;
  output [0:31] Sl_DBus;
  output        Sl_errAck;
  output        Sl_retry;
  output        Sl_toutSup;
  output        Sl_xferAck;
  // PLB Master Signals
  input         PLB_MnAddrAck;
  input         PLB_MnBusy;
  input         PLB_MnErr;
  input         PLB_MnRdBTerm;
  input         PLB_MnRdDAck;
  input  [0:63] PLB_MnRdDBus;
  input  [0:3]  PLB_MnRdWdAddr;
  input         PLB_MnRearbitrate;
  input  [0:1]  PLB_Mnssize;
  input         PLB_MnWrBTerm;
  input         PLB_MnWrDAck;
  input  [0:1]  PLB_pendPri;
  input         PLB_pendReq;
  input  [0:1]  PLB_reqPri;
  output        Mn_abort;
  output [0:31] Mn_ABus;
  output [0:7]  Mn_BE;
  output        Mn_busLock;
  output        Mn_compress;
  output        Mn_guarded;
  output        Mn_lockErr;
  output [0:1]  Mn_msize;
  output        Mn_ordered;
  output [0:1]  Mn_priority;
  output        Mn_rdBurst;
  output        Mn_request;
  output        Mn_RNW;
  output [0:3]  Mn_size;
  output [0:2]  Mn_type;
  output        Mn_wrBurst;
  output [0:63] Mn_wrDBus;
  // DCR Slave Interface (For Error Status Registers)
  input  [0:9]  DCR_ABus;
  input  [0:31] DCR_DBusIn;
  input         DCR_Read;
  input         DCR_Write;
  output        DCR_Ack;
  output [0:31] DCR_DBusOut;


///////////////////////////////////////////////////////////////////////////////
// Parameter Declaration
///////////////////////////////////////////////////////////////////////////////

  parameter  C_OPB_AWIDTH     = 32;     
  parameter  C_PLB_AWIDTH     = 32;     
  parameter  C_OPB_DWIDTH     = 32;     
  parameter  C_PLB_DWIDTH     = 64;     
  parameter  C_DCR_AWIDTH     = 10;     
  parameter  C_DCR_DWIDTH     = 32;     

  parameter  C_CLK_ASYNC      = 1'b0;
  parameter  C_CLK_SAME       = 1'b1;
  parameter  C_PRECISE_ABORTS = 1'b0;

  parameter  C_NUM_ADDR_RNG  = 2;
  parameter  C_RNG0_BASEADDR = 32'h00000000;
  parameter  C_RNG0_HIGHADDR = 32'h1FFFFFFF;
  parameter  C_RNG0_ADDR_LSB = 3;
  parameter  C_RNG1_BASEADDR = 32'hF0000000;
  parameter  C_RNG1_HIGHADDR = 32'hFFFFFFFF;
  parameter  C_RNG1_ADDR_LSB = 3;

//  parameter  C_DCR_BASE_ADDR  = 10'b00_0000_1000;

///////////////////////////////////////////////////////////////////////////////
// Signal Declaration
///////////////////////////////////////////////////////////////////////////////

  reg         Bus_Error_Det;
  wire [0:31] bus2ip_Addr;
  wire [0:3]  bus2ip_BE;
  wire        bus2ip_Clk;
  wire [0:31] bus2ip_Data;
  wire        bus2ip_RdReq;
  wire        bus2ip_RdReq_i;
  wire        bus2ip_Reset;
  wire        bus2ip_SRAM_CE;
  wire        bus2ip_WrReq;
  reg         data_mux_sel;
  reg  [0:31] ip2bus_Data;
  wire        ip2bus_Error;
  wire        ip2bus_RdAck;
  wire        ip2bus_Retry;
  wire        ip2bus_ToutSup;
  wire        ip2bus_WrAck;
  reg  [0:31] Mn_ABus;
  reg  [0:7]  Mn_BE;
  reg         Mn_request;
  reg         Mn_RNW;
  reg  [0:63] Mn_wrDBus;
  wire        opb_start_xfer;
  reg         opb_start_xfer_d1;
  reg         opb_start_xfer_d2;
  reg         opb_start_xfer_d3;
  reg         opb_start_xfer_d4;
  wire        opb_xfer_done;
  wire        opb_xfer_done_i;
  reg         opb_xfer_done_i_d1;
  wire        plb_start_xfer;
  wire        plb_start_xfer_i;
  reg         plb_start_xfer_i_d1;
  reg         plb_xfer_done;
  reg         plb_xfer_done_d1;
  wire        plb_xfer_done_d1_clr;
  reg         plb_xfer_done_d2;
  reg         plb_xfer_done_d3;
  reg         rd_busy;
  reg         rd_error_det;
  reg         read_hold;
  reg         wr_busy;

  wire        SYS_plbReset;
  wire        SYS_plbClk;


  assign      SYS_plbClk = PLB_Clk;
  assign      SYS_plbReset = PLB_Rst;


///////////////////////////////////////////////////////////////////////////////
// Instantiate IPIF
///////////////////////////////////////////////////////////////////////////////
opb2plb_bgi_ipif
  #(C_NUM_ADDR_RNG,  C_RNG0_BASEADDR, C_RNG0_HIGHADDR, C_RNG0_ADDR_LSB,
    C_RNG1_BASEADDR, C_RNG1_HIGHADDR, C_RNG1_ADDR_LSB, C_PRECISE_ABORTS)
 opb2plb_bgi_ipif (
  // OPB Slave Signals
  .OPB_ABus       (OPB_ABus),       // I [0:31]
  .OPB_BE         (OPB_BE),         // I [0:3]
  .OPB_Clk        (OPB_Clk),        // I
  .OPB_DBus       (OPB_DBus),       // I [0:31]
  .OPB_RNW        (OPB_RNW),        // I
  .OPB_Rst        (OPB_Rst),        // I
  .OPB_select     (OPB_select),     // I
  .OPB_seqAddr    (OPB_seqAddr),    // I
  .Sl_DBus        (Sl_DBus),        // O [0:31]
  .Sl_DBusEn      (),               // O
  .Sl_errAck      (Sl_errAck),      // O
  .Sl_retry       (Sl_retry),       // O
  .Sl_toutSup     (Sl_toutSup),     // O
  .Sl_xferAck     (Sl_xferAck),     // O
  // IP Related Signals
  .Bus2IP_Addr    (bus2ip_Addr),    // O [0:31]
  .Bus2IP_BE      (bus2ip_BE),      // O [0:3]
  .Bus2IP_Clk     (bus2ip_Clk),     // O
  .Bus2IP_Data    (bus2ip_Data),    // O [0:31]
  .Bus2IP_RdReq   (bus2ip_RdReq_i), // O
  .Bus2IP_Reset   (bus2ip_Reset),   // O
  .Bus2IP_SRAM_CE (bus2ip_SRAM_CE), // O
  .Bus2IP_WrReq   (bus2ip_WrReq),   // O
  .IP2Bus_Data    (ip2bus_Data),    // I [0:31]
  .IP2Bus_Error   (ip2bus_Error),   // I
  .IP2Bus_RdAck   (ip2bus_RdAck),   // I
  .IP2Bus_Retry   (ip2bus_Retry),   // I
  .IP2Bus_ToutSup (ip2bus_ToutSup), // I
  .IP2Bus_WrAck   (ip2bus_WrAck)    // I
  );


///////////////////////////////////////////////////////////////////////////////
// Tie off Constants
///////////////////////////////////////////////////////////////////////////////

assign Mn_abort    = 1'b0;
assign Mn_busLock  = 1'b0;
assign Mn_compress = 1'b0;
assign Mn_guarded  = 1'b0;
assign Mn_lockErr  = 1'b0;
assign Mn_msize    = 2'b01;   // 64 bit PLB transfer
assign Mn_ordered  = 1'b0;
assign Mn_priority = 2'b00;
assign Mn_rdBurst  = 1'b0;
assign Mn_size     = 4'b0000; // non-cacheline, non-burst
assign Mn_type     = 3'b000;  // memory type
assign Mn_wrBurst  = 1'b0;

assign ip2bus_ToutSup = 1'b1;


///////////////////////////////////////////////////////////////////////////////
// OPB Side Interface Logic
///////////////////////////////////////////////////////////////////////////////
 
// Writes are fire and forget so assert busy when the transaction is pending
// over PLB
always @(posedge OPB_Clk)
  if (OPB_Rst | SYS_plbReset)
    wr_busy <= 1'b0;
  else if (ip2bus_WrAck & bus2ip_SRAM_CE)
    wr_busy <= 1'b1;
  else if (wr_busy & opb_xfer_done)
    wr_busy <= 1'b0;

always @(posedge OPB_Clk)
  if (ip2bus_Retry | ip2bus_RdAck | OPB_Rst | SYS_plbReset | ~bus2ip_SRAM_CE)
    read_hold <= 1'b0;
  else if (bus2ip_RdReq)
    read_hold <= 1'b1;

assign bus2ip_RdReq   = bus2ip_RdReq_i | read_hold;

assign ip2bus_Retry   = (bus2ip_WrReq & (rd_busy | wr_busy))
                      | (bus2ip_RdReq & (rd_busy | wr_busy
                                         | (BGI_Trans_Abort & ~opb_xfer_done)));
assign ip2bus_WrAck   = bus2ip_WrReq & ~rd_busy & ~wr_busy;
assign ip2bus_RdAck   = bus2ip_RdReq & opb_xfer_done
                      & ~rd_busy & ~wr_busy;
assign ip2bus_Error   = ip2bus_RdAck & rd_error_det;

assign opb_start_xfer = bus2ip_SRAM_CE
                      & ( ip2bus_WrAck
                        | (bus2ip_RdReq & ~(rd_busy | BGI_Trans_Abort | wr_busy)));

// Register PLB write data from OPB - also handle 64 bit data mirroring 
//always @(posedge OPB_Clk)
//  if (!wr_busy)
//    Mn_wrDBus <= {bus2ip_Data, bus2ip_Data};

// Register PLB transfer quals / write data from OPB
// Also handle 64 bit data mirroring 
always @(posedge OPB_Clk)
  if (opb_start_xfer) begin
    Mn_ABus[0:29] <= bus2ip_Addr[0:29];
    Mn_ABus[30:31]<= bus2ip_BE[0]? 2'b00 :
                     bus2ip_BE[1]? 2'b01 :
                     bus2ip_BE[2]? 2'b10 :
                                   2'b11;
    Mn_BE[0:3]    <= bus2ip_Addr[29]? 4'b0 : bus2ip_BE;
    Mn_BE[4:7]    <= bus2ip_Addr[29]? bus2ip_BE : 4'b0;
    Mn_RNW        <= bus2ip_RdReq;
    Mn_wrDBus     <= {bus2ip_Data, bus2ip_Data};
  end

// assert read busy only when a read transaction has already been sent off
// on PLB but Retry Request/Abort was asserted forcing the OPB side to abort the
// transaction
always @(posedge OPB_Clk)
  if (OPB_Rst | SYS_plbReset)
    rd_busy <= 1'b0;
//  else if (bus2ip_RdReq & ~opb_xfer_done & BGI_Trans_Abort & ~bus2ip_SRAM_CE)
  else if (read_hold & ~opb_xfer_done & ~rd_busy & ~wr_busy
           & (BGI_Trans_Abort | ~bus2ip_SRAM_CE))
    rd_busy <= 1'b1;
  else if (opb_xfer_done & ~wr_busy)
    rd_busy <= 1'b0;


///////////////////////////////////////////////////////////////////////////////
// Transfer Interface Logic
///////////////////////////////////////////////////////////////////////////////

// Resync and remove meta-stability on OPB to PLB clock domain
always @(posedge OPB_Clk)
begin
  opb_start_xfer_d1 <= opb_start_xfer;
  opb_start_xfer_d2 <= opb_start_xfer | opb_start_xfer_d1;
end

always @(negedge SYS_plbClk)
  opb_start_xfer_d3 <= opb_start_xfer_d2;
  
always @(posedge SYS_plbClk)
  opb_start_xfer_d4 <= opb_start_xfer_d3;
  
// Detect synchronous pos edge in PLB clock domain
assign #0.1 plb_start_xfer_i = (C_CLK_ASYNC)? opb_start_xfer_d4 :
                               (C_CLK_SAME)?  opb_start_xfer    :
                               opb_start_xfer_d1;

always @(posedge SYS_plbClk)
  plb_start_xfer_i_d1 <= plb_start_xfer_i;

assign plb_start_xfer = plb_start_xfer_i & ~plb_start_xfer_i_d1;

// Resync and remove meta-stability on PLB to OPB clock domain
assign plb_xfer_done_d1_clr = plb_xfer_done_d3 | OPB_Rst | SYS_plbReset;

always @(posedge plb_xfer_done_d1_clr or posedge SYS_plbClk)
  if (plb_xfer_done_d1_clr)
    plb_xfer_done_d1 <= 1'b0;
  else if (plb_xfer_done)
    plb_xfer_done_d1 <= 1'b1;

always @(negedge OPB_Clk)
  plb_xfer_done_d2 <= plb_xfer_done_d1;

always @(posedge OPB_Clk)
  plb_xfer_done_d3 <= plb_xfer_done_d2;

// Detect synchronous pos edge in OPB clock domain
assign #0.1 opb_xfer_done_i = (C_CLK_ASYNC)? plb_xfer_done_d3 : plb_xfer_done;

always @(posedge OPB_Clk)
  opb_xfer_done_i_d1 <= opb_xfer_done_i;

assign opb_xfer_done = opb_xfer_done_i & ~opb_xfer_done_i_d1;


///////////////////////////////////////////////////////////////////////////////
// PLB Side Interface Logic
///////////////////////////////////////////////////////////////////////////////

always @(posedge SYS_plbClk)
  if (PLB_MnAddrAck)
    data_mux_sel <= (PLB_Mnssize != 2'b00) & Mn_ABus[29];

always @(posedge SYS_plbClk)
  if (PLB_MnRdDAck) begin
    ip2bus_Data  <= (data_mux_sel)? PLB_MnRdDBus[32:63] : PLB_MnRdDBus[0:31];
    rd_error_det <= PLB_MnErr;
  end

always @(posedge SYS_plbClk)
  if (OPB_Rst | SYS_plbReset)
    Mn_request <= 1'b0;
  else if (plb_start_xfer)
    Mn_request <= 1'b1;
  else if (PLB_MnAddrAck)
    Mn_request <= 1'b0;

always @(posedge SYS_plbClk)
  if (OPB_Rst | SYS_plbReset)
    plb_xfer_done <= 1'b0;
  else
    plb_xfer_done <= PLB_MnRdDAck | PLB_MnWrDAck;


///////////////////////////////////////////////////////////////////////////////
// DCR Logic
///////////////////////////////////////////////////////////////////////////////

  assign DCR_Ack     = 1'b0;
  assign DCR_DBusOut = DCR_DBusIn;


///////////////////////////////////////////////////////////////////////////////
// Error Interrupt
///////////////////////////////////////////////////////////////////////////////

always @(posedge OPB_Clk)
  if (OPB_Rst | SYS_plbReset)
    Bus_Error_Det <= 1'b0;
  else if (PLB_MnBusy & PLB_MnErr)
    Bus_Error_Det <= 1'b1;

endmodule
