///////////////////////////////////////////////////////////////////////////////
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////

module par_port_ipif (
  // IPIF Signals
  Bus2IP_Addr,     // I  [0:31]
  Bus2IP_BE,       // I  [0:3]
  Bus2IP_Clk,      // I
  Bus2IP_Data,     // I  [0:31]
  Bus2IP_RdReq,    // I
  Bus2IP_Reset,    // I
  Bus2IP_SRAM_CE,  // I
  Bus2IP_WrReq,    // I
  IP2Bus_Data,     // O  [0:31]
  IP2Bus_Error,    // O
  IP2Bus_RdAck,    // O
  IP2Bus_Retry,    // O
  IP2Bus_ToutSup,  // O
  IP2Bus_WrAck,    // O
  // Parallel Port
  PP_INTR,         // I       SPP : nAck
  PP_nWAIT,        // I       SPP : Busy
  PP_USER1,        // I       SPP : Out of Paper or PError
  PP_USER3,        // I       SPP : Select
  PP_DATA,         // O [7:0] SPP : Data
  PP_DIR,          // O       Transceiver Chip Direction
  PP_HD,           // O       Transceiver Chip High Drive
  PP_nASTRB,       // O       SPP : nSelect
  PP_nDSTRB,       // O       SPP : nAutoFeed
  PP_nINIT,        // O       SPP : nInitialize 
  PP_nWRITE,       // O       SPP : nStrobe
  PP_USER2         // I       SPP : nError
  );

///////////////////////////////////////////////////////////////////////////////
// Port Declarations
///////////////////////////////////////////////////////////////////////////////

  // IPIF Signals
  input  [0:31] Bus2IP_Addr;
  input  [0:3]  Bus2IP_BE;
  input         Bus2IP_Clk;
  input  [0:31] Bus2IP_Data;
  input         Bus2IP_RdReq;
  input         Bus2IP_Reset;
  input         Bus2IP_SRAM_CE;
  input         Bus2IP_WrReq;
  output [0:31] IP2Bus_Data;
  output        IP2Bus_Error;
  output        IP2Bus_RdAck;
  output        IP2Bus_Retry;
  output        IP2Bus_ToutSup;
  output        IP2Bus_WrAck;
  // Parallel Port Signals
  input         PP_INTR;
  input         PP_nWAIT;
  input         PP_USER1;
  input         PP_USER3;
  output [7:0]  PP_DATA;
  output        PP_DIR;
  output        PP_HD;
  output        PP_nASTRB;
  output        PP_nDSTRB;
  output        PP_nINIT;
  output        PP_nWRITE;
  input         PP_USER2;

///////////////////////////////////////////////////////////////////////////////
// Signal Declarations
///////////////////////////////////////////////////////////////////////////////

  reg   [0:31]  IP2Bus_Data;
  reg    [7:0]  PP_DATA;
  reg           PP_DIR;
  reg           PP_HD;
  reg           PP_nASTRB;
  reg           PP_nDSTRB;
  reg           PP_nINIT;
  reg           PP_nWRITE;


///////////////////////////////////////////////////////////////////////////////
// Main Body of Code
///////////////////////////////////////////////////////////////////////////////

// Tie off unused IPIF signals
assign IP2Bus_Error   = 1'b0;
assign IP2Bus_Retry   = 1'b0;
assign IP2Bus_ToutSup = 1'b0;

assign IP2Bus_RdAck = Bus2IP_RdReq;
assign IP2Bus_WrAck = Bus2IP_WrReq;

always @(posedge Bus2IP_Clk)
  if (Bus2IP_Reset) begin
    PP_DATA[7:0] <= 8'b0; // SPP : Data
    PP_nASTRB    <= 1'b0; // SPP : nSelect
    PP_nINIT     <= 1'b1; // SPP : nInitialize 
    PP_nDSTRB    <= 1'b1; // SPP : nAutoFeed
    PP_nWRITE    <= 1'b1; // SPP : nStrobe
    PP_DIR       <= 1'b1; // Transceiver Chip Direction
    PP_HD        <= 1'b0; // Transceiver Chip High Drive
  end
  else if (IP2Bus_WrAck) begin
    // SPP Data Register, Base + 0 (Byte)
    if (Bus2IP_BE[0]) begin
      PP_DATA[7:0] <= Bus2IP_Data[0:7]; // SPP : Data
    end
    // SPP Status Register, Base + 1 (Byte), Writes Ignored
    //    if (Bus2IP_BE[1]) begin
    //    end
    // SPP Control Register, Base + 2 (Byte)
    if (Bus2IP_BE[2]) begin
      PP_nASTRB    <= ~Bus2IP_Data[20];    // SPP : nSelect
      PP_nINIT     <= Bus2IP_Data[21];    // SPP : nInitialize 
      PP_nDSTRB    <= ~Bus2IP_Data[22];    // SPP : nAutoFeed
      PP_nWRITE    <= ~Bus2IP_Data[23];    // SPP : nStrobe
    end
    // Misc Control Register, Base + 3 (Byte)
    if (Bus2IP_BE[3]) begin
      PP_DIR       <= 1'b1;               // Transceiver Chip Direction - Fixed (Prevents Damage)
      PP_HD        <= Bus2IP_Data[25];    // Transceiver Chip High Drive
    end
  end

always @(posedge Bus2IP_Clk) begin
  // SPP Data Register, Base + 0 (Byte)
  IP2Bus_Data[0:7]   <= PP_DATA[7:0]; // SPP : Data
  // SPP Status Register, Base + 1 (Byte)
  IP2Bus_Data[8]     <= ~PP_nWAIT;    // SPP : Busy
  IP2Bus_Data[9]     <= PP_INTR;      // SPP : nAck
  IP2Bus_Data[10]    <= PP_USER1;     // SPP : Out of Paper or PError
  IP2Bus_Data[11]    <= PP_USER3;     // SPP : Select
  IP2Bus_Data[12]    <= PP_USER2;     // SPP : nError
  IP2Bus_Data[13:15] <= 3'b0;         // SPP : Undefined
  // SPP Control Register, Base + 2 (Byte)
  IP2Bus_Data[16:17] <= 2'b0;         // SPP : Undefined
  IP2Bus_Data[18]    <= 1'b0;         // SPP : Output Mode Only
  IP2Bus_Data[19]    <= 1'b0;         // SPP : Interrupt Mode Disabled Only
  IP2Bus_Data[20]    <= ~PP_nASTRB;   // SPP : nSelect
  IP2Bus_Data[21]    <= PP_nINIT;     // SPP : nInitialize 
  IP2Bus_Data[22]    <= ~PP_nDSTRB;   // SPP : nAutoFeed
  IP2Bus_Data[23]    <= ~PP_nWRITE;   // SPP : nStrobe
  // SPP Misc Control Register, Base + 3 (Byte)
  IP2Bus_Data[24]    <= PP_DIR;       // Transceiver Chip Direction
  IP2Bus_Data[25]    <= PP_HD;        // Transceiver Chip High Drive
  IP2Bus_Data[26:31] <= 6'b0;         // SPP : Undefined
end

endmodule

