//-----------------------------------------------------------------------------
// PCI Bridge Top IPIF-Based Logic - Module
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
// 
//-----------------------------------------------------------------------------
// Filename:     pci_brg_ipif.v
// 
// Description:    
//   This module instantiates a PCI Bridge using an IPIF interface. The
//   for the initiator is taken from the Xilinx PCI Core Design Guide
//
//
//-----------------------------------------------------------------------------
// Structure:   
// 
//            -- (opb/plb/etc)_pci_brg_top.v
//                -- (opb/plb/etc)_ipif_slv_sram.v
//                -- pcim_lc_33_3_s.v
//                    -- pci_lc_i.v (PCI Core)
//                -- pci_cfg.v
//                -- pci_brg_ipif.v
//
//-----------------------------------------------------------------------------
// Author:      KD
// History:
//   KD        5/18/01 -- EA 2 Release
//
//-----------------------------------------------------------------------------
// Naming Conventions:
//      active low signals:                     "*_n"
//      clock signals:                          "clk", "clk_div#", "clk_#x" 
//      reset signals:                          "rst", "rst_n" 
//      generics/parameters:                    "C_*" 
//      user defined types:                     "*_TYPE" 
//      state machine next state:               "*_ns" 
//      state machine current state:            "*_cs" 
//      combinatorial signals:                  "*_com" 
//      pipelined or register delay signals:    "*_d#" 
//      counter signals:                        "*cnt*"
//      clock enable signals:                   "*_ce" 
//      internal version of output port         "*_i"
//      device pins:                            "*_pin" 
//      ports:                                  - Names begin with Uppercase 
//      processes:                              "*_PROCESS" 
//      component instantiations:               "<ENTITY_>I_<#|FUNC>
//-----------------------------------------------------------------------------

`timescale 1ns/1ps

///////////////////////////////////////////////////////////////////////////////
// Module Declaration
///////////////////////////////////////////////////////////////////////////////

module pci_brg_ipif (
  // PCI Signals
  ACK64Q_N,        // I
  ADDR,            // I  [31:0]
  ADDR_VLD,        // I
  B_BUSY,          // I
  BACKOFF,         // I
  BASE_HIT,        // I  [7:0]
  CFG,             // I  [255:0]
  CFG_HIT,         // I
  CFG_VLD,         // I
  CSR,             // I  [39:0]
  DEVSELQ_N,       // I
  DR_BUS,          // I
  FRAMEQ_N,        // I
  IDLE,            // I
  IRDYQ_N,         // I
  I_IDLE,          // I
  M_ADDR_N,        // I
  M_DATA,          // I
  M_DATA_VLD,      // I
  M_FAIL64,        // I
  M_SRC_EN,        // I
  PCI_CMD,         // I  [15:0]
  PERRQ_N,         // I
  REQ64Q_N,        // I
  S_CBE,           // I  [7:0]
  S_CYCLE64,       // I
  S_DATA,          // I
  S_DATA_VLD,      // I
  S_SRC_EN,        // I
  S_WRDN,          // I
  SERRQ_N,         // I
  STOPQ_N,         // I
  TIME_OUT,        // I
  TRDYQ_N,         // I
  C_READY,         // O
  C_TERM,          // O
  CFG_SELF,        // O
  COMPLETE,        // O
  INTR_N,          // O
  KEEPOUT,         // O
  M_CBE,           // O  [7:0]
  M_READY,         // O
  M_WRDN,          // O
  REQUEST64,       // O
  REQUEST,         // O
  REQUESTHOLD,     // O
  S_ABORT,         // O
  S_READY,         // O
  S_TERM,          // O
  SLOT64,          // O
  SUB_DATA,        // O  [31:0]
  ADIO,            // IO [63:0]
  // IPIF Slave Signals
  Bus2IP_Addr,     // I  [0:31]
  Bus2IP_BE,       // I  [0:3]
  Bus2IP_Clk,      // I
  Bus2IP_Data,     // I  [0:31]
  Bus2IP_RdReq,    // I
  Bus2IP_Reset,    // I
  Bus2IP_SRAM_CE,  // I
  Bus2IP_WrReq,    // I
  IP2Bus_Data,     // O  [0:31]
  IP2Bus_Error,    // O
  IP2Bus_RdAck,    // O
  IP2Bus_Retry,    // O
  IP2Bus_ToutSup,  // O
  IP2Bus_WrAck,    // O
  // IPIF Master Signals
  Bus2IP_MstData,  // I [0:31]
  Bus2IP_Error,    // I
  Bus2IP_MstRdAck, // I
  Bus2IP_MstWrAck, // I
  Bus2IP_Retry,    // I
  Bus2IP_TimeOut,  // I
  IP2Bus_Addr,     // O [0:31]
  IP2Bus_BusLock,  // O
  IP2Bus_MstData,  // O [0:31]
  IP2Bus_MstRdReq, // O
  IP2Bus_MstWrReq, // O
  IP2Bus_MstBE     // O [0:3]
  );
  // synthesis syn_noclockbuf = 1
  // synthesis syn_edif_bit_format = "%u<%i>"
  // synthesis syn_edif_scalar_format = "%u"


///////////////////////////////////////////////////////////////////////////////
// Port Declarations
///////////////////////////////////////////////////////////////////////////////

  // PCI Signals
  input         ACK64Q_N;
  input  [31:0] ADDR;
  input         ADDR_VLD;
  input         B_BUSY;
  input         BACKOFF;
  input   [7:0] BASE_HIT;
  input [255:0] CFG;
  input         CFG_HIT;
  input         CFG_VLD;
  input  [39:0] CSR;
  input         DEVSELQ_N;
  input         DR_BUS;
  input         FRAMEQ_N;
  input         IDLE;
  input         IRDYQ_N;
  input         I_IDLE;
  input         M_ADDR_N;
  input         M_DATA;
  input         M_DATA_VLD;
  input         M_FAIL64;
  input         M_SRC_EN;
  input  [15:0] PCI_CMD;
  input         PERRQ_N;
  input         REQ64Q_N;
  input   [7:0] S_CBE;
  input         S_CYCLE64;
  input         S_DATA;
  input         S_DATA_VLD;
  input         S_SRC_EN;
  input         S_WRDN;
  input         SERRQ_N;
  input         STOPQ_N;
  input         TIME_OUT;
  input         TRDYQ_N;
  output        C_READY;
  output        C_TERM;
  output        CFG_SELF;
  output        COMPLETE;
  output        INTR_N;
  output        KEEPOUT;
  output  [7:0] M_CBE;
  output        M_READY;
  output        M_WRDN;
  output        REQUEST64;
  output        REQUEST;
  output        REQUESTHOLD;
  output        S_ABORT;
  output        S_READY;
  output        S_TERM;
  output        SLOT64;
  output [31:0] SUB_DATA;
  inout  [63:0] ADIO;
  // IPIF Slave Signals
  input  [0:31] Bus2IP_Addr;
  input  [0:3]  Bus2IP_BE;
  input         Bus2IP_Clk;
  input  [0:31] Bus2IP_Data;
  input         Bus2IP_RdReq;
  input         Bus2IP_Reset;
  input         Bus2IP_SRAM_CE;
  input         Bus2IP_WrReq;
  output [0:31] IP2Bus_Data;
  output        IP2Bus_Error;
  output        IP2Bus_RdAck;
  output        IP2Bus_Retry;
  output        IP2Bus_ToutSup;
  output        IP2Bus_WrAck;
  // IPIF Master Signals
  input  [0:31] Bus2IP_MstData;
  input         Bus2IP_Error;
  input         Bus2IP_MstRdAck;
  input         Bus2IP_MstWrAck;
  input         Bus2IP_Retry;
  input         Bus2IP_TimeOut;
  output [0:31] IP2Bus_Addr;
  output        IP2Bus_BusLock;
  output [0:31] IP2Bus_MstData;
  output        IP2Bus_MstRdReq;
  output        IP2Bus_MstWrReq;
  output [0:3]  IP2Bus_MstBE;

///////////////////////////////////////////////////////////////////////////////
// Signal Declarations
///////////////////////////////////////////////////////////////////////////////

// Signals used by PCI Initiator Logic in OPB->PCI xfers
  reg          M_DATAQ;
  wire         M_DATA_FELL;
  reg          FATAL;
  reg          RETRY;
  reg  [2:0]   STATE;
  reg          COMPLETE;
  wire         ADDR_OE;
  wire [3:0]   COMMAND;
  wire [3:0]   BYTE_ENABLE;
  wire         LOAD;
  wire         OE;
  reg  [0:31]  IP2Bus_Data;
  wire         START;
  reg          DIR;
  reg  [31:0]  write_data;
  reg          M_READY;
  reg  [63:0]  addr_i;

// Signals Used for CFG Address/Data Register Pair
  reg  [31:0]  cfg_addr_reg;  
  wire [15:0]  idsel;
  wire [31:0]  addr_cfg_type0;
  wire [31:0]  addr_cfg_type1;
  wire [31:0]  addr_cfg;
  wire         cfg_addr_reg_sel;
  wire         cfg_data_reg_sel;
  wire         no_pci_xfer;

// Signals used by PCI Target Logic in PCI->OPB xfers
  reg          IP2Bus_MstRdReq;
  reg          IP2Bus_MstRdReq_hold;
  reg          IP2Bus_MstWrReq;
  reg          IP2Bus_MstWrReq_hold;
  // asserted during OPB Master transaction from req to ack
  reg          xfer_busy;
  // asserted only if transaction on PCI->OPB direction had to be aborted to 
  // prevent deadlock. This remembers that OPB transaction is still pedning
  // but the PCI side has been issued retry. This is basically now a stale
  // transaction that we are waiting to finish
  reg          xfer_retried;
  wire         gen_pci_retry;
  reg  [0:31]  IP2Bus_Addr;
  reg  [0:31]  IP2Bus_MstData;
  reg  [0:3]   IP2Bus_MstBE;
  wire         mst_rd_req_i;
  wire         mst_wr_req_i;
  reg  [4:0]   retry_holdoff;
  wire         gen_auto_retry; // after PCI retry, auto gen retry for some period of time

///////////////////////////////////////////////////////////////////////////////
// Parameter Declarations
///////////////////////////////////////////////////////////////////////////////

  parameter     IDLE_S   = 3'b000,
                REQ_S    = 3'b001,
                WRITE_S  = 3'b010,
                READ_S   = 3'b011,
                RETRY_S  = 3'b100,
                DONE_S   = 3'b101,
                DEAD_S   = 3'b110,
                RETRY_S2 = 3'b111;

///////////////////////////////////////////////////////////////////////////////
// Main Body of Code
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// PCI Initiator Functionality (OPB->PCI Transfers)
///////////////////////////////////////////////////////////////////////////////

// Edge detect for falling edge of M_DATA indicating data transfer was 
// finished
always @(posedge Bus2IP_Clk)
  if (Bus2IP_Reset) M_DATAQ <= 1'b0;
  else M_DATAQ <= M_DATA; 

assign M_DATA_FELL = !M_DATA & M_DATAQ;

// Detect PCI Retry or Abort Conditions
always @(posedge Bus2IP_Clk)
  begin : watch_status
    if (Bus2IP_Reset | (STATE == IDLE_S))
    begin
      FATAL <= 1'b0;
      RETRY <= 1'b0;
    end
    else if (!M_ADDR_N) // clear at beginning
    begin
      FATAL <= 1'b0;
      RETRY <= 1'b0; 
    end 
    else if (M_DATA) // latch until end
    begin
      FATAL <= CSR[39] | CSR[38];
      RETRY <= CSR[36];
    end
 end

// PCI Initiator Main Control State Machine. Control the process
// of requesting access to the PCI Bus, Performing the transfer,
// and handling errors/retries if they occur
always @(posedge Bus2IP_Clk)
  begin : initiator_fsm
    if (Bus2IP_Reset) STATE <= IDLE_S;
    else case (STATE)
      IDLE_S : begin
        if (START) STATE <= REQ_S;
        else STATE <= IDLE_S;
      end
      REQ_S : begin
        if (DIR) STATE <= WRITE_S;
        else STATE <= READ_S;
      end
      WRITE_S : begin
        if (M_DATA_FELL) begin
          if (FATAL) STATE <= DEAD_S;
          else if (RETRY) STATE <= RETRY_S;
          else STATE <= DONE_S;
        end
        else STATE <= WRITE_S;
      end
      READ_S : begin
        if (M_DATA_FELL) begin
          if (FATAL) STATE <= DEAD_S;
          else if (RETRY) STATE <= RETRY_S;
          else STATE <= DONE_S;
        end
        else
          STATE <= READ_S;
      end
      RETRY_S : STATE <= RETRY_S2;
      RETRY_S2 : STATE <= IDLE_S;
      DONE_S : STATE <= IDLE_S;
      DEAD_S : STATE <= IDLE_S;
      default : STATE <= IDLE_S;
    endcase
  end

always @(posedge Bus2IP_Clk)
  if (Bus2IP_Reset) M_READY <= 1'b0;
  else M_READY <= 1'b1;

always @(posedge Bus2IP_Clk)
       if (Bus2IP_Reset)   retry_holdoff <= 5'b0;
  else if (RETRY)          retry_holdoff <= 5'b11111;
  else if (|retry_holdoff) retry_holdoff <= retry_holdoff - 1;

assign gen_auto_retry = (|retry_holdoff)
 | (IP2Bus_MstWrReq_hold & (STATE == IDLE_S) & Bus2IP_SRAM_CE & ~no_pci_xfer);

assign REQUEST = (STATE == REQ_S);
assign REQUESTHOLD = 1'b0;   

assign ADDR_OE = M_ADDR_N;

// mask off bit [22] to zero in config or I/O space
assign ADIO = ADDR_OE ? 64'bz : addr_i;

// Handle Config Registers
assign idsel[0]  = (cfg_addr_reg[15:11] == 5'h0);
assign idsel[1]  = (cfg_addr_reg[15:11] == 5'h1);
assign idsel[2]  = (cfg_addr_reg[15:11] == 5'h2);
assign idsel[3]  = (cfg_addr_reg[15:11] == 5'h3);
assign idsel[4]  = (cfg_addr_reg[15:11] == 5'h4);
assign idsel[5]  = (cfg_addr_reg[15:11] == 5'h5);
assign idsel[6]  = (cfg_addr_reg[15:11] == 5'h6);
assign idsel[7]  = (cfg_addr_reg[15:11] == 5'h7);
assign idsel[8]  = (cfg_addr_reg[15:11] == 5'h8);
assign idsel[9]  = (cfg_addr_reg[15:11] == 5'h9);
assign idsel[10] = (cfg_addr_reg[15:11] == 5'hA);
assign idsel[11] = (cfg_addr_reg[15:11] == 5'hB);
assign idsel[12] = (cfg_addr_reg[15:11] == 5'hC);
assign idsel[13] = (cfg_addr_reg[15:11] == 5'hD);
assign idsel[14] = (cfg_addr_reg[15:11] == 5'hE);
assign idsel[15] = (cfg_addr_reg[15:11] == 5'hF);

assign addr_cfg_type0 = {idsel, 5'b0, cfg_addr_reg[10:2], 2'b00};
assign addr_cfg_type1 = {8'b0, cfg_addr_reg[23:2], 2'b01};
assign addr_cfg       = (cfg_addr_reg[23:16] == 8'b0)? addr_cfg_type0 : addr_cfg_type1;

assign cfg_addr_reg_sel = Bus2IP_Addr[3:29] == 27'h700_0000;
assign cfg_data_reg_sel = Bus2IP_Addr[3:29] == 27'h700_0001;
assign no_pci_xfer      = cfg_addr_reg_sel | (cfg_data_reg_sel & ~cfg_addr_reg[31]);

always @(posedge Bus2IP_Clk)
  if (cfg_addr_reg_sel & Bus2IP_WrReq) begin
    if (Bus2IP_BE[3])
      cfg_addr_reg[31]    = Bus2IP_Data[24];
    if (Bus2IP_BE[2])
      cfg_addr_reg[23:16] = Bus2IP_Data[16:23];
    if (Bus2IP_BE[1])
      cfg_addr_reg[15:8]  = Bus2IP_Data[8:15];
    if (Bus2IP_BE[0])
      cfg_addr_reg[7:2]   = Bus2IP_Data[0:5];
  end

// Generate PCI Address and mask off upper address bits
always @(posedge Bus2IP_Clk)
  if (cfg_data_reg_sel)
    addr_i[63:0] <= {32'b0, addr_cfg};
  else begin
    addr_i[63:2] <= {32'b0,
                     ((Bus2IP_Addr[3:4] == 2'b11)?  6'b0 : Bus2IP_Addr[0:5]),
                     ((Bus2IP_Addr[3:5] == 3'b111)? 1'b0 : Bus2IP_Addr[6]),
                     Bus2IP_Addr[7:29]};

    addr_i[1:0]  <= (Bus2IP_Addr[3:5] != 3'b110)? 2'b00 :
                    (Bus2IP_BE[0])? 2'b00 :
                    (Bus2IP_BE[1])? 2'b01 :
                    (Bus2IP_BE[2])? 2'b10 :
                    2'b11;
  end
     
// Generate PCI command between Mem, Config, or I/O cycles depending
// on address (PCI commands are accessed as memory mapped locations)
assign COMMAND = (Bus2IP_Addr[3:4] != 2'b11)?  {3'b011, DIR} : // Mem Space
                 (Bus2IP_Addr[3:5] == 3'b111)? {3'b101, DIR} : // Config Space
                 {3'b001, DIR}; // I/O Space

// Swap byte lanes to account for big/little endian
assign BYTE_ENABLE = ~{Bus2IP_BE[3], Bus2IP_BE[2], Bus2IP_BE[1], Bus2IP_BE[0]};
assign M_CBE[7:4] = 4'b0000;
assign M_CBE[3:0] = M_ADDR_N ? BYTE_ENABLE : COMMAND;
assign M_WRDN = DIR;

always @(posedge Bus2IP_Clk)
  begin : driving_complete
    if (Bus2IP_Reset) COMPLETE <= 1'b0;
    else
      case (STATE)
        REQ_S : COMPLETE <= 1'b1;
        READ_S : COMPLETE <= 1'b1;
        WRITE_S : COMPLETE <= 1'b1;
        default : COMPLETE <= 1'b0;
      endcase
  end

assign LOAD = (STATE == READ_S) & M_DATA_VLD;
assign OE = (STATE == WRITE_S) & M_DATA;
assign START = (Bus2IP_SRAM_CE & ~no_pci_xfer) & ~gen_auto_retry & ~IP2Bus_MstWrReq_hold;

// Pipeline read/write data and reverse data byte lanes to convert
// between big and little endian addressing
always @(posedge Bus2IP_Clk)
if (Bus2IP_Reset)
  IP2Bus_Data <= 32'b0;
else if (STATE == REQ_S)
  IP2Bus_Data <= 32'hFFFF_FFFF;
else if (STATE == IDLE_S)
  IP2Bus_Data <= {cfg_addr_reg[7:2], 2'b0, cfg_addr_reg[15:8],
                  cfg_addr_reg[23:16], cfg_addr_reg[31], 7'b0};
else if (LOAD)
  IP2Bus_Data <= {ADIO[7:0], ADIO[15:8], ADIO[23:16], ADIO[31:24]};

always @(posedge Bus2IP_Clk)
if (Bus2IP_Reset) begin
  write_data <= 32'b0;
  DIR <= 1'b0;
end 
else begin
  if (START) begin
    write_data <= {Bus2IP_Data[24:31], Bus2IP_Data[16:23],
                   Bus2IP_Data[8:15], Bus2IP_Data[0:7]};
  end
  if (Bus2IP_WrReq)
    DIR <= 1'b1;
  else if (Bus2IP_RdReq)
    DIR <= 1'b0;
end

assign ADIO = (OE)? {32'b0, write_data} : 64'bz;

// Generate Control Signals back to IPIF
assign IP2Bus_Error = (STATE == DEAD_S) & (Bus2IP_Addr[3:6] != 4'b1110);
assign IP2Bus_Retry = (STATE == RETRY_S) | (gen_auto_retry & ~no_pci_xfer & Bus2IP_SRAM_CE)
                    | (IP2Bus_MstWrReq_hold & (STATE == IDLE_S) & Bus2IP_SRAM_CE & ~no_pci_xfer);
assign IP2Bus_ToutSup = 1'b1;
assign IP2Bus_RdAck = (~DIR & ((STATE == DEAD_S) | (STATE == DONE_S)))
                    | (no_pci_xfer & Bus2IP_RdReq);
assign IP2Bus_WrAck = ( DIR & ((STATE == DEAD_S) | (STATE == DONE_S)))
                    | (no_pci_xfer & Bus2IP_WrReq);

// Must allow for self config to enable PCI Core Master Functionality
assign CFG_SELF = Bus2IP_SRAM_CE & (Bus2IP_Addr[3:6] == 4'b1111);

// Tie of unused signals
assign C_TERM = 1'b1;
assign C_READY = 1'b1;
assign REQUEST64 = 1'b0;
assign SLOT64 = 1'b0;
assign INTR_N = 1'b1;
assign KEEPOUT = 1'b0;
assign SUB_DATA = 32'b0;

///////////////////////////////////////////////////////////////////////////////
// PCI Target Functionality (PCI->OPB Transfers)
///////////////////////////////////////////////////////////////////////////////

assign mst_rd_req_i = BASE_HIT[0] & ~S_WRDN & ~Bus2IP_SRAM_CE & ~xfer_busy
                    & (PCI_CMD[6] | PCI_CMD[12] | PCI_CMD[14]);
assign mst_wr_req_i = BASE_HIT[0] &  S_WRDN & ~Bus2IP_SRAM_CE & ~xfer_busy
                    & (PCI_CMD[7] | PCI_CMD[15]);

always @(posedge Bus2IP_Clk)
begin
  IP2Bus_MstRdReq <= mst_rd_req_i;
  if (Bus2IP_Reset | Bus2IP_MstRdAck | Bus2IP_MstWrAck | Bus2IP_Retry
    | Bus2IP_TimeOut)
    IP2Bus_MstRdReq_hold <= 1'b0;
  else if (BASE_HIT[0] & ~S_WRDN & ~Bus2IP_SRAM_CE & ~xfer_busy
         & (PCI_CMD[6] | PCI_CMD[12] | PCI_CMD[14]))
    IP2Bus_MstRdReq_hold <= 1'b1;
  else if (!S_DATA)
    IP2Bus_MstRdReq_hold <= 1'b0;

  IP2Bus_MstWrReq <= mst_wr_req_i;
  if (Bus2IP_Reset | Bus2IP_MstRdAck | Bus2IP_MstWrAck | Bus2IP_Retry
    | Bus2IP_TimeOut)
    IP2Bus_MstWrReq_hold <= 1'b0;
  else if (mst_wr_req_i)
    IP2Bus_MstWrReq_hold <= 1'b1;

  if (Bus2IP_Reset | Bus2IP_MstRdAck | Bus2IP_MstWrAck | Bus2IP_Retry
    | Bus2IP_TimeOut)
    xfer_busy <= 1'b0;
  else if (IP2Bus_MstRdReq | IP2Bus_MstWrReq)
    xfer_busy <= 1'b1;

  if (Bus2IP_Reset | Bus2IP_MstRdAck | Bus2IP_MstWrAck | Bus2IP_Retry
    | Bus2IP_TimeOut)
    xfer_retried <= 1'b0;
  else if (xfer_busy & Bus2IP_SRAM_CE & ~gen_auto_retry)
    xfer_retried <= 1'b1;
    
end

assign gen_pci_retry = (Bus2IP_SRAM_CE & ~gen_auto_retry) | xfer_retried;

assign S_TERM  = ((Bus2IP_MstRdAck | Bus2IP_MstWrAck) & ~Bus2IP_Error)
               | Bus2IP_Retry | gen_pci_retry;
assign S_READY = ((Bus2IP_MstRdAck | Bus2IP_MstWrAck) & ~Bus2IP_Error)
               & ~gen_pci_retry;
assign S_ABORT = (Bus2IP_TimeOut
                | ((Bus2IP_MstRdAck | Bus2IP_MstWrAck) &  Bus2IP_Error))
               & ~gen_pci_retry;

// Read Data, Byte swap PCI to OPB bytes
assign ADIO[31:0] = (IP2Bus_MstRdReq_hold & ~xfer_retried)?
                        { Bus2IP_MstData[24:31], Bus2IP_MstData[16:23],
                          Bus2IP_MstData[8:15],  Bus2IP_MstData[0:7] } :
                        32'bz;

// Latch Write Data, Byte swap PCI to OPB bytes in data bus
always @(posedge Bus2IP_Clk)
  if (mst_wr_req_i)
    IP2Bus_MstData <= {ADIO[7:0], ADIO[15:8], ADIO[23:16], ADIO[31:24]};

always @(posedge Bus2IP_Clk)
  if (mst_wr_req_i | mst_rd_req_i) begin
    // Latch Address
    IP2Bus_Addr[0:31] <= ADDR;
    // Byte swap PCI to OPB byte enables
    IP2Bus_MstBE[0]   <= ~S_CBE[0];
    IP2Bus_MstBE[1]   <= ~S_CBE[1];
    IP2Bus_MstBE[2]   <= ~S_CBE[2];
    IP2Bus_MstBE[3]   <= ~S_CBE[3];
  end

assign IP2Bus_BusLock = 1'b0;

endmodule

