//-----------------------------------------------------------------------------
// PCI I/O Buffer and Core Wrapper - Module
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
// 
//-----------------------------------------------------------------------------
// Filename:     pcim_lc_33_3_s.v
// 
// Description:    
//   This module instantiates the I/O Buffers and PCI Core together. It
//   acts as a wrapper to connect I/O to Core. The I/O Buffers are 33 MHz
//   3.3V PCI type. A DCM is instantiated to remove skew from the PCI clock
//   being driven externally.
//
//-----------------------------------------------------------------------------
// Structure:   
// 
//            -- opb_pci_brg_top.v
//                -- pcim_lc_33_3_s.v
//                    -- pci_lc_i.v (PCI Core)
//                -- pci_cfg.v
//                -- pci_brg_ipif.v
//                -- opb_ipif_slv_sram.v
//
//-----------------------------------------------------------------------------
// Author:      KD
// History:
//   KD        5/18/01 -- EA 2 Release
//
//-----------------------------------------------------------------------------
// Naming Conventions:
//      active low signals:                     "*_n"
//      clock signals:                          "clk", "clk_div#", "clk_#x" 
//      reset signals:                          "rst", "rst_n" 
//      generics/parameters:                    "C_*" 
//      user defined types:                     "*_TYPE" 
//      state machine next state:               "*_ns" 
//      state machine current state:            "*_cs" 
//      combinatorial signals:                  "*_com" 
//      pipelined or register delay signals:    "*_d#" 
//      counter signals:                        "*cnt*"
//      clock enable signals:                   "*_ce" 
//      internal version of output port         "*_i"
//      device pins:                            "*_pin" 
//      ports:                                  - Names begin with Uppercase 
//      processes:                              "*_PROCESS" 
//      component instantiations:               "<ENTITY_>I_<#|FUNC>
//-----------------------------------------------------------------------------

`timescale 1ns/1ps

///////////////////////////////////////////////////////////////////////////////
// Module Declaration
///////////////////////////////////////////////////////////////////////////////

module pcim_lc (
  C_READY,     // I
  C_TERM,      // I
  CFG,         // I  [255:0]
  CFG_SELF,    // I
  CLK,         // I
  COMPLETE,    // I
  GNT_I,       // I
  INTR_N,      // I
  KEEPOUT,     // I
  M_CBE,       // I  [7:0]
  M_READY,     // I
  M_WRDN,      // I
  REQUEST64,   // I
  REQUEST,     // I
  REQUESTHOLD, // I
  RST,         // I
  S_ABORT,     // I
  S_READY,     // I
  S_TERM,      // I
  SLOT64,      // I
  SUB_DATA,    // I  [31:0]
  ACK64Q_N,    // O
  ADDR,        // O  [31:0]
  ADDR_VLD,    // O
  B_BUSY,      // O
  BACKOFF,     // O
  BASE_HIT,    // O  [7:0]
  CFG_HIT,     // O
  CFG_VLD,     // O
  CSR,         // O  [39:0]
  DEVSELQ_N,   // O
  DR_BUS,      // O
  FRAMEQ_N,    // O
  I_IDLE,      // O
  IDLE,        // O
  INTA_O,      // O
  IRDYQ_N,     // O
  M_ADDR_N,    // O
  M_DATA,      // O
  M_DATA_VLD,  // O
  M_FAIL64,    // O
  M_SRC_EN,    // O
  PCI_CMD,     // O  [15:0]
  PERRQ_N,     // O
  REQ64Q_N,    // O
  REQ_O,       // O
  RST_O,       // O
  S_CBE,       // O  [7:0]
  S_CYCLE64,   // O
  S_DATA,      // O
  S_DATA_VLD,  // O
  S_SRC_EN,    // O
  S_WRDN,      // O
  SERRQ_N,     // O
  STOPQ_N,     // O
  TIME_OUT,    // O
  TRDYQ_N,     // O
  ACK64_IO,    // IO
  AD_IO,       // IO  [63:0]
  ADIO,        // IO  [63:0]
  CBE_IO,      // IO  [7:0]
  DEVSEL_IO,   // IO
  FRAME_IO,    // IO
  IDSEL_IO,    // IO
  IRDY_IO,     // IO
  PAR64_IO,    // IO
  PAR_IO,      // IO
  PERR_IO,     // IO
  REQ64_IO,    // IO
  SERR_IO,     // IO
  STOP_IO,     // IO
  TRDY_IO      // IO
  );
  // synthesis syn_edif_bit_format = "%u<%i>"
  // synthesis syn_edif_scalar_format = "%u"


///////////////////////////////////////////////////////////////////////////////
// Port Declarations
///////////////////////////////////////////////////////////////////////////////

  input         C_READY;
  input         C_TERM;
  input [255:0] CFG;
  input         CFG_SELF;
  input         CLK;
  input         COMPLETE;
  input         GNT_I;
  input         INTR_N;
  input         KEEPOUT;
  input   [7:0] M_CBE;
  input         M_READY;
  input         M_WRDN;
  input         REQUEST64;
  input         REQUEST;
  input         REQUESTHOLD;
  input         RST;
  input         S_ABORT;
  input         S_READY;
  input         S_TERM;
  input         SLOT64;
  input  [31:0] SUB_DATA;

  output        ACK64Q_N;
  output [31:0] ADDR;
  output        ADDR_VLD;
  output        B_BUSY;
  output        BACKOFF;
  output  [7:0] BASE_HIT;
  output        CFG_HIT;
  output        CFG_VLD;
  output [39:0] CSR;
  output        DEVSELQ_N;
  output        DR_BUS;
  output        FRAMEQ_N;
  output        I_IDLE;
  output        IDLE;
  output        INTA_O;
  output        IRDYQ_N;
  output        M_ADDR_N;
  output        M_DATA;
  output        M_DATA_VLD;
  output        M_FAIL64;
  output        M_SRC_EN;
  output [15:0] PCI_CMD;
  output        PERRQ_N;
  output        REQ64Q_N;
  output        REQ_O;
  output        RST_O;
  output  [7:0] S_CBE;
  output        S_CYCLE64;
  output        S_DATA;
  output        S_DATA_VLD;
  output        S_SRC_EN;
  output        S_WRDN;
  output        SERRQ_N;
  output        STOPQ_N;
  output        TIME_OUT;
  output        TRDYQ_N;

  inout         ACK64_IO;
  inout  [63:0] AD_IO;
  inout  [63:0] ADIO;
  inout   [7:0] CBE_IO;
  inout         DEVSEL_IO;
  inout         FRAME_IO;
  inout         IDSEL_IO;
  inout         IRDY_IO;
  inout         PAR64_IO;
  inout         PAR_IO;
  inout         PERR_IO;
  inout         REQ64_IO;
  inout         SERR_IO;
  inout         STOP_IO;
  inout         TRDY_IO;


///////////////////////////////////////////////////////////////////////////////
// Signal Declarations
///////////////////////////////////////////////////////////////////////////////

  wire          IDSEL_IN;
  wire          GNT_IN;

///////////////////////////////////////////////////////////////////////////////
// I/O structure instantiations
///////////////////////////////////////////////////////////////////////////////
/*
IOBUF_PCI33_3 XPCI_ADB63 (.O(AD_I63),.IO(AD_IO[63]),.I(AD_O63),.T(OE_ADO_T64 ));
IOBUF_PCI33_3 XPCI_ADB62 (.O(AD_I62),.IO(AD_IO[62]),.I(AD_O62),.T(OE_ADO_T64 ));
IOBUF_PCI33_3 XPCI_ADB61 (.O(AD_I61),.IO(AD_IO[61]),.I(AD_O61),.T(OE_ADO_T64 ));
IOBUF_PCI33_3 XPCI_ADB60 (.O(AD_I60),.IO(AD_IO[60]),.I(AD_O60),.T(OE_ADO_T64 ));
IOBUF_PCI33_3 XPCI_ADB59 (.O(AD_I59),.IO(AD_IO[59]),.I(AD_O59),.T(OE_ADO_T64 ));
IOBUF_PCI33_3 XPCI_ADB58 (.O(AD_I58),.IO(AD_IO[58]),.I(AD_O58),.T(OE_ADO_T64 ));
IOBUF_PCI33_3 XPCI_ADB57 (.O(AD_I57),.IO(AD_IO[57]),.I(AD_O57),.T(OE_ADO_T64 ));
IOBUF_PCI33_3 XPCI_ADB56 (.O(AD_I56),.IO(AD_IO[56]),.I(AD_O56),.T(OE_ADO_T64 ));

IOBUF_PCI33_3 XPCI_ADB55 (.O(AD_I55),.IO(AD_IO[55]),.I(AD_O55),.T(OE_ADO_LT64));
IOBUF_PCI33_3 XPCI_ADB54 (.O(AD_I54),.IO(AD_IO[54]),.I(AD_O54),.T(OE_ADO_LT64));
IOBUF_PCI33_3 XPCI_ADB53 (.O(AD_I53),.IO(AD_IO[53]),.I(AD_O53),.T(OE_ADO_LT64));
IOBUF_PCI33_3 XPCI_ADB52 (.O(AD_I52),.IO(AD_IO[52]),.I(AD_O52),.T(OE_ADO_LT64));
IOBUF_PCI33_3 XPCI_ADB51 (.O(AD_I51),.IO(AD_IO[51]),.I(AD_O51),.T(OE_ADO_LT64));
IOBUF_PCI33_3 XPCI_ADB50 (.O(AD_I50),.IO(AD_IO[50]),.I(AD_O50),.T(OE_ADO_LT64));
IOBUF_PCI33_3 XPCI_ADB49 (.O(AD_I49),.IO(AD_IO[49]),.I(AD_O49),.T(OE_ADO_LT64));
IOBUF_PCI33_3 XPCI_ADB48 (.O(AD_I48),.IO(AD_IO[48]),.I(AD_O48),.T(OE_ADO_LT64));

IOBUF_PCI33_3 XPCI_ADB47 (.O(AD_I47),.IO(AD_IO[47]),.I(AD_O47),.T(OE_ADO_LB64));
IOBUF_PCI33_3 XPCI_ADB46 (.O(AD_I46),.IO(AD_IO[46]),.I(AD_O46),.T(OE_ADO_LB64));
IOBUF_PCI33_3 XPCI_ADB45 (.O(AD_I45),.IO(AD_IO[45]),.I(AD_O45),.T(OE_ADO_LB64));
IOBUF_PCI33_3 XPCI_ADB44 (.O(AD_I44),.IO(AD_IO[44]),.I(AD_O44),.T(OE_ADO_LB64));
IOBUF_PCI33_3 XPCI_ADB43 (.O(AD_I43),.IO(AD_IO[43]),.I(AD_O43),.T(OE_ADO_LB64));
IOBUF_PCI33_3 XPCI_ADB42 (.O(AD_I42),.IO(AD_IO[42]),.I(AD_O42),.T(OE_ADO_LB64));
IOBUF_PCI33_3 XPCI_ADB41 (.O(AD_I41),.IO(AD_IO[41]),.I(AD_O41),.T(OE_ADO_LB64));
IOBUF_PCI33_3 XPCI_ADB40 (.O(AD_I40),.IO(AD_IO[40]),.I(AD_O40),.T(OE_ADO_LB64));

IOBUF_PCI33_3 XPCI_ADB39 (.O(AD_I39),.IO(AD_IO[39]),.I(AD_O39),.T(OE_ADO_B64 ));
IOBUF_PCI33_3 XPCI_ADB38 (.O(AD_I38),.IO(AD_IO[38]),.I(AD_O38),.T(OE_ADO_B64 ));
IOBUF_PCI33_3 XPCI_ADB37 (.O(AD_I37),.IO(AD_IO[37]),.I(AD_O37),.T(OE_ADO_B64 ));
IOBUF_PCI33_3 XPCI_ADB36 (.O(AD_I36),.IO(AD_IO[36]),.I(AD_O36),.T(OE_ADO_B64 ));
IOBUF_PCI33_3 XPCI_ADB35 (.O(AD_I35),.IO(AD_IO[35]),.I(AD_O35),.T(OE_ADO_B64 ));
IOBUF_PCI33_3 XPCI_ADB34 (.O(AD_I34),.IO(AD_IO[34]),.I(AD_O34),.T(OE_ADO_B64 ));
IOBUF_PCI33_3 XPCI_ADB33 (.O(AD_I33),.IO(AD_IO[33]),.I(AD_O33),.T(OE_ADO_B64 ));
IOBUF_PCI33_3 XPCI_ADB32 (.O(AD_I32),.IO(AD_IO[32]),.I(AD_O32),.T(OE_ADO_B64 ));
*/
wire AD_I32;
wire AD_I33;
wire AD_I34;
wire AD_I35;
wire AD_I36;
wire AD_I37;
wire AD_I38;
wire AD_I39;
wire AD_I40;
wire AD_I41;
wire AD_I42;
wire AD_I43;
wire AD_I44;
wire AD_I45;
wire AD_I46;
wire AD_I47;
wire AD_I48;
wire AD_I49;
wire AD_I50;
wire AD_I51;
wire AD_I52;
wire AD_I53;
wire AD_I54;
wire AD_I55;
wire AD_I56;
wire AD_I57;
wire AD_I58;
wire AD_I59;
wire AD_I60;
wire AD_I61;
wire AD_I62;
wire AD_I63;
   
assign AD_I32 = 1'b0;
assign AD_I33 = 1'b0;
assign AD_I34 = 1'b0;
assign AD_I35 = 1'b0;
assign AD_I36 = 1'b0;
assign AD_I37 = 1'b0;
assign AD_I38 = 1'b0;
assign AD_I39 = 1'b0;
assign AD_I40 = 1'b0;
assign AD_I41 = 1'b0;
assign AD_I42 = 1'b0;
assign AD_I43 = 1'b0;
assign AD_I44 = 1'b0;
assign AD_I45 = 1'b0;
assign AD_I46 = 1'b0;
assign AD_I47 = 1'b0;
assign AD_I48 = 1'b0;
assign AD_I49 = 1'b0;
assign AD_I50 = 1'b0;
assign AD_I51 = 1'b0;
assign AD_I52 = 1'b0;
assign AD_I53 = 1'b0;
assign AD_I54 = 1'b0;
assign AD_I55 = 1'b0;
assign AD_I56 = 1'b0;
assign AD_I57 = 1'b0;
assign AD_I58 = 1'b0;
assign AD_I59 = 1'b0;
assign AD_I60 = 1'b0;
assign AD_I61 = 1'b0;
assign AD_I62 = 1'b0;
assign AD_I63 = 1'b0;

IOBUF_PCI33_3 XPCI_ADB31 (.O(AD_I31),.IO(AD_IO[31]),.I(AD_O31),.T(OE_ADO_T   ));
IOBUF_PCI33_3 XPCI_ADB30 (.O(AD_I30),.IO(AD_IO[30]),.I(AD_O30),.T(OE_ADO_T   ));
IOBUF_PCI33_3 XPCI_ADB29 (.O(AD_I29),.IO(AD_IO[29]),.I(AD_O29),.T(OE_ADO_T   ));
IOBUF_PCI33_3 XPCI_ADB28 (.O(AD_I28),.IO(AD_IO[28]),.I(AD_O28),.T(OE_ADO_T   ));
IOBUF_PCI33_3 XPCI_ADB27 (.O(AD_I27),.IO(AD_IO[27]),.I(AD_O27),.T(OE_ADO_T   ));
IOBUF_PCI33_3 XPCI_ADB26 (.O(AD_I26),.IO(AD_IO[26]),.I(AD_O26),.T(OE_ADO_T   ));
IOBUF_PCI33_3 XPCI_ADB25 (.O(AD_I25),.IO(AD_IO[25]),.I(AD_O25),.T(OE_ADO_T   ));
IOBUF_PCI33_3 XPCI_ADB24 (.O(AD_I24),.IO(AD_IO[24]),.I(AD_O24),.T(OE_ADO_T   ));

IOBUF_PCI33_3 XPCI_ADB23 (.O(AD_I23),.IO(AD_IO[23]),.I(AD_O23),.T(OE_ADO_LT  ));
IOBUF_PCI33_3 XPCI_ADB22 (.O(AD_I22),.IO(AD_IO[22]),.I(AD_O22),.T(OE_ADO_LT  ));
IOBUF_PCI33_3 XPCI_ADB21 (.O(AD_I21),.IO(AD_IO[21]),.I(AD_O21),.T(OE_ADO_LT  ));
IOBUF_PCI33_3 XPCI_ADB20 (.O(AD_I20),.IO(AD_IO[20]),.I(AD_O20),.T(OE_ADO_LT  ));
IOBUF_PCI33_3 XPCI_ADB19 (.O(AD_I19),.IO(AD_IO[19]),.I(AD_O19),.T(OE_ADO_LT  ));
IOBUF_PCI33_3 XPCI_ADB18 (.O(AD_I18),.IO(AD_IO[18]),.I(AD_O18),.T(OE_ADO_LT  ));
IOBUF_PCI33_3 XPCI_ADB17 (.O(AD_I17),.IO(AD_IO[17]),.I(AD_O17),.T(OE_ADO_LT  ));
IOBUF_PCI33_3 XPCI_ADB16 (.O(AD_I16),.IO(AD_IO[16]),.I(AD_O16),.T(OE_ADO_LT  ));

IOBUF_PCI33_3 XPCI_ADB15 (.O(AD_I15),.IO(AD_IO[15]),.I(AD_O15),.T(OE_ADO_LB  ));
IOBUF_PCI33_3 XPCI_ADB14 (.O(AD_I14),.IO(AD_IO[14]),.I(AD_O14),.T(OE_ADO_LB  ));
IOBUF_PCI33_3 XPCI_ADB13 (.O(AD_I13),.IO(AD_IO[13]),.I(AD_O13),.T(OE_ADO_LB  ));
IOBUF_PCI33_3 XPCI_ADB12 (.O(AD_I12),.IO(AD_IO[12]),.I(AD_O12),.T(OE_ADO_LB  ));
IOBUF_PCI33_3 XPCI_ADB11 (.O(AD_I11),.IO(AD_IO[11]),.I(AD_O11),.T(OE_ADO_LB  ));
IOBUF_PCI33_3 XPCI_ADB10 (.O(AD_I10),.IO(AD_IO[10]),.I(AD_O10),.T(OE_ADO_LB  ));
IOBUF_PCI33_3 XPCI_ADB9  (.O(AD_I9 ),.IO(AD_IO[9 ]),.I(AD_O9 ),.T(OE_ADO_LB  ));
IOBUF_PCI33_3 XPCI_ADB8  (.O(AD_I8 ),.IO(AD_IO[8 ]),.I(AD_O8 ),.T(OE_ADO_LB  ));

IOBUF_PCI33_3 XPCI_ADB7  (.O(AD_I7 ),.IO(AD_IO[7 ]),.I(AD_O7 ),.T(OE_ADO_B   ));
IOBUF_PCI33_3 XPCI_ADB6  (.O(AD_I6 ),.IO(AD_IO[6 ]),.I(AD_O6 ),.T(OE_ADO_B   ));
IOBUF_PCI33_3 XPCI_ADB5  (.O(AD_I5 ),.IO(AD_IO[5 ]),.I(AD_O5 ),.T(OE_ADO_B   ));
IOBUF_PCI33_3 XPCI_ADB4  (.O(AD_I4 ),.IO(AD_IO[4 ]),.I(AD_O4 ),.T(OE_ADO_B   ));
IOBUF_PCI33_3 XPCI_ADB3  (.O(AD_I3 ),.IO(AD_IO[3 ]),.I(AD_O3 ),.T(OE_ADO_B   ));
IOBUF_PCI33_3 XPCI_ADB2  (.O(AD_I2 ),.IO(AD_IO[2 ]),.I(AD_O2 ),.T(OE_ADO_B   ));
IOBUF_PCI33_3 XPCI_ADB1  (.O(AD_I1 ),.IO(AD_IO[1 ]),.I(AD_O1 ),.T(OE_ADO_B   ));
IOBUF_PCI33_3 XPCI_ADB0  (.O(AD_I0 ),.IO(AD_IO[0 ]),.I(AD_O0 ),.T(OE_ADO_B   ));

FDPE XPCI_ADQ63 (.Q(AD63),.D(AD_I63),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ62 (.Q(AD62),.D(AD_I62),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ61 (.Q(AD61),.D(AD_I61),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ60 (.Q(AD60),.D(AD_I60),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ59 (.Q(AD59),.D(AD_I59),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ58 (.Q(AD58),.D(AD_I58),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ57 (.Q(AD57),.D(AD_I57),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ56 (.Q(AD56),.D(AD_I56),.C(CLK),.CE(1'b1),.PRE(RST));

FDPE XPCI_ADQ55 (.Q(AD55),.D(AD_I55),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ54 (.Q(AD54),.D(AD_I54),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ53 (.Q(AD53),.D(AD_I53),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ52 (.Q(AD52),.D(AD_I52),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ51 (.Q(AD51),.D(AD_I51),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ50 (.Q(AD50),.D(AD_I50),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ49 (.Q(AD49),.D(AD_I49),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ48 (.Q(AD48),.D(AD_I48),.C(CLK),.CE(1'b1),.PRE(RST));

FDPE XPCI_ADQ47 (.Q(AD47),.D(AD_I47),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ46 (.Q(AD46),.D(AD_I46),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ45 (.Q(AD45),.D(AD_I45),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ44 (.Q(AD44),.D(AD_I44),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ43 (.Q(AD43),.D(AD_I43),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ42 (.Q(AD42),.D(AD_I42),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ41 (.Q(AD41),.D(AD_I41),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ40 (.Q(AD40),.D(AD_I40),.C(CLK),.CE(1'b1),.PRE(RST));

FDPE XPCI_ADQ39 (.Q(AD39),.D(AD_I39),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ38 (.Q(AD38),.D(AD_I38),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ37 (.Q(AD37),.D(AD_I37),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ36 (.Q(AD36),.D(AD_I36),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ35 (.Q(AD35),.D(AD_I35),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ34 (.Q(AD34),.D(AD_I34),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ33 (.Q(AD33),.D(AD_I33),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ32 (.Q(AD32),.D(AD_I32),.C(CLK),.CE(1'b1),.PRE(RST));

FDPE XPCI_ADQ31 (.Q(AD31),.D(AD_I31),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ30 (.Q(AD30),.D(AD_I30),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ29 (.Q(AD29),.D(AD_I29),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ28 (.Q(AD28),.D(AD_I28),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ27 (.Q(AD27),.D(AD_I27),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ26 (.Q(AD26),.D(AD_I26),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ25 (.Q(AD25),.D(AD_I25),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ24 (.Q(AD24),.D(AD_I24),.C(CLK),.CE(1'b1),.PRE(RST));

FDPE XPCI_ADQ23 (.Q(AD23),.D(AD_I23),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ22 (.Q(AD22),.D(AD_I22),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ21 (.Q(AD21),.D(AD_I21),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ20 (.Q(AD20),.D(AD_I20),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ19 (.Q(AD19),.D(AD_I19),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ18 (.Q(AD18),.D(AD_I18),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ17 (.Q(AD17),.D(AD_I17),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ16 (.Q(AD16),.D(AD_I16),.C(CLK),.CE(1'b1),.PRE(RST));

FDPE XPCI_ADQ15 (.Q(AD15),.D(AD_I15),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ14 (.Q(AD14),.D(AD_I14),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ13 (.Q(AD13),.D(AD_I13),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ12 (.Q(AD12),.D(AD_I12),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ11 (.Q(AD11),.D(AD_I11),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ10 (.Q(AD10),.D(AD_I10),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ9  (.Q(AD9 ),.D(AD_I9 ),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ8  (.Q(AD8 ),.D(AD_I8 ),.C(CLK),.CE(1'b1),.PRE(RST));

FDPE XPCI_ADQ7  (.Q(AD7 ),.D(AD_I7 ),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ6  (.Q(AD6 ),.D(AD_I6 ),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ5  (.Q(AD5 ),.D(AD_I5 ),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ4  (.Q(AD4 ),.D(AD_I4 ),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ3  (.Q(AD3 ),.D(AD_I3 ),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ2  (.Q(AD2 ),.D(AD_I2 ),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ1  (.Q(AD1 ),.D(AD_I1 ),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_ADQ0  (.Q(AD0 ),.D(AD_I0 ),.C(CLK),.CE(1'b1),.PRE(RST));

/*
IOBUF_PCI33_3 XPCI_CBB7 (.O(CBE_I7),.IO(CBE_IO[7]),.I(CBE_O7),.T(OE_CBE64));
IOBUF_PCI33_3 XPCI_CBB6 (.O(CBE_I6),.IO(CBE_IO[6]),.I(CBE_O6),.T(OE_CBE64));
IOBUF_PCI33_3 XPCI_CBB5 (.O(CBE_I5),.IO(CBE_IO[5]),.I(CBE_O5),.T(OE_CBE64));
IOBUF_PCI33_3 XPCI_CBB4 (.O(CBE_I4),.IO(CBE_IO[4]),.I(CBE_O4),.T(OE_CBE64));
*/
wire CBE_I4;
wire CBE_I5;
wire CBE_I6;
wire CBE_I7;
assign CBE_I4 = 1'b0;
assign CBE_I5 = 1'b0;
assign CBE_I6 = 1'b0;
assign CBE_I7 = 1'b0;

IOBUF_PCI33_3 XPCI_CBB3 (.O(CBE_I3),.IO(CBE_IO[3]),.I(CBE_O3),.T(OE_CBE  ));
IOBUF_PCI33_3 XPCI_CBB2 (.O(CBE_I2),.IO(CBE_IO[2]),.I(CBE_O2),.T(OE_CBE  ));
IOBUF_PCI33_3 XPCI_CBB1 (.O(CBE_I1),.IO(CBE_IO[1]),.I(CBE_O1),.T(OE_CBE  ));
IOBUF_PCI33_3 XPCI_CBB0 (.O(CBE_I0),.IO(CBE_IO[0]),.I(CBE_O0),.T(OE_CBE  ));

FDPE XPCI_CBQ7 (.Q(CBE_IN7),.D(CBE_I7),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_CBQ6 (.Q(CBE_IN6),.D(CBE_I6),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_CBQ5 (.Q(CBE_IN5),.D(CBE_I5),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_CBQ4 (.Q(CBE_IN4),.D(CBE_I4),.C(CLK),.CE(1'b1),.PRE(RST));

FDPE XPCI_CBQ3 (.Q(CBE_IN3),.D(CBE_I3),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_CBQ2 (.Q(CBE_IN2),.D(CBE_I2),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_CBQ1 (.Q(CBE_IN1),.D(CBE_I1),.C(CLK),.CE(1'b1),.PRE(RST));
FDPE XPCI_CBQ0 (.Q(CBE_IN0),.D(CBE_I0),.C(CLK),.CE(1'b1),.PRE(RST));

/*
IOBUF_PCI33_3 XPCI_PAR64    (.O(PAR64_I),.IO(PAR64_IO),
                             .I(PAR64_O),.T(OE_PAR64));
*/
wire PAR64_I;
assign PAR64_I = 1'b0;

IOBUF_PCI33_3 XPCI_PAR      (.O(PAR_I),.IO(PAR_IO),
                             .I(PAR_O),.T(OE_PAR));

IOBUF_PCI33_3 XPCI_FRAME    (.O(FRAME_I),.IO(FRAME_IO),
                             .I(FRAME_O),.T(OE_FRAME));

IOBUF_PCI33_3 XPCI_REQ64    (.O(REQ64_I),.IO(REQ64_IO),
                             .I(REQ64_O),.T(OE_REQ64));

IOBUF_PCI33_3 XPCI_TRDY     (.O(TRDY_I),.IO(TRDY_IO),
                             .I(TRDY_O),.T(OE_TRDY));

IOBUF_PCI33_3 XPCI_IRDY     (.O(IRDY_I),.IO(IRDY_IO),
                             .I(IRDY_O),.T(OE_IRDY));

IOBUF_PCI33_3 XPCI_STOP     (.O(STOP_I),.IO(STOP_IO),
                             .I(STOP_O),.T(OE_STOP));

IOBUF_PCI33_3 XPCI_DEVSEL   (.O(DEVSEL_I),.IO(DEVSEL_IO),
                             .I(DEVSEL_O),.T(OE_DEVSEL));

IOBUF_PCI33_3 XPCI_ACK64    (.O(ACK64_I),.IO(ACK64_IO),
                             .I(ACK64_O),.T(OE_ACK64));

IOBUF_PCI33_3 XPCI_PERR     (.O(PERR_I),.IO(PERR_IO),
                             .I(PERR_O),.T(OE_PERR));

IOBUF_PCI33_3 XPCI_SERR     (.O(SERR_I),.IO(SERR_IO),
                             .I( 1'b0 ),.T(OE_SERR));

wire REQ_OUT, OE_REQ;
//OBUFT_PCI33_3 XPCI_REQ      (.O(REQ_O),.T(OE_REQ),.I(REQ_OUT));
assign REQ_O = REQ_OUT | OE_REQ;

//OBUFT_PCI33_3 XPCI_INTA     (.O(INTA_O),.T(OE_INTA),.I( 1'b0 ));
wire OE_INTA;
assign INTA_O = OE_INTA;

//Turn off external IDSEL when performing PCI  configuration cycle on self
wire IDSEL_I;
IOBUF_PCI33_3  XPCI_IDSEL_I (.O(IDSEL_I), .IO(IDSEL_IO), .I(1'b0), .T(1'b1));
assign IDSEL_IN = CFG_SELF | IDSEL_I;

//IBUF_PCI33_3  XPCI_GNT    (.O(GNT_IN),.I(GNT_I));
assign GNT_IN = GNT_I;

//OBUF_PCI33_3  XPCI_RST      (.O(RST_O), .I(RST_N));
assign RST_N = ~RST;
assign RST_O = RST_N;

///////////////////////////////////////////////////////////////////////////////
// PCI Core interface instantiation
///////////////////////////////////////////////////////////////////////////////

  PCI_LC_I PCI_LC(
    .OE_ADO_T64  ( OE_ADO_T64  ),
    .OE_ADO_T    ( OE_ADO_T    ),
    .OE_ADO_LT64 ( OE_ADO_LT64 ),
    .OE_ADO_LT   ( OE_ADO_LT   ),
    .OE_ADO_LB64 ( OE_ADO_LB64 ),
    .OE_ADO_LB   ( OE_ADO_LB   ),
    .OE_ADO_B64  ( OE_ADO_B64  ),
    .OE_ADO_B    ( OE_ADO_B    ),
    .AD63 ( AD63 ),
    .AD62 ( AD62 ),
    .AD61 ( AD61 ),
    .AD60 ( AD60 ),
    .AD59 ( AD59 ),
    .AD58 ( AD58 ),
    .AD57 ( AD57 ),
    .AD56 ( AD56 ),
    .AD55 ( AD55 ),
    .AD54 ( AD54 ),
    .AD53 ( AD53 ),
    .AD52 ( AD52 ),
    .AD51 ( AD51 ),
    .AD50 ( AD50 ),
    .AD49 ( AD49 ),
    .AD48 ( AD48 ),
    .AD47 ( AD47 ),
    .AD46 ( AD46 ),
    .AD45 ( AD45 ),
    .AD44 ( AD44 ),
    .AD43 ( AD43 ),
    .AD42 ( AD42 ),
    .AD41 ( AD41 ),
    .AD40 ( AD40 ),
    .AD39 ( AD39 ),
    .AD38 ( AD38 ),
    .AD37 ( AD37 ),
    .AD36 ( AD36 ),
    .AD35 ( AD35 ),
    .AD34 ( AD34 ),
    .AD33 ( AD33 ),
    .AD32 ( AD32 ),
    .AD31 ( AD31 ),
    .AD30 ( AD30 ),
    .AD29 ( AD29 ),
    .AD28 ( AD28 ),
    .AD27 ( AD27 ),
    .AD26 ( AD26 ),
    .AD25 ( AD25 ),
    .AD24 ( AD24 ),
    .AD23 ( AD23 ),
    .AD22 ( AD22 ),
    .AD21 ( AD21 ),
    .AD20 ( AD20 ),
    .AD19 ( AD19 ),
    .AD18 ( AD18 ),
    .AD17 ( AD17 ),
    .AD16 ( AD16 ),
    .AD15 ( AD15 ),
    .AD14 ( AD14 ),
    .AD13 ( AD13 ),
    .AD12 ( AD12 ),
    .AD11 ( AD11 ),
    .AD10 ( AD10 ),
    .AD9  ( AD9  ),
    .AD8  ( AD8  ),
    .AD7  ( AD7  ),
    .AD6  ( AD6  ),
    .AD5  ( AD5  ),
    .AD4  ( AD4  ),
    .AD3  ( AD3  ),
    .AD2  ( AD2  ),
    .AD1  ( AD1  ),
    .AD0  ( AD0  ),
    .AD_O63 ( AD_O63 ),
    .AD_O62 ( AD_O62 ),
    .AD_O61 ( AD_O61 ),
    .AD_O60 ( AD_O60 ),
    .AD_O59 ( AD_O59 ),
    .AD_O58 ( AD_O58 ),
    .AD_O57 ( AD_O57 ),
    .AD_O56 ( AD_O56 ),
    .AD_O55 ( AD_O55 ),
    .AD_O54 ( AD_O54 ),
    .AD_O53 ( AD_O53 ),
    .AD_O52 ( AD_O52 ),
    .AD_O51 ( AD_O51 ),
    .AD_O50 ( AD_O50 ),
    .AD_O49 ( AD_O49 ),
    .AD_O48 ( AD_O48 ),
    .AD_O47 ( AD_O47 ),
    .AD_O46 ( AD_O46 ),
    .AD_O45 ( AD_O45 ),
    .AD_O44 ( AD_O44 ),
    .AD_O43 ( AD_O43 ),
    .AD_O42 ( AD_O42 ),
    .AD_O41 ( AD_O41 ),
    .AD_O40 ( AD_O40 ),
    .AD_O39 ( AD_O39 ),
    .AD_O38 ( AD_O38 ),
    .AD_O37 ( AD_O37 ),
    .AD_O36 ( AD_O36 ),
    .AD_O35 ( AD_O35 ),
    .AD_O34 ( AD_O34 ),
    .AD_O33 ( AD_O33 ),
    .AD_O32 ( AD_O32 ),
    .AD_O31 ( AD_O31 ),
    .AD_O30 ( AD_O30 ),
    .AD_O29 ( AD_O29 ),
    .AD_O28 ( AD_O28 ),
    .AD_O27 ( AD_O27 ),
    .AD_O26 ( AD_O26 ),
    .AD_O25 ( AD_O25 ),
    .AD_O24 ( AD_O24 ),
    .AD_O23 ( AD_O23 ),
    .AD_O22 ( AD_O22 ),
    .AD_O21 ( AD_O21 ),
    .AD_O20 ( AD_O20 ),
    .AD_O19 ( AD_O19 ),
    .AD_O18 ( AD_O18 ),
    .AD_O17 ( AD_O17 ),
    .AD_O16 ( AD_O16 ),
    .AD_O15 ( AD_O15 ),
    .AD_O14 ( AD_O14 ),
    .AD_O13 ( AD_O13 ),
    .AD_O12 ( AD_O12 ),
    .AD_O11 ( AD_O11 ),
    .AD_O10 ( AD_O10 ),
    .AD_O9  ( AD_O9  ),
    .AD_O8  ( AD_O8  ),
    .AD_O7  ( AD_O7  ),
    .AD_O6  ( AD_O6  ),
    .AD_O5  ( AD_O5  ),
    .AD_O4  ( AD_O4  ),
    .AD_O3  ( AD_O3  ),
    .AD_O2  ( AD_O2  ),
    .AD_O1  ( AD_O1  ),
    .AD_O0  ( AD_O0  ),

    .OE_CBE64  ( OE_CBE64 ),
    .OE_CBE    ( OE_CBE   ),
    .CBE_I7  ( CBE_I7  ),
    .CBE_I6  ( CBE_I6  ),
    .CBE_I5  ( CBE_I5  ),
    .CBE_I4  ( CBE_I4  ),
    .CBE_I3  ( CBE_I3  ),
    .CBE_I2  ( CBE_I2  ),
    .CBE_I1  ( CBE_I1  ),
    .CBE_I0  ( CBE_I0  ),
    .CBE_IN7 ( CBE_IN7 ),
    .CBE_IN6 ( CBE_IN6 ),
    .CBE_IN5 ( CBE_IN5 ),
    .CBE_IN4 ( CBE_IN4 ),
    .CBE_IN3 ( CBE_IN3 ),
    .CBE_IN2 ( CBE_IN2 ),
    .CBE_IN1 ( CBE_IN1 ),
    .CBE_IN0 ( CBE_IN0 ),
    .CBE_O7  ( CBE_O7  ),
    .CBE_O6  ( CBE_O6  ),
    .CBE_O5  ( CBE_O5  ),
    .CBE_O4  ( CBE_O4  ),
    .CBE_O3  ( CBE_O3  ),
    .CBE_O2  ( CBE_O2  ),
    .CBE_O1  ( CBE_O1  ),
    .CBE_O0  ( CBE_O0  ),

    .OE_PAR64( OE_PAR64 ),
    .PAR64_I ( PAR64_I ),
    .PAR64_O ( PAR64_O ),

    .OE_PAR ( OE_PAR ),
    .PAR_I ( PAR_I ),
    .PAR_O ( PAR_O ),

    .OE_FRAME ( OE_FRAME ),
    .FRAME_I ( FRAME_I ),
    .FRAME_O ( FRAME_O ),

    .OE_REQ64 ( OE_REQ64 ),
    .REQ64_I ( REQ64_I ),
    .REQ64_O ( REQ64_O ),

    .OE_TRDY ( OE_TRDY ),
    .TRDY_I ( TRDY_I ),
    .TRDY_O ( TRDY_O ),

    .OE_IRDY ( OE_IRDY ),
    .IRDY_I ( IRDY_I ),
    .IRDY_O ( IRDY_O ),

    .OE_STOP ( OE_STOP ),
    .STOP_I ( STOP_I ),
    .STOP_O ( STOP_O ),

    .OE_DEVSEL ( OE_DEVSEL ),
    .DEVSEL_I ( DEVSEL_I ),
    .DEVSEL_O ( DEVSEL_O ),

    .OE_ACK64 ( OE_ACK64 ),
    .ACK64_I ( ACK64_I ),
    .ACK64_O ( ACK64_O ),

    .IDSEL_IN ( IDSEL_IN ),

    .OE_INTA ( OE_INTA ),

    .OE_PERR ( OE_PERR ),
    .PERR_I ( PERR_I ),
    .PERR_O ( PERR_O ),

    .OE_SERR ( OE_SERR ),
    .SERR_I ( SERR_I ),

    .OE_REQ ( OE_REQ ),
    .REQ_OUT ( REQ_OUT ),

    .GNT_IN ( GNT_IN ),

    .RST_N ( RST_N ),

    .CFG255 ( CFG[255] ),
    .CFG254 ( CFG[254] ),
    .CFG253 ( CFG[253] ),
    .CFG252 ( CFG[252] ),
    .CFG251 ( CFG[251] ),
    .CFG250 ( CFG[250] ),
    .CFG249 ( CFG[249] ),
    .CFG248 ( CFG[248] ),
    .CFG247 ( CFG[247] ),
    .CFG246 ( CFG[246] ),
    .CFG245 ( CFG[245] ),
    .CFG244 ( CFG[244] ),
    .CFG243 ( CFG[243] ),
    .CFG242 ( CFG[242] ),
    .CFG241 ( CFG[241] ),
    .CFG240 ( CFG[240] ),
    .CFG239 ( CFG[239] ),
    .CFG238 ( CFG[238] ),
    .CFG237 ( CFG[237] ),
    .CFG236 ( CFG[236] ),
    .CFG235 ( CFG[235] ),
    .CFG234 ( CFG[234] ),
    .CFG233 ( CFG[233] ),
    .CFG232 ( CFG[232] ),
    .CFG231 ( CFG[231] ),
    .CFG230 ( CFG[230] ),
    .CFG229 ( CFG[229] ),
    .CFG228 ( CFG[228] ),
    .CFG227 ( CFG[227] ),
    .CFG226 ( CFG[226] ),
    .CFG225 ( CFG[225] ),
    .CFG224 ( CFG[224] ),
    .CFG223 ( CFG[223] ),
    .CFG222 ( CFG[222] ),
    .CFG221 ( CFG[221] ),
    .CFG220 ( CFG[220] ),
    .CFG219 ( CFG[219] ),
    .CFG218 ( CFG[218] ),
    .CFG217 ( CFG[217] ),
    .CFG216 ( CFG[216] ),
    .CFG215 ( CFG[215] ),
    .CFG214 ( CFG[214] ),
    .CFG213 ( CFG[213] ),
    .CFG212 ( CFG[212] ),
    .CFG211 ( CFG[211] ),
    .CFG210 ( CFG[210] ),
    .CFG209 ( CFG[209] ),
    .CFG208 ( CFG[208] ),
    .CFG207 ( CFG[207] ),
    .CFG206 ( CFG[206] ),
    .CFG205 ( CFG[205] ),
    .CFG204 ( CFG[204] ),
    .CFG203 ( CFG[203] ),
    .CFG202 ( CFG[202] ),
    .CFG201 ( CFG[201] ),
    .CFG200 ( CFG[200] ),
    .CFG199 ( CFG[199] ),
    .CFG198 ( CFG[198] ),
    .CFG197 ( CFG[197] ),
    .CFG196 ( CFG[196] ),
    .CFG195 ( CFG[195] ),
    .CFG194 ( CFG[194] ),
    .CFG193 ( CFG[193] ),
    .CFG192 ( CFG[192] ),
    .CFG191 ( CFG[191] ),
    .CFG190 ( CFG[190] ),
    .CFG189 ( CFG[189] ),
    .CFG188 ( CFG[188] ),
    .CFG187 ( CFG[187] ),
    .CFG186 ( CFG[186] ),
    .CFG185 ( CFG[185] ),
    .CFG184 ( CFG[184] ),
    .CFG183 ( CFG[183] ),
    .CFG182 ( CFG[182] ),
    .CFG181 ( CFG[181] ),
    .CFG180 ( CFG[180] ),
    .CFG179 ( CFG[179] ),
    .CFG178 ( CFG[178] ),
    .CFG177 ( CFG[177] ),
    .CFG176 ( CFG[176] ),
    .CFG175 ( CFG[175] ),
    .CFG174 ( CFG[174] ),
    .CFG173 ( CFG[173] ),
    .CFG172 ( CFG[172] ),
    .CFG171 ( CFG[171] ),
    .CFG170 ( CFG[170] ),
    .CFG169 ( CFG[169] ),
    .CFG168 ( CFG[168] ),
    .CFG167 ( CFG[167] ),
    .CFG166 ( CFG[166] ),
    .CFG165 ( CFG[165] ),
    .CFG164 ( CFG[164] ),
    .CFG163 ( CFG[163] ),
    .CFG162 ( CFG[162] ),
    .CFG161 ( CFG[161] ),
    .CFG160 ( CFG[160] ),
    .CFG159 ( CFG[159] ),
    .CFG158 ( CFG[158] ),
    .CFG157 ( CFG[157] ),
    .CFG156 ( CFG[156] ),
    .CFG155 ( CFG[155] ),
    .CFG154 ( CFG[154] ),
    .CFG153 ( CFG[153] ),
    .CFG152 ( CFG[152] ),
    .CFG151 ( CFG[151] ),
    .CFG150 ( CFG[150] ),
    .CFG149 ( CFG[149] ),
    .CFG148 ( CFG[148] ),
    .CFG147 ( CFG[147] ),
    .CFG146 ( CFG[146] ),
    .CFG145 ( CFG[145] ),
    .CFG144 ( CFG[144] ),
    .CFG143 ( CFG[143] ),
    .CFG142 ( CFG[142] ),
    .CFG141 ( CFG[141] ),
    .CFG140 ( CFG[140] ),
    .CFG139 ( CFG[139] ),
    .CFG138 ( CFG[138] ),
    .CFG137 ( CFG[137] ),
    .CFG136 ( CFG[136] ),
    .CFG135 ( CFG[135] ),
    .CFG134 ( CFG[134] ),
    .CFG133 ( CFG[133] ),
    .CFG132 ( CFG[132] ),
    .CFG131 ( CFG[131] ),
    .CFG130 ( CFG[130] ),
    .CFG129 ( CFG[129] ),
    .CFG128 ( CFG[128] ),
    .CFG127 ( CFG[127] ),
    .CFG126 ( CFG[126] ),
    .CFG125 ( CFG[125] ),
    .CFG124 ( CFG[124] ),
    .CFG123 ( CFG[123] ),
    .CFG122 ( CFG[122] ),
    .CFG121 ( CFG[121] ),
    .CFG120 ( CFG[120] ),
    .CFG119 ( CFG[119] ),
    .CFG118 ( CFG[118] ),
    .CFG117 ( CFG[117] ),
    .CFG116 ( CFG[116] ),
    .CFG115 ( CFG[115] ),
    .CFG114 ( CFG[114] ),
    .CFG113 ( CFG[113] ),
    .CFG112 ( CFG[112] ),
    .CFG111 ( CFG[111] ),
    .CFG110 ( CFG[110] ),
    .CFG109 ( CFG[109] ),
    .CFG108 ( CFG[108] ),
    .CFG107 ( CFG[107] ),
    .CFG106 ( CFG[106] ),
    .CFG105 ( CFG[105] ),
    .CFG104 ( CFG[104] ),
    .CFG103 ( CFG[103] ),
    .CFG102 ( CFG[102] ),
    .CFG101 ( CFG[101] ),
    .CFG100 ( CFG[100] ),
    .CFG99  ( CFG[99] ),
    .CFG98  ( CFG[98] ),
    .CFG97  ( CFG[97] ),
    .CFG96  ( CFG[96] ),
    .CFG95  ( CFG[95] ),
    .CFG94  ( CFG[94] ),
    .CFG93  ( CFG[93] ),
    .CFG92  ( CFG[92] ),
    .CFG91  ( CFG[91] ),
    .CFG90  ( CFG[90] ),
    .CFG89  ( CFG[89] ),
    .CFG88  ( CFG[88] ),
    .CFG87  ( CFG[87] ),
    .CFG86  ( CFG[86] ),
    .CFG85  ( CFG[85] ),
    .CFG84  ( CFG[84] ),
    .CFG83  ( CFG[83] ),
    .CFG82  ( CFG[82] ),
    .CFG81  ( CFG[81] ),
    .CFG80  ( CFG[80] ),
    .CFG79  ( CFG[79] ),
    .CFG78  ( CFG[78] ),
    .CFG77  ( CFG[77] ),
    .CFG76  ( CFG[76] ),
    .CFG75  ( CFG[75] ),
    .CFG74  ( CFG[74] ),
    .CFG73  ( CFG[73] ),
    .CFG72  ( CFG[72] ),
    .CFG71  ( CFG[71] ),
    .CFG70  ( CFG[70] ),
    .CFG69  ( CFG[69] ),
    .CFG68  ( CFG[68] ),
    .CFG67  ( CFG[67] ),
    .CFG66  ( CFG[66] ),
    .CFG65  ( CFG[65] ),
    .CFG64  ( CFG[64] ),
    .CFG63  ( CFG[63] ),
    .CFG62  ( CFG[62] ),
    .CFG61  ( CFG[61] ),
    .CFG60  ( CFG[60] ),
    .CFG59  ( CFG[59] ),
    .CFG58  ( CFG[58] ),
    .CFG57  ( CFG[57] ),
    .CFG56  ( CFG[56] ),
    .CFG55  ( CFG[55] ),
    .CFG54  ( CFG[54] ),
    .CFG53  ( CFG[53] ),
    .CFG52  ( CFG[52] ),
    .CFG51  ( CFG[51] ),
    .CFG50  ( CFG[50] ),
    .CFG49  ( CFG[49] ),
    .CFG48  ( CFG[48] ),
    .CFG47  ( CFG[47] ),
    .CFG46  ( CFG[46] ),
    .CFG45  ( CFG[45] ),
    .CFG44  ( CFG[44] ),
    .CFG43  ( CFG[43] ),
    .CFG42  ( CFG[42] ),
    .CFG41  ( CFG[41] ),
    .CFG40  ( CFG[40] ),
    .CFG39  ( CFG[39] ),
    .CFG38  ( CFG[38] ),
    .CFG37  ( CFG[37] ),
    .CFG36  ( CFG[36] ),
    .CFG35  ( CFG[35] ),
    .CFG34  ( CFG[34] ),
    .CFG33  ( CFG[33] ),
    .CFG32  ( CFG[32] ),
    .CFG31  ( CFG[31] ),
    .CFG30  ( CFG[30] ),
    .CFG29  ( CFG[29] ),
    .CFG28  ( CFG[28] ),
    .CFG27  ( CFG[27] ),
    .CFG26  ( CFG[26] ),
    .CFG25  ( CFG[25] ),
    .CFG24  ( CFG[24] ),
    .CFG23  ( CFG[23] ),
    .CFG22  ( CFG[22] ),
    .CFG21  ( CFG[21] ),
    .CFG20  ( CFG[20] ),
    .CFG19  ( CFG[19] ),
    .CFG18  ( CFG[18] ),
    .CFG17  ( CFG[17] ),
    .CFG16  ( CFG[16] ),
    .CFG15  ( CFG[15] ),
    .CFG14  ( CFG[14] ),
    .CFG13  ( CFG[13] ),
    .CFG12  ( CFG[12] ),
    .CFG11  ( CFG[11] ),
    .CFG10  ( CFG[10] ),
    .CFG9   ( CFG[9] ),
    .CFG8   ( CFG[8] ),
    .CFG7   ( CFG[7] ),
    .CFG6   ( CFG[6] ),
    .CFG5   ( CFG[5] ),
    .CFG4   ( CFG[4] ),
    .CFG3   ( CFG[3] ),
    .CFG2   ( CFG[2] ),
    .CFG1   ( CFG[1] ),
    .CFG0   ( CFG[0] ),
    .FRAMEQ_N ( FRAMEQ_N ),
    .REQ64Q_N ( REQ64Q_N ),
    .TRDYQ_N ( TRDYQ_N ),
    .IRDYQ_N ( IRDYQ_N ),
    .STOPQ_N ( STOPQ_N ),
    .DEVSELQ_N ( DEVSELQ_N ),
    .ACK64Q_N ( ACK64Q_N ),
    .ADDR31 ( ADDR[31] ),
    .ADDR30 ( ADDR[30] ),
    .ADDR29 ( ADDR[29] ),
    .ADDR28 ( ADDR[28] ),
    .ADDR27 ( ADDR[27] ),
    .ADDR26 ( ADDR[26] ),
    .ADDR25 ( ADDR[25] ),
    .ADDR24 ( ADDR[24] ),
    .ADDR23 ( ADDR[23] ),
    .ADDR22 ( ADDR[22] ),
    .ADDR21 ( ADDR[21] ),
    .ADDR20 ( ADDR[20] ),
    .ADDR19 ( ADDR[19] ),
    .ADDR18 ( ADDR[18] ),
    .ADDR17 ( ADDR[17] ),
    .ADDR16 ( ADDR[16] ),
    .ADDR15 ( ADDR[15] ),
    .ADDR14 ( ADDR[14] ),
    .ADDR13 ( ADDR[13] ),
    .ADDR12 ( ADDR[12] ),
    .ADDR11 ( ADDR[11] ),
    .ADDR10 ( ADDR[10] ),
    .ADDR9  ( ADDR[ 9] ),
    .ADDR8  ( ADDR[ 8] ),
    .ADDR7  ( ADDR[ 7] ),
    .ADDR6  ( ADDR[ 6] ),
    .ADDR5  ( ADDR[ 5] ),
    .ADDR4  ( ADDR[ 4] ),
    .ADDR3  ( ADDR[ 3] ),
    .ADDR2  ( ADDR[ 2] ),
    .ADDR1  ( ADDR[ 1] ),
    .ADDR0  ( ADDR[ 0] ),
    .ADIO63 ( ADIO[63] ),
    .ADIO62 ( ADIO[62] ),
    .ADIO61 ( ADIO[61] ),
    .ADIO60 ( ADIO[60] ),
    .ADIO59 ( ADIO[59] ),
    .ADIO58 ( ADIO[58] ),
    .ADIO57 ( ADIO[57] ),
    .ADIO56 ( ADIO[56] ),
    .ADIO55 ( ADIO[55] ),
    .ADIO54 ( ADIO[54] ),
    .ADIO53 ( ADIO[53] ),
    .ADIO52 ( ADIO[52] ),
    .ADIO51 ( ADIO[51] ),
    .ADIO50 ( ADIO[50] ),
    .ADIO49 ( ADIO[49] ),
    .ADIO48 ( ADIO[48] ),
    .ADIO47 ( ADIO[47] ),
    .ADIO46 ( ADIO[46] ),
    .ADIO45 ( ADIO[45] ),
    .ADIO44 ( ADIO[44] ),
    .ADIO43 ( ADIO[43] ),
    .ADIO42 ( ADIO[42] ),
    .ADIO41 ( ADIO[41] ),
    .ADIO40 ( ADIO[40] ),
    .ADIO39 ( ADIO[39] ),
    .ADIO38 ( ADIO[38] ),
    .ADIO37 ( ADIO[37] ),
    .ADIO36 ( ADIO[36] ),
    .ADIO35 ( ADIO[35] ),
    .ADIO34 ( ADIO[34] ),
    .ADIO33 ( ADIO[33] ),
    .ADIO32 ( ADIO[32] ),
    .ADIO31 ( ADIO[31] ),
    .ADIO30 ( ADIO[30] ),
    .ADIO29 ( ADIO[29] ),
    .ADIO28 ( ADIO[28] ),
    .ADIO27 ( ADIO[27] ),
    .ADIO26 ( ADIO[26] ),
    .ADIO25 ( ADIO[25] ),
    .ADIO24 ( ADIO[24] ),
    .ADIO23 ( ADIO[23] ),
    .ADIO22 ( ADIO[22] ),
    .ADIO21 ( ADIO[21] ),
    .ADIO20 ( ADIO[20] ),
    .ADIO19 ( ADIO[19] ),
    .ADIO18 ( ADIO[18] ),
    .ADIO17 ( ADIO[17] ),
    .ADIO16 ( ADIO[16] ),
    .ADIO15 ( ADIO[15] ),
    .ADIO14 ( ADIO[14] ),
    .ADIO13 ( ADIO[13] ),
    .ADIO12 ( ADIO[12] ),
    .ADIO11 ( ADIO[11] ),
    .ADIO10 ( ADIO[10] ),
    .ADIO9  ( ADIO[ 9] ),
    .ADIO8  ( ADIO[ 8] ),
    .ADIO7  ( ADIO[ 7] ),
    .ADIO6  ( ADIO[ 6] ),
    .ADIO5  ( ADIO[ 5] ),
    .ADIO4  ( ADIO[ 4] ),
    .ADIO3  ( ADIO[ 3] ),
    .ADIO2  ( ADIO[ 2] ),
    .ADIO1  ( ADIO[ 1] ),
    .ADIO0  ( ADIO[ 0] ),
    .CFG_VLD ( CFG_VLD ),
    .CFG_HIT ( CFG_HIT ),
    .C_TERM ( C_TERM ),
    .C_READY ( C_READY ),
    .ADDR_VLD ( ADDR_VLD ),
    .BASE_HIT7 ( BASE_HIT[7] ),
    .BASE_HIT6 ( BASE_HIT[6] ),
    .BASE_HIT5 ( BASE_HIT[5] ),
    .BASE_HIT4 ( BASE_HIT[4] ),
    .BASE_HIT3 ( BASE_HIT[3] ),
    .BASE_HIT2 ( BASE_HIT[2] ),
    .BASE_HIT1 ( BASE_HIT[1] ),
    .BASE_HIT0 ( BASE_HIT[0] ),
    .S_CYCLE64 ( S_CYCLE64 ),
    .S_TERM ( S_TERM ),
    .S_READY ( S_READY ),
    .S_ABORT ( S_ABORT ),
    .S_WRDN ( S_WRDN ),
    .S_SRC_EN ( S_SRC_EN ),
    .S_DATA_VLD ( S_DATA_VLD ),
    .S_CBE7 ( S_CBE[7] ),
    .S_CBE6 ( S_CBE[6] ),
    .S_CBE5 ( S_CBE[5] ),
    .S_CBE4 ( S_CBE[4] ),
    .S_CBE3 ( S_CBE[3] ),
    .S_CBE2 ( S_CBE[2] ),
    .S_CBE1 ( S_CBE[1] ),
    .S_CBE0 ( S_CBE[0] ),
    .PCI_CMD15 ( PCI_CMD[15] ),
    .PCI_CMD14 ( PCI_CMD[14] ),
    .PCI_CMD13 ( PCI_CMD[13] ),
    .PCI_CMD12 ( PCI_CMD[12] ),
    .PCI_CMD11 ( PCI_CMD[11] ),
    .PCI_CMD10 ( PCI_CMD[10] ),
    .PCI_CMD9  ( PCI_CMD[ 9] ),
    .PCI_CMD8  ( PCI_CMD[ 8] ),
    .PCI_CMD7  ( PCI_CMD[ 7] ),
    .PCI_CMD6  ( PCI_CMD[ 6] ),
    .PCI_CMD5  ( PCI_CMD[ 5] ),
    .PCI_CMD4  ( PCI_CMD[ 4] ),
    .PCI_CMD3  ( PCI_CMD[ 3] ),
    .PCI_CMD2  ( PCI_CMD[ 2] ),
    .PCI_CMD1  ( PCI_CMD[ 1] ),
    .PCI_CMD0  ( PCI_CMD[ 0] ),
    .REQUEST ( REQUEST ),
    .REQUEST64 ( REQUEST64 ),
    .REQUESTHOLD ( REQUESTHOLD ),
    .COMPLETE ( COMPLETE ),
    .M_WRDN ( M_WRDN ),
    .M_READY ( M_READY ),
    .M_SRC_EN ( M_SRC_EN ),
    .M_DATA_VLD ( M_DATA_VLD ),
    .M_CBE7 ( M_CBE[7] ),
    .M_CBE6 ( M_CBE[6] ),
    .M_CBE5 ( M_CBE[5] ),
    .M_CBE4 ( M_CBE[4] ),
    .M_CBE3 ( M_CBE[3] ),
    .M_CBE2 ( M_CBE[2] ),
    .M_CBE1 ( M_CBE[1] ),
    .M_CBE0 ( M_CBE[0] ),
    .TIME_OUT ( TIME_OUT ),
    .M_FAIL64 ( M_FAIL64 ),
    .CFG_SELF ( CFG_SELF ),
    .M_DATA ( M_DATA ),
    .DR_BUS ( DR_BUS ),
    .I_IDLE ( I_IDLE ),
    .M_ADDR_N ( M_ADDR_N ),
    .IDLE ( IDLE ),
    .B_BUSY ( B_BUSY ),
    .S_DATA ( S_DATA ),
    .BACKOFF ( BACKOFF ),
    .SLOT64 ( SLOT64 ),
    .INTR_N  ( INTR_N ),
    .PERRQ_N ( PERRQ_N ),
    .SERRQ_N ( SERRQ_N ),
    .KEEPOUT ( KEEPOUT ),
    .CSR39 ( CSR[39] ),
    .CSR38 ( CSR[38] ),
    .CSR37 ( CSR[37] ),
    .CSR36 ( CSR[36] ),
    .CSR35 ( CSR[35] ),
    .CSR34 ( CSR[34] ),
    .CSR33 ( CSR[33] ),
    .CSR32 ( CSR[32] ),
    .CSR31 ( CSR[31] ),
    .CSR30 ( CSR[30] ),
    .CSR29 ( CSR[29] ),
    .CSR28 ( CSR[28] ),
    .CSR27 ( CSR[27] ),
    .CSR26 ( CSR[26] ),
    .CSR25 ( CSR[25] ),
    .CSR24 ( CSR[24] ),
    .CSR23 ( CSR[23] ),
    .CSR22 ( CSR[22] ),
    .CSR21 ( CSR[21] ),
    .CSR20 ( CSR[20] ),
    .CSR19 ( CSR[19] ),
    .CSR18 ( CSR[18] ),
    .CSR17 ( CSR[17] ),
    .CSR16 ( CSR[16] ),
    .CSR15 ( CSR[15] ),
    .CSR14 ( CSR[14] ),
    .CSR13 ( CSR[13] ),
    .CSR12 ( CSR[12] ),
    .CSR11 ( CSR[11] ),
    .CSR10 ( CSR[10] ),
    .CSR9 ( CSR[9] ),
    .CSR8 ( CSR[8] ),
    .CSR7 ( CSR[7] ),
    .CSR6 ( CSR[6] ),
    .CSR5 ( CSR[5] ),
    .CSR4 ( CSR[4] ),
    .CSR3 ( CSR[3] ),
    .CSR2 ( CSR[2] ),
    .CSR1 ( CSR[1] ),
    .CSR0 ( CSR[0] ),
    .SUB_DATA31 ( SUB_DATA[31] ),
    .SUB_DATA30 ( SUB_DATA[30] ),
    .SUB_DATA29 ( SUB_DATA[29] ),
    .SUB_DATA28 ( SUB_DATA[28] ),
    .SUB_DATA27 ( SUB_DATA[27] ),
    .SUB_DATA26 ( SUB_DATA[26] ),
    .SUB_DATA25 ( SUB_DATA[25] ),
    .SUB_DATA24 ( SUB_DATA[24] ),
    .SUB_DATA23 ( SUB_DATA[23] ),
    .SUB_DATA22 ( SUB_DATA[22] ),
    .SUB_DATA21 ( SUB_DATA[21] ),
    .SUB_DATA20 ( SUB_DATA[20] ),
    .SUB_DATA19 ( SUB_DATA[19] ),
    .SUB_DATA18 ( SUB_DATA[18] ),
    .SUB_DATA17 ( SUB_DATA[17] ),
    .SUB_DATA16 ( SUB_DATA[16] ),
    .SUB_DATA15 ( SUB_DATA[15] ),
    .SUB_DATA14 ( SUB_DATA[14] ),
    .SUB_DATA13 ( SUB_DATA[13] ),
    .SUB_DATA12 ( SUB_DATA[12] ),
    .SUB_DATA11 ( SUB_DATA[11] ),
    .SUB_DATA10 ( SUB_DATA[10] ),
    .SUB_DATA9  ( SUB_DATA[9] ),
    .SUB_DATA8  ( SUB_DATA[8] ),
    .SUB_DATA7  ( SUB_DATA[7] ),
    .SUB_DATA6  ( SUB_DATA[6] ),
    .SUB_DATA5  ( SUB_DATA[5] ),
    .SUB_DATA4  ( SUB_DATA[4] ),
    .SUB_DATA3  ( SUB_DATA[3] ),
    .SUB_DATA2  ( SUB_DATA[2] ),
    .SUB_DATA1  ( SUB_DATA[1] ),
    .SUB_DATA0  ( SUB_DATA[0] ),
    .CLK ( CLK ),
    .CLKX ( CLK ),
    .RST ( )
  );

endmodule
