//----------------------------------------------------------------------------
// pciarbr6.v - Simple PCI Arbiter
//----------------------------------------------------------------------------
// 
// Copyright (c) 1998, 2003, John J. Sullivan
// All Rights Reserved.
// 
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, 
//    this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright 
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the distribution.
// 3. The names of the contributors shall not be used to endorse or promote
//    products derived from this software without specific prior written 
//    permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
//   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT 
//   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
//   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
//   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
//   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
//----------------------------------------------------------------------------

module pciarbr6(
        pci_req5_in, pci_req4_in, pci_req3_in, 
        pci_req2_in, pci_req1_in, pci_req0_in,

        pci_gnt5_out, pci_gnt4_out, pci_gnt3_out,
        pci_gnt2_out, pci_gnt1_out, pci_gnt0_out,

        pci_frame_in, pci_irdy_in,
        pci_clk, pci_reset
);

input           pci_req5_in, pci_req4_in, pci_req3_in;
input           pci_req2_in, pci_req1_in, pci_req0_in;

output          pci_gnt5_out, pci_gnt4_out, pci_gnt3_out;
output          pci_gnt2_out, pci_gnt1_out, pci_gnt0_out;

input           pci_frame_in, pci_irdy_in;
input           pci_clk, pci_reset;

//----------------------------------------------------------------------------

function [2:0] gnt_func;
input   [2:0]   gnt_cur;
input   [5:0]   req_cur;
input   [2:0]   park;
begin
        // Enumeration may not be elegant, but it's efficient.

        casex ({gnt_cur, req_cur})
        {3'h0, 6'bxxxx1x }:     gnt_func = 3'h1;
        {3'h0, 6'bxxx10x }:     gnt_func = 3'h2;
        {3'h0, 6'bxx100x }:     gnt_func = 3'h3;
        {3'h0, 6'bx1000x }:     gnt_func = 3'h4;
        {3'h0, 6'b10000x }:     gnt_func = 3'h5;
        {3'h0, 6'b000001 }:     gnt_func = 3'h0;

        {3'h1, 6'bxxx1xx }:     gnt_func = 3'h2;
        {3'h1, 6'bxx10xx }:     gnt_func = 3'h3;
        {3'h1, 6'bx100xx }:     gnt_func = 3'h4;
        {3'h1, 6'b1000xx }:     gnt_func = 3'h5;
        {3'h1, 6'b0000x1 }:     gnt_func = 3'h0;
        {3'h1, 6'b000010 }:     gnt_func = 3'h1;

        {3'h2, 6'bxx1xxx }:     gnt_func = 3'h3;
        {3'h2, 6'bx10xxx }:     gnt_func = 3'h4;
        {3'h2, 6'b100xxx }:     gnt_func = 3'h5;
        {3'h2, 6'b000xx1 }:     gnt_func = 3'h0;
        {3'h2, 6'b000x10 }:     gnt_func = 3'h1;
        {3'h2, 6'b000100 }:     gnt_func = 3'h2;

        {3'h3, 6'bx1xxxx }:     gnt_func = 3'h4;
        {3'h3, 6'b10xxxx }:     gnt_func = 3'h5;
        {3'h3, 6'b00xxx1 }:     gnt_func = 3'h0;
        {3'h3, 6'b00xx10 }:     gnt_func = 3'h1;
        {3'h3, 6'b00x100 }:     gnt_func = 3'h2;
        {3'h3, 6'b001000 }:     gnt_func = 3'h3;

        {3'h4, 6'b1xxxxx }:     gnt_func = 3'h5;
        {3'h4, 6'b0xxxx1 }:     gnt_func = 3'h0;
        {3'h4, 6'b0xxx10 }:     gnt_func = 3'h1;
        {3'h4, 6'b0xx100 }:     gnt_func = 3'h2;
        {3'h4, 6'b0x1000 }:     gnt_func = 3'h3;
        {3'h4, 6'b010000 }:     gnt_func = 3'h4;

        {3'h5, 6'bxxxxx1 }:     gnt_func = 3'h0;
        {3'h5, 6'bxxxx10 }:     gnt_func = 3'h1;
        {3'h5, 6'bxxx100 }:     gnt_func = 3'h2;
        {3'h5, 6'bxx1000 }:     gnt_func = 3'h3;
        {3'h5, 6'bx10000 }:     gnt_func = 3'h4;
        {3'h5, 6'b100000 }:     gnt_func = 3'h5;

        default:                gnt_func = park;

        endcase
end
endfunction

//----------------------------------------------------------------------------
// Configuration signals.

wire            park_last = 1'b1;
wire    [2:0]   park_host = 3'h5;
wire            pci_delay = 1'b0;

//----------------------------------------------------------------------------

wire    [5:0]   req_in;
reg     [5:0]   req_dly;
wire    [5:0]   req_cur;

assign  req_in = { pci_req5_in, pci_req4_in, pci_req3_in, 
                   pci_req2_in, pci_req1_in, pci_req0_in };

always @(posedge pci_clk) begin
        req_dly <= req_in;
end

//assign        req_cur = (pci_delay) ? req_dly : req_in;
assign  req_cur = (1'b1) ? req_dly : req_in;

//----------------------------------------------------------------------------

reg     pci_frame_dly;
reg     pci_irdy_dly;
reg     pci_frame_reg;

always @(posedge pci_clk) begin
        pci_frame_dly  <= pci_frame_in;
        pci_irdy_dly   <= pci_irdy_in;
end

wire    pci_frame_sel = (pci_delay) ? pci_frame_dly : pci_frame_in;
wire    pci_irdy_sel  = (pci_delay) ? pci_irdy_dly  : pci_irdy_in;

always @(posedge pci_clk) begin
        pci_frame_reg <= pci_frame_sel;
end

//----------------------------------------------------------------------------

wire    pci_start = pci_frame_sel && !pci_frame_reg;
wire    pci_idle = !pci_frame_sel && !pci_irdy_sel;

//----------------------------------------------------------------------------

reg             gnt_act;
reg     [2:0]   gnt_cur;
reg     [5:0]   gnt_dec;

wire    [2:0]   gnt_cur_next;
wire            gnt_act_next;

reg     gnt_active, gnt_assert;
wire    gnt_start, gnt_active_next, gnt_assert_next;
wire    [2:0]   gnt_calc;
wire    [2:0]   gnt_park;

assign  gnt_start = !gnt_active && (req_cur != 5'b0) && (req_cur != gnt_dec);
assign  gnt_active_next = gnt_start || (gnt_active && !pci_start);

assign  gnt_park = (park_last) ? gnt_cur : park_host;
assign  gnt_calc = gnt_func(gnt_cur, req_cur, gnt_park);

wire    gnt_enable = gnt_start || (!gnt_active && !park_last);

assign  gnt_assert_next = !(gnt_enable && (gnt_calc != gnt_cur));
assign  gnt_cur_next = (gnt_enable) ? gnt_calc : gnt_cur;

always @(posedge pci_clk) begin
        if (pci_reset) begin
                gnt_active <= 1'b0;
                gnt_assert <= 1'b0;
                gnt_cur <= 3'h0;
                gnt_dec <= 5'b0;
        end else begin
                gnt_active <= gnt_active_next;
                gnt_assert <= gnt_assert_next;
                gnt_cur <= gnt_cur_next;
                gnt_dec[5] <= (gnt_cur_next == 3'h5) && gnt_assert_next;
                gnt_dec[4] <= (gnt_cur_next == 3'h4) && gnt_assert_next;
                gnt_dec[3] <= (gnt_cur_next == 3'h3) && gnt_assert_next;
                gnt_dec[2] <= (gnt_cur_next == 3'h2) && gnt_assert_next;
                gnt_dec[1] <= (gnt_cur_next == 3'h1) && gnt_assert_next;
                gnt_dec[0] <= (gnt_cur_next == 3'h0) && gnt_assert_next;
        end
end

assign { pci_gnt5_out, pci_gnt4_out, pci_gnt3_out,
         pci_gnt2_out, pci_gnt1_out, pci_gnt0_out } = gnt_dec;

//----------------------------------------------------------------------------

endmodule

//----------------------------------------------------------------------------
