//////////////////////////////////////////////////////////////////////////
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//////////////////////////////////////////////////////////////////////////

module ppc_trace
  (
  // CPU Trace Signals
  cpu_clk,                    // I
  c405trccycle,               // I
  c405trcevenexecutionstatus, // I [0:1]
  c405trctracestatus,         // I [0:3]
  c405trctriggereventout,     // I
  c405trctriggereventtype,    // I [0:10]
  c405trcoddexecutionstatus,  // I [0:1]
  trcc405tracedisable,        // O
  trcc405triggereventin,      // O
  // External Trace Connector Signals
  trc_clk,                    // O
  trc_ts1o,                   // O
  trc_ts2o,                   // O
  trc_ts1e,                   // O
  trc_ts2e,                   // O
  trc_ts3,                    // O
  trc_ts4,                    // O
  trc_ts5,                    // O
  trc_ts6                     // O
  );

  // CPU Trace Signals
  input         cpu_clk;
  input         c405trccycle;
  input  [0:1]  c405trcevenexecutionstatus;
  input  [0:3]  c405trctracestatus;
  input         c405trctriggereventout;
  input  [0:10] c405trctriggereventtype;
  input  [0:1]  c405trcoddexecutionstatus;
  output        trcc405tracedisable;
  output        trcc405triggereventin;
  // External Trace Connector Signals
  output        trc_clk;
  output        trc_ts1o;
  output        trc_ts2o;
  output        trc_ts1e;
  output        trc_ts2e;
  output        trc_ts3;
  output        trc_ts4;
  output        trc_ts5;
  output        trc_ts6;

  // Trace Pipeline Regs
  reg         c405trccycle_d1;
  reg         c405trccycle_d2;  
  reg         c405trccycle_d3;  
  reg         c405trccycle_d4;  
  reg         c405trccycle_d5;  
  reg         c405trccycle_d6;  
  reg         c405trccycle_d7;  

  reg         trc_pipe_ts1o;
  reg         trc_pipe_ts2o;
  reg         trc_pipe_ts1e;
  reg         trc_pipe_ts2e;
  reg         trc_pipe_ts3;
  reg         trc_pipe_ts4;
  reg         trc_pipe_ts5;
  reg         trc_pipe_ts6;

  reg         trc_pipe_ts1o_d1;
  reg         trc_pipe_ts2o_d1;
  reg         trc_pipe_ts1e_d1;
  reg         trc_pipe_ts2e_d1;
  reg         trc_pipe_ts3_d1;
  reg         trc_pipe_ts4_d1;
  reg         trc_pipe_ts5_d1;
  reg         trc_pipe_ts6_d1;

always @(posedge cpu_clk)
begin
  c405trccycle_d1 <= c405trccycle;
  c405trccycle_d2 <= c405trccycle_d1;
  c405trccycle_d3 <= c405trccycle_d2;
  c405trccycle_d4 <= c405trccycle_d3;
  c405trccycle_d5 <= c405trccycle_d4;
  c405trccycle_d6 <= c405trccycle_d5;
  c405trccycle_d7 <= c405trccycle_d6;
end

always @(posedge cpu_clk)
  if (c405trccycle_d3) begin
    trc_pipe_ts1o <= c405trcoddexecutionstatus[0];
    trc_pipe_ts2o <= c405trcoddexecutionstatus[1];
    trc_pipe_ts1e <= c405trcevenexecutionstatus[0];
    trc_pipe_ts2e <= c405trcevenexecutionstatus[1];
    trc_pipe_ts3  <= c405trctracestatus[0];
    trc_pipe_ts4  <= c405trctracestatus[1];
    trc_pipe_ts5  <= c405trctracestatus[2];
    trc_pipe_ts6  <= c405trctracestatus[3];
  end

always @(posedge cpu_clk)
  if (c405trccycle_d5) begin
    trc_pipe_ts1o_d1 <= trc_pipe_ts1o;
    trc_pipe_ts2o_d1 <= trc_pipe_ts2o;
    trc_pipe_ts1e_d1 <= trc_pipe_ts1e;
    trc_pipe_ts2e_d1 <= trc_pipe_ts2e;
    trc_pipe_ts3_d1  <= trc_pipe_ts3;
    trc_pipe_ts4_d1  <= trc_pipe_ts4;
    trc_pipe_ts5_d1  <= trc_pipe_ts5;
    trc_pipe_ts6_d1  <= trc_pipe_ts6;
  end   

assign trc_clk  = c405trccycle_d7;
assign trc_ts1o = trc_pipe_ts1o_d1;
assign trc_ts2o = trc_pipe_ts2o_d1;
assign trc_ts1e = trc_pipe_ts1e_d1;
assign trc_ts2e = trc_pipe_ts2e_d1;
assign trc_ts3  = trc_pipe_ts3_d1;
assign trc_ts4  = trc_pipe_ts4_d1;
assign trc_ts5  = trc_pipe_ts5_d1;
assign trc_ts6  = trc_pipe_ts6_d1;

assign trcc405triggereventin = c405trctriggereventout;
assign trcc405tracedisable   = 1'b0;

endmodule

  
