/*-----------------------------------------------------------------------------
//     $Date: 2004/03/10 23:30:00 $
//     $RCSfile: bootload.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/
#include "xio.h"
#include "xbasic_types.h"
#include "sleep.h"
#include "xstatus.h"
#include "xcache_l.h"
#include "xtft.h"
#include "xtft_l.h"
#include "xsysace_l.h"
#include "xtouchscreen.h"
#include "xparameters.h"
#include "bootload_graphics.h"
#include "bootload_boot_boxdef.h"
#include "xilinx_logo_greyongrey_bmp.h"
#include "ml300_txt_bmp.h"
#include "ml300splashsmallbmp.h"

#define UART_BASEADDR              XPAR_OPB_UART16550_0_BASEADDR
#define UART_CLOCK                 XPAR_XUARTNS550_CLOCK_HZ
#define UART_BAUDRATE              9600

#define SYSACE_BASEADDR         XPAR_OPB_SYSACE_0_BASEADDR
#define TFT_DEVICE_ID           XPAR_PLB_TFT_CNTLR_REF_0_DEVICE_ID
#define TOUCHSCREEN_DEVICE_ID   XPAR_OPB_TSD_REF_0_DEVICE_ID

#define STATUS_CHECK(Status) { if (Status != XST_SUCCESS) { return Status; } }

//#define BOOTLOAD_DEBUG
#ifdef BOOTLOAD_DEBUG
#include <stdio.h>
#include "xuartns550_l.h"
#endif

/************************ Function prototypes ***********************/
static void LoadProgram(XTft *InstancePtr, int num);
static int DetectBox(XTft *TftPtr, XTouchscreen *TouchscreenPtr);
void DisplayBitMapImage1 (XTft *InstancePtr,
			  Xuint32 pixel_upper_left_corn_start_x,
			  Xuint32 pixel_upper_left_corn_start_y,
			  unsigned long *myBitmap1);
void DisplayBitMapImage2 (XTft *InstancePtr,
			  Xuint32 pixel_upper_left_corn_start_x,
			  Xuint32 pixel_upper_left_corn_start_y,
			  unsigned long *myBitmap);
void DisplayBitMapPicture (XTft *InstancePtr);


/********************** Function definitions ************************/
static void LoadProgram(XTft *InstancePtr, int num)
{
  XSysAce_mSetControlReg(SYSACE_BASEADDR, XSA_CR_CFGRESET_MASK);
  XSysAce_mSetControlReg(SYSACE_BASEADDR, ((((char *)(&num))[3] << XSA_CR_CFGADDR_SHIFT) |
					   XSA_CR_FORCECFGADDR_MASK |
					   XSA_CR_CFGMODE_MASK |
					   XSA_CR_CFGSTART_MASK));
  
  XTft_ClearScreen(InstancePtr);
  switch (num) {
  case 1:
    XTft_WriteCenteredString(InstancePtr, "Loading XROM_ML300..... ", 100);
    break;
  case 2:
    XTft_WriteCenteredString(InstancePtr, "Loading Linux..... ", 100);
    break;
  case 3:
    XTft_WriteCenteredString(InstancePtr, "Loading VxWorks Bootloader..... ", 100);
    break;
  case 4:
    XTft_WriteCenteredString(InstancePtr, "Loading My Own ACE File..... ", 100);
    break;
  case 5:
    XTft_WriteCenteredString(InstancePtr, "Loading V2PDraw..... ", 100);
    break;
  case 6:
    XTft_WriteCenteredString(InstancePtr, "Loading Tic-Tac-Toe..... ", 100);
    break;
  case 7:
    XTft_WriteCenteredString(InstancePtr, "Loading VxWorks..... ", 100);
    break;
  default:
    XTft_WriteCenteredString(InstancePtr, "Error in Bootloader!", 100);
  }
  while(1);
}

/* Detects which menu box on the TFT was touched 
 * and returns an appropriate indicator value    */
static int DetectBox(XTft *TftPtr, XTouchscreen *TouchscreenPtr)
{
  Xuint32 x, y;
  Xuint32 ctrlReg;
  
  while (1) {
    
    ctrlReg = XTouchscreen_mReadCtrlReg(TouchscreenPtr->BaseAddress);
    /* if touchscreen is touched */
    if (ctrlReg & 0x1) {
      
      /* get coordinates */
      XTouchscreen_GetCoord(TouchscreenPtr, &x, &y);
#ifdef BOOTLOAD_DEBUG
      printf("x, y: %5u, %5u\n\r", x, y);
      continue;
#endif
      
      /* Diagnostics */
      if (x >= BOX_1_X_LEFT && x <= BOX_1_X_RIGHT && y >= BOX_1_Y_LEFT && y <= BOX_1_Y_RIGHT) {
	XTft_DrawFixedSizeFlashButton(TftPtr, BOX_1_X_LEFT, BOX_1_Y_LEFT, BOX_1_X_RIGHT, BOX_1_Y_RIGHT, 
				      "XROM_ML300", 10, 0x00FFFF00, 0x0000FF00);
	return 1;
      }
      /* Linux */
      if (x >= BOX_2_X_LEFT && x <= BOX_2_X_RIGHT && y >= BOX_2_Y_LEFT && y <= BOX_2_Y_RIGHT) {
	XTft_DrawFixedSizeFlashButton(TftPtr, BOX_2_X_LEFT, BOX_2_Y_LEFT, BOX_2_X_RIGHT, BOX_2_Y_RIGHT, 
				      "Linux", 5, 0x00FFFF00, 0x0000FF00);
	return 2;
      }
      /* VxWorks Bootloader */
      if (x >= BOX_3_X_LEFT && x <= BOX_3_X_RIGHT && y >= BOX_3_Y_LEFT && y <= BOX_3_Y_RIGHT) {
	XTft_DrawFixedSizeFlashButton(TftPtr, BOX_3_X_LEFT, BOX_3_Y_LEFT, BOX_3_X_RIGHT, BOX_3_Y_RIGHT, 
				      "Vxworks Bootloader", 18, 0x00FFFF00, 0x0000FF00);
	return 3;
      }
      /* My Own ACE File */
      if (x >= BOX_4_X_LEFT && x <= BOX_4_X_RIGHT && y >= BOX_4_Y_LEFT && y <= BOX_4_Y_RIGHT) {
	XTft_DrawFixedSizeFlashButton(TftPtr, BOX_4_X_LEFT, BOX_4_Y_LEFT, BOX_4_X_RIGHT, BOX_4_Y_RIGHT, 
				      "My Own ACE File", 15, 0x00FFFF00, 0x0000FF00);
	return 4;
      }
      /* V2PDraw */
      if (x >= BOX_5_X_LEFT && x <= BOX_5_X_RIGHT && y >= BOX_5_Y_LEFT && y <= BOX_5_Y_RIGHT) {
	XTft_DrawFixedSizeFlashButton(TftPtr, BOX_5_X_LEFT, BOX_5_Y_LEFT, BOX_5_X_RIGHT, BOX_5_Y_RIGHT, 
				      "V2PDraw", 7, 0x00FFFF00, 0x0000FF00);
	return 5;
      }
      /* Tic-Tac-Toe */
      if (x >= BOX_6_X_LEFT && x <= BOX_6_X_RIGHT && y >= BOX_6_Y_LEFT && y <= BOX_6_Y_RIGHT) {
	XTft_DrawFixedSizeFlashButton(TftPtr, BOX_6_X_LEFT, BOX_6_Y_LEFT, BOX_6_X_RIGHT, BOX_6_Y_RIGHT, 
				      "Tic-Tac-Toe", 11, 0x00FFFF00, 0x0000FF00);
	return 6;
      }
      /* VxWorks */
      if (x >= BOX_7_X_LEFT && x <= BOX_7_X_RIGHT && y >= BOX_7_Y_LEFT && y <= BOX_7_Y_RIGHT) {
	XTft_DrawFixedSizeFlashButton(TftPtr, BOX_7_X_LEFT, BOX_7_Y_LEFT, BOX_7_X_RIGHT, BOX_7_Y_RIGHT, 
				      "VxWorks", 7, 0x00FFFF00, 0x0000FF00);
	return 7;
      }
      
      /* detect Xilinx logo being touched */
      if (x >= 439 && x <= 639 && y >= 419 && y <= 479) {
	XTft_ClearScreen(TftPtr);
	DisplayBitMapPicture(TftPtr);
	XTft_DrawFixedSizeFlashButton(TftPtr, 444, 424, 634, 474, 
				      "DOH!", 4, 0x00FFFF00, 0x0000FF00);
	
	while (1) {
	  ctrlReg = XTouchscreen_mReadCtrlReg(TouchscreenPtr->BaseAddress);
	  if (ctrlReg & 0x1) {
	    InitGraphics(TftPtr, TouchscreenPtr, 0x00ffffff, 0x00404040);
	    DisplayBitMapImage2(TftPtr, 245, 25, ml300_txt_bmp); /* "ML300 Bootloader" header */
	    
	    XTft_DrawFixedSizeButton(TftPtr, BOX_1_X_LEFT, BOX_1_Y_LEFT, BOX_1_X_RIGHT, BOX_1_Y_RIGHT, 
				     "XROM_ML300", 10, 0x00FFFF00);
	    XTft_DrawFixedSizeButton(TftPtr, BOX_2_X_LEFT, BOX_2_Y_LEFT, BOX_2_X_RIGHT, BOX_2_Y_RIGHT, 
				     "Linux", 5, 0x00FFFF00);
	    XTft_DrawFixedSizeButton(TftPtr, BOX_3_X_LEFT, BOX_3_Y_LEFT, BOX_3_X_RIGHT, BOX_3_Y_RIGHT, 
				     "Vxworks Bootloader", 18, 0x00FFFF00);
	    XTft_DrawFixedSizeButton(TftPtr, BOX_4_X_LEFT, BOX_4_Y_LEFT, BOX_4_X_RIGHT, BOX_4_Y_RIGHT, 
				     "My Own ACE File", 15, 0x00FFFF00);
	    XTft_DrawFixedSizeButton(TftPtr, BOX_5_X_LEFT, BOX_5_Y_LEFT, BOX_5_X_RIGHT, BOX_5_Y_RIGHT, 
				     "V2PDraw", 7, 0x00FFFF00);
	    XTft_DrawFixedSizeButton(TftPtr, BOX_6_X_LEFT, BOX_6_Y_LEFT, BOX_6_X_RIGHT, BOX_6_Y_RIGHT, 
				     "Tic-Tac-Toe", 11, 0x00FFFF00);
	    XTft_DrawFixedSizeButton(TftPtr, BOX_7_X_LEFT, BOX_7_Y_LEFT, BOX_7_X_RIGHT, BOX_7_Y_RIGHT, 
				     "VxWorks", 7, 0x00FFFF00);
	    usleep(500);
	    DisplayBitMapImage1(TftPtr, 439, 419, logo_bitmap);
	    usleep(500);
	    break;
	  }
	}
      }
    }
  }

  return -1; /* not reached */

}

void DisplayBitMapImage1 (XTft *InstancePtr,
			  Xuint32 pixel_upper_left_corn_start_x,
			  Xuint32 pixel_upper_left_corn_start_y,
			  unsigned long *myBitmap)
{
  Xuint32 count = 0;
  Xuint32 x, y;
  
  for(y=pixel_upper_left_corn_start_y; y<(pixel_upper_left_corn_start_y+myBitmap[1]); y++) {
    for(x=pixel_upper_left_corn_start_x; x<(pixel_upper_left_corn_start_x+myBitmap[0]); x++) {
      XTft_mSetPixel(*(Xuint32 *)InstancePtr->BaseAddress, x, y, myBitmap[count+4]);
      count++;
    }
    count++;
  }
  
}

void DisplayBitMapImage2 (XTft *InstancePtr,
			  Xuint32 pixel_upper_left_corn_start_x,
			  Xuint32 pixel_upper_left_corn_start_y,
			  unsigned long *myBitmap)
{
  Xuint32 count = 0;
  Xuint32 x, y;
  count = 0;
  
  for(y=pixel_upper_left_corn_start_y; y<((pixel_upper_left_corn_start_y)+(myBitmap[1])); y++) {
    for(x=pixel_upper_left_corn_start_x; x<((pixel_upper_left_corn_start_x)+(myBitmap[0])); x++) {
      XTft_mSetPixel(*(Xuint32 *)InstancePtr->BaseAddress, x, y, myBitmap[count+4]);
      count++;
    }
  }

}

void DisplayBitMapPicture (XTft *InstancePtr)
{
  Xuint32 count, x, y;
  count = 0;

  for(y=120; y<(120+picture[1]); y++) {
    for(x=160; x<(160+picture[0]); x++) {
      XTft_mSetPixel(*(Xuint32 *)InstancePtr->BaseAddress, x, y, picture[count+4]);
      count++;
    }
  }

}

int main()
{
  XTft Tft;
  XTouchscreen Touchscreen;
  XStatus Status;
  
  int boxClicked;

  /* Initialize caches */
  XCache_EnableICache(0x80000001);
  XCache_EnableDCache(0x80000001);

#ifdef BOOTLOAD_DEBUG
  XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
  XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);
#endif

  /* Initialize TFT */
  Status = XTft_Initialize(&Tft, TFT_DEVICE_ID);
  STATUS_CHECK(Status);
  
  /* Initialize touchscreen */
  Status = XTouchscreen_Initialize(&Touchscreen, TOUCHSCREEN_DEVICE_ID);
  STATUS_CHECK(Status);
  
  /* Initialize TFT, touchscreen and display */
  InitGraphics(&Tft, &Touchscreen, 0x00ffffff, 0x00404040); /* foreground, background */
  
  DisplayBitMapImage2(&Tft, 245, 25, ml300_txt_bmp); /* "ML300 Bootloader" header */
  XTft_DrawFixedSizeButton(&Tft, BOX_1_X_LEFT, BOX_1_Y_LEFT, BOX_1_X_RIGHT, BOX_1_Y_RIGHT, "XROM_ML300", 10, 0x00FFFF00);
  XTft_DrawFixedSizeButton(&Tft, BOX_2_X_LEFT, BOX_2_Y_LEFT, BOX_2_X_RIGHT, BOX_2_Y_RIGHT, "Linux", 5, 0x00FFFF00);
  XTft_DrawFixedSizeButton(&Tft, BOX_3_X_LEFT, BOX_3_Y_LEFT, BOX_3_X_RIGHT, BOX_3_Y_RIGHT, "VxWorks Bootloader", 18, 0x00FFFF00);
  XTft_DrawFixedSizeButton(&Tft, BOX_4_X_LEFT, BOX_4_Y_LEFT, BOX_4_X_RIGHT, BOX_4_Y_RIGHT, "My Own ACE File", 15, 0x00FFFF00);
  XTft_DrawFixedSizeButton(&Tft, BOX_5_X_LEFT, BOX_5_Y_LEFT, BOX_5_X_RIGHT, BOX_5_Y_RIGHT, "V2PDraw", 7, 0x00FFFF00);
  XTft_DrawFixedSizeButton(&Tft, BOX_6_X_LEFT, BOX_6_Y_LEFT, BOX_6_X_RIGHT, BOX_6_Y_RIGHT, "Tic-Tac-Toe", 11, 0x00FFFF00);
  XTft_DrawFixedSizeButton(&Tft, BOX_7_X_LEFT, BOX_7_Y_LEFT, BOX_7_X_RIGHT, BOX_7_Y_RIGHT, "VxWorks", 7, 0x00FFFF00);
  /* Place the Xilinx logo on the screen */
  DisplayBitMapImage1(&Tft, 439, 419, logo_bitmap);

  boxClicked = DetectBox(&Tft, &Touchscreen);

  LoadProgram(&Tft, boxClicked);

  return 0;
}
