/******************************************************************************
*     $Date: 2004/01/13 00:32:42 $
*     $RCSfile: hello_cache.c,v $
*******************************************************************************

*******************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2004 Xilinx, Inc.
*     All rights reserved.
*
******************************************************************************/
#include "xparameters.h"
#include "xuartns550_l.h"

#define UART_BASEADDR XPAR_OPB_UART16550_0_BASEADDR
#define UART_CLOCK    XPAR_XUARTNS550_CLOCK_HZ

#define UART_BAUDRATE 9600                      /* real hardware */

#define ESCAPE          0x1b

#if MULTI_PROCESSOR == 1
#include "xpseudo_asm.h"
#include "sleep.h"

static void ProcInit(void)
{
  unsigned int proc_no;

  proc_no = mfdcr(0x6*4 + 2);
  proc_no >>= 24;
  sleep(proc_no * 5);
}
#endif  // MULTI_PROCESSOR

static void printf(char *s)
{
  while (*s) {
    XUartNs550_SendByte(UART_BASEADDR, *s);
    s++;
  }
}

void echo()
{
  char s;

  do {
    s = XUartNs550_RecvByte(UART_BASEADDR);
    if(s == '\r')
      XUartNs550_SendByte(UART_BASEADDR, '\n');
    XUartNs550_SendByte(UART_BASEADDR, s); 
  } while (s != ESCAPE);
}

int main()
{
#if MULTI_PROCESSOR == 1
  ProcInit();
#endif
  XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
  XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);
  printf("Hello world demo program running out of cache!\r\n");
  printf("============================================================"
	 "===================\r\n");
  printf("\r\nThis demo program runs completely out of cache. "
	 "The instructions are stored in the instruction cache "
	 "while the data and the stack reside in the data cache.\r\n\r\n"
	 "As a result, no internal BRAM or external memory is "
	 "required. This results in two important points:\r\n\r\n"
	 "1) The hardware design is less complex. You don't need "
	 "any memory controllers in your design.\r\n"
	 "2) Your software runs optimally fast. Cache is the fastest "
	 "memory in your system. Accesses to instruction and data "
	 "occur at maximum speed.\r\n\r\n"
	 "Accesses to peripherals are still possible. For example, this "
	 "program writes to the UART while printing this message.\r\n\r\n"
	 "How can I run my program out of caches?\r\n"
	 "There are two important steps. First, when linking your "
	 "program you have to separate your program into an instruction "
	 "part and a data part. In the linker script you map the "
	 "instructions and the data into two distinct memory sections. "
	 "As an example for a linker script have a look at \r\n"
	 "sw/standalone/mapfiles/mapfile5.\r\n"
	 "Second, you tell XMD the address ranges for the instruction "
	 "and the data cache when loading your program or generating "
	 "ACE files. (Refer to xmd.ini as an example) "
	 "You have to tell XMD that the instruction cache occupies 16KB "
	 "starting at address 0x70000000 and the data cache "
	 "occupies 16KB starting at address 0x78000000. When XMD detects "
	 "accesses to these address ranges it performs special "
	 "transactions to enable, read and write the caches.\r\n\r\n"
	 "What should I be aware of?\r\n"
	 "In your program do not enable or disable the caches. The "
	 "caches are used by your program and cannot cache any other "
	 "memory besides that.\r\n\r\n"
	 "With this example the Systems Engineering Group at Xilinx "
	 "hopes to get you started implementing your own software "
	 "completely running out of caches.\r\n\r\n"
	 "Type your message here:\r\n"
	 );
  echo();

  return 0;
}

