/*-----------------------------------------------------------------------------
//     $Date: 2004/01/17 00:54:05 $
//     $RCSfile: hello.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/

#include "xexception_l.h"
#include "xtft.h"
#include "xuartns550_l.h"
#include "xpseudo_asm.h"
#include "xtime_l.h"
#include "xparameters.h"

#define UART_BASEADDR              XPAR_OPB_UART16550_0_BASEADDR
#define UART_CLOCK                 XPAR_XUARTNS550_CLOCK_HZ
#define UART_BAUDRATE              9600
#define TFT_DEVICE_ID              XPAR_PLB_TFT_CNTLR_REF_0_DEVICE_ID

#define INTERVAL  0x00ffffff  // PIT constant
#define ESCAPE    0x1b

#define STATUS_CHECK(Status)       {if(Status != XST_SUCCESS) {return Status;}}

/* static variables */
static XTft Tft;

/* local prototypes */
static void tft_printf(char *s);
static void uart_printf(char *s);
static void uart_echo(void);

static void tft_printf(char *s)
{
  while (*s) {
    XTft_Write(&Tft, *s);
    s++;
  }
}

static void uart_printf(char *s)
{
  while (*s) {
    XUartNs550_SendByte(UART_BASEADDR, *s);
    s++;
  }
}

static void uart_echo(void)
{
  char s;

  do {
    s = XUartNs550_RecvByte(UART_BASEADDR);
    if (s == '\r')
      XUartNs550_SendByte(UART_BASEADDR, '\n');
    XUartNs550_SendByte(UART_BASEADDR, s); 
  } while (s != ESCAPE);
}

void TimerExceptionHandler (void *CallBackRef, Xuint32 Event, unsigned int EventData)
{
  static int i=0;

  if (i == 64) {
    i = 0;
  }
  XTime_PITClearInterrupt();
  XTime_PITSetInterval(INTERVAL);
  XTft_SetPos(&Tft, i, i);
  tft_printf("PIT");
  i++;
}

int main (void)
{
  XStatus Status;

  /* Initialize UART and TFT */
  XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
  XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);  
  Status = XTft_Initialize(&Tft, TFT_DEVICE_ID);
  STATUS_CHECK(Status);
  XTft_SetColor(&Tft, 0xffffff, 0x000000);
  XTft_ClearScreen(&Tft);
  XTft_SetPos(&Tft, 120, 10);
  tft_printf("hello_pit: Test Programmable-Interval Timer (PIT)\n\r");
  XTft_SetPos(&Tft, 120, 25);
  tft_printf("Prints \"PIT\" on TFT when PIT interrupt occurs\n\r");
  XTft_SetPos(&Tft, 120, 40);
  tft_printf("and echoes characters on UART until ESC is pressed.\n\r");

  uart_printf(" hello_pit: Test Programmable-Interval Timer (PIT)\n\r");
  uart_printf("            Prints \"PIT\" on TFT when PIT interrupt occurs\n\r");
  uart_printf("            and echoes characters on UART until ESC is pressed.\n\r");

  XExc_Init(); // This initializes the Exception Handler. 
  XExc_RegisterHandler(XEXC_ID_PIT_INT, 
		       (XExceptionHandler)TimerExceptionHandler, 
		       (void *) XNULL);
  /* Initialize PIT */
  XTime_PITSetInterval(INTERVAL);
  XTime_TSRClearStatusBits(XREG_TSR_CLEAR_ALL);
  XTime_PITEnableAutoReload();
  XTime_PITEnableInterrupt();

  XExc_mEnableExceptions(XEXC_ALL);

  uart_echo();
  XTime_PITDisableInterrupt();
  XExc_mDisableExceptions(XEXC_ALL);
  XExc_RemoveHandler(XEXC_ID_PIT_INT);

  XTft_SetPos(&Tft, 250, 70);
  tft_printf("Bye...");
  return 0;
}

