/******************************************************************************
*     $Date: 2004/01/01 01:26:20 $
*     $RCSfile: iic_tft_brightness.c,v $
*******************************************************************************

*******************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2004 Xilinx, Inc.
*     All rights reserved.
*
******************************************************************************/
#include "xbasic_types.h"
#include "xiic_l.h"
#include "xparameters.h"
#include "xuartns550_l.h"
//#include "exit.h"

/************************** Function Prototypes ****************************/

/************************** Constant Definitions ***************************/

/* IIC Base Address
 */
#define IIC_BASE_ADDRESS            XPAR_OPB_IIC_0_BASEADDR

/* Potentiometer IIC Bus Address.
 */
#define SCREEN_POTI_ADDRESS         (0xac/2)

/* Potentiometer setting address within the devices address range.
 */
#define SCREEN_POTI_SETTING_ADDRESS 0xf8

#define UART_BASEADDR  XPAR_OPB_UART16550_0_BASEADDR
#define UART_CLOCK     XPAR_XUARTNS550_CLOCK_HZ
#if !SIM
#define UART_BAUDRATE 9600                      /* real hardware */
#else
#define UART_BAUDRATE (UART_CLOCK / 16 / 3)     /* simulation */
#endif

#define ESCAPE        0x1b

/**************************** Type Definitions *****************************/

/***************** Macros (Inline Functions) Definitions *******************/

/************************** Variable Definitions **************************/

/*****************************************************************************/

static void printstr(char *s)
{
  while (*s)
  {
    XUartNs550_SendByte(UART_BASEADDR, *s);
    s++;
  }
}

static void printnum(unsigned int x)
{
  static char hex[]="0123456789ABCDEF";
  char digit[10];
  int i;

  i = 0;
  do {
    digit[i] = hex[x % 10];
    x = x / 10;
    i++;
  } while (x != 0);

  while (i > 0) {
    i--;
    XUartNs550_SendByte(UART_BASEADDR, digit[i]);
  }
}

static int GetBrightness(Xuint8 *value)
{
    int ByteCount;
    Xuint8 Address = 0xf8;

    ByteCount = XIic_Send(IIC_BASE_ADDRESS, SCREEN_POTI_ADDRESS,
                          (Xuint8 *)&Address, 1);
    ByteCount = XIic_Recv(IIC_BASE_ADDRESS, SCREEN_POTI_ADDRESS,
                          value, 1);
    return ByteCount;
}

static int SetBrightness(Xuint8 value)
{
    int ByteCount;
    Xuint8 Data[2];

    Data[0] = 0xf8;
    Data[1] = value;

    ByteCount = XIic_Send(IIC_BASE_ADDRESS, SCREEN_POTI_ADDRESS,
                          Data, 2);
    return ByteCount;
}

int main(void)
{
    Xuint8 s;
    Xuint8 count;
    Xuint8 value;   

    XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
    XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);
    printstr("\r\n\n\n\nI2C Demo using Level0 driver \r\n\n");
    printstr("Use the '+' and '-' keys to adjust the screen brightness.\r\n");
    printstr("Press ESC to exit. The last value will be saved\r\n\n");
    do {
      count=GetBrightness(&value);
      
      //printstr("Current Setting:    \r");
      printstr("Current Setting: ");
      printnum(value);
      printstr("\r");
      
      s = XUartNs550_RecvByte(UART_BASEADDR);
      
      switch(s) {
      case '+':
	printstr("Increase\n\r");
	if (value < 255) {
	  value++;
	  count=SetBrightness(value);
	}
	break;
	
      case '-':
	printstr("Decrease\n\r");
	if (value > 0) {
	  value--;
	  count=SetBrightness(value);
	}
	break;
	
      default:
      }
    } while (s != ESCAPE);
    
    return 0;
}


