/*-----------------------------------------------------------------------------
//     $Date: 2003/04/29 04:03:20 $
//     $RCSfile: ml300_set_eeprom.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include "sleep.h"
#include "xbasic_types.h"
#include "xparameters.h"
#include "xiic_l.h"
#include "xuartns550_l.h"

#define UART_BASEADDR XPAR_OPB_UART16550_0_BASEADDR
#define UART_CLOCK    XPAR_XUARTNS550_CLOCK_HZ

#define START_ADDR    1024
#define END_ADDR      2047
#define IIC_SIZE      (END_ADDR-START_ADDR+1)
#define IIC_ADDR      0xA0

#define IIC_DELAY     5000 /* useconds */

#if !SIM
#define UART_BAUDRATE 9600                      /* real hardware */
#else
#define UART_BAUDRATE (UART_CLOCK / 16 / 3)     /* simulation */
#endif

Xint8 defaultVal[] = "C=\0\0\0E=000A35002200\0";


static void send(Xuint32 adr, Xint8 *data, Xuint32 len)
{
  Xint8 sendBuf[34];
  Xuint32 pos, wlen;
  Xuint32 ret;

  wlen = 32;
  for (pos=0; pos < len; pos+=32)
  {
    if ((len - pos) < 32)
      wlen = len-pos;
    
    sendBuf[0] = (Xint8) ((adr+pos) >> 8);
    sendBuf[1] = (Xint8) (adr+pos);
    memcpy(&sendBuf[2], &data[pos], wlen);
    ret = XIic_Send(XPAR_OPB_IIC_0_BASEADDR, IIC_ADDR>>1, sendBuf, wlen+2);
    usleep(IIC_DELAY);
  }
}

static void receive(Xuint32 adr, Xint8 *data, Xuint32 len)
{
  Xint8 address[2];
  Xuint32 ret;

  address[0] = (Xint8) (adr >> 8);
  address[1] = (Xint8) adr;
  ret = XIic_Send(XPAR_OPB_IIC_0_BASEADDR, IIC_ADDR>>1, address, 2);
  ret = XIic_Recv(XPAR_OPB_IIC_0_BASEADDR, IIC_ADDR>>1, data, len);
}

static void ask(Xint8 *question)
{
  Xint8 c[2];
  
  printf("%s (y/N)? ", question);
  scanf("%1s", c);
  
  if ((c[0]!='y') && (c[0]!='Y'))
  {
    printf("\nYou said 'No'. Good bye!\n");
    exit(-1);
  }
}

static void findKeys(Xint8 *data, Xuint32 len,
                     Xuint32* crcStart, Xuint32* macStart)
{
  /* we assume that 'E=' and 'C=' only occur once */
  Xuint32 i;

  *macStart = 0xffffffff;
  *crcStart = 0xffffffff;
  for(i=1; i < len; i++)
  {
    if(data[i] == '=')
    {
      switch (data[i-1])
      {
      case 'E':
        *macStart = i-1;
        break;
      case 'C':
        *crcStart = i-1;
        break;
      default:
        break;
      }
    }
  }
}

int main()
{
  Xint32 i;
  Xuint32 crc;
  Xuint32 crcStart, macStart;
  Xint8 data[IIC_SIZE];
  Xint8 addr[13];
  Xint8 valid;
  static const Xint8 hex[]="0123456789ABCDEF";

  XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
  XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);

  printf(
    "This program will store the Ethernet MAC address into the IIC EEPROM.\n"
    "As part of this process the EEPROM content in the memory range "
    "[%d,%d] will be deleted.\n", START_ADDR, END_ADDR);
  ask("Do you want to proceed");

  /* read current MAC address */
  receive(START_ADDR, data, IIC_SIZE);

  findKeys(data, sizeof(data), &crcStart, &macStart);
  if ((crcStart == 0xffffffff) || (macStart == 0xffffffff))
  {
    printf("No current settings found\n");
    ask("Do you want to use new default settings");

    /* yes */
    memcpy(data, defaultVal, sizeof(defaultVal));
    findKeys(data, sizeof(data), &crcStart, &macStart);
  }
  
  printf("Current MAC address: %s\n", &data[macStart+2]);
  ask("Do you want to overwrite this setting");
  
  /* yes */
  while (1)
  {
    printf("Enter the Ethernet MAC address for your board: ");
    addr[12]=0;
    scanf("%12s", addr);
    valid = 1;
    for (i = 0; addr[i] ; i++)
      valid &= (((addr[i] >= '0') && (addr[i] <= '9')) ||
                ((addr[i] >= 'a') && (addr[i] <= 'f')) ||
                ((addr[i] >= 'A') && (addr[i] <= 'F')));
    valid &= (i==12);
    if (valid)
      break;
    printf("Invalid address: %s\n. Try again.\n", addr);
  }

  /* set new mac address */
  memcpy(&data[macStart+2], addr, 12);
    
  /* new CRC */
  printf("Calculating CRC\n");

  /* set temporarily to ease parsing for end of data */
  data[crcStart+2] = 0xff;
  data[crcStart+3] = 0xff;
  
  crc = 0;
  i = 0;
  do
  {
    crc += data[i];
    i++;
  } while (data[i] || data[i-1]);
  
  crc -= 2*0xff;
  data[crcStart+2] = hex[(crc >> 4) & 0xf];
  data[crcStart+3] = hex[crc & 0xf];

  printf("Programming\n");
  send(START_ADDR, data, IIC_SIZE);

  printf("Verifying\n");
  receive(START_ADDR+macStart+2, data, 12);
  data[12]=0;
  for (i=0; i < 12; i++)
  {
    if (addr[i] != data[i])
    {
      printf("ERROR; MAC address read back: %s\n", data);
      return -2;
    }
  }

  printf("OK.\n");
  
  return 0;
}
