/******************************************************************************
*     $Date: 2004/01/01 01:29:24 $
*     $RCSfile: scalechar.c,v $
*******************************************************************************

*******************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2004 Xilinx, Inc.
*     All rights reserved.
*
******************************************************************************/
#include "xbasic_types.h"
#include "xcache_l.h"
#include "xtft.h"
#include "xtft_charcode.h"
#include "xparameters.h"
//#include "memaccess.h"
//#include "assert.h"
//#include "exit.h"

#define BUFFER_COLUMNS   1024
#define DISPLAY_COLUMNS  640
#define DISPLAY_ROWS     480
#define CHAR_WIDTH       XTFT_CHAR_WIDTH
#define CHAR_HEIGHT      XTFT_CHAR_HEIGHT
#define MAXSIZE          4

#define TFT_DEVICE_ID           XPAR_PLB_TFT_CNTLR_REF_0_DEVICE_ID

#define STATUS_CHECK(Status) { if (Status != XST_SUCCESS) { return Status; } }

static XTft Tft;
static unsigned int curPositionX, curPositionY;
static unsigned int currentFg, currentBg;

/* Program specific functions */

/***********************************/
/*  genScaledPixels returns a line */
/*  of the new scaled character    */
/***********************************/
unsigned long genScaledPixels(unsigned int size,
                              unsigned char a7,
                              unsigned char a6,
                              unsigned char a5,
                              unsigned char a4,
                              unsigned char a3,
                              unsigned char a2,
                              unsigned char a1,
                              unsigned char a0)
{
   unsigned long a7l,a6l,a5l,a4l,a3l,a2l,a1l,a0l,result;
   int i;
   result = 0;

   // Assign chars to longs
   a7l = a7;
   a6l = a6;
   a5l = a5;
   a4l = a4;
   a3l = a3;
   a2l = a2;
   a1l = a1;
   a0l = a0;

   // Scale this line based on size
   for (i=0;i<size;i++) {
     if (a7l)
       result = result | (a7l << (7*size+(size-(i+1))));
     if (a6l) result = result | (a6l << (6*size+(size-(i+1))));
     if (a5l) result = result | (a5l << (5*size+(size-(i+1))));
     if (a4l) result = result | (a4l << (4*size+(size-(i+1))));
     if (a3l) result = result | (a3l << (3*size+(size-(i+1))));
     if (a2l) result = result | (a2l << (2*size+(size-(i+1))));
     if (a1l) result = result | (a1l << (1*size+(size-(i+1))));
     if (a0l) result = result | (a0l << (size-(i+1)));
   }

   return result;
}


/***********************************/
/*  tftWriteChar writes a scaled   */
/*  character to the tft display   */
/***********************************/
static void tft_WriteScaledChar(
  unsigned char ch,
  unsigned int fg,
  unsigned int bg,
  short int size)
{
  unsigned int adr, col, x, y;
  unsigned long val;
  unsigned long scaledChar[CHAR_HEIGHT*MAXSIZE];

  // Calculate the address to start writing to
  adr = BUFFER_COLUMNS * curPositionY + curPositionX;

  // Fill array with scaled character
  for(y=0; y<CHAR_HEIGHT; y++) {
    for (x=0; x<size; x++) {
      scaledChar[y*size+x] = genScaledPixels(size,
					     (XTft_vidChars[ch][y] & 0x80)?1:0,
					     (XTft_vidChars[ch][y] & 0x40)?1:0,
					     (XTft_vidChars[ch][y] & 0x20)?1:0,
					     (XTft_vidChars[ch][y] & 0x10)?1:0,
					     (XTft_vidChars[ch][y] & 0x08)?1:0,
					     (XTft_vidChars[ch][y] & 0x04)?1:0,
					     (XTft_vidChars[ch][y] & 0x02)?1:0,
					     (XTft_vidChars[ch][y] & 0x01)?1:0);
    }
  }
  
  // Print character to screen
  for (y=0; y<(CHAR_HEIGHT*size); y++) {
     val = scaledChar[y];
     for (x=0; x<CHAR_WIDTH*size; x++) {
       if (val & (1 << ((CHAR_WIDTH * size) - x - 1)))
         col = fg;
       else
         col = bg;
       XTft_SetPixel(&Tft, curPositionX + x, curPositionY+y, col);
     }
  }
  return;
}


void tft_WriteChar(unsigned char val,unsigned int size)
{
  if (size > MAXSIZE) return;

  switch (val) {
  case 0xd:  /* carriage return */
    curPositionX = 0;
    break;
  case 0xa:  /* line feed */
    curPositionX = 0;
    curPositionY += (CHAR_HEIGHT * size);
    break;
  default:
    tft_WriteScaledChar(val, currentFg, currentBg, size);
    curPositionX += (CHAR_WIDTH * size);
    break;
  }
}

void tft_WriteString(unsigned char *val, unsigned int size)
{
  while (*val) {
    tft_WriteChar(*val,size);
    val++;
  }
}

int main (void)
{
  XStatus Status;

  /* Initialize caches */
  XCache_EnableICache(0x80000001);
  XCache_EnableDCache(0x80000001);

  /* Initialize TFT */
  Status = XTft_Initialize(&Tft, TFT_DEVICE_ID);
  STATUS_CHECK(Status);

  XTft_ClearScreen(&Tft);
  curPositionX = 1;
  curPositionY = 1;
  currentFg = 0xffffffff;
  currentBg = 0x00000000;
  tft_WriteString("the quick brown fox jumps over the lazy dog\n",1);
  tft_WriteString("THE QUICK BROWN FOX JUMPS OVER THE LAZY DOG 1234567890\n",1);
  tft_WriteString("the quick brown fox jumps \nover the lazy dog\n",2);
  tft_WriteString("THE QUICK BROWN FOX JUMPS \nOVER THE LAZY DOG 1234567890\n",2);
  tft_WriteString("the quick brown fox jumps \nover the lazy dog\n",3);
  tft_WriteString("THE QUICK BROWN FOX JUMPS \nOVER THE LAZY DOG 1234567890\n",3);
  tft_WriteString("the quick brown fox jumps \nover the lazy dog\n",4);
  tft_WriteString("THE QUICK BROWN FOX JUMPS \nOVER THE LAZY DOG 1234567890\n",4);
  return 0;
}
