/*-----------------------------------------------------------------------------
//     $Date: 2004/01/15 14:57:38 $
//     $RCSfile: scan_pci.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/

#include <stdio.h>
#include "xuartns550_l.h"
#include "xpci_l.h"
#include "xparameters.h"
#include "xpseudo_asm.h"
#include "sleep.h"

#define PCI_BASEADDR  XPAR_OPB_PCI_REF_0_BASEADDR
#define UART_BASEADDR XPAR_OPB_UART16550_0_BASEADDR
#define UART_CLOCK    XPAR_XUARTNS550_CLOCK_HZ
#if !SIM
#define UART_BAUDRATE 9600                      /* real hardware */
#else
#define UART_BAUDRATE (UART_CLOCK / 16 / 3)     /* simulation */
#endif

/* from V2PDK */
#define PCI_CTRL_OFFSET        0x1E000004
#define PCI_CFG_VAL            0xffff1047
// redefine two constants in xpci_l.h so that XPci_mConfigIn works
#undef  XPCI_CAR_OFFSET
#undef  XPCI_CDR_OFFSET
#define XPCI_CAR_OFFSET        0x1C000000
#define XPCI_CDR_OFFSET        0x1C000004

#define MAX_NUM_BUSSES         1
#define MAX_NUM_DEVICES        32

/* local prototypes */

static void pci_dump_device(int value, int busDevFunc);
int ByteSwap(int b);

int ByteSwap(int b) {
  return ( ((b & 0xff) << 24)      | 
	   ((b & 0xff00) << 8)     |
	   ((b & 0xff0000) >> 8)   |
	   ((b & 0xff000000) >> 24) );
}

static void pci_dump_device(int value, int busDevFunc)
{
  unsigned int bus;
  unsigned int device;
  int i;
  
  bus = (busDevFunc >> 16) & 0x1f;
  device = (busDevFunc >> 11) & 0x1f;

  printf("*********************************\r\n");
  printf("Bus: %d, Device: %d\r\n", bus, device);
  printf("*********************************\r\n");

  printf("Vendor ID:      0x%04x, Device ID:  0x%04x\r\n", 
	 value & 0xffff, (value >> 16) & 0xffff);

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x2 << 2), value);
  printf("Rev ID:         0x%02x,   Class code: 0x%06x\r\n",
	 value & 0xff, (value >> 8) & 0xffffff);

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x3 << 2), value);
  printf("Header Type:    0x%02x\r\n",
	 (value >> 16) & 0xff);

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x4 << 2), value);
  printf("Base Address 0: 0x%08x\r\n", value);

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x5 << 2), value);
  printf("Base Address 1: 0x%08x\r\n", value);

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x6 << 2), value);
  printf("Base Address 2: 0x%08x\r\n", value);

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x7 << 2), value);
  printf("Base Address 3: 0x%08x\r\n", value);

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x8 << 2), value);
  printf("Base Address 4: 0x%08x\r\n", value);

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x9 << 2), value);
  printf("Base Address 5: 0x%08x\r\n", value);

  printf("Raw data (words 0 to 15):\r\n");
  // dump raw data
  for(i=0; i<16; i++) {
    XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (i << 2), value);
    printf("Data word %2d: 0x%08x\r\n", i, value); 
  }

}

int main()
{
  int busDevFunc;
  int devices;
  int value;     // value read from PCI bus
  int i, j;

  /* Initialize UART */
  XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
  XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);

  /* Initialize PCI by self-configuration */
  XPci_mWriteReg(PCI_BASEADDR, PCI_CTRL_OFFSET, PCI_CFG_VAL);

  // read config data reg - expected value: 0xee100003
  busDevFunc = XIo_In32(PCI_BASEADDR+0x1E000000);
  printf("Device ID and vendor ID: 0x%x\r\n", busDevFunc);

  printf("Scanning PCI bus\r\n");
  busDevFunc = 0;
  devices = 0;
  value = -1;

  // scan devices
  for (i=0; i<MAX_NUM_BUSSES; i++) {
    for (j=0; j<MAX_NUM_DEVICES; j++) {
      // bus number
      busDevFunc = (i << 16);
      // device number and set enable bit
      busDevFunc |= (j << 11) | 0x80000000;
      // scan bus for device
      XPci_mConfigIn(PCI_BASEADDR, busDevFunc, value);
      // if value == -1, then no device
      if (value != -1) {
	devices++; // increment total;
	printf("Device %d found: %x\r\n", j, value);
	pci_dump_device(value, busDevFunc);
      }
    }
  }

  printf("\r\nNumber of PCI devices found while scanning: %d\r\n\r\n", devices);

  return 0;
}
