/*-----------------------------------------------------------------------------
//     $Date: 2003/04/29 04:54:43 $
//     $RCSfile: sysace_sector_ops_polled.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/

/***************************** Include Files *********************************/

#include <stdio.h>
#include "xsysace.h"
#include "xstatus.h"
#include "xparameters.h"
#include "xuartns550_l.h"
#include "sleep.h"

/************************** Constant Definitions *****************************/
#define UART_BASEADDR XPAR_OPB_UART16550_0_BASEADDR
#define UART_CLOCK    XPAR_XUARTNS550_CLOCK_HZ
#define UART_BAUDRATE 9600

#define NUM_SECTORS   1000
#define ENABLE_WRITE  0
#define TIMEOUT       60 /* seconds */
/**************************** Type Definitions *******************************/

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Function Prototypes ******************************/

/************************** Variable Definitions *****************************/


int main()
{
  XSysAce Ace;                            /* an instance of the device */
  XStatus Result;
  Xuint8 Buffer[XSA_CF_SECTOR_SIZE];      /* fits one sector of data */
  Xuint32 i, j;
  
  /* Initialize the UART to print some messages */
  XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
  XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);

  printf("\nThis program demonstrates how data is read from or "
         "written to a CompactFlash card or a MicroDrive in "
         "the System ACE card slot.\r\n\r\n"
         "Per default writing is not enabled. To enable writing you "
         "have to modify the source code and change\r\n"
         "#define ENABLE_WRITE 0\r\n"
         "to\r\n"
         "#define ENABLE_WRITE 1\r\n\r\n"
         "Writing will delete the contents of your device including all "
         "partition information.\r\n"
         "Currently, the setting is:\r\n"
         "#define ENABLE_WRITE %d\r\n"
         "If this is not what you want turn the power off now!\r\n\r\n"
         "The program execution will continue in %d seconds...\r\n\r\n",
         ENABLE_WRITE, TIMEOUT);
  sleep(TIMEOUT);

  /* print out the sector size */
  printf("The sector size is %d bytes\r\n", XSA_CF_SECTOR_SIZE);
  
  /*
   * Initialize the instance. The device defaults to polled mode.
   */
  Result = XSysAce_Initialize(&Ace, XPAR_OPB_SYSACE_0_DEVICE_ID);
  if (Result != XST_SUCCESS)
  {
    printf("Could not initialize the System ACE driver!\r\n");
    return -1;
  }

  /*
   * Get a lock (this is a non-blocking function call to allow the user
   * to do other things while waiting for the lock)
   */
  printf("Getting a lock on the System ACE device...");
  while ((Result = XSysAce_Lock(&Ace, XFALSE)) != XST_SUCCESS);
  printf("done\r\n");
  
  /*
   * Read the sector of data. The starting sector is 0 and the number
   * of sectors to read is 1.
   *
   * In polled mode, this is a blocking call that returns when the sector
   * has been read (or an error occurred).
   */

  printf("Reading the first %d sectors on the device...", NUM_SECTORS);  
  for (i=0; i < NUM_SECTORS; i++)
  {
    while (!XSysAce_IsCFReady(&Ace)) ;
    Result = XSysAce_SectorRead(&Ace, i, 1, Buffer);
    if (Result != XST_SUCCESS)
    {
      printf("\r\nError during read operation. Quitting...\r\n");
      return -2;
    }
  }    
  printf("done.\r\n");

  if (ENABLE_WRITE)
  {
    printf("Writing the first %d sectors on the device...", NUM_SECTORS);  
    for (i=0; i < NUM_SECTORS; i++)
    {
      for (j = 0; j < XSA_CF_SECTOR_SIZE; j++)
        Buffer[j] = (Xuint8) (i+j);   /* every sector starts with +1 */
      
      while (!XSysAce_IsCFReady(&Ace)) ;
      Result = XSysAce_SectorWrite(&Ace, i, 1, Buffer);
      if (Result != XST_SUCCESS)
      {
        printf("\r\nError during write operation. Quitting...\r\n");
        return -3;
      }
    }    
    printf("done.\r\n");
  }
  else
    printf("Writing skipped!\r\n");

  /*
   * Release the lock
   */
  printf("Releasing the lock on the System ACE device...");
  XSysAce_Unlock(&Ace);
  printf("done.\r\n");
  
  return 0;
}

