/*-----------------------------------------------------------------------------
//     $Date: 2004/03/10 23:31:44 $
//     $RCSfile: test_spi.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2002 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/

#include <stdio.h>
#include "xuartns550_l.h"
#include "xspi_l.h"
#include "xparameters.h"
#include "sleep.h"

#define SPI_BASEADDR  XPAR_OPB_SPI_0_BASEADDR
#define UART_BASEADDR XPAR_OPB_UART16550_0_BASEADDR
#define UART_CLOCK    XPAR_XUARTNS550_CLOCK_HZ
#if !SIM
#define UART_BAUDRATE 9600                      /* real hardware */
#else
#define UART_BAUDRATE (UART_CLOCK / 16 / 3)     /* simulation */
#endif

#define ESCAPE    0x1b

/* Functions to access and set up SPI */
/* Set Write Enable */
void set_wren (void)
{
  unsigned int j;
  usleep (100);
  XSpi_mSetSlaveSelectReg(SPI_BASEADDR, 0x00000000); // CS Low
  XSpi_mSendByte(SPI_BASEADDR, 0x06); // Enable Writes
  usleep (100);
  j = XSpi_mRecvByte(SPI_BASEADDR); // Dummy read
  usleep (100);
  XSpi_mSetSlaveSelectReg(SPI_BASEADDR, 0x00000001);  // CS High
  usleep (100);
}

/* Read and return value of SPI Status register */
unsigned int rd_spi_stat_reg (void)
{
  unsigned int j;
  usleep (100);
  XSpi_mSetSlaveSelectReg(SPI_BASEADDR, 0x00000000); // CS Low
  XSpi_mSendByte(SPI_BASEADDR, 0x05);  // Send Read status reg Command
  XSpi_mSendByte(SPI_BASEADDR, 0x00); // Dummy Write
  usleep (100);
  j = XSpi_mRecvByte(SPI_BASEADDR); // Dummy read
  usleep (100);
  j = XSpi_mRecvByte(SPI_BASEADDR); // Read data
  usleep (100);
  XSpi_mSetSlaveSelectReg(SPI_BASEADDR, 0x00000001);  // CS High
  usleep (100);
  return j;
}

/* Write wr_data to SPI Status register */
void wr_spi_stat_reg (unsigned int wr_data)
{
  unsigned int j;
  usleep (100);
  set_wren();
  usleep (100);
  XSpi_mSetSlaveSelectReg(SPI_BASEADDR, 0x00000000); // CS Low
  XSpi_mSendByte(SPI_BASEADDR, 0x01);  // Send Write status reg Command
  XSpi_mSendByte(SPI_BASEADDR, wr_data);     // Write to status Reg
  usleep (100);
  j = XSpi_mRecvByte(SPI_BASEADDR); // Dummy read
  usleep (100);
  j = XSpi_mRecvByte(SPI_BASEADDR); // Dummy read
  usleep (100);
  XSpi_mSetSlaveSelectReg(SPI_BASEADDR, 0x00000001);  // CS High
  usleep (100);
  usleep (50000);                        // Takes 5ms to do a write, otherwise poll on WIP bit
}

/* Transmit specified SPI data to a specified address */
void wr_spi_data(unsigned int wr_addr, unsigned int wr_data)
{
  unsigned int j;
  usleep (100);
  set_wren ();
  usleep (100);
  XSpi_mSetSlaveSelectReg(SPI_BASEADDR, 0x00000000); // CS Low
  XSpi_mSendByte(SPI_BASEADDR, 0x02); // Issue Write Instruction
  XSpi_mSendByte(SPI_BASEADDR, (wr_addr >> 8)); // Send  Write Addr MS
  XSpi_mSendByte(SPI_BASEADDR, wr_addr);  // Send  Write Addr LSB
  XSpi_mSendByte(SPI_BASEADDR, wr_data);  // Send  Data
  usleep (100);
  j = XSpi_mRecvByte(SPI_BASEADDR); // Dummy read
  usleep (100);
  j = XSpi_mRecvByte(SPI_BASEADDR); // Dummy read
  usleep (100);
  j = XSpi_mRecvByte(SPI_BASEADDR); // Dummy read
  usleep (100);
  j = XSpi_mRecvByte(SPI_BASEADDR); // Dummy read
  usleep (100);
  XSpi_mSetSlaveSelectReg(SPI_BASEADDR, 0x00000001);  // CS High
  usleep (5000);                           // Takes 5ms to do a write, otherwise poll on WIP bit
}

/* Read SPI data from a specified location */
unsigned int rd_spi_data(unsigned int rd_addr)
{
  unsigned int j;
  usleep (100);
  XSpi_mSetSlaveSelectReg(SPI_BASEADDR, 0x00000000); // CS Low
  XSpi_mSendByte(SPI_BASEADDR, 0x03); // Issue Read Instruction
  XSpi_mSendByte(SPI_BASEADDR, (rd_addr >> 8)); // Send Read Addr MS
  XSpi_mSendByte(SPI_BASEADDR, rd_addr); // Send Read Addr LSB
  XSpi_mSendByte(SPI_BASEADDR, 0x00); // Issue dummy write
  usleep (100);
  j = XSpi_mRecvByte(SPI_BASEADDR); // Dummy read
  usleep (100);
  j = XSpi_mRecvByte(SPI_BASEADDR); // Dummy read
  usleep (100);
  j = XSpi_mRecvByte(SPI_BASEADDR); // Dummy read
  usleep (100);
  j = XSpi_mRecvByte(SPI_BASEADDR); // Dummy read
  usleep (100);
  XSpi_mSetSlaveSelectReg(SPI_BASEADDR, 0x00000001);  // CS High
  usleep (100);
  return j;
}

int main()
{
  unsigned int i;
  
  /* Initialize UART */
  XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
  XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);
  
  printf ("\n\r----- Start ----\n\r");
  printf("This program demonstrates how to set up and access the SPI\r\n");
  printf("SPI memory is zero-ed and then written to sequentially, with the\r\n");
  printf("data being (displayed in hexadecimal) bytes from 0x0 to 0xff\r\n");
  printf ("\n\r----------------\n\r");

  // Enable system, master mode, reset Tx, Rx FIFOs, inhibit master transactions
  XSpi_mSetControlReg(SPI_BASEADDR, (XSP_CR_MASTER_MODE_MASK |
				     XSP_CR_TXFIFO_RESET_MASK | 
				     XSP_CR_RXFIFO_RESET_MASK));
  XSpi_mEnable(SPI_BASEADDR); // Set System Enable bit
  
  /* Set up SPI correctly for subsequent operations */
  printf ("Write to SPI status reg to Write-Protect all Data\n\r");
  wr_spi_stat_reg (0x0000000e);
  // Value read back will not be 0x0000000e
  printf ("Read SPI status reg = %x\n\r", XSpi_mGetStatusReg(SPI_BASEADDR));
  
  printf ("Write to SPI status reg to make all data Writeable\n\r");
  wr_spi_stat_reg (0x00000000);
  // Value read back will not be 0x0000000e
  printf ("Read SPI status reg = %x\n\r", XSpi_mGetStatusReg(SPI_BASEADDR));
  
  printf ("Set Write Enable Latch\n\r");
  set_wren ();
  printf ("Read SPI status reg = %x\n\r", rd_spi_stat_reg());
  
  // Clear Mem
  printf ("\n\rClearing data in memory to zeros:\n\r");
  for (i = 0; i < 2048; i++) {
    wr_spi_data (i, 0);
  }
  
  // Check Clear
  printf ("\n\rRead Data (should be all zeros):\n\r");
  for (i = 0; i < 2048; i++) {
    printf ("%2x", rd_spi_data (i));
  }

  usleep (1000000);
  
  // Write Data
  printf ("\n\rWriting Data (0,1,2...,a,b,c,...,10,11,12,..,a0,a1,..,ff):\n\r");
  for (i = 0; i < 2048; i++) {
    wr_spi_data (i, i);
  }

  // Read Data
  printf ("\n\rRead Data:\n\r");
  for (i = 0; i < 2048; i++) {
    printf ("%2x", rd_spi_data (i));
  }

  printf ("\n\r------ End -----\n\r");

  return 0;
}
