/******************************************************************************
*     $Date: 2004/01/01 01:30:43 $
*     $RCSfile: touchcalibrate.c,v $
*******************************************************************************

*******************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2004 Xilinx, Inc.
*     All rights reserved.
*
******************************************************************************/
#include <stdio.h>
#include "xuartns550_l.h"
#include "xtouchscreen.h"
#include "xtft.h"
#include "sleep.h"
#include "xcache_l.h"
#include "xparameters.h"
#include "math.h"

#define UART_BASEADDR           XPAR_OPB_UART16550_0_BASEADDR
#define UART_CLOCK              XPAR_XUARTNS550_CLOCK_HZ
#if !SIM
#define UART_BAUDRATE           9600                      /* real hardware */
#else
#define UART_BAUDRATE           (UART_CLOCK / 16 / 3)     /* simulation */
#endif
#define XPAR_TFT_0_DEVICE_ID    XPAR_PLB_TFT_CNTLR_REF_0_DEVICE_ID
#define TOUCHSCREEN_DEVICE_ID   XPAR_OPB_TSD_REF_0_DEVICE_ID

#define DISPLAY_COLUMNS         640
#define DISPLAY_ROWS            480
#define TFT_WIDTH               640
#define CHAR_WIDTH              8
#define CHAR_HEIGHT             12
#define NORM_FACTOR             1000000

#define NUM_SAMPLES             100

#define STATUS_CHECK(Status)         \
    {                                \
        if (Status != XST_SUCCESS)   \
        {                            \
            return Status;           \
        }                            \
    }

/* XTft_WriteString: Writes a string of characters to the TFT */
void XTft_WriteString(XTft *InstancePtr, unsigned char *val) 
{
  while (*val) {
    XTft_Write(InstancePtr, *val);
    val++;
  }
}

/* XTft_WriteCenteredString: Writes a string centered along a specified column */
void XTft_WriteCenteredString(XTft *InstancePtr, char * str, int ypos)
{
	int totalWidth = strlen(str) * CHAR_WIDTH;

	XTft_SetPos(InstancePtr, TFT_WIDTH / 2 - totalWidth / 2, ypos);
	XTft_WriteString(InstancePtr, str);
}

/* XTft_DrawCircle: Draws a circle on a centerpoint with
 * a specified radius and thickness                      */
int XTft_DrawCircle(XTft *InstancePtr, 
		    int x, int y, int r, int thickness, unsigned int col)
{
  int i, j, dx, dx2, dy, distance;
  
  if (x >= 0 &&
      x <= DISPLAY_COLUMNS-1 &&
      y >= 0 &&
      y <= DISPLAY_ROWS-1 &&
      thickness <= r) {
    
    for (i=x-r; i<=(x+r); i++) {
      if ((i >= 0) && (i <= DISPLAY_COLUMNS-1)) {
	dx = x-i;
	dx2 = dx*dx;
	for (j=y-r; j<=y+r; j++) {
	  if ((j > 0) && (j <= DISPLAY_ROWS-1)) {
	    dy = y-j;
	    distance = sqrt(dx2+(dy * dy));
	    if (thickness == 0) {
	      if (distance <= r) {
		XTft_SetPixel(InstancePtr, i, j, col);
	      }
	    }
	    else {
	      if (distance <= r && distance >= r-thickness) {
		XTft_SetPixel(InstancePtr, i, j, col);
	      }
	    }
	  }
	}
      }
    }
    return 0;
  }
  return 1;

}

/* XTft_DrawBox: Draws a box on the TFT of a specified
 * color between two points                            */
int XTft_DrawBox(XTft *InstancePtr,
		 int x1, int y1, int x2, int y2, unsigned int col)
{
  int xmin, xmax, ymin, ymax, i, j;
  
  if (x1 >= 0 &&
      x1 <= DISPLAY_COLUMNS-1 &&
      x2 >= 0 &&
      x2 <= DISPLAY_COLUMNS-1 &&
      y1 >= 0 &&
      y1 <= DISPLAY_ROWS-1 &&
      y2 >= 0 &&
      y2 <= DISPLAY_ROWS-1) {
    if (x2 < x1) {
      xmin = x2;
      xmax = x1;
    }
    else {
      xmin = x1;
      xmax = x2;
    }
    if (y2 < y1) {
      ymin = y2;
      ymax = y1;
    }
    else {
      ymin = y1;
      ymax = y2;
    }

    for (i=xmin; i<=xmax; i++) {
      for (j=ymin; j<=ymax; j++) {
	XTft_SetPixel(InstancePtr, i, j, col);
      }
    }
    return 0;
  }
  return 1;

}

/* XTft_DrawLine: Draws a line between two points with
 * with a specified color                              */
int XTft_DrawLine(XTft *InstancePtr,
		  int x1, int y1, int x2, int y2, unsigned int col)
{
  
  int slope, yintercept;
  int xmin, ymin, xmax, ymax, i, j, mx;
  
  if (x1 >= 0 &&
      x1 <= DISPLAY_COLUMNS-1 &&
      x2 >= 0 &&
      x2 <= DISPLAY_COLUMNS-1 &&
      y1 >= 0 &&
      y1 <= DISPLAY_ROWS-1 &&
      y2 >= 0 &&
      y2 <= DISPLAY_ROWS-1) {
    
    if (x2-x1 != 0) {
      slope = ((y2-y1)/(x2-x1)*100000);     /* Calculate slope */
      yintercept = y1-((slope/100000)*x1);  /* Calculate yintercept */
    }
    else { /* divide by zero */
      slope = 0;
      yintercept = (-1);
    }
    
    if (x2 < x1) {
      xmin = x2;
      xmax = x1;
    }
    else {
      xmin = x1;
      xmax = x2;
    }
    if (y2 < y1) {
      ymin = y2;
      ymax = y1;
    }
    else {
      ymin = y1;
      ymax = y2;
    }

    /*
      printf("xmin, ymin, xmax, ymax : ");
      PrintInt(xmin);
      printf(", ");
      PrintInt(ymin);
      printf(", ");
      PrintInt(xmax);
      printf(", ");
      PrintInt(ymax);
      printf("\n\r");
      printf("slope, y-intercept : ");
      PrintInt(slope);
      printf(", ");
      PrintInt(yintercept);
      printf(" ");
    */

    for (i=xmin; i<=xmax; i++) {
      mx = (slope * i) / 100000;
      for (j=ymin; j<=ymax; j++) {
	if ((j - mx) == yintercept) { /* Calculate visible line */
	  XTft_SetPixel(InstancePtr, i, j, col);
	}
	else {
	  if ((slope == 0) && (yintercept == -1)) { /* divide by zero */
	    XTft_SetPixel(InstancePtr, i, j, col);  /* a.k.a vertical line */
	  }
	}
      }
    }
    return 0;
  }
  else {
    return 1;
  }
}

/* Draws a grid, initializes touchscreen, draws touchscreen
 * cursor on the grid.                                      */
int main(void) {

  XStatus Status;
  XTft Tft;
  XTouchscreen Touchscreen;

  Xuint32 x, y, i, z;
  Xuint32 ctrlReg;
  
  int Xmult = 0;
  int Ymult = 0;
  int Xrcorn = 0;
  int Yrcorn = 0;
  int Xlcorn = 0;
  int Ylcorn = 0;
  int Ydif = 0;
  int Xdif = 0;
  
  int tl_Xpixelsample[NUM_SAMPLES];
  int tl_Ypixelsample[NUM_SAMPLES];
  int br_Xpixelsample[NUM_SAMPLES];
  int br_Ypixelsample[NUM_SAMPLES];

  char msg[100]; // output message

  /* Initialize caches */
  XCache_EnableICache(0x80000001);
  XCache_EnableDCache(0x80000001);

  /* Initialize UART */
  XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
  XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);
  
  /* Initialize TFT */
  Status = XTft_Initialize(&Tft, XPAR_TFT_0_DEVICE_ID);
  STATUS_CHECK(Status);
  
  /* Initialize touchscreen */
  Status = XTouchscreen_Initialize(&Touchscreen, TOUCHSCREEN_DEVICE_ID);
  STATUS_CHECK(Status);

  /* Set foreground=White and background=Black */
  XTft_SetColor(&Tft, 0xffffff, 0x000000);
  XTft_ClearScreen(&Tft);
  /* Draw Grid */
  for (i=0; i<640; i++) {
    if (i%16 == 0) {
      XTft_DrawLine(&Tft, i, 0, i, 479, 0x00ff00);
    }
  }
  for (i=0;i<480;i++) {
    if (i%16 == 0) {
      XTft_DrawLine(&Tft, 0, i, 639, i, 0x00ff00);
    }
  }

  sleep(2);
  /* Start calibration */
  printf("Calibrating upper left corner\n\r");
  XTft_WriteCenteredString(&Tft, "Calibrating upper left corner", 100);
  /* Draw Crosshairs */
  XTft_DrawCircle(&Tft, 16, 16, 16, 2, 0xffffff);
  XTft_DrawCircle(&Tft, 16, 16, 8, 2, 0xffffff);
  XTft_DrawCircle(&Tft, 16, 16, 1, 1, 0xffffff);
  /* gather samples */
  printf("Touch and hold the target until it disappears...\n");
  XTft_WriteCenteredString(&Tft, "Touch and hold the target until it disappears...", 115);
  for (i=0; i<NUM_SAMPLES; i++) {
    do {
      ctrlReg = XTouchscreen_mReadCtrlReg(Touchscreen.BaseAddress);
    } while (!(ctrlReg & 0x1));
    XTouchscreen_GetPosition_3D(&Touchscreen, &x, &y, &z);
    /* normalize values */
    x = NORM_FACTOR - ((x * NORM_FACTOR) / 0x1000);
    y = (y * NORM_FACTOR) / 0x1000;
    printf("x, y : %5lu, %5lu\n\r", x, y);
    sprintf(msg, "x, y: %5lu, %5lu", x, y);
    XTft_WriteCenteredString(&Tft, msg, 130);
    tl_Xpixelsample[i] = x;
    tl_Ypixelsample[i] = y;
    usleep(100000);
  }
  
  XTft_DrawCircle(&Tft, 16, 16, 16, 2, 0x0); /* Erase Crosshairs */
  XTft_DrawCircle(&Tft, 16, 16, 8, 2, 0x0);
  XTft_DrawCircle(&Tft, 16, 16, 1, 1, 0x0);
  
  printf("Calibrating lower right corner\n\r");
  XTft_WriteCenteredString(&Tft, "Calibrating lower right corner", 160);
  sleep(2);
  XTft_DrawCircle(&Tft, 624, 464, 16, 2, 0xffffff); /* Draw Crosshairs */
  XTft_DrawCircle(&Tft, 624, 464, 8, 2, 0xffffff);
  XTft_DrawCircle(&Tft, 624, 464, 1, 1, 0xffffff);
  printf("Touch and hold the target until it disappears...");
  XTft_WriteCenteredString(&Tft, "Touch and hold the target until it disappears...", 175);
  for (i=0; i<NUM_SAMPLES; i++) {
    do {
      ctrlReg = XTouchscreen_mReadCtrlReg(Touchscreen.BaseAddress);
    } while ((ctrlReg & 0x1) == 0);
    XTouchscreen_GetPosition_3D(&Touchscreen, &x, &y, &z);
    /* normalize values */
    x = NORM_FACTOR - ((x * NORM_FACTOR) / 0x1000);
    y = (y * NORM_FACTOR) / 0x1000;
    printf("x, y : %5lu, %5lu\n\r", x, y);
    sprintf(msg, "x, y: %5lu, %5lu", x, y);
    XTft_WriteCenteredString(&Tft, msg, 190);
    br_Xpixelsample[i] = x;
    br_Ypixelsample[i] = y;
    usleep(100000);
  }

  XTft_DrawCircle(&Tft, 624, 464, 16, 2, 0x0); /* Erase Crosshairs */
  XTft_DrawCircle(&Tft, 624, 464, 8, 2, 0x0);
  XTft_DrawCircle(&Tft, 624, 464, 1, 1, 0x0);

  /* Calculate average values */
  for (i=1; i<NUM_SAMPLES; i++) {
    Yrcorn += br_Ypixelsample[i];
    Xrcorn += br_Xpixelsample[i];
    Ylcorn += tl_Ypixelsample[i];
    Xlcorn += tl_Xpixelsample[i];
  }
  Yrcorn /= NUM_SAMPLES-1;
  Xrcorn /= NUM_SAMPLES-1;
  Ylcorn /= NUM_SAMPLES-1;
  Xlcorn /= NUM_SAMPLES-1;

  printf("Xlcorn, Ylcorn : %5u, %5u\n\r", Xlcorn, Ylcorn);
  printf("Xrcorn, Yrcorn : %5u, %5u\n\r", Xrcorn, Yrcorn);
  XTft_ClearScreen(&Tft);
  XTft_WriteCenteredString(&Tft, "Your display constants are:", 100);
  sprintf(msg, "Xlcorn, Ylcorn: %5u, %5u", Xlcorn, Ylcorn);
  XTft_WriteCenteredString(&Tft, msg, 115);
  sprintf(msg, "Xrcorn, Yrcorn: %5u, %5u", Xrcorn, Yrcorn);
  XTft_WriteCenteredString(&Tft, msg, 130);
  Xdif = Xrcorn - Xlcorn;
  Ydif = Yrcorn - Ylcorn;
  
  Xmult = Xdif/608;  /*  /640 */
  Ymult = Ydif/448;  /*  /480 */
  
  printf("Xdif, Ydif : %5u, %5u\n\r", Xdif, Ydif);
  printf("Xmult, Ymult : %5u, %5u\n\r", Xmult, Ymult);
  sprintf(msg, "Xdif, Ydif: %5u, %5u", Xdif, Ydif);
  XTft_WriteCenteredString(&Tft, msg, 145);
  sprintf(msg, "Xmult, Ymult: %5u, %5u", Xmult, Ymult);
  XTft_WriteCenteredString(&Tft, msg, 160);

  return 0;
}
