/******************************************************************************
*     $Date: 2004/01/01 01:30:04 $
*     $RCSfile: touchscreen_polled.c,v $
*******************************************************************************

*******************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2004 Xilinx, Inc.
*     All rights reserved.
*
******************************************************************************/
/***************************** Include Files ********************************/

#include <stdio.h>
#include "sleep.h"
#include "xbasic_types.h"
#include "xparameters.h"
#include "xtouchscreen.h"
#include "xuartns550_l.h"

/************************** Constant Definitions ****************************/
#define UART_BASEADDR              XPAR_OPB_UART16550_0_BASEADDR
#define UART_CLOCK                 XPAR_XUARTNS550_CLOCK_HZ
#define UART_BAUDRATE              9600

#define TOUCHSCREEN_DEVICE_ID      XPAR_OPB_TSD_REF_0_DEVICE_ID

/**************************** Type Definitions ******************************/

#define STATUS_CHECK(Status)         \
    {                                \
        if (Status != XST_SUCCESS)   \
        {                            \
            return Status;           \
        }                            \
    }                                

/***************** Macros (Inline Functions) Definitions ********************/

/************************** Variable Definitions ****************************/

static XTouchscreen Touchscreen;

/****************************************************************************/

void PrintInfo(void)
{
    Xuint32 ctrlReg;
    Xuint32 intrReg;
    Xuint32 x;
    Xuint32 y;
    Xuint32 z;

    ctrlReg = XTouchscreen_mReadCtrlReg(Touchscreen.BaseAddress);
    intrReg = XTouchscreen_mGetIntrStatus(Touchscreen.BaseAddress);
    /* if touchscreen is touched, print contents of Control and 
       Interrupt registers, along with the coordinates where it's
       touched */
    if (ctrlReg & 0x1) {
      XTouchscreen_GetPosition_3D(&Touchscreen, &x, &y, &z);
      printf("CtrlReg=0x%x, IntrReg=0x%x\n", (unsigned int)ctrlReg, (unsigned int)intrReg);
      printf("X=%5lu   Y=%5lu   Z=%5lu\n", x, y, z);
    }
}

/****************************************************************************/

int main()
{
    XStatus Status;

    /* Initialize the UART driver, so we can use printf */
    XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
    XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);
    printf("\n\nTouchscreen Demo using polled mode\n\r");
    printf("When the screen is touched, the UART terminal displays the:\n\r");
    printf("(1) display (x,y) and pressure (z) coordinates; and\n\r");
    printf("(2) contents of the Control & Interrupt registers\n\r");

    /* Initialize the Touchscreen driver so that it's ready to use,
     * specify the device ID that is generated in xparameters.h */
    Status = XTouchscreen_Initialize(&Touchscreen, TOUCHSCREEN_DEVICE_ID);
    STATUS_CHECK(Status);
    
    /* Continuously poll the touchscreen and print:
     * 1) Coordinates and 2) Control & Interrupt registers */
    printf("Start polling:\n\r");
    while(1) {
      PrintInfo();
      usleep(100000);
    }
    return Status;
}


