/*-----------------------------------------------------------------------------
//     $Date: 2004/03/19 19:52:36 $
//     $RCSfile: v2pdraw.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/

#include <stdio.h>
#include "xuartns550_l.h"

#include "xbasic_types.h"
#include "xexception_l.h"
#include "xintc.h"
#include "xparameters.h"
#include "xtouchscreen.h"
#include "sleep.h"
#include "xcache_l.h"
#include "xtft_basicgraphics.h"
#include "xtouchscreen_utils.h"

#define STATUS_CHECK(Status)       {if(Status != XST_SUCCESS) {return Status;}}
/* Uncomment the following to use interrupt-driven mode */
//#define V2PDRAW_INT

#define UART_BASEADDR              XPAR_OPB_UART16550_0_BASEADDR
#define UART_CLOCK                 XPAR_XUARTNS550_CLOCK_HZ
#if !SIM
#define UART_BAUDRATE              9600 /* real hardware */
#else
#define UART_BAUDRATE              (UART_CLOCK / 16 / 3) /* simulation */
#endif

#define TFT_DEVICE_ID              XPAR_PLB_TFT_CNTLR_REF_0_DEVICE_ID
#define TOUCHSCREEN_DEVICE_ID      XPAR_OPB_TSD_REF_0_DEVICE_ID
#define TOUCHSCREEN_INTERRUPT_ID   XPAR_DCR_INTC_0_OPB_TSD_REF_0_INTR_INTR
#define INTC_DEVICE_ID             XPAR_DCR_INTC_0_DEVICE_ID

#define UP                XFALSE
#define DOWN              XTRUE

/* ML2/ML300 Defaults */
#define DEFAULT_XMULT    869
#define DEFAULT_YMULT    1192
#define DEFAULT_XLCORN   227916
#define DEFAULT_YLCORN   185388

#define DISPLAY_COLUMNS   640
#define BUFFER_COLUMNS    1024
#define TFT_COLOR_PLANES  TFT_COLOR_PLANES_COMBINED
#define NORM_FACTOR       1000000

#define PRESS_THRESHOLD   5000
#define NUM_SAMPLES       100

/************************** Variable Definitions ****************************/

static XTft Tft;
static XTouchscreen Touchscreen;
static Xboolean PenDown;
static int Xmult, Ymult, Xlcorn, Ylcorn;
static Xuint32 Xpixel, Ypixel;
static int BrushType;
static unsigned int BrushColor;

/************************** Function Prototypes *****************************/
void CanvasDraw(Xuint32 Xpixel, Xuint32 Ypixel);
void DrawMenu();
#ifdef V2PDRAW_INT
static XStatus SetupInterruptSystem(XIntc *Intc, XTouchscreen *TouchscreenPtr);
#endif
void Handler(void *CallBackRef, Xuint32 Event, unsigned int EventData);
void SetBrush(int brush);

int touchscreen_GetCoord(void);
void touchscreen_Calibrate(void);
void CanvasClear(unsigned int col);

/****************************************************************************/

/**************************** Type Definitions ******************************/

/*************************************************************/
/* CanvasClear:  Clears the canvas                           */
/*************************************************************/
void CanvasClear(unsigned int col)
{

#if TFT_COLOR_PLANES == TFT_COLOR_PLANES_COMBINED

  unsigned int adr;
  unsigned int x;
  unsigned int fromY,toY;

  fromY = 0;
  toY = 446;

  while (fromY <= toY) {
    adr = (126*1024*1024) + 4 * fromY * BUFFER_COLUMNS;
    for (x = 0; x < DISPLAY_COLUMNS; x++) {
      //m_ww(adr, col);
      *(unsigned int *)adr = col;
      adr += 4;
    }
    fromY++;
  }

#elif TFT_COLOR_PLANES == TFT_COLOR_PLANES_SPLIT

  unsigned int adr, x, i;

  col |= (col << 8);
  col |= (col << 16);

  i = 0;

  while (i <= 446) {
    adr = 0x30000000 + i * BUFFER_COLUMNS;
    for (x = 0; x < DISPLAY_COLUMNS; x += 4) {
      //m_ww(adr, col);
      *(unsigned int *)adr = col;
      adr += 4;
    }
    i++;
  }

  i = 0;

  while (i <= 446) {
    adr = (0x30000000 + 512*1024) + i * BUFFER_COLUMNS;
    for (x = 0; x < DISPLAY_COLUMNS; x += 4) {
      //m_ww(adr, col);
      *(unsigned int *)adr = col;
      adr += 4;
    }
    i++;
  }

  i = 0;

  while (i <= 446) {
    adr = (0x30000000 + 1024*1024) + i * BUFFER_COLUMNS;
    for (x = 0; x < DISPLAY_COLUMNS; x += 4) {
      //m_ww(adr, col);
      *(unsigned int *)adr = col;
      adr += 4;
    }
    i++;
  }

  Xpixel = 0;
  Ypixel = 0;

#endif

}


/*************************************************************/
/* CanvasDraw:  Draws Brush on canvas                        */
/*************************************************************/
void CanvasDraw(Xuint32 Xpixel, Xuint32 Ypixel)
{
   switch(BrushType) {
   case 1:
     XTft_DrawSolidBox(&Tft, Xpixel-1,Ypixel-1,Xpixel+1,Ypixel+1, BrushColor);
     break;
   case 2:
     XTft_DrawSolidBox(&Tft, Xpixel-2,Ypixel-2,Xpixel+2,Ypixel+2, BrushColor);
     break;
   case 3:
     XTft_DrawSolidBox(&Tft, Xpixel-3,Ypixel-3,Xpixel+3,Ypixel+3, BrushColor);
     break;
   case 4:
     XTft_DrawCircle(&Tft, Xpixel,Ypixel,3,0,BrushColor);
     break;
   case 5:
     XTft_DrawCircle(&Tft, Xpixel,Ypixel,4,0,BrushColor);
     break;
   case 6:
     XTft_DrawCircle(&Tft, Xpixel,Ypixel,5,0,BrushColor);
     break;
   default:
     XTft_DrawPixel(&Tft, Xpixel,Ypixel, BrushColor);
   }
}


/*************************************************************/
/* DrawMenu:  Draws and refreshes the menu                   */
/*************************************************************/
void DrawMenu()
{

  XTft_DrawSolidBox(&Tft, 0,447,639,479,0xC6C3C6);
  
  XTft_DrawEmptyBox(&Tft, 0,447,57,479,0x000000,1);
  XTft_DrawLine(&Tft, 0,448,0,479,0xffffff);
  XTft_DrawLine(&Tft, 0,448,56,448,0xffffff);
  if (BrushType == 0)
    XTft_DrawSolidBox(&Tft, 0,448,56,479,0x848284);
  XTft_DrawSolidBox(&Tft, 27,463,30,466,0x000000);
  
  XTft_DrawEmptyBox(&Tft, 58,447,115,479,0x000000,1);
  XTft_DrawLine(&Tft, 59,448,59,479,0xffffff);
  XTft_DrawLine(&Tft, 59,448,114,448,0xffffff);
  if (BrushType == 2)
    XTft_DrawSolidBox(&Tft, 59,448,115,479,0x848284);
  XTft_DrawSolidBox(&Tft, 83,461,88,466,0x000000);
  
  XTft_DrawEmptyBox(&Tft, 116,447,173,479,0x000000,1);
  XTft_DrawLine(&Tft, 117,448,117,479,0xffffff);
  XTft_DrawLine(&Tft, 117,448,172,448,0xffffff);
  if (BrushType == 3)
    XTft_DrawSolidBox(&Tft, 117,448,172,479,0x848284);
  XTft_DrawSolidBox(&Tft, 140,460,147,467,0x000000);
  
  XTft_DrawEmptyBox(&Tft, 174,447,231,479,0x000000,1);
  XTft_DrawLine(&Tft, 175,448,175,479,0xffffff);
  XTft_DrawLine(&Tft, 175,448,230,448,0xffffff);
  if (BrushType == 4)
    XTft_DrawSolidBox(&Tft, 175,448,230,479,0x848284);
  XTft_DrawCircle(&Tft, 202,465,3,0,0x000000);
  
  XTft_DrawEmptyBox(&Tft, 232,447,289,479,0x000000,1);
  XTft_DrawLine(&Tft, 233,448,233,479,0xffffff);
  XTft_DrawLine(&Tft, 233,448,288,448,0xffffff);
  if (BrushType == 5)
    XTft_DrawSolidBox(&Tft, 233,448,288,479,0x848284);
  XTft_DrawCircle(&Tft, 260,465,4,0,0x000000);
  
  XTft_DrawEmptyBox(&Tft, 290,447,347,479,0x000000,1);
  XTft_DrawLine(&Tft, 291,448,291,479,0xffffff);
  XTft_DrawLine(&Tft, 291,448,346,448,0xffffff);
  if (BrushType == 6)
    XTft_DrawSolidBox(&Tft, 291,448,346,479,0x848284);
  XTft_DrawCircle(&Tft, 320,465,5,0,0x000000);
  
  XTft_DrawEmptyBox(&Tft, 348,447,405,479,0x000000,1);
  XTft_DrawLine(&Tft, 349,448,349,479,0xffffff);
  XTft_DrawLine(&Tft, 349,448,404,448,0xffffff);
  if (BrushColor == 0xff0000)
    XTft_DrawSolidBox(&Tft, 349,448,404,479,0x848284);
  XTft_DrawSolidBox(&Tft, 352,451,401,475,0xff0000);
  
  XTft_DrawEmptyBox(&Tft, 406,447,463,479,0x000000,1);
  XTft_DrawLine(&Tft, 407,448,407,479,0xffffff);
  XTft_DrawLine(&Tft, 407,448,462,448,0xffffff);
  if (BrushColor == 0x00ff00)
    XTft_DrawSolidBox(&Tft, 407,448,462,479,0x848284);
  XTft_DrawSolidBox(&Tft, 410,451,459,475,0x00ff00);
  
  XTft_DrawEmptyBox(&Tft, 464,447,521,479,0x000000,1);
  XTft_DrawLine(&Tft, 465,448,465,479,0xffffff);
  XTft_DrawLine(&Tft, 465,448,520,448,0xffffff);
  if (BrushColor == 0x0000ff)
    XTft_DrawSolidBox(&Tft, 465,448,520,479,0x848284);
  XTft_DrawSolidBox(&Tft, 468,451,517,475,0x0000ff);
  
  XTft_DrawEmptyBox(&Tft, 522,447,579,479,0x000000,1);
  XTft_DrawLine(&Tft, 523,448,523,479,0xffffff);
  XTft_DrawLine(&Tft, 523,448,578,448,0xffffff);
  XTft_SetColor(&Tft, 0x000000,0xC6C3C6);
  XTft_SetPos(&Tft, 538, 460);
  XTft_WriteString(&Tft, "CLR");
  
  XTft_DrawEmptyBox(&Tft, 580,447,637,479,0x000000,1);
  XTft_DrawLine(&Tft, 581,448,581,479,0xffffff);
  XTft_DrawLine(&Tft, 581,448,636,448,0xffffff);
  XTft_SetPos(&Tft, 598, 460);
  XTft_WriteString(&Tft, "CAL");
  
  Xpixel = 0;
  Ypixel = 0;
}

/*************************************************************/
/* SetBrush:  Sets the BrushType and refreshes the menu      */
/*************************************************************/
void SetBrush(int brush)
{
   switch (brush) {
   case 1:
     BrushType = 1;
     break;
   case 2:
     BrushType = 2;
     break;
   case 3:
     BrushType = 3;
     break;
   case 4:
     BrushType = 4;
     break;
   case 5:
     BrushType = 5;
     break;
   case 6:
     BrushType = 6;
     break;
   default:
     BrushType = 0;
   }
   
   DrawMenu();
}


/* touchscreen_Calibrate: Calculates constants to map the         
 * touchscreen coordinates to a 640x480 grid                      */
void touchscreen_Calibrate(void) {

  Xuint32 x, y, i, z;
  Xuint32 touchscreen_CtrlReg;
  int Xrcorn = 0;
  int Yrcorn = 0;
  int Ydif = 0;
  int Xdif = 0;
  int tl_Xpixelsample[NUM_SAMPLES];
  int tl_Ypixelsample[NUM_SAMPLES];
  int br_Xpixelsample[NUM_SAMPLES];
  int br_Ypixelsample[NUM_SAMPLES];
  char msg[100];
  Xuint32 ctrlReg;

  /* Initialize return values */
  Xmult = 0;
  Ymult = 0;
  Xlcorn = 0;
  Ylcorn = 0;

  /* Set foreground=White and background=Black */
  XTft_SetColor(&Tft, 0xffffff, 0x000000);
  XTft_ClearScreen(&Tft);
  /* Draw Grid */
  for (i=0; i<640; i++) {
    if (i%16 == 0) {
      XTft_DrawLine(&Tft, i, 0, i, 479, 0x00ff00);
    }
  }
  for (i=0;i<480;i++) {
    if (i%16 == 0) {
      XTft_DrawLine(&Tft, 0, i, 639, i, 0x00ff00);
    }
  }

  sleep(2);
  /* Start calibration */
  printf("Calibrating upper left corner\n\r");
  XTft_WriteCenteredString(&Tft, "Calibrating upper left corner", 100);
  /* Draw Crosshairs */
  XTft_DrawCircle(&Tft, 16, 16, 16, 2, 0xffffff);
  XTft_DrawCircle(&Tft, 16, 16, 8, 2, 0xffffff);
  XTft_DrawCircle(&Tft, 16, 16, 1, 1, 0xffffff);
  /* gather NUM_SAMPLES samples */
  printf("Touch and hold the target until it disappears...\n");
  XTft_WriteCenteredString(&Tft, "Touch and hold the target until it disappears...", 115);
  for (i=0; i<NUM_SAMPLES; i++) {
    do {
      touchscreen_CtrlReg = XTouchscreen_mReadCtrlReg(Touchscreen.BaseAddress);
    } while ((touchscreen_CtrlReg & 1) == 0);
    XTouchscreen_GetPosition_3D(&Touchscreen, &x, &y, &z);
    /* normalize values */
    x = NORM_FACTOR - ((x * NORM_FACTOR) / 0x1000);
    y = (y * NORM_FACTOR) / 0x1000;
    printf("x, y : %5lu, %5lu\n\r", x, y);
    sprintf(msg, "x, y : %5lu, %5lu\n\r", x, y);
    XTft_WriteCenteredString(&Tft, msg, 130);
    tl_Xpixelsample[i] = x;
    tl_Ypixelsample[i] = y;
    usleep(100000);
  }
  
  XTft_DrawCircle(&Tft, 16, 16, 16, 2, 0x0); /* Erase Crosshairs */
  XTft_DrawCircle(&Tft, 16, 16, 8, 2, 0x0);
  XTft_DrawCircle(&Tft, 16, 16, 1, 1, 0x0);
  
  printf("Calibrating lower right corner\n\r");
  XTft_WriteCenteredString(&Tft, "Calibrating lower right corner", 160);
  sleep(2);
  XTft_DrawCircle(&Tft, 624, 464, 16, 2, 0xffffff); /* Draw Crosshairs */
  XTft_DrawCircle(&Tft, 624, 464, 8, 2, 0xffffff);
  XTft_DrawCircle(&Tft, 624, 464, 1, 1, 0xffffff);
  printf("Touch and hold the target until it disappears...");
  XTft_WriteCenteredString(&Tft, "Touch and hold the target until it disappears...", 175);
  for (i=0; i<NUM_SAMPLES; i++) {
    do {
      touchscreen_CtrlReg = XTouchscreen_mReadCtrlReg(Touchscreen.BaseAddress);
    } while ((touchscreen_CtrlReg & 1) == 0);
    XTouchscreen_GetPosition_3D(&Touchscreen, &x, &y, &z);
    /* normalize values */
    x = NORM_FACTOR - ((x * NORM_FACTOR) / 0x1000);
    y = (y * NORM_FACTOR) / 0x1000;
    printf("x, y : %5lu, %5lu\n\r", x, y);
    sprintf(msg, "x, y: %5lu, %5lu", x, y);
    XTft_WriteCenteredString(&Tft, msg, 190);
    br_Xpixelsample[i] = x;
    br_Ypixelsample[i] = y;
    usleep(100000);
  }

  XTft_DrawCircle(&Tft, 624, 464, 16, 2, 0x0); /* Erase Crosshairs */
  XTft_DrawCircle(&Tft, 624, 464, 8, 2, 0x0);
  XTft_DrawCircle(&Tft, 624, 464, 1, 1, 0x0);

  /* Calculate average values */
  for (i=1; i<NUM_SAMPLES; i++) {
    Yrcorn += br_Ypixelsample[i];
    Xrcorn += br_Xpixelsample[i];
    Ylcorn += tl_Ypixelsample[i];
    Xlcorn += tl_Xpixelsample[i];
  }
  Yrcorn /= NUM_SAMPLES-1;
  Xrcorn /= NUM_SAMPLES-1;
  Ylcorn /= NUM_SAMPLES-1;
  Xlcorn /= NUM_SAMPLES-1;

  printf("Xlcorn, Ylcorn : %5u, %5u\n\r", Xlcorn, Ylcorn);
  printf("Xrcorn, Yrcorn : %5u, %5u\n\r", Xrcorn, Yrcorn);
  XTft_ClearScreen(&Tft);
  XTft_WriteCenteredString(&Tft, "Your display constants are:", 100);
  sprintf(msg, "Xlcorn, Ylcorn : %5u, %5u", Xlcorn, Ylcorn);
  XTft_WriteCenteredString(&Tft, msg, 115);
  sprintf(msg, "Xrcorn, Yrcorn : %5u, %5u", Xrcorn, Yrcorn);
  XTft_WriteCenteredString(&Tft, msg, 130);

  Xdif = Xrcorn - Xlcorn;
  Ydif = Yrcorn - Ylcorn;
  
  Xmult = Xdif/608;  /*  /640 */
  Ymult = Ydif/448;  /*  /480 */
  
  printf("Xdif, Ydif : %5u, %5u\n\r", Xdif, Ydif);
  printf("Xmult, Ymult : %5u, %5u\n\r", Xmult, Ymult);
  sprintf(msg, "Xdif, Ydif : %5u, %5u\n\r", Xdif, Ydif);
  XTft_WriteCenteredString(&Tft, msg, 145);
  sprintf(msg, "Xmult, Ymult : %5u, %5u\n\r", Xmult, Ymult);
  XTft_WriteCenteredString(&Tft, msg, 160);
  sleep(3);

  do {
    ctrlReg = XTouchscreen_mReadCtrlReg(Touchscreen.BaseAddress);
  }
  while (!(ctrlReg & 0x1));

  return;
}

/* touchscreen_GetCoord: assign touchscreen coordinates */
int touchscreen_GetCoord(void)
{
  /*  Get the Touchscreen Coordinates */
  Xuint32 x, y, z;

  /* if touchscreen is touched, get coordinates */
  XTouchscreen_GetPosition_3D(&Touchscreen, &x, &y, &z);
  if (z < PRESS_THRESHOLD) {
    // normalize values
    x = NORM_FACTOR - ((x * NORM_FACTOR) / 0x1000);
    y = (y * NORM_FACTOR) / 0x1000;
    Xpixel = (x - Xlcorn+(16*Xmult)) / Xmult;
    Ypixel = (y - Ylcorn+(16*Ymult)) / Ymult;
    //printf("Xpixel, Ypixel: %5u, %5u\n\r", Xpixel, Ypixel);
    if (Xpixel <= 0 )
      Xpixel = 1;
    else if (Xpixel > 639)
      Xpixel = 639;
    
    if (Ypixel <= 0 )
      Ypixel = 1;
  else if (Ypixel > 479)
    Ypixel = 479;

    return 1;
  }
  else {
    return 0;
  }
}


/* Interrupt handler */
void Handler(void *CallBackRef, Xuint32 Event, unsigned int EventData)
{ 
#if !defined(V2PDRAW_INT)
  Xuint32 ctrlReg;
#endif
  if (Event == XTOUCHSCREEN_EVENT_PEN_DOWN) {
    PenDown = XTRUE;
  } 
  if (Event == XTOUCHSCREEN_EVENT_PEN_UP) {
    PenDown = XFALSE;
  }

#ifdef V2PDRAW_INT  
  if (PenDown) {
#else
  ctrlReg = XTouchscreen_mReadCtrlReg(Touchscreen.BaseAddress);
  if (ctrlReg & 0x1) {
#endif
    if (touchscreen_GetCoord()) {
      if (Ypixel >= 447) {
	if(Xpixel <= 57)
	  SetBrush(0);
	else if(Xpixel <= 115)
	  SetBrush(2);
	else if(Xpixel <= 173)
	  SetBrush(3);
	else if(Xpixel <= 231)
	  SetBrush(4);
	else if(Xpixel <= 289)
	  SetBrush(5);
	else if(Xpixel <= 347)
	  SetBrush(6);
	else if(Xpixel <= 405) {
	  BrushColor = 0xff0000;
	  DrawMenu();
	}
	else if(Xpixel <= 463) {
	  BrushColor = 0x00ff00;
	  DrawMenu();
	}
	else if(Xpixel <= 521) {
	  BrushColor = 0x0000ff;
	  DrawMenu();
	}
	else if(Xpixel <= 579)
	  CanvasClear(0xffffff);
	else if(Xpixel <= 637) {
	  touchscreen_Calibrate();
	  CanvasClear(0xffffff);
	  DrawMenu();
	}
      }
      else CanvasDraw(Xpixel,Ypixel);
    }
  }
}

#ifdef V2PDRAW_INT
static XStatus SetupInterruptSystem(XIntc *Intc, XTouchscreen *TouchscreenPtr)
{
    XStatus Status;

    /* Initialize the exception handler */
    XExc_Init();

    /* Initialize the interrupt controller driver so that
     * it's ready to use, specify the device ID that is generated in
     * xparameters.h */
    Status = XIntc_Initialize(Intc, INTC_DEVICE_ID);
    STATUS_CHECK(Status);

    /* Connect the Intc handler to the exception handler */
    XExc_RegisterHandler(XEXC_ID_NON_CRITICAL_INT,
                         (XExceptionHandler)XIntc_InterruptHandler,
                         (void *)Intc);

    /* Connect a device driver handler that will be called when an interrupt
     * for the device occurs, the device driver handler performs the specifc
     * interrupt processing for the device */
    Status = XIntc_Connect(Intc, TOUCHSCREEN_INTERRUPT_ID,
                           (XInterruptHandler)XTouchscreen_InterruptHandler,
                           (void *)TouchscreenPtr);
    STATUS_CHECK(Status);

    /* Start the interrupt controller such that interrupts are enabled for
     * all devices that cause interrupts, specific real mode so that
     * TSD can cause interrupt thru the interrupts controller. */
    Status = XIntc_Start(Intc, XIN_REAL_MODE);
    STATUS_CHECK(Status);

    /* Enable the interrupts for Touchscreen */
    XIntc_Enable(Intc, TOUCHSCREEN_INTERRUPT_ID);

    /* Enable non-critical exceptions */
    XExc_mEnableExceptions(XEXC_ALL);

    return Status;
}
#endif

/************************************************************/
/* V2P Draw                                                 */
/*                                                          */
/* A simple paint program designed for V2P which            */
/* demonstrates Touchscreen and TFT functionality.          */
/*                                                          */
/* 9/9/02:  BF                                             */
/************************************************************/

int main(void)
{
#ifdef V2PDRAW_INT
  static XIntc Intc;
#endif
  XStatus Status;

  /* Initialize the UART driver, so we can use printf */
  XUartNs550_SetBaud(UART_BASEADDR, UART_CLOCK, UART_BAUDRATE);
  XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);

  /* Initialize caches */
  XCache_EnableICache(0x80000001);
  XCache_EnableDCache(0x80000001);
  
  /* Initialize Globals */
  Xpixel=0;
  Ypixel=0;
  Xmult=DEFAULT_XMULT;
  Ymult=DEFAULT_YMULT;
  Xlcorn=DEFAULT_XLCORN;
  Ylcorn=DEFAULT_YLCORN;
  printf("xmult, ymult, xlcorn, ylcorn: %d, %d, %d, %d\n\r", Xmult, Ymult, Xlcorn, Ylcorn);
  
  PenDown = XFALSE;
  BrushType = 0;

  /* Initialize TFT */
  Status = XTft_Initialize(&Tft, TFT_DEVICE_ID);
  STATUS_CHECK(Status);

  /* Initialize touchscreen */
  Status = XTouchscreen_Initialize(&Touchscreen, TOUCHSCREEN_DEVICE_ID);
  STATUS_CHECK(Status);

  /* paint initial menu and canvas */
  DrawMenu();
  CanvasClear(0xffffff);
  
  /*
   * Setup the handlers that will be called from the interrupt
   * context when data has been sent and received, specify a pointer to the
   * driver instance as the callback reference so that handlers are able
   * to access the instance data
   */
#ifdef V2PDRAW_INT
  XTouchscreen_SetHandler(&Touchscreen, Handler, &Touchscreen);
  
  /* Make Intc aware of touchscreen interrupts */
  Status = SetupInterruptSystem(&Intc, &Touchscreen);
  STATUS_CHECK(Status);
#endif
  
  /* loop continously, handling interrupt activity on the touchscreen */
  while (1) {
#ifdef V2PDRAW_INT
    while(PenDown) {
#endif
      Handler((void *)XNULL, 0xFFFFFFFF, 0);
#ifdef V2PDRAW_INT
    }
#endif
  }
  return 0;
}
