/******************************************************************************
*     $Date: 2004/01/01 01:30:20 $
*     $RCSfile: xtouchscreen_utils.c,v $
*******************************************************************************

*******************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2004 Xilinx, Inc.
*     All rights reserved.
*
******************************************************************************/
#include "xtouchscreen.h"
#include "xtft.h"
#include "sleep.h"
#include "xparameters.h"
#include "math.h"

#define DISPLAY_COLUMNS  640
#define DISPLAY_ROWS     480

/* XTouchscreen_Calibrate: Calculates constants to map the         
 * touchscreen coordinates to a 640x480 grid                      */
void XTouchscreen_Calibrate(XTft*Tft, XTouchscreen *Touchscreen,
			    int *Xmult, int *Ymult, int *Xlcorn, int *Ylcorn) {

  Xuint32 x, y, i, z;
  Xuint32 touchscreen_CtrlReg;
  int Xrcorn = 0;
  int Yrcorn = 0;
  int Ydif = 0;
  int Xdif = 0;
  int tl_Xpixelsample[10];
  int tl_Ypixelsample[10];
  int br_Xpixelsample[10];
  int br_Ypixelsample[10];
  char msg[100];

  /* Initialize return values */
  *Xmult = 0;
  *Ymult = 0;
  *Xlcorn = 0;
  *Ylcorn = 0;

  /* Set foreground=White and background=Black */
  XTft_SetColor(Tft, 0xffffff, 0x000000);
  XTft_ClearScreen(Tft);
  /* Draw Grid */
  for (i=0; i<640; i++) {
    if (i%16 == 0) {
      XTft_DrawLine(Tft, i, 0, i, 479, 0x00ff00);
    }
  }
  for (i=0;i<480;i++) {
    if (i%16 == 0) {
      XTft_DrawLine(Tft, 0, i, 639, i, 0x00ff00);
    }
  }

  sleep(2);
  /* Start calibration */
  printf("Calibrating upper left corner\n\r");
  XTft_WriteCenteredString(Tft, "Calibrating upper left corner", 100);
  /* Draw Crosshairs */
  XTft_DrawCircle(Tft, 16, 16, 16, 2, 0xffffff);
  XTft_DrawCircle(Tft, 16, 16, 8, 2, 0xffffff);
  XTft_DrawCircle(Tft, 16, 16, 1, 1, 0xffffff);
  for (i=0; i<10; i++) {
    usleep(100000);
    printf("Tap upper left-hand corner: ");
    XTft_WriteCenteredString(Tft, "Tap upper left corner 10 times...", 115);
    do {
      touchscreen_CtrlReg = XTouchscreen_mReadCtrlReg(Touchscreen->BaseAddress);
    } while ((touchscreen_CtrlReg & 1) == 0);
    XTouchscreen_GetPosition_3D(Touchscreen, &x, &y, &z);
    printf("x, y : %5lu, %5lu\n\r", x, y);
    sprintf(msg, "x, y : %5lu, %5lu\n\r", x, y);
    XTft_WriteCenteredString(Tft, msg, 130);
    tl_Xpixelsample[i] = x;
    tl_Ypixelsample[i] = y;
  }
  
  XTft_DrawCircle(Tft, 16, 16, 16, 2, 0x0); /* Erase Crosshairs */
  XTft_DrawCircle(Tft, 16, 16, 8, 2, 0x0);
  XTft_DrawCircle(Tft, 16, 16, 1, 1, 0x0);
  
  printf("Calibrating lower right corner\n\r");
  XTft_WriteCenteredString(Tft, "Calibrating lower right corner", 160);
  sleep(2);
  XTft_DrawCircle(Tft, 624, 464, 16, 2, 0xffffff); /* Draw Crosshairs */
  XTft_DrawCircle(Tft, 624, 464, 8, 2, 0xffffff);
  XTft_DrawCircle(Tft, 624, 464, 1, 1, 0xffffff);
  for (i=0; i<10; i++) {
    usleep(100000);
    printf("Tap the lower right corner: ");
    XTft_WriteCenteredString(Tft, "Tap lower right corner 10 times...", 175);
    do {
      touchscreen_CtrlReg = XTouchscreen_mReadCtrlReg(Touchscreen->BaseAddress);
    } while ((touchscreen_CtrlReg & 1) == 0);
    XTouchscreen_GetPosition_3D(Touchscreen, &x, &y, &z);
    printf("x, y : %5lu, %5lu\n\r", x, y);
    sprintf(msg, "x, y: %5lu, %5lu", x, y);
    XTft_WriteCenteredString(Tft, msg, 190);
    br_Xpixelsample[i] = x;
    br_Ypixelsample[i] = y;
  }

  XTft_DrawCircle(Tft, 624, 464, 16, 2, 0x0); /* Erase Crosshairs */
  XTft_DrawCircle(Tft, 624, 464, 8, 2, 0x0);
  XTft_DrawCircle(Tft, 624, 464, 1, 1, 0x0);

  /* Calculate average values */
  for (i=1; i<10; i++) {
    Yrcorn += br_Ypixelsample[i];
    Xrcorn += br_Xpixelsample[i];
    *Ylcorn += tl_Ypixelsample[i];
    *Xlcorn += tl_Xpixelsample[i];
  }
  Yrcorn /= 9;
  Xrcorn /= 9;
  *Ylcorn /= 9;
  *Xlcorn /= 9;

  printf("Xlcorn, Ylcorn : %5u, %5u\n\r", *Xlcorn, *Ylcorn);
  printf("Xrcorn, Yrcorn : %5u, %5u\n\r", Xrcorn, Yrcorn);
  XTft_ClearScreen(Tft);
  XTft_WriteCenteredString(Tft, "Your display constants are:", 100);
  sprintf(msg, "Xlcorn, Ylcorn : %5u, %5u", *Xlcorn, *Ylcorn);
  XTft_WriteCenteredString(Tft, msg, 115);
  sprintf(msg, "Xrcorn, Yrcorn : %5u, %5u", Xrcorn, Yrcorn);
  XTft_WriteCenteredString(Tft, msg, 130);

  Xdif = Xrcorn - *Xlcorn;
  Ydif = Yrcorn - *Ylcorn;
  
  *Xmult = Xdif/608;  /*  /640 */
  *Ymult = Ydif/448;  /*  /480 */
  
  printf("Xdif, Ydif : %5u, %5u\n\r", Xdif, Ydif);
  printf("Xmult, Ymult : %5u, %5u\n\r", *Xmult, *Ymult);
  sprintf(msg, "Xdif, Ydif : %5u, %5u\n\r", Xdif, Ydif);
  XTft_WriteCenteredString(Tft, msg, 145);
  sprintf(msg, "Xmult, Ymult : %5u, %5u\n\r", *Xmult, *Ymult);
  XTft_WriteCenteredString(Tft, msg, 160);

  return;
}

/* XTouchscreen_GetCoord:  return the touchscreen coordinates */
void XTouchscreen_GetCoord(XTouchscreen *InstancePtr, Xuint32 *xval, Xuint32 *yval,
			   int Xmult, int Ymult, int Xlcorn, int Ylcorn)
{
  /*  Get the Touchscreen Coordinates */
  Xuint32 x, y, z;
  Xuint32 Xpixel, Ypixel;

  /* if touchscreen is touched, get coordinates */
  XTouchscreen_GetPosition_3D(InstancePtr, &x, &y, &z);
  
  /* signed computation */
  Xpixel = (x - Xlcorn+(16*Xmult)) / Xmult;
  Ypixel = (y - Ylcorn+(16*Ymult)) / Ymult;
  
  if (Xpixel <= 0 )
    Xpixel = 1;
  else if (Xpixel > 639)
    Xpixel = 639;
  
  if (Ypixel <= 0 )
    Ypixel = 1;
  else if (Ypixel > 479)
    Ypixel = 479;
  
  *xval = Xpixel;
  *yval = Ypixel;
  
  return;

}
