/******************************************************************************
*     $Date: 2004/03/10 23:33:40 $
*     $RCSfile: v2ptictac.c,v $
*******************************************************************************

*******************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2004 Xilinx, Inc.
*     All rights reserved.
*
******************************************************************************/
#include "xio.h"
#include "xintc.h"
#include "xtouchscreen.h"
#include "xtft.h"
#include "sleep.h"
#include "xcache_l.h"
#include "xparameters.h"
#include "xtft_basicgraphics.h"
#include "math.h"
#include <stdio.h>

// GPIO
#include "xgpio_l.h"

#define TFT_DEVICE_ID              XPAR_PLB_TFT_CNTLR_REF_0_DEVICE_ID
#define TOUCHSCREEN_DEVICE_ID      XPAR_OPB_TSD_REF_0_DEVICE_ID
#define TOUCHSCREEN_INTERRUPT_ID   XPAR_DCR_INTC_0_OPB_TSD_REF_0_INTR_INTR
#define INTC_DEVICE_ID             XPAR_DCR_INTC_0_DEVICE_ID

#define STATUS_CHECK(Status)       {if(Status != XST_SUCCESS) {return Status;}}

// UNCOMMENT THE FOLLOWING LINE TO ENABLE NETWORK SERIAL MODE
//#define NETWORK_SERIAL

#ifdef NETWORK_SERIAL
// PIT timer libraries
#include "xexception_l.h"
#include "xuartns550_l.h"
//#include "ioaccess.h"
#include "xtime_l.h"

#define UART1_BASEADDR  XPAR_OPB_UART16550_0_BASEADDR
#define UART2_BASEADDR  XPAR_OPB_UART16550_1_BASEADDR
#define UART_CLOCK      XPAR_XUARTNS550_CLOCK_HZ
#if !SIM
#define UART_BAUDRATE 9600                      /* real hardware */
#else
#define UART_BAUDRATE (UART_CLOCK / 16 / 3)     /* simulation */
#endif

// PIT timer constants
#define BAUDRATE        115000
#define INTERVAL        0x00ffffff

// UART port constants
#define PORT1		UART1_BASEADDR
#define PORT2		UART2_BASEADDR
#define STATUS_OFFSET	0x14

// network packet constants
#define TO		0x1	// 001 N NNNN where NNNNN is a 5b number (32 targets)
#define FROM		0x2	// 010 N NNNN where NNNNN is a 5b number (32 targets)
#define SET		0x3	// 011 N NNNN where NNNNN is a 5b number (32 targets)
#define ACKMAX		0x4	// 100 N NNNN where NNNNN is a 5b number (32 targets)
#define INIT		0xA0	// 101 0 0000
#define ACK		0xB0	// 101 1 0000
#define SETPOS		0x6	// 110 N NNNN where NNNNN is a 5b number for position (9 needed)
#define ADDR_INSTR	0xC	// 110 0 to test whether instructions have a from/to requirement
#define NGRQ		0xE1	// 111 0 0001 - new game request
#define NGACK		0xF1	// 111 1 0001 - new game request acknowledge
#define NGDEN		0xF2	// 111 1 0010 - new game request denial
#define SWAPOR		0xE5	// 111 0 0101 - swap order command

// network state constants
#define s0_no_connect		0
#define s1_send_init		1
#define s2_init			2
#define s3_init_a		3
#define s4_idle			4
#define s5_idle_a		5
#define s6_send_NGRQ		6
#define s7_wait_NGACK_DEN	7
#define s8_hold_NG		8
#define s9_send_NGACK		9
#define s10_send_NGDEN		10
#define s11_wait_user_move	11
#define s12_send_SETPOS		12
#define s13_wait_SETPOS		13
#define s14_send_SWAPOR		14
#define s15_hold_SWAPOR		15

// opponent number constants
#define	DIGIT0	10
#define DIGIT1	1
#define	DIGIT2	2
#define DIGIT3	3
#define	DIGIT4	4
#define DIGIT5	5
#define	DIGIT6	6
#define DIGIT7	7
#define	DIGIT8	8
#define DIGIT9	9
#define	OK	100
#define CANCEL	200
#endif

// tft display constants
#define DISPLAY_COLUMNS  	640
#define DISPLAY_ROWS     	480
#define DEFAULT_XMULT  		856
#define DEFAULT_YMULT  		1193
#define DEFAULT_XLCORN 		254057
#define DEFAULT_YLCORN 		197499

// overall game constants
#define BOARD_SIDE		3
#define BOARD_SIZE  		9
#define NUM_PLAYERS 		2
#define MAX_TURNS 		9
#define	MAX_WINS		2

// menu dimensions constants
#define MENU_RIGHT        	160
#define MENU_BUTTONS_LEFT 	30
#define BUTTON_HEIGHT     	30
#define BUTTON_WIDTH	  	120

// menu button selection constants
#define SELECT_ABOUT		50
#define SELECT_NEW_GAME 	100
#define SELECT_CLEAR_SCORES 200
#define SELECT_SWAP_PIECE 	300
#define SELECT_SWAP_ORDER 	400
#define SELECT_NETWORK_GAME 500

// tic-tac-toe grid selection constants
#define	INVALID_SELECTION	0
#define TOP_LEFT    		1
#define TOP_MID     		2
#define TOP_RIGHT   		3
#define MID_LEFT    		4
#define MID_MID     		5
#define MID_RIGHT   		6
#define BOT_LEFT    		7
#define BOT_MID     		8
#define BOT_RIGHT   		9

// X / O piece constants
#define EMPTY_SPACE		0
#define	X_PIECE			1
#define	O_PIECE			2
#define X_PIECE_THICKNESS	6
#define O_PIECE_THICKNESS	5
#define PIECE_WIDTH		50

// color constants
#define NORMAL_COLOR		0
#define	INVERT_COLOR 		1
#define MENU_COLOR		0xD4D0C8	// grey
#define NORMAL_O_PIECE_COLOR  	0xC80000	// red
#define INVERT_O_PIECE_COLOR  	0x37FFFF	// green
#define NORMAL_X_PIECE_COLOR  	0x0000FF	// blue
#define INVERT_X_PIECE_COLOR  	0xFFFF00	// yellow
#define NORMAL_BOARD_COLOR   	0x008000	// green
#define INVERT_BOARD_COLOR   	0xFF7FFF	// yellow
#define NORMAL_GRID_COLOR	0xFFFF00	// yellow
#define INVERT_GRID_COLOR	0x0000FF	// blue
#define	STATUS_MESSAGE_COLOR	0xFFFF00	// yellow
#define ABOUT_BOX_TEXT_COLOR	0xC80000	// red
#define NORMAL_WIN_LINE_COLOR	0xFF70CC	// pink
#define INVERT_WIN_LINE_COLOR	0x70FFCC	// aqua

// game results constants
#define	NO_WIN			0
#define	TOP_HORIZ_WIN		1
#define	MID_HORIZ_WIN		2
#define	BOT_HORIZ_WIN		3
#define	LEFT_VERT_WIN		4
#define	MID_VERT_WIN		5
#define	RIGHT_VERT_WIN		6
#define	LR_DIAG_WIN		7
#define	RL_DIAG_WIN		8

// game status constants
#define PLAYER1_TURN			1
#define PLAYER2_TURN			2
#define GAME_OVER_PLAYER1_WIN		3
#define GAME_OVER_PLAYER2_WIN		4
#define GAME_OVER_TIE			5
#define INVALID_TIME			6
#define NOT_YOUR_TURN			7
#define	NETWORK_GAME_IN_PROGRESS	8

/*   GLOBALS    */
static XTft Tft;
static XTouchscreen Touchscreen;
static int Xmult, Ymult, Xlcorn, Ylcorn;
static Xuint32 Xpixel, Ypixel;

struct player {
	int number;
	int piece;
	int wins;
	int losses;
	int ties;
};

#ifdef NETWORK_SERIAL
struct connection {
  unsigned int netState;
  unsigned int uart1;
  unsigned int uart2;
  unsigned int number;
  unsigned int maxNum;
  unsigned int opponentNum;
  unsigned int currMove;
  unsigned int timerCount;
  unsigned int maxTimerCount;
} netConnect;

// inbound packet data structure
struct iPacket {
  unsigned char to;
  unsigned char from;
  unsigned char data;
};

struct port {
  struct iPacket inbound;
} netPort1, netPort2;


/*************************************************************/
/**************** Port access function calls *****************/
/*************************************************************/

/*************************************************************/
/* PortWrite:  Writes data to given port, wrapper interface	 */
/*                                                           */
/*************************************************************/
void PortWrite(unsigned int port, unsigned char data)
{
  XUartNs550_SendByte(port, data);
}


/*************************************************************/
/* PacketWrite:  Writes packet to given port				 */
/*                                                           */
/*************************************************************/
void PacketWrite(unsigned int port, unsigned char data)
{
  PortWrite(port, data);
}


/*************************************************************/
/* PacketBundleWrite:  Writes 3 packets to given port		 */
/*                                                           */
/*************************************************************/
void PacketBundleWrite(unsigned int port, unsigned char to, unsigned char from, unsigned char data)
{
  PacketWrite(port, to);
  PacketWrite(port, from);
  PacketWrite(port, data);
}


/*************************************************************/
/* PortRead:  Reads packet on given port, wrapper interface	 */
/*                                                           */
/*************************************************************/
unsigned char PortRead(unsigned int port)
{
  //return ((io_rb(port+STATUS_OFFSET) & 1) ? io_rb(port) : 0);
  return ((XIo_In8(port+STATUS_OFFSET) & 1) ? XIo_In8(port) : 0);
}


/*************************************************************/
/* PacketRead:  Reads packet on given port	   				 */
/*                                                           */
/*************************************************************/
unsigned char PacketRead(unsigned int port)
{
  unsigned char data;
  
  data = PortRead(port);
  
  if(port == PORT1) {
    // store to/from address information prior to instruction data packet
    if ((data >> 5) == TO)
      netPort1.inbound.to = data;
    else if ((data >> 5) == FROM)
      netPort1.inbound.from = data;
    else {
      netPort1.inbound.data = data;
      // if data is destined for another target, pass it along
      if (((data >> 4) >= ADDR_INSTR) && ((netPort1.inbound.to & 0x1F) > netConnect.number)) {
	PacketBundleWrite(PORT2,
			  netPort1.inbound.to,
			  netPort1.inbound.from,
			  netPort1.inbound.data);
	netPort1.inbound.data = 0;
	return 0;
      }
      // if a new game request is received but we're busy, automatically deny it
      else if ((data == NGRQ) && (netConnect.netState > s4_idle)) {
	PacketBundleWrite(PORT1,
			  ((TO << 5) | (netPort1.inbound.from & 0x1F)),
			  ((FROM << 5) | netConnect.number),
			  NGDEN);
	netPort1.inbound.data = 0;
	return 0;
      }
    }
  }
  else {
    if ((data >> 5) == TO)
      netPort2.inbound.to = data;
    else if ((data >> 5) == FROM)
      netPort2.inbound.from = data;
    else
      netPort2.inbound.data = data;
    // if data is destined for another target, pass it along
    if (((data >> 4) >= ADDR_INSTR) && ((netPort2.inbound.to & 0x1F) < netConnect.number)) {
      PacketBundleWrite(PORT1,
			netPort2.inbound.to,
			netPort2.inbound.from,
			netPort2.inbound.data);
      netPort2.inbound.data = 0;
      return 0;
    }
    // if a new game request is received but we're busy, automatically deny it
    else if ((data == NGRQ) && (netConnect.netState > s4_idle)) {
      PacketBundleWrite(PORT2,
			((TO << 5) | (netPort2.inbound.from & 0x1F)),
			((FROM << 5) | netConnect.number),
			NGDEN);
      netPort2.inbound.data = 0;
      return 0;
    }
  }
  return data;
}
#endif


/*************************************************************/
/* touchscreen_GetCoord:  return the touchscreen coordinates */
/*                                                           */
/*************************************************************/
/*int touchscreen_GetCoord()
  {
  //  Get the Touchscreen Coordinates
  unsigned int x, y, z1, z2;
  unsigned int readmask, writemask, excpmask;
  
  readmask = 0;
  writemask = 0;
  excpmask = 0;
  touchscreen_Select(TouchscreenBaseAdr, &readmask, &writemask, &excpmask);
  if (readmask)
  {
  touchscreen_GetPosition(TouchscreenBaseAdr, &x, &y, &z1, &z2);
  
  
  Xpixel = (x - Xlcorn+(16*Xmult)) / Xmult;
  Ypixel = (y - Ylcorn+(16*Ymult)) / Ymult;
  
  if (Xpixel <= 0 )
  Xpixel = 1;
  else if (Xpixel > 639)
  Xpixel = 639;
  
  if (Ypixel <= 0 )
  Ypixel = 1;
  else if (Ypixel > 479)
  Ypixel = 479;
  
  return 0;
  }
  else return 1;
  }
*/


/* touchscreen_GetCoord: return touchscreen coordinates */
int touchscreen_GetCoord(void)
{
  /*  Get the Touchscreen Coordinates */
  Xuint32 x, y, z;
  Xuint32 ctrlReg;

  /* if touchscreen is touched, get coordinates */
  ctrlReg = XTouchscreen_mReadCtrlReg(Touchscreen.BaseAddress);
  if (ctrlReg & 1) {
    XTouchscreen_GetPosition_3D(&Touchscreen, &x, &y, &z);    
    /* normalize values */
    x = NORM_FACTOR - ((x * NORM_FACTOR) / 0x1000);
    y = (y * NORM_FACTOR) / 0x1000;
    /* signed computation */
    Xpixel = (x - Xlcorn+(16*Xmult)) / Xmult;
    Ypixel = (y - Ylcorn+(16*Ymult)) / Ymult;
    if (Xpixel <= 0 )
      Xpixel = 1;
    else if (Xpixel > 639)
      Xpixel = 639;
    if (Ypixel <= 0 )
      Ypixel = 1;
    else if (Ypixel > 479)
      Ypixel = 479;
    return 0;
  }
  else {
    return 1;
  }
}


/*************************************************************/
/******** Additional graphics library function calls *********/
/*************************************************************/

/*************************************************************/
/********** Game specific graphical function calls ***********/
/*************************************************************/

/*************************************************************/
/* DrawButton:  Draws button at x,y (upper left corner)		 */
/*                                                           */
/*************************************************************/
void DrawButton(int x, int y, int width, int height/*, int cond*/)
{
  // draw white lines from upper left corner
  XTft_DrawLine(&Tft, x,y,x + width,y,0xffffff);
  XTft_DrawLine(&Tft, x,y,x,y + height,0xffffff);
  
  // draw black lines on lower right corner
  XTft_DrawLine(&Tft, x,y + height + 1,x + width + 1,y + height + 1,0x404040);
  XTft_DrawLine(&Tft, x + width + 1,y,x + width + 1,y + height + 1,0x404040);
  
  // draw grey lines on lower right corner
  XTft_DrawLine(&Tft, x + 1,y + height,x + width,y + height,0x808080);
  XTft_DrawLine(&Tft, x + width,y + 1,x + width,y + height,0x808080);
}


/*************************************************************/
/* DrawMenu:  Draws and refreshes the menu (plus scores)     */
/*                                                           */
/*************************************************************/
void DrawMenu(struct player *players)
{
  // draw gray menu bar on left side
  XTft_DrawSolidBox(&Tft, 0,0,159,479,MENU_COLOR);
  
  // draw TIC-TAC-TOE name
  DrawButton(20, 20, BUTTON_WIDTH, BUTTON_HEIGHT);
  XTft_SetColor(&Tft, 0x000000,MENU_COLOR);
  XTft_SetPos(&Tft, 35, 29);
  XTft_WriteString(&Tft, "TIC-TAC-TOE");
  
  XTft_DrawLine (&Tft, 15, 69, 144, 69, 0x000000);
  XTft_DrawLine (&Tft, 15, 245, 144, 245, 0x000000);
  
  // draw "NEW GAME" button
  DrawButton (20, 274, BUTTON_WIDTH, BUTTON_HEIGHT);
  XTft_SetColor(&Tft, 0x000000,MENU_COLOR);
  XTft_SetPos(&Tft, 47, 283);
  XTft_WriteString(&Tft, "NEW GAME");
  
  // draw "CLEAR SCORES" button
  DrawButton (20, 313, BUTTON_WIDTH, BUTTON_HEIGHT);
  XTft_SetColor(&Tft, 0x000000,MENU_COLOR);
  XTft_SetPos(&Tft, 32, 322);
  XTft_WriteString(&Tft, "CLEAR SCORES");
  
  // draw "SWAP PIECE" button
  DrawButton (20, 352, BUTTON_WIDTH, BUTTON_HEIGHT);
  XTft_SetColor(&Tft, 0x000000,MENU_COLOR);
  XTft_SetPos(&Tft, 40, 361);
  XTft_WriteString(&Tft, "SWAP PIECE");
  
  // draw "SWAP ORDER" button
  DrawButton (20, 391, BUTTON_WIDTH, BUTTON_HEIGHT);
  XTft_SetColor(&Tft, 0x000000,MENU_COLOR);
  XTft_SetPos(&Tft, 40, 400);
  XTft_WriteString(&Tft, "SWAP ORDER");
  
#ifdef NETWORK_SERIAL
  // draw "NETWORK GAME" button
  DrawButton (20, 430, BUTTON_WIDTH, BUTTON_HEIGHT);
  XTft_SetColor(&Tft, 0x000000,MENU_COLOR);
  XTft_SetPos(&Tft, 32, 439);
  // 12 CHARS * 8 = 96
  XTft_WriteString(&Tft, "NETWORK GAME");
#endif

  Xpixel = 0;
  Ypixel = 0;
}


/*************************************************************/
/* DrawScores:  Draws/ refreshes player scores            	 */
/*                                                           */
/*************************************************************/
void DrawScores(struct player *players)
{
  int i;
  int x = 30;
  int y = 89;
  unsigned int tempVal  = 0x00000000;
  unsigned int tempVal2 = 0x00000000;
  //    unsigned long bitmask = 0x00000001;
  
  unsigned char scoreLine [15] = "";
  
  // draw gray menu box behind updated score values
  XTft_DrawSolidBox(&Tft, 10,70,144,230,MENU_COLOR);
  
  for (i = 0; i< NUM_PLAYERS; i++) {
    XTft_SetColor(&Tft, 0x000000,MENU_COLOR);
    
    XTft_SetPos(&Tft, x, y);
    sprintf (scoreLine, "PLAYER %d", players[i].number);
    XTft_WriteString (&Tft, scoreLine);
    
    if(players[i].piece == X_PIECE)
      XTft_DrawX (&Tft, x-13,y+4,NORMAL_X_PIECE_COLOR,6,2);
    else
      XTft_DrawCircle(&Tft, x-13,y+4,6,2,NORMAL_O_PIECE_COLOR);
    
    XTft_DrawLine (&Tft, x - 1, y + 16, x + 82, y + 16, 0x000000);
    
    XTft_SetPos(&Tft, x, y + 22);
    sprintf (scoreLine, "WINS     %d", players[i].wins);
    XTft_WriteString (&Tft, scoreLine);
    
    XTft_SetPos(&Tft, x, y + 34);
    sprintf (scoreLine, "LOSSES   %d", players[i].losses);
    XTft_WriteString (&Tft, scoreLine);
    
    XTft_SetPos(&Tft, x, y + 46);
    sprintf (scoreLine, "TIES     %d", players[i].ties);
    XTft_WriteString (&Tft, scoreLine);
    
    y = y + 78;
  }
  
  // Send the score to the GPIOs
  tempVal = (players[0].wins <<24 | (players[1].wins & 0x000000ff));
  // insert bit twist code from codesnip.c here.
  
  
  tempVal = tempVal2;  // just to preserve our original code  
  
  
  // Write the score to the LEDs
  //	gpio_Write( 8, tempVal);
  // Set the output enable for the LEDs
  //	gpio_Write(12, 0xFFFFFFFF);
  
  Xpixel = 0;
  Ypixel = 0;
}


/*************************************************************/
/* ClearScores:  Clears scores & refreshes (via  DrawScores) */
/*                                                           */
/*************************************************************/
void ClearScores(struct player *players)
{
  int i;
  
  for (i = 0; i < NUM_PLAYERS; i++) {
    players[i].wins = 0;
    players[i].losses = 0;
    players[i].ties = 0;
  }
  DrawScores(players);
}


/*************************************************************/
/* DrawXO:  Draws X or O at select board position			 */
/*                                                           */
/*************************************************************/
void DrawXO(int boardPos, int selectXO, int color)
{
  if (selectXO == X_PIECE) {
    int xColor = (color == NORMAL_COLOR) ? NORMAL_X_PIECE_COLOR : INVERT_X_PIECE_COLOR;
    switch (boardPos) {
    case TOP_LEFT:
      XTft_DrawX(&Tft, 275,110,xColor,PIECE_WIDTH, X_PIECE_THICKNESS);
      break;
    case TOP_MID:
      XTft_DrawX(&Tft, 399,110,xColor,PIECE_WIDTH, X_PIECE_THICKNESS);
      break;
    case TOP_RIGHT:
      XTft_DrawX(&Tft, 524,110,xColor,PIECE_WIDTH, X_PIECE_THICKNESS);
      break;
    case MID_LEFT:
      XTft_DrawX(&Tft, 275,234,xColor,PIECE_WIDTH, X_PIECE_THICKNESS);
      break;
    case MID_MID:
      XTft_DrawX(&Tft, 399,234,xColor,PIECE_WIDTH, X_PIECE_THICKNESS);
      break;
    case MID_RIGHT:
      XTft_DrawX(&Tft, 524,234,xColor,PIECE_WIDTH, X_PIECE_THICKNESS);
      break;
    case BOT_LEFT:
      XTft_DrawX(&Tft, 275,359,xColor,PIECE_WIDTH, X_PIECE_THICKNESS);
      break;
    case BOT_MID:
      XTft_DrawX(&Tft, 399,359,xColor,PIECE_WIDTH, X_PIECE_THICKNESS);
      break;
    case BOT_RIGHT:
      XTft_DrawX(&Tft, 524,359,xColor,PIECE_WIDTH, X_PIECE_THICKNESS);
      break;
    }
  }
  else if (selectXO == O_PIECE) {
    int oColor = (color == NORMAL_COLOR) ? NORMAL_O_PIECE_COLOR : INVERT_O_PIECE_COLOR;
    switch (boardPos) {
    case TOP_LEFT:
      XTft_DrawCircle(&Tft, 275,110,PIECE_WIDTH,O_PIECE_THICKNESS,oColor);
      break;
    case TOP_MID:
      XTft_DrawCircle(&Tft, 399,110,PIECE_WIDTH,O_PIECE_THICKNESS,oColor);
      break;
    case TOP_RIGHT:
      XTft_DrawCircle(&Tft, 524,110,PIECE_WIDTH,O_PIECE_THICKNESS,oColor);
      break;
    case MID_LEFT:
      XTft_DrawCircle(&Tft, 275,234,PIECE_WIDTH,O_PIECE_THICKNESS,oColor);
      break;
    case MID_MID:
      XTft_DrawCircle(&Tft, 399,234,PIECE_WIDTH,O_PIECE_THICKNESS,oColor);
      break;
    case MID_RIGHT:
      XTft_DrawCircle(&Tft, 524,234,PIECE_WIDTH,O_PIECE_THICKNESS,oColor);
      break;
    case BOT_LEFT:
      XTft_DrawCircle(&Tft, 275,359,PIECE_WIDTH,O_PIECE_THICKNESS,oColor);
      break;
    case BOT_MID:
      XTft_DrawCircle(&Tft, 399,359,PIECE_WIDTH,O_PIECE_THICKNESS,oColor);
      break;
    case BOT_RIGHT:
      XTft_DrawCircle(&Tft, 524,359,PIECE_WIDTH,O_PIECE_THICKNESS,oColor);
      break;
    }
  }
  
  Xpixel = 0;
  Ypixel = 0;
}


/*************************************************************/
/* DrawWinLines:  Draws winning lines given winning position */
/*                                                           */
/*************************************************************/
void DrawWinLines(int winPos, int color)
{
  int lineColor = (color == NORMAL_COLOR) ? NORMAL_WIN_LINE_COLOR : INVERT_WIN_LINE_COLOR;
  
  switch (winPos) {
  case TOP_HORIZ_WIN:
    XTft_DrawThickLine (&Tft, 215,110,584,110,lineColor,5);
    break;
  case MID_HORIZ_WIN:
    XTft_DrawThickLine (&Tft, 215,234,584,234,lineColor,5);
    break;
  case BOT_HORIZ_WIN:
    XTft_DrawThickLine (&Tft, 215,359,584,359,lineColor,5);
    break;
  case LEFT_VERT_WIN:
    XTft_DrawThickLine (&Tft, 275,50,275,419,lineColor,5);
    break;
  case MID_VERT_WIN:
    XTft_DrawThickLine (&Tft, 399,50,399,419,lineColor,5);
    break;
  case RIGHT_VERT_WIN:
    XTft_DrawThickLine (&Tft, 524,50,524,419,lineColor,5);
    break;
  case LR_DIAG_WIN:
    XTft_DrawThickLine (&Tft, 215,50,584,419,lineColor,7);
    break;
  case RL_DIAG_WIN:
    XTft_DrawThickLine (&Tft, 215,419,584,50,lineColor,7);
    break;
  }
  
  Xpixel = 0;
  Ypixel = 0;
}


/*************************************************************/
/* DrawBoard:  Draws/ refreshes tic-tac-toe board         */
/*                                                           */
/*************************************************************/
void DrawBoard(int *board, int color)
{
  int i;
  
  int boardColor = (color == NORMAL_COLOR) ?  NORMAL_BOARD_COLOR : INVERT_BOARD_COLOR;
  int gridColor = (color == NORMAL_COLOR) ? NORMAL_GRID_COLOR : INVERT_GRID_COLOR;
	
  // draw green background playing surface
  XTft_DrawSolidBox(&Tft, 160,0,639,479,boardColor);

  // draw tic-tac-toe grid lines
  XTft_DrawSolidBox(&Tft, 215,170,584,174,gridColor);
  XTft_DrawSolidBox(&Tft, 215,295,584,299,gridColor);
  XTft_DrawSolidBox(&Tft, 335,50,339,419,gridColor);
  XTft_DrawSolidBox(&Tft, 460,50,464,419,gridColor);
  
  // refreshes board given board array
  for (i = 0; i < BOARD_SIZE; i++) {
    DrawXO (i+1, board[i], color);
  }
  
  Xpixel = 0;
  Ypixel = 0;
}


/*************************************************************/
/* ClearBoard:  Clears board array & draws empty board       */
/*                                                           */
/*************************************************************/
void ClearBoard(int *board, int *win)
{
  int i;
  
  for (i = 0; i < BOARD_SIZE; i++) {
    board[i] = EMPTY_SPACE;
  }
  
  for (i = 0; i < MAX_WINS; i++) {
    win[i] = NO_WIN;
  }
  
  DrawBoard (board, NORMAL_COLOR);
}


/*************************************************************/
/* DrawStatusMessage:  Draw staus message given message type */
/*                                                           */
/*************************************************************/
void DrawStatusMessage(int messageType)
{
  // draw green background box for message
  XTft_DrawSolidBox(&Tft, 163,455,635,475,NORMAL_BOARD_COLOR);
  XTft_SetColor(&Tft, STATUS_MESSAGE_COLOR,NORMAL_BOARD_COLOR);
  
  switch (messageType) {
  case PLAYER1_TURN:
    XTft_SetPos(&Tft, 347,458);
    XTft_WriteString(&Tft, "1st PLAYER TURN");
    break;
  case PLAYER2_TURN:
    XTft_SetPos(&Tft, 347,458);
    XTft_WriteString(&Tft, "2nd PLAYER TURN");
    break;
  case GAME_OVER_PLAYER1_WIN:
    XTft_SetPos(&Tft, 299,458);
    XTft_WriteString(&Tft, "GAME OVER - 1st PLAYER WINS");
    break;
  case GAME_OVER_PLAYER2_WIN:
    XTft_SetPos(&Tft, 299,458);
    XTft_WriteString(&Tft, "GAME OVER - 2nd PLAYER WINS");
    break;
  case GAME_OVER_TIE:
    XTft_SetPos(&Tft, 339,458);
    XTft_WriteString(&Tft, "GAME OVER - TIE");
    break;
  case INVALID_TIME:
    XTft_SetPos(&Tft, 183,458);
    XTft_WriteString(&Tft, "SWAP CAN ONLY BE MADE AT THE BEGINNING OF A NEW GAME");
    break;
    
#ifdef NETWORK_SERIAL
  case NOT_YOUR_TURN:
    XTft_SetPos(&Tft, 255,458);
    XTft_WriteString(&Tft, "INVALID - WAITING FOR OPPONENT INPUT");
    break;
  case NETWORK_GAME_IN_PROGRESS:
    XTft_SetPos(&Tft, 299,458);
    XTft_WriteString(&Tft, "NETWORK GAME IN PROGRESS");
    break;
#endif
    
  default:
    XTft_SetPos(&Tft, 283,458);
    XTft_WriteString(&Tft, "INVALID SELECTION - TRY AGAIN");
  }
  
  Xpixel = 0;
  Ypixel = 0;
}


/*************************************************************/
/* DrawAboutBox:  Draws about box			     */
/*************************************************************/
void DrawAboutBox()
{
  // draw grey background box plus red box outline
  XTft_DrawSolidBox(&Tft, 184,189,454,289,MENU_COLOR);
  XTft_DrawEmptyBox(&Tft, 189,193,449,284,ABOUT_BOX_TEXT_COLOR,1);
  
  DrawButton (184, 189, 270, 100);
  
  // Xilinx logo has tic-tac-toe X in front
  XTft_DrawX (&Tft, 293,225,ABOUT_BOX_TEXT_COLOR,6,3);
  XTft_SetColor(&Tft, ABOUT_BOX_TEXT_COLOR,MENU_COLOR);
  XTft_SetPos(&Tft, 304,221);
  XTft_WriteString(&Tft, "ILINX");
  
  XTft_SetColor(&Tft, 0x000000,MENU_COLOR);
  XTft_SetPos(&Tft, 212,245);
  XTft_WriteString(&Tft, "TIC-TAC-TOE TUTORIAL v.1.0");
}


/*************************************************************/
/*************** Game specific function calls ****************/
/*************************************************************/

#ifdef NETWORK_SERIAL
/*************************************************************/
/* OkTouchCheck:  Check for valid touched area for ok box	 */
/*                                                           */
/*************************************************************/
int OkTouchCheck()
{
  return ((Xpixel > 183) && (Xpixel < 455) && (Ypixel > 254) && (Ypixel < 286));
}


/*************************************************************/
/* OkCancelTouchCheck:  Check for valid touched area         */
/*						in ok/ cancel box					 */
/*                                                           */
/*************************************************************/
int OkCancelTouchCheck()
{
  if ((Xpixel > 183) && (Xpixel < 455) && (Ypixel > 254) && (Ypixel < 286))
    if (Xpixel < 319)
      return OK;
    else
      return CANCEL;
  else
    return 0;
}
#endif


/*************************************************************/
/* TouchCheck:  Check for valid touched area in main game 	 */
/*                                                           */
/*************************************************************/
int TouchCheck()
{
  // check if touch is inside grid area
  if ((Xpixel > 214) && (Xpixel < 585) && (Ypixel > 49) && (Ypixel < 420)) {
    if (Ypixel < 170) {
      if (Xpixel < 335)
	return TOP_LEFT;
      else if (Xpixel < 460)
	return TOP_MID;
      else
	return TOP_RIGHT;
    }
    else if (Ypixel < 295) {
      if (Xpixel < 335)
	return MID_LEFT;
      else if (Xpixel < 460)
	return MID_MID;
      else
	return MID_RIGHT;
    }
    else {
      if (Xpixel < 335)
	return BOT_LEFT;
      else if (Xpixel < 460)
	return BOT_MID;
      else
	return BOT_RIGHT;
    }
  }
  // check if touch is on control buttons
  else if ((Xpixel < 150) && (Xpixel > 19) && (Ypixel > 19) && (Ypixel < 460)) {
    if (Ypixel < 60)
      return SELECT_ABOUT;
    else if (Ypixel > 273) {
      if (Ypixel < 304)
	return SELECT_NEW_GAME;
      else if (Ypixel > 312) {
	if(Ypixel < 343)
	  return SELECT_CLEAR_SCORES;
	else if (Ypixel > 351) {
	  if (Ypixel < 382)
	    return SELECT_SWAP_PIECE;
	  else if (Ypixel > 390) {
	    if (Ypixel < 421)
	      return SELECT_SWAP_ORDER;
#ifdef NETWORK_SERIAL
	    else if (Ypixel > 429)
	      return SELECT_NETWORK_GAME;
#endif
	    
	  }
	}
      }
    }
  }
  // all other areas are considered invalid
  return INVALID_SELECTION;
}


/*************************************************************/
/* ProcessWin:  Update board / score for a win			     */
/*                                                           */
/*************************************************************/
void ProcessWin(int *board, struct player *players, int currPlayer,int *currMessage, int *win, int *gameOver)
{
  int i;
  
  players[currPlayer].wins++;
  players[!currPlayer].losses++;
  DrawScores(players);
  
  for (i = 0; i < 1; i ++) {
    DrawBoard (board, INVERT_COLOR);
    DrawBoard (board, NORMAL_COLOR);
  }
  for (i = 0; i < MAX_WINS; i++) {
    DrawWinLines (win[i], NORMAL_COLOR);
  }
  
  *currMessage = (currPlayer) ? GAME_OVER_PLAYER2_WIN : GAME_OVER_PLAYER1_WIN;
  *gameOver = 1;
}


/*************************************************************/
/* ProcessTie:  Updae board / score for a tie 				 */
/*                                                           */
/*************************************************************/
void ProcessTie (int *board, struct player *players,int *currMessage, int *currColor, int *win, int *gameOver)
{
  players[0].ties++;
  players[1].ties++;
  DrawScores(players);
  
  *currColor = INVERT_COLOR;
  DrawBoard (board, INVERT_COLOR);
  
  *currMessage = GAME_OVER_TIE;
  *gameOver = 1;
}


/*************************************************************/
/* CheckForWin:  Check board for possible win				 */
/*                                                           */
/*************************************************************/
int CheckForWin(int *board, int *win)
{
  int i = 0;
  
  // check for horizontal wins
  if (board[0] && (board[0] == board[1]) && (board[1] == board[2])) {
    win[i] = TOP_HORIZ_WIN;
    i++;
  }
  if (board[3] && (board[3] == board[4]) && (board[4] == board[5])) {
    win[i] = MID_HORIZ_WIN;
    i++;
  }
  if (board[6] && (board[6] == board[7]) && (board[7] == board[8])) {
    win[i] = BOT_HORIZ_WIN;
    i++;
  }
  
  // check for vertical wins
  if (board[0] && (board[0] == board[3]) && (board[3] == board[6])) {
    win[i] = LEFT_VERT_WIN;
    i++;
  }
  if (board[1] && (board[1] == board[4]) && (board[4] == board[7])) {
    win[i] = MID_VERT_WIN;
    i++;
  }
  if (board[2] && (board[2] == board[5]) && (board[5] == board[8])) {
    win[i] = RIGHT_VERT_WIN;
    i++;
  }

  // check for upper-left to lower-right diagonal win
  if (board[0] && (board[0] == board[4]) && (board[4] == board[8])) {
    win[i] = LR_DIAG_WIN;
    i++;
  }

  // check for lower-left to upper-right diagonal win
  if (board[2] && (board[2] == board[4]) && (board[4] == board[6])) {
    win[i] = RL_DIAG_WIN;
  }
  
  return (win[0]);
}

#ifdef NETWORK_SERIAL
/*************************************************************/
/*************** Network library function calls **************/
/*************************************************************/

/*************************************************************/
/* InitNetworkSettings:  Initialize network variables		 */
/*                                                           */
/*************************************************************/
void InitNetworkSettings()
{
  netConnect.netState = s0_no_connect;
  netConnect.number = 0;
  netConnect.uart1 = 0;
  netConnect.uart2 = 0;
  netConnect.maxNum = 0;
  netConnect.opponentNum = 0;
  netConnect.currMove = 0;
  netConnect.timerCount = 0;
  netConnect.maxTimerCount = 1;
  
  netPort1.inbound.to = 0;
  netPort1.inbound.from = 0;
  netPort1.inbound.data = 0;
  
  netPort2.inbound.to = 0;
  netPort2.inbound.from = 0;
  netPort2.inbound.data = 0;
  
  // initialize UART ports
  XUartNs550_SetBaud(UART1_BASEADDR, UART_CLOCK, UART_BAUDRATE);
  XUartNs550_mSetLineControlReg(UART1_BASEADDR, XUN_LCR_8_DATA_BITS);
  XUartNs550_SetBaud(UART2_BASEADDR, UART_CLOCK, UART_BAUDRATE);
  XUartNs550_mSetLineControlReg(UART2_BASEADDR, XUN_LCR_8_DATA_BITS);
  
  // write null data to ports
  XUartNs550_SendByte(PORT1, 0);
  XUartNs550_SendByte(PORT2, 0);
}


/*************************************************************/
/* DrawNetworkBox:  Draws box given network messages		 */
/*                                                           */
/*************************************************************/
void DrawNetworkBox(unsigned char *message1, unsigned char *message2)
{
  XTft_DrawSolidBox(&Tft, 184,188,454,253,MENU_COLOR);
  
  DrawButton (184, 188, 270, 65);
  
  XTft_SetColor(&Tft, 0x000000,MENU_COLOR);
  XTft_SetPos(&Tft, 212,209);
  XTft_WriteString(&Tft, message1);
  
  XTft_SetPos(&Tft, 212,223);
  XTft_WriteString(&Tft, message2);
}


/*************************************************************/
/* DrawOkBox:  Draws box with ok button 		 			 */
/*                                                           */
/*************************************************************/
void DrawOkBox(char *line1, char *line2)
{
  int x = 184;
  int y = 255;
  
  DrawNetworkBox (line1, line2);
  
  XTft_DrawSolidBox(&Tft, x,y,x + 270,y + 30,MENU_COLOR);
  XTft_SetColor(&Tft, 0x000000,MENU_COLOR);
  
  DrawButton (184,255,270,30);
  XTft_SetPos(&Tft, 313,264);
  XTft_WriteString(&Tft, "OK");
}


/*************************************************************/
/* DrawOkCancelBox:  Draws box with ok/ cancel buttons 		 */
/*                                                           */
/*************************************************************/
void DrawOkCancelBox(char *line1, char *line2)
{
  int x = 184;
  int y = 255;
  
  DrawNetworkBox (line1, line2);
  
  XTft_DrawSolidBox(&Tft, x,y,x + 270,y + 30,MENU_COLOR);
  XTft_SetColor(&Tft, 0x000000,MENU_COLOR);
  
  DrawButton (184,255,134,30);
  XTft_SetPos(&Tft, 243,264);
  XTft_WriteString(&Tft, "OK");
  
  DrawButton (319,255,134,30);
  XTft_SetPos(&Tft, 362,264);
  XTft_WriteString(&Tft, "CANCEL");
}


/*************************************************************/
/* ConfirmOk:  Request user input to confirm message	     */
/*                                                           */
/*************************************************************/
int ConfirmOk(char *line1, char *line2, int currMessage)
{
  DrawOkBox(line1, line2);
  while (1) {
    if (touchscreen_GetCoord() == 0) {
      if(OkTouchCheck())
	return 1;
      else {
	DrawStatusMessage(INVALID_SELECTION);
	usleep (200000);
	DrawStatusMessage(currMessage);
      }
      usleep (500000);
    }
  }
}


/*************************************************************/
/* InitNetwork:  Initialize network connections				 */
/*                                                           */
/*************************************************************/
void InitNetwork(struct player *players, int currMessage)
{
  unsigned char message[40] = "";
  
  DrawNetworkBox ("Initializing network ...","");
  
  // set state (let timer send the packets
  netConnect.netState = (netConnect.netState) ? netConnect.netState : s1_send_init;
  while((netConnect.netState == s1_send_init) ||
	(netConnect.netState == s2_init) ||
	(netConnect.netState == s3_init_a)) {}
  if (netConnect.netState == s4_idle) {
    players[0].number = netConnect.number;
    players[1].number = 0;
    DrawScores(players);
    
    sprintf (message, "%d useres online. You are #%d", netConnect.maxNum, netConnect.number);
    ConfirmOk("Initialization done!", message, currMessage);
  }
  else if (!netConnect.netState)	// s0_no_connect or timed out
    ConfirmOk("Initialization failed.", "No one found on network.", currMessage);
  else { // some other error, print out current netState
    sprintf (message, "netState = %d", netConnect.netState);
    ConfirmOk("Initialization failed.", message, currMessage);
  }
}


/*************************************************************/
/* DrawNetworkNewGameBox:  Draws new game box to select game */
/*                                                           */
/*************************************************************/
void DrawNetworkNewGameBox()
{
  int i, j;
  int x = 184;
  int y = 255;
  int size = 54;
  int xoff = (size / 2) - 4;
  int yoff = (size / 2) - 6;
  char val[2] = "";
  
  DrawNetworkBox ("Please select an opponent.", "");
  
  XTft_DrawSolidBox(&Tft, x,y,x + 270,y + (2 * size) + 30,MENU_COLOR);
  
  XTft_SetColor(&Tft, 0x000000,MENU_COLOR);
  
  // draws number buttons to select opponent
  for (i = 0; i < 2; i++) {
    for (j = 0; j < 5; j++) {
      DrawButton (x + (j * size), y + (i * size), size - 1, size - 1);
      XTft_SetPos(&Tft, x + (j * size) + xoff, y + (i * size) + yoff);
      sprintf(val, "%d", (i*5)+j);
      XTft_WriteString(&Tft, val);
    }
  }
  
  DrawButton (184,363,134,30);
  XTft_SetPos(&Tft, 243,372);
  XTft_WriteString(&Tft, "OK");
  
  DrawButton (319,363,134,30);
  XTft_SetPos(&Tft, 362,372);
  XTft_WriteString(&Tft, "CANCEL");
}



/*************************************************************/
/* NetworkNewGameTouchCheck:  Check for valid touched area   */
/*                            during new game command        */
/*                                                           */
/*************************************************************/
int NetworkNewGameTouchCheck()
{
  // check if touch is inside grid area
  if ((Xpixel > 183) && (Xpixel < 455) && (Ypixel > 253) && (Ypixel < 394)) {
    if (Ypixel < 308) {
      if (Xpixel < 238)
	return DIGIT0;
      else if (Xpixel < 292)
	return DIGIT1;
      else if (Xpixel < 346)
	return DIGIT2;
      else if (Xpixel < 400)
	return DIGIT3;
      else
	return DIGIT4;
    }
    else if (Ypixel < 363) {
      if (Xpixel < 238)
	return DIGIT5;
      else if (Xpixel < 292)
	return DIGIT6;
      else if (Xpixel < 346)
	return DIGIT7;
      else if (Xpixel < 400)
	return DIGIT8;
      else
	return DIGIT9;
    }
    else if (Xpixel < 319)
      return OK;
    else
      return CANCEL;
  }
  else
    return 0;
}


/*************************************************************/
/* SelectOpponent:  Request user input to select opponent  	 */
/*                                                           */
/*************************************************************/
int SelectOpponent(int currMessage)
{
  int touchResult;
  char message1[40] = "";
  char message2[40] = "";
  
  DrawNetworkNewGameBox();
  netConnect.opponentNum = 0;
  netConnect.netState = s5_idle_a;
  
  while (1) {
    if (touchscreen_GetCoord() == 0) {
      switch (touchResult = NetworkNewGameTouchCheck()) {
      case DIGIT0:
	touchResult = 0;
      case DIGIT1:
      case DIGIT2:
      case DIGIT3:
      case DIGIT4:
      case DIGIT5:
      case DIGIT6:
      case DIGIT7:
      case DIGIT8:
      case DIGIT9:
	netConnect.opponentNum *= 10;
	netConnect.opponentNum += touchResult;
	sprintf(message2,"%d", netConnect.opponentNum);
	DrawNetworkBox ("Please select an opponent.", message2);
	break;
      case OK:
	return (netConnect.opponentNum);
	break;
      case CANCEL:
	netConnect.opponentNum = 0;
	return 0;
	break;
      default:
	DrawStatusMessage(INVALID_SELECTION);
	usleep (200000);
	DrawStatusMessage(currMessage);
      }
      usleep (500000);
      // cannot select somebody higher than the last player
      if(netConnect.opponentNum > netConnect.maxNum) {
	sprintf (message1, "Opponent #%d out of range.", netConnect.opponentNum);
	sprintf (message2, "Please select # between 1-%d", netConnect.maxNum);
	DrawNetworkBox (message1, message2);
	netConnect.opponentNum = 0;
	usleep (1500000);
	DrawNetworkBox ("Please select an opponent.", "");
      }
      // cannot select yourself
      else if(netConnect.opponentNum == netConnect.number) {
	sprintf (message2, "yourself (player #%d).", netConnect.number);
	DrawNetworkBox ("Please select # other than", message2);
	netConnect.opponentNum = 0;
	usleep (1500000);
	DrawNetworkBox ("Please select an opponent.", "");
      }
    }
  }
}


/*************************************************************/
/* ConfirmOpponent:  Request user input to confirm opponent  */
/*                                                           */
/*************************************************************/
int ConfirmOpponent(int currMessage, struct player *players)
{
  int touchResult;
  char message[40] = "";
  
  sprintf(message, "from opponent #%d?", netConnect.opponentNum);
  DrawOkCancelBox("Do you accept the challenge", message);
  
  while (1) {
    if (touchscreen_GetCoord() == 0) {
      switch (touchResult = OkCancelTouchCheck()) {
      case OK:
	// sets local values and sends back ack to new game request
	players[1].number = netConnect.opponentNum;
	DrawScores(players);
	netConnect.netState = s9_send_NGACK;
	return 1;
	break;
      case CANCEL:
	// sends back deny command to new game request
	netConnect.netState = s10_send_NGDEN;
	return 0;
	break;
      default:
	DrawStatusMessage(INVALID_SELECTION);
	usleep (200000);
	DrawStatusMessage(currMessage);
      }
      usleep (500000);
    }
  }
}


/*************************************************************/
/* ConfirmReinitialize: Reinitialize based on user's input	 */
/*                                                           */
/*************************************************************/
int ConfirmReinitialize(int currMessage)
{
  int touchResult;
  
  DrawOkCancelBox("Network already initialized.", "Reinitialize?");
  while (1) {
    if (touchscreen_GetCoord() == 0) {
      switch (touchResult = OkCancelTouchCheck()) {
      case OK:
	netConnect.uart1 = 0;
	netConnect.uart2 = 0;
	netConnect.netState = s1_send_init;
	return 1;
	break;
      case CANCEL:
	return 0;
	break;
      default:
	DrawStatusMessage(INVALID_SELECTION);
	usleep (200000);
	DrawStatusMessage(currMessage);
      }
      usleep (500000);
    }
  }
}


/*************************************************************/
/* NegotiateNewGame:  Negotiate a new game with opponent	 */
/*                                                           */
/*************************************************************/
void NegotiateNewGame (int currMessage, struct player *players)
{
  unsigned char message [40] = "";
  
  sprintf (message, "with player #%d ...", netConnect.opponentNum);
  DrawNetworkBox ("Negotiating a new game", message);
  
  // lets timer send out new game request packets
  netConnect.netState = s6_send_NGRQ;
  
  while((netConnect.netState == s6_send_NGRQ) ||
	(netConnect.netState == s7_wait_NGACK_DEN)) {}
  if (netConnect.netState == s11_wait_user_move) {
    players[1].number = netConnect.opponentNum;
    DrawScores(players);
    sprintf (message, "Connected to player #%d", players[1].number);
    ConfirmOk("New game request accepted!", message, currMessage);
  }
  else
    ConfirmOk("New game request rejected.","Please try again.", currMessage);
}


/*************************************************************/
/* NotifyOrderSwap:  1st player sends order swap command to  */
/*					 2nd player. 2nd player can switch back. */
/*                                                           */
/*************************************************************/
void NotifyOrderSwap (int *currPlayer)
{
  unsigned char message [40] = "";
  
  *currPlayer = !(*currPlayer);
  netConnect.netState = s14_send_SWAPOR;
  
  while(netConnect.netState == s14_send_SWAPOR) {}
  sprintf (message, "Notifying player #%d", netConnect.opponentNum);
  DrawNetworkBox (message, "of order swap.");
}


/*************************************************************/
/* TimerExceptionHandler:  Process Timer Exception			 */
/*                                                           */
/*************************************************************/
void TimerExceptionHandler (void *CallBackRef, Xuint32 Event, unsigned int EventData)
{
  unsigned char packet1, packet2;
  
  packet1 = PacketRead(PORT1);
  packet2 = PacketRead(PORT2);
  
  if (packet1 == INIT) {
    // all INIT commands receives an ACK back
    PacketWrite(PORT1, ACK);
    
    // new INIT forces a system wide reinitialization
    if(netConnect.netState >= s4_idle) {
      netConnect.netState = s0_no_connect;
      netConnect.uart2 = 0;
    }
    
    // if never initialized, set to initialize state
    if(netConnect.netState == s0_no_connect) {
      netConnect.netState = s1_send_init;
      netConnect.uart1 = 1;
    }
  }
  
  if (packet2 == INIT) {
    // all INIT commands receives an ACK back
    PacketWrite(PORT2, ACK);
    
    // new INIT forces a system wide reinitialization
    if(netConnect.netState >= s4_idle) {
      netConnect.netState = s0_no_connect;
      netConnect.uart1 = 0;
    }
    
    // if never initialized, set to initialize state
    if(netConnect.netState == s0_no_connect) {
      netConnect.netState = s1_send_init;
      netConnect.uart2 = 1;
    }
  }
  
  switch (netConnect.netState) {
  case s1_send_init:		// sends out INIT packets and resets network variables
    netConnect.number = 0;
    netConnect.maxNum = 0;
    netConnect.opponentNum = 0;
    netConnect.currMove = 0;
    netConnect.timerCount = 1;
    netConnect.maxTimerCount = 20;
    
    if (!netConnect.uart1)
      PacketWrite(PORT1, INIT);
    if (!netConnect.uart2)
      PacketWrite(PORT2, INIT);
    netConnect.netState = s2_init;
    break;
    
  case s2_init:		// waits for ACK packets, SET command and ACKMAX values
    netConnect.uart1 = (packet1 == ACK) ? 1 : netConnect.uart1;
    netConnect.uart2 = (packet2 == ACK) ? 1 : netConnect.uart2;
    netConnect.number = ((packet1 >> 5) == SET) ? (packet1 & 0x1F) : netConnect.number;
    
    if (netConnect.timerCount > netConnect.maxTimerCount) {
      if (netConnect.uart1) {
	if ((packet1 >> 5) == SET)
	  netConnect.number = (packet1 & 0x1F);
	if (netConnect.uart2) {
	  if (netConnect.number) {
	    PacketWrite(PORT2, ((SET << 5) | (netConnect.number + 1)));
	    netConnect.netState = s3_init_a;
	  }
	}
	else if (netConnect.number) {
	  netConnect.maxNum = netConnect.number;
	  PacketWrite(PORT1, ((ACKMAX << 5) | (netConnect.maxNum)));
	  netConnect.netState = s4_idle;
	}
      }
      else if (netConnect.uart2) {
	if (!netConnect.number) {
	  netConnect.number = 1;
	  PacketWrite(PORT2, ((SET << 5) | 0x2));
	}
	else if ((packet2 >> 5) == ACKMAX) {
	  netConnect.maxNum = (packet2 & 0x1F);
	  netConnect.netState = s4_idle;
	}
      }
      else
	netConnect.netState = s0_no_connect;
    }
    else
      netConnect.timerCount++;
    break;
    
  case s3_init_a:	// holding state for middle node to wait for ACKMAX
    if ((packet2 >> 5) == ACKMAX) {
      netConnect.maxNum = (packet2 & 0x1F);
      PacketWrite(PORT1, packet2);
      netConnect.netState = s4_idle;
    }
    break;

  case s4_idle:		// default state to wait for new game request
			// Port1 requests always handled first
    if(packet1 == NGRQ) {
      netConnect.opponentNum = (netPort1.inbound.from & 0x1F);
      netConnect.netState = s8_hold_NG;
      if (packet2 == NGRQ) {
	PacketBundleWrite(PORT2,
			  ((TO << 5) | (netPort2.inbound.from & 0x1F)),
			  ((FROM << 5) | netConnect.number),
			  NGDEN);
      }
    }
    else if (packet2 == NGRQ) {
      netConnect.opponentNum = (netPort2.inbound.from & 0x1F);
      netConnect.netState = s8_hold_NG;
    }
    break;
    
  case s5_idle_a: 	// holding state for opponent selection during a new game request
    break;
    
  case s6_send_NGRQ:	// sends new game request
    if ((netConnect.uart1) && (netConnect.number > netConnect.opponentNum)) {
      PacketBundleWrite(PORT1,
			((TO << 5) | netConnect.opponentNum),
			((FROM << 5) | netConnect.number),
			NGRQ);
    }
    else if ((netConnect.uart2) && (netConnect.number < netConnect.opponentNum)) {
      PacketBundleWrite(PORT2,
			((TO << 5) | netConnect.opponentNum),
			((FROM << 5) | netConnect.number),
			NGRQ);
    }
    netConnect.netState = s7_wait_NGACK_DEN;
    break;

  case s7_wait_NGACK_DEN:		// waits for new game acknowledgement or denial
    if (netConnect.number < netConnect.opponentNum) {
      if((packet2 == NGACK) &&
	 ((netPort2.inbound.from & 0x1F) == netConnect.opponentNum))
	netConnect.netState = s11_wait_user_move;
      else if ((packet2 == NGDEN) &&
	       ((netPort2.inbound.from & 0x1F) == netConnect.opponentNum)) {
	netConnect.opponentNum = 0;
	netConnect.netState = s4_idle;
      }
    }
    else if (netConnect.number > netConnect.opponentNum) {
      if((packet1 == NGACK) &&
	 ((netPort1.inbound.from & 0x1F) == netConnect.opponentNum))
	netConnect.netState = s11_wait_user_move;
      else if ((packet1 == NGDEN) &&
	       ((netPort1.inbound.from & 0x1F) == netConnect.opponentNum)) {
	netConnect.opponentNum = 0;
	netConnect.netState = s4_idle;
      }
    }
    break;
    
  case s8_hold_NG: 	// just a holding state to request user input given a new game request
    break;
    
  case s9_send_NGACK:	// sends new game acknowledge when user agrees
    if (netConnect.number < netConnect.opponentNum) {
      PacketBundleWrite(PORT2,
			((TO << 5) | netConnect.opponentNum),
			((FROM << 5) | netConnect.number),
			NGACK);
    }
    else {
      PacketBundleWrite(PORT1,
			((TO << 5) | netConnect.opponentNum),
			((FROM << 5) | netConnect.number),
			NGACK);
    }
    netConnect.netState = s13_wait_SETPOS;
    break;
    
  case s10_send_NGDEN:	// sends new game denial when user disagrees
    if (netConnect.number < netConnect.opponentNum) {
      PacketBundleWrite(PORT2,
			((TO << 5) | netConnect.opponentNum),
			((FROM << 5) | netConnect.number),
			NGDEN);
    }
    else {
      PacketBundleWrite(PORT1,
			((TO << 5) | netConnect.opponentNum),
			((FROM << 5) | netConnect.number),
			NGDEN);
    }
    netConnect.opponentNum = 0;
    netConnect.netState = s4_idle;
    break;
    
  case s11_wait_user_move: 	// waiting for user input during a game - holding state
    break;
    
  case s12_send_SETPOS:	// sends user input position to opponent
    if (netConnect.number < netConnect.opponentNum) {
      PacketBundleWrite(PORT2,
			((TO << 5) | netConnect.opponentNum),
			((FROM << 5) | netConnect.number),
			((SETPOS << 5) | netConnect.currMove));
    }
    else {
      PacketBundleWrite(PORT1,
			((TO << 5) | netConnect.opponentNum),
			((FROM << 5) | netConnect.number),
			((SETPOS << 5) | netConnect.currMove));
    }
    netConnect.currMove = 0;
    netConnect.netState = s13_wait_SETPOS;
    break;
    
  case s13_wait_SETPOS:		// waits for opponent to send back their position
    if (netConnect.number < netConnect.opponentNum) {
      if ((packet2 >> 5) == SETPOS)
	netConnect.currMove = (packet2 & 0x1F);
      else if (packet2 == SWAPOR) // initiator forces this to be turn 1
	netConnect.netState = s15_hold_SWAPOR;
    }
    else if (netConnect.number > netConnect.opponentNum) {
      if ((packet1 >> 5) == SETPOS)
	netConnect.currMove = (packet1 & 0x1F);
      else if (packet1 == SWAPOR)
	netConnect.netState = s15_hold_SWAPOR;
    }
    break;
    
  case s14_send_SWAPOR:	// sends out order swap command
    if (netConnect.number < netConnect.opponentNum) {
      PacketBundleWrite(PORT2,
			((TO << 5) | netConnect.opponentNum),
			((FROM << 5) | netConnect.number),
			SWAPOR);
    }
    else {
      PacketBundleWrite(PORT1,
			((TO << 5) | netConnect.opponentNum),
			((FROM << 5) | netConnect.number),
			SWAPOR);
    }
    netConnect.netState = s13_wait_SETPOS;
    
  case s15_hold_SWAPOR: // another holding state for swap order command
    break;
  default:
  }
  
  XTime_PITClearInterrupt();
  XTime_PITSetInterval(INTERVAL);
}
#endif


/************************************************************/
/* V2P Tic-Tac-Toe                                          */
/*                                                          */
/* A simple tic-tac-toe game designed for V2P which         */
/* demonstrates Touchscreen and TFT functionality.          */
/*                                                          */
/* 8/26/02:  JL                                             */
/************************************************************/

int main(void)
{
  XStatus Status;

  int temp;
  int turnNum = 1;			// turn count is initialized to 1
  int startPlayer = 0;			// starting player is set by default to 1st player
  int currPlayer = 0;			// current player is set by default to 1st player
  int touchResult = 0;			// touch result by default set to invalid
  int currMessage = PLAYER1_TURN;		// current message is set to "1st Player TURN"
  int currColor = NORMAL_COLOR;		// colors initialized to normal
  int gameOver = 0;			// game is initialized to be NOT over
  
  int board[BOARD_SIZE];			// tic-tac-toe board array
  int win[MAX_WINS];			// winning lines array (max is 2 simultaneous)
  struct player players[NUM_PLAYERS];	// players struct (wins, losses, ties)
  
  // sets initial player numbers
  players[0].number = 1;
  players[1].number = 2;
  
  // set initial player piece types
  players[0].piece = X_PIECE;		// 1st player by default is X
  players[1].piece = O_PIECE;		// 2nd player by default is Y
  
  XCache_EnableICache(0x80000001);
  XCache_EnableDCache(0x80000001);
  
  // Init Globals
  Xpixel=0;
  Ypixel=0;
  Xmult=DEFAULT_XMULT;
  Ymult=DEFAULT_YMULT;
  Xlcorn=DEFAULT_XLCORN;
  Ylcorn=DEFAULT_YLCORN;
  
#ifdef NETWORK_SERIAL
  
  InitNetworkSettings();
  
  // Exception Handler initialization
  XExc_Init();
  XExc_RegisterHandler(XEXC_ID_PIT_INT, 
		       (XExceptionHandler)TimerExceptionHandler, 
		       (void *)XNULL);
  XTime_PITSetInterval(INTERVAL);
  XTime_TSRClearStatusBits(XREG_TSR_CLEAR_ALL);
  XTime_PITEnableAutoReload();
  XTime_PITEnableInterrupt();
  XExc_mEnableExceptions(XEXC_ALL);
  
#endif
  // TFT Initialization
  Status = XTft_Initialize(&Tft, TFT_DEVICE_ID);
  STATUS_CHECK(Status);
  
  // Touchscreen initialization
  Status = XTouchscreen_Initialize(&Touchscreen, TOUCHSCREEN_DEVICE_ID);
  STATUS_CHECK(Status);
  
  XTft_SetColor(&Tft, 0x000000, 0xffffff);
  XTft_ClearScreen(&Tft);
  
  // game initialization
  DrawMenu(players);
  ClearScores (players);
  ClearBoard (board, win);
  DrawStatusMessage(currMessage);
  
  while (1) {
      
#ifdef NETWORK_SERIAL
    
    // if receiver of initialization command, move to initialize
    if (netConnect.netState == s2_init) {
      InitNetwork(players, currMessage);
      startPlayer = 0;
      currPlayer = startPlayer;
      
      DrawBoard (board, currColor);
      DrawStatusMessage(currMessage);
    }
    // if receive new game request, ask for user response
    else if (netConnect.netState == s8_hold_NG) {
      ClearBoard (board, win);
      turnNum = 1;
      currPlayer = startPlayer + 1;
      currMessage = currPlayer + 1;
      currColor = NORMAL_COLOR;
      gameOver = 0;
      
      ConfirmOpponent(currMessage, players);
      DrawBoard (board, currColor);
      DrawStatusMessage(currMessage);
    }
    // if receive opponent move, mark move value and check for win/ tie
    else if ((netConnect.netState == s13_wait_SETPOS) && 
	     (netConnect.currMove) &&
	     !board[netConnect.currMove-1]) {
      board[netConnect.currMove-1] = players[currPlayer].piece;
      DrawXO (netConnect.currMove, players[currPlayer].piece, NORMAL_COLOR);
      currMessage = (!currPlayer) + 1;
      
      if (CheckForWin (board, win)) {
	ProcessWin (board, players, currPlayer, &currMessage, win, &gameOver);
	netConnect.currMove = 0;
	netConnect.opponentNum = 0;
	netConnect.netState = s4_idle;
      }
      else if (turnNum == MAX_TURNS) {
	ProcessTie(board, players, &currMessage, &currColor, win, &gameOver);
	netConnect.currMove = 0;
	netConnect.opponentNum = 0;
	netConnect.netState = s4_idle;
      }
      else {
	currPlayer = !currPlayer;
	turnNum++;
	netConnect.netState = s11_wait_user_move;
      }
      DrawStatusMessage(currMessage);
    }
    // if receive swap order command, swap player order values
    else if (netConnect.netState == s15_hold_SWAPOR) {
      currPlayer = !currPlayer;
      currMessage = currPlayer + 1;
      netConnect.netState = s11_wait_user_move;
      DrawStatusMessage(currMessage);
    }
    // otherwise, check for touch on screen
    else
      
#endif

      if (touchscreen_GetCoord() == 0) {
	switch (touchResult = TouchCheck()) {
	case SELECT_ABOUT:
	  DrawAboutBox();
	  usleep (500000);
	  while (touchscreen_GetCoord()) {}
	  DrawBoard (board, currColor);
	  break;
	  
	case SELECT_NEW_GAME:
	  
#ifdef NETWORK_SERIAL
	  // new game command not allowed during network game
	  if (netConnect.netState <= s4_idle) {
#endif
	    ClearBoard (board, win);
	    turnNum = 1;
	    currPlayer = startPlayer;
	    currMessage = currPlayer + 1;
	    currColor = NORMAL_COLOR;
	    gameOver = 0;
#ifdef NETWORK_SERIAL
	    // if network initialized, request opponent selection
	    if (netConnect.netState == s4_idle)	{
	      if (SelectOpponent(currMessage)) {
		DrawBoard(board, currColor);
		startPlayer = 0;
		currPlayer = startPlayer;
		NegotiateNewGame(currMessage, players);
	      }
	      else
		netConnect.netState = s4_idle;
	      DrawBoard (board, currColor);
	    }
	  }
	  else {
	    DrawStatusMessage(NETWORK_GAME_IN_PROGRESS);
	    usleep (500000);
	  }
#endif
	  break;
	  
	case SELECT_CLEAR_SCORES:
	  ClearScores (players);
	  break;
	  
	case SELECT_SWAP_PIECE:
	  if(turnNum == 1) {
	    temp =	players[0].piece;
	    players[0].piece = players[1].piece;
	    players[1].piece = temp;
	    DrawScores(players);
	  }
	  else {
	    DrawStatusMessage(INVALID_TIME);
	    usleep (1000000);
	  }
	  break;

	case SELECT_SWAP_ORDER:
	  if(turnNum == 1) {
#ifdef NETWORK_SERIAL
	    if(netConnect.netState == s11_wait_user_move) {
	      NotifyOrderSwap(&currPlayer);
	      usleep (1000000);
	      DrawBoard (board, currColor);
	    }
	    else if (netConnect.netState != s13_wait_SETPOS)
#endif
	      {
		startPlayer = !startPlayer;
		currPlayer = startPlayer;
	      }
	    currMessage = currPlayer + 1;
	  }
	  else {
	    DrawStatusMessage(INVALID_TIME);
	    usleep (1000000);
	  }
	  break;
#ifdef NETWORK_SERIAL
	case SELECT_NETWORK_GAME:
	  if ((!netConnect.netState) ||
	      ((netConnect.netState) && ConfirmReinitialize(currMessage)))
	    InitNetwork(players, currMessage);
	  DrawBoard (board, currColor);
	  break;
#endif
	case TOP_LEFT:
	case TOP_MID:
	case TOP_RIGHT:
	case MID_LEFT:
	case MID_MID:
	case MID_RIGHT:
	case BOT_LEFT:
	case BOT_MID:
	case BOT_RIGHT:
#ifdef NETWORK_SERIAL
	  if ((netConnect.netState == s0_no_connect) ||
	      (netConnect.netState == s11_wait_user_move)) {
#endif
	    if ((!gameOver) &&
		(board[touchResult-1] == EMPTY_SPACE)) {
	      board[touchResult-1] = players[currPlayer].piece;
	      DrawXO (touchResult, players[currPlayer].piece, NORMAL_COLOR);
	      currMessage = (!currPlayer) + 1;
#ifdef NETWORK_SERIAL
	      if(netConnect.netState == s11_wait_user_move) {
		netConnect.currMove = touchResult;
		netConnect.netState = s12_send_SETPOS;
		while (netConnect.netState != s13_wait_SETPOS) {}
	      }
#endif
	      if (CheckForWin (board, win)) {
		ProcessWin (board, players, currPlayer, &currMessage, win, &gameOver);
#ifdef NETWORK_SERIAL
		if(netConnect.netState == s13_wait_SETPOS) {
		  netConnect.currMove = 0;
		  netConnect.opponentNum = 0;
		  netConnect.netState = s4_idle;
		}
#endif
	      }
	      else if (turnNum == MAX_TURNS) {
		ProcessTie(board, players, &currMessage, &currColor, win, &gameOver);
#ifdef NETWORK_SERIAL
		if(netConnect.netState == s13_wait_SETPOS) {
		  netConnect.currMove = 0;
		  netConnect.opponentNum = 0;
		  netConnect.netState = s4_idle;
		}
#endif
	      }
	      
	      currPlayer = !currPlayer;
	      turnNum++;
	    }
	    else {
	      DrawStatusMessage(INVALID_SELECTION);
	      usleep (500000);
	    }
#ifdef NETWORK_SERIAL
	  }
	  else if (netConnect.netState > s11_wait_user_move) {
	    DrawStatusMessage(NOT_YOUR_TURN);
	    usleep(500000);
	  }
	  else {
	    DrawStatusMessage(INVALID_SELECTION);
	    usleep (500000);
	  }
#endif
	  break;
	default:
	  DrawStatusMessage(INVALID_SELECTION);
	  usleep (500000);
	}
	usleep (200000);
	DrawStatusMessage(currMessage);
      }
  }
  return 0;
  
#ifdef NETWORK_SERIAL
  // ending function calls (that are never called but are here for completeness)
  XTime_PITDisableInterrupt();
  XExc_mDisableExceptions(XEXC_ALL);
  XExc_RemoveHandler(XEXC_ID_PIT_INT);
#endif

}
