/*-----------------------------------------------------------------------------
//     $Date: 2003/05/30 18:49:11 $
//     $RCSfile: xrom.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/

#include "globaltypes.h"
#include "version.h"
#include "xrom.h"
#include "xromlog.h"
#include "xrominput.h"
#include "xromtests.h"
#include "xromled.h"
#include "xromcache.h"
#include "xromiic.h"

#define HEADER_STRING    "XROM for Virtex-II Pro\r\n"

MenuProc printMenu;
InputProc handleInput;
bool doPrintMenu;
static bool runOutOfCache;

typedef struct MenuStackEntry
{
  MenuProc menu;
  InputProc input;
} MenuStackEntry;

struct MenuStackEntry MenuStack[5];
int menuStackPtr = 0;

void PushMenu(MenuProc menu, InputProc input)
{
  /*  assert(menuStackPtr < 5); */
  MenuStack[menuStackPtr].menu = menu;
  MenuStack[menuStackPtr].input = input;
  menuStackPtr++;
}

void PopMenu(MenuProc *menu, InputProc *input)
{
  /*  assert(menuStackPtr > 0); */
  menuStackPtr--;
  *menu = MenuStack[menuStackPtr].menu;
  *input = MenuStack[menuStackPtr].input;
}

static void XromMenu(void)
{
#if SIM==0
  XromLogLn();
  XromLogString(HR);
  XromLogString(HEADER_STRING);
  XromLogString(VERSION_STRING);
  if (runOutOfCache)
    XromLogString("Running out of Caches.\r\n");
  XromLogString(HR);
  XromLogLn();
#endif

  XromLogString("  1. Tests\r\n");
#if SIM==0
  XromLogString("  2. Communication settings\r\n");
#endif
  if (!runOutOfCache)
    XromLogString("  9. Cache settings\r\n"); 
}

static void XromInput(char ch)
{
  switch(ch)
  {
  case '1':
    printMenu = TestsMenu;
    handleInput = TestsInput;
    PushMenu(XromMenu, XromInput);
    doPrintMenu = true;
    break;
#if SIM==0
  case '2':
    printMenu = XromLogMenu;
    handleInput = XromLogInput;
    PushMenu(XromMenu, XromInput);
    doPrintMenu = true;
    break;
#endif
  case '9':
    if (!runOutOfCache)
    {
      printMenu = XromCacheMenu;
      handleInput = XromCacheInput;
      PushMenu(XromMenu, XromInput);
      doPrintMenu = true;
      break;
    }
  default:
    doPrintMenu = true;
    break;
  } 
}


int main()
{
  XromLogInit();
  XromLedInit();
  XromCacheInit();
  XromIicInit();

  printMenu = XromMenu;
  handleInput = XromInput;
  doPrintMenu = true;

  /* Test whether the program is run out of cache.
     We assume that the data cache is mapped at 0x78000000
     and that instruction and data caches are used.
  */
  runOutOfCache = (((unsigned int) &runOutOfCache) >= 0x78000000) &&
      (((unsigned int) &runOutOfCache) < (0x78000000+16*1024));

  for(;;)
  {
    if(doPrintMenu)
    {
      printMenu();
      doPrintMenu = false;
    }
    
    XromLedToggle();
    InputGetMenu();
  }

  return 0;
}
