/*-----------------------------------------------------------------------------
//     $Date: 2003/05/29 16:25:42 $
//     $RCSfile: xromlog.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/

#include "xparameters.h"
#include "xuartns550_l.h"
#include "xromlog.h"
#include "xrom.h"

#if SIM==1
/* set the UART divisor latch to 3 for the simulation */
static unsigned int baudRates[] = {XPAR_XUARTNS550_CLOCK_HZ / 16 / 3};
#define DEFAULTBAUDRATE      0
#else
static unsigned int baudRates[] =
{
    300,
    600,
    1200,
    2400,
    4800,
    9600,
    19200,
    38400,
    57600,
    115200
};

#define DEFAULTBAUDRATE      5
#endif
#define MAXBAUDRATES      (sizeof(baudRates) / sizeof(unsigned int))

unsigned int UartBaseAdr;
unsigned int UartBaud;
static unsigned int NewUartBaseAdr;
static unsigned int NewUartBaud;

static void putc(char c)
{
  XUartNs550_SendByte(UartBaseAdr, c);
}


void XromLogString(char *s)
{
  for(;*s;s++)
    putc(*s);
}

void XromLogLn(void)
{
  XromLogString("\r\n");
}

static void LogNum(unsigned int x, unsigned int base)
{
  static char hex[]="0123456789ABCDEF";
  char digit[10];
  int i;

  i = 0;
  do
  {
    digit[i] = hex[x % base];
    x = x / base;
    i++;
  } while (x != 0);

  while (i > 0)
  {
    i--;
    putc(digit[i]);
  }
}
  
void XromLogHex(unsigned int x)
{
  XromLogString("0x");
  LogNum(x, 16);
}

void XromLogInt(int x)
{
  unsigned int val;
  
  if (x < 0)
  {
    putc('-');
    val = ((unsigned int) ~x ) + 1;
  }
  else
    val = (unsigned int) x;

  LogNum(val, 10);
}

void XromLogInit(void)
{
  UartBaseAdr = XPAR_OPB_UART16550_0_BASEADDR;
  UartBaud = DEFAULTBAUDRATE;
  NewUartBaseAdr = UartBaseAdr;
  NewUartBaud = UartBaud;
  XUartNs550_SetBaud(UartBaseAdr, XPAR_XUARTNS550_CLOCK_HZ,
                     baudRates[UartBaud]);
  XUartNs550_mSetLineControlReg(UartBaseAdr, XUN_LCR_8_DATA_BITS);
}

void XromLogMenu()
{
  XromLogString("\r\n");

  XromLogString("Main.Communication Settings\r\n");
  XromLogString("  2. Select baud rate: ");
  XromLogInt(baudRates[NewUartBaud]);
  XromLogLn();
  XromLogString("  0. Set and Exit to Main Menu\r\n");
}

void XromLogInput(char ch)
{
  switch(ch)
  {
  case '2':
    NewUartBaud = (NewUartBaud + 1) % MAXBAUDRATES;
    doPrintMenu = true;
    break;
  case '0':
    UartBaseAdr = NewUartBaseAdr;
    UartBaud = NewUartBaud;
    XUartNs550_SetBaud(UartBaseAdr, XPAR_XUARTNS550_CLOCK_HZ,
                       baudRates[UartBaud]);
    XUartNs550_mSetLineControlReg(UartBaseAdr, XUN_LCR_8_DATA_BITS);
    PopMenu(&printMenu, &handleInput);
    doPrintMenu = true;
    break;
  case 0x1b:
    NewUartBaseAdr = UartBaseAdr;
    NewUartBaud = UartBaud;
    PopMenu(&printMenu, &handleInput);
    doPrintMenu = true;
    break;
  default:
    doPrintMenu = true;
    break;
  } 
}
