/*-----------------------------------------------------------------------------
//     $Date: 2004/01/13 19:57:52 $
//     $RCSfile: xromtestram.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/

#include "globaltypes.h"
#include "xromtestram.h"
#include "xromlog.h"
#include "xromled.h"
#include "xromabort.h"
#include "sleep.h"
#include "xpseudo_asm.h"

unsigned int code4[] =
{
  0x3863fffb, /* 0:   38 63 ff fb     addi    r3,r3,-5  */
  0x7c8400d0, /* 4:   7c 84 00 d0     neg     r4,r4     */
  0x7c632214, /* 8:   7c 63 22 14     add     r3,r3,r4  */
  0x4e800020  /* c:   4e 80 00 20     blr               */
};

static bool compError(unsigned int adr, unsigned int exp, unsigned int got,
                      unsigned int *errors, unsigned int maxerrors)
{
  (*errors)++;
  XromLogString("error; adr: ");
  XromLogHex(adr);
  XromLogString(", expected: ");
  XromLogHex(exp);
  XromLogString(", got: ");
  XromLogHex(got);
  XromLogLn();

  return (maxerrors && (*errors >= maxerrors));
}

static bool TestRamBusError(void)
{
  unsigned int val;

  val = 0;        // replace this line and check the bus error registers
  if (val != 0)
  {
    //XromLogBusError();
    XromLogString("error; bus error\r\n");
  }

  return false;
}


static bool TestRamWrite(
  unsigned int startAdr,
  unsigned int size,
  unsigned char startVal,
  unsigned char incr,
  unsigned char tsize,
  unsigned int *errors,
  unsigned int maxerrors)
{
  unsigned int i, j;
  unsigned int val;
  
  if (maxerrors && (*errors >= maxerrors))
    return false;
  
  /* write phase */
  i = 0;
  while (i < size)
  {
    val = 0;
    for(j=0; j < tsize; j++)
    {
      val = (val << 8) | startVal;
      startVal += incr;
    }
    switch(tsize)
    {
    case 1:
      *(volatile unsigned char *) startAdr = (unsigned char) val;
      startAdr++;
      break;
    case 2:
      *(volatile unsigned short *) startAdr = (unsigned short) val;
      startAdr += 2;
      break;
    case 4:
      *(volatile unsigned int *) startAdr = val;
      startAdr += 4;
      break;
    }
    i++;
    XromLedToggle();
    if (((i & 1023) == 0) && AbortByKeyboard())
      return true;
  }

  TestRamBusError();
  return false;
}

static bool TestRamCompare(
  unsigned int startAdr,
  unsigned int size,
  unsigned char startVal,
  unsigned char incr,
  unsigned char tsize,
  unsigned int *errors,
  unsigned int maxerrors)
{
  unsigned int i, j;
  unsigned int val;
  unsigned int getVal=0;
  unsigned int curAdr;
  
  /* compare phase */
  i = 0;
  while (i < size)
  {
    val = 0;
    for(j=0; j < tsize; j++)
    {
      val = (val << 8) | startVal;
      startVal += incr;
    }

    curAdr = startAdr;
    switch(tsize)
    {
    case 1:
      getVal = (unsigned int) *(volatile unsigned char *) startAdr;
      startAdr++;
      break;
    case 2:
      getVal = (unsigned int) *(volatile unsigned short *) startAdr;
      startAdr += 2;
      break;
    case 4:
      getVal = *(volatile unsigned int *) startAdr;
      startAdr += 4;
      break;
    }

    if (getVal != val)
      if (compError(curAdr, val, getVal, errors, maxerrors))
        return false;

    i++;
    XromLedToggle();
    if (((i & 1023) == 0) && AbortByKeyboard())
      return true;
  }

  TestRamBusError();
  return false;
}

static bool TestRamReadByte(
  unsigned int startAdr,
  unsigned int size,
  unsigned int *errors,
  unsigned int maxerrors)
{
  unsigned int i;
  unsigned char val;
  
  if (maxerrors && (*errors >= maxerrors))
    return false;
  
  /* read phase */
  i = 0;
  while (i < size)
  {
    val = *(volatile unsigned char *) startAdr;
    i++;
    startAdr++;
    XromLedToggle();
    if (((i & 1023) == 0) && AbortByKeyboard())
      return true;
  }

  TestRamBusError();
  return false;
}

static bool TestRamWalkByte(
  unsigned int startAdr,
  unsigned int size,
  bool one,   /* true for walking ones, false for walking zeros */
  unsigned int *errors,
  unsigned int maxErrors)
{
  unsigned int i, j;
  unsigned char byte;
  unsigned char bit;
  unsigned char initVal;
  unsigned char bitVal;
  
  if (maxErrors && (*errors >= maxErrors))
    return false;
  
  if (one)
    initVal = 0x0;
  else
    initVal = 0xff;

  i = 0;
  while (i < size * 8)
  {
    if(TestRamWrite(startAdr, size, initVal, 0x0, 1, errors, maxErrors))
      return true;
    byte = i / 8;
    bit = i % 8;
    bitVal = 1 << bit;
    if (!one)
      bitVal = ~bitVal;
    
    if(TestRamWrite(startAdr+byte, 1, bitVal, 0x0, 1, errors, maxErrors))
      return true;

    /* compare */
    j = 0;
    while(j < byte)
    {
      if (TestRamCompare(startAdr+j, 1, initVal, 0, 1, errors, maxErrors))
        return true;
      j++;
    }
    if (TestRamCompare(startAdr+j, 1, bitVal, 0, 1, errors, maxErrors))
      return true;
    j++;
    while (j < size)
    {
      if (TestRamCompare(startAdr+j, 1, initVal, 0, 1, errors, maxErrors))
        return true;
      j++;
    }

    if (maxErrors && (*errors >= maxErrors))
      return false;

    i++;
    XromLedToggle();
    if (((i & 1023) == 0) && AbortByKeyboard())
      return true;
  }
  return false;
}

static unsigned int CalcWalkAddress(unsigned int startAdr, unsigned int size,
                                    unsigned int adr, bool one)
{

  if (one)
    return (startAdr + ((~adr) % size));
  else
    return (startAdr + adr);
}

static bool TestRamWalkAddress(
  unsigned int startAdr,
  unsigned int size,
  bool one,   /* true for walking ones, false for walking zeros */
  unsigned int *errors,
  unsigned int maxErrors)
{
  unsigned int adri, adrj;
  unsigned int adr;
  
  if (maxErrors && (*errors >= maxErrors))
    return false;
  
  adri = 1;
  while (adri < size)
  {
    if(TestRamWrite(startAdr, size, 0, 0x0, 1, errors, maxErrors))
      return true;

    adr = CalcWalkAddress(startAdr, size, adri, one);
    if(TestRamWrite(adr, 1, 0xff, 0x0, 1, errors, maxErrors))
      return true;

    /* compare */
    adrj = 1;
    while(adrj < adri)
    {
      adr = CalcWalkAddress(startAdr, size, adrj, one);
      if (TestRamCompare(adr, 1, 0, 0, 1, errors, maxErrors))
        return true;
      adrj *= 2;
    }
    adr = CalcWalkAddress(startAdr, size, adrj, one);
    if (TestRamCompare(adr, 1, 0xff, 0, 1, errors, maxErrors))
      return true;
    adrj *= 2;
    while (adrj < size)
    {
      adr = CalcWalkAddress(startAdr, size, adrj, one);
      if (TestRamCompare(adr, 1, 0, 0, 1, errors, maxErrors))
        return true;
      adrj *= 2;
    }

    if (maxErrors && (*errors >= maxErrors))
      return false;

    adri *= 2;
    XromLedToggle();

    if (AbortByKeyboard())
      return true;
  }
  return false;
}

static bool ExecuteCode4(unsigned int startAdr,
                         unsigned int *errors,
                         unsigned int maxErrors)
{
  unsigned int i, j;
  unsigned int got;
  unsigned int exp;
  unsigned int (*f)(unsigned int, unsigned int);

  if (maxErrors && (*errors >= maxErrors))
    return false;

  for(i = 0; i < 4; i++)
  {
    for (j = 0; j < 4; j++)
      *(volatile unsigned int*) (startAdr+j*4) = code4[j];

    sync;
    isync;
    f = (void *) startAdr;
    got = f(startAdr, i);
    exp = (startAdr - 5) + (-i);
    if (exp != got)
      if (compError(startAdr, exp, got, errors, maxErrors))
        return false;

    startAdr += 9*4;
  }
  return false;
}

void TestRam(
  unsigned int startAdr,
  unsigned int size,
  unsigned int maxErrors,
  unsigned int sleepTime)
{
  unsigned int errors;
  unsigned int i;

  errors = 0;

  /**************************************************************************/
  /* do single byte/half-word/word accesses                                 */
  /**************************************************************************/

  XromLogString("  \r\n* Single byte/half-word/word accesses\r\n");

  TestRamBusError();
  
  /* read only test */
  XromLogString("  Testing for memory availability\r\n");
  if (TestRamReadByte(startAdr, size, &errors, maxErrors))
    return;

  /* test for stuck at 1 errors; write all zeros and compare */
  XromLogString("  Testing for data lines stuck at one\r\n");
  if (TestRamWrite(startAdr, 8, 0x0, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamCompare(startAdr, 8, 0x0, 0x0, 1, &errors, maxErrors))
    return;

  /* test for stuck at 0 errors; write all ones and compare */
  XromLogString("  Testing for data lines stuck at zero\r\n");
  if (TestRamWrite(startAdr, 8, 0xff, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamCompare(startAdr, 8, 0xff, 0x0, 1, &errors, maxErrors))
    return;

  /* test for shorts in adjacent data lines; do walking ones */
  XromLogString("  Testing for walking one data errors\r\n");
  if (TestRamWalkByte(startAdr, 8, true, &errors, maxErrors))
    return;

  /* test for shorts in adjacent data lines; do walking zeros */
  XromLogString("  Testing for walking zero data errors\r\n");
  if (TestRamWalkByte(startAdr, 8, false, &errors, maxErrors))
    return;

  /* test for address lines stuck at 1 errors */
  /* set memory to zero, write 0xff to zero, find change at other address */
  XromLogString("  Testing for address lines stuck at one\r\n");
  if (TestRamWrite(startAdr, size, 0x0, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamWrite(startAdr, 1, 0xff, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamCompare(startAdr, 1, 0xff, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamCompare(startAdr+1, size-1, 0x0, 0x0, 1, &errors, maxErrors))
    return;

  /* test for address lines stuck at 0 errors */
  /* set memory to zero, write 0xff to size-1, find change at other address */
  XromLogString("  Testing for address lines stuck at zero\r\n");
  if (TestRamWrite(startAdr, size, 0x0, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamWrite(startAdr+size-1, 1, 0xff, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamCompare(startAdr, size-1, 0x0, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamCompare(startAdr+size-1, 1, 0xff, 0x0, 1, &errors, maxErrors))
    return;

  /* test for shorts in adjacent address lines; do walking ones */
  XromLogString("  Testing for walking one address errors\r\n");
  if (TestRamWalkAddress(startAdr, size, true, &errors, maxErrors))
    return;

  /* test for shorts in adjacent address lines; do walking zeros */
  XromLogString("  Testing for walking zero address errors\r\n");
  if (TestRamWalkAddress(startAdr, size, false, &errors, maxErrors))
    return;

  /* test for pseudo-random data */
  XromLogString("  Testing for pseudo-random data errors\r\n");
  if (TestRamWrite(startAdr, size, 0x0, 0x19, 1, &errors, maxErrors))
    return;
  if (TestRamCompare(startAdr, size, 0x0, 0x19, 1, &errors, maxErrors))
    return;

  /* test for refresh errors */
#if SIM==0
  if (sleepTime)
  {
    XromLogString("  Testing for refresh errors\r\n    sleeping...");
    sleep(sleepTime);
    XromLogString("comparing\r\n");
    if (TestRamCompare(startAdr, size, 0x0, 0x19, 1, &errors, maxErrors))
      return;
  }
#endif

  /* test for writing bytes and reading halfwords */
  XromLogString("  Testing for writing bytes / reading halfwords\r\n");
  if (TestRamWrite(startAdr, 9, 0x2, 0x27, 1, &errors, maxErrors))
    return;
  for(i=0; i < 2; i++)
  {
    XromLogString("    comparing with offset ");
    XromLogInt(i);
    XromLogLn();
    if (TestRamCompare(startAdr+i, 4, i*0x27 + 0x2, 0x27, 2, &errors, maxErrors))
      return;
  }

  /* test for writing bytes and reading words */
  XromLogString("  Testing for writing bytes / reading words\r\n");
  if (TestRamWrite(startAdr, 11, 0xb7, 0x5, 1, &errors, maxErrors))
    return;
  for(i=0; i < 4; i++)
  {
    XromLogString("    comparing with offset ");
    XromLogInt(i);
    XromLogLn();
    if (TestRamCompare(startAdr+i, 2, i*0x5 + 0xb7, 0x5, 4, &errors, maxErrors))
      return;
  }

  /* test for writing halfwords and reading bytes */
  XromLogString("  Testing for writing halfwords / reading bytes\r\n");
  for(i=0; i < 2; i++)
  {
    XromLogString("    comparing with offset ");
    XromLogInt(i);
    XromLogLn();
    if (TestRamWrite(startAdr+i, 4, i*0x53, 5*i+0x8b, 2, &errors, maxErrors))
      return;
    if (TestRamCompare(startAdr+i, 8, i*0x53, 5*i+0x8b, 1, &errors, maxErrors))
      return;
  }

  /* test for writing words and reading bytes */
  XromLogString("  Testing for writing words / reading bytes\r\n");
  for(i=0; i < 4; i++)
  {
    XromLogString("    comparing with offset ");
    XromLogInt(i);
    XromLogLn();
    if (TestRamWrite(startAdr+i, 2, i*0x2a, 0x65*i+0x44, 4, &errors, maxErrors))
      return;
    if (TestRamCompare(startAdr+i, 8, i*0x2a, 0x65*i+0x44, 1, &errors, maxErrors))
      return;
  }

  /**************************************************************************/
  /* do four-word transactions                                              */
  /**************************************************************************/

  XromLogString("  \r\n* Four-word transactions\r\n");

  /* Execute code */
  XromLogString("  Execute code\r\n");
  if (ExecuteCode4(startAdr, &errors, maxErrors))
    return;


  XromLogString("\r\nRAM test completed; errors: ");
  XromLogInt(errors);
  XromLogLn();
}




