/*-----------------------------------------------------------------------------
//     $Date: 2004/03/10 23:35:36 $
//     $RCSfile: xrom.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/

#include "globaltypes.h"
#include "version.h"
#include "xuartns550_l.h"
#include "xtft.h"
#include "xrom.h"
#include "xromlog.h"
#include "xrominput.h"
#include "xromtests.h"
#include "xromled.h"
#include "xromcache.h"
#include "xrompci.h"
#include "xromtft.h"
#include "xromps2.h"
#include "xromtouchscreen.h"
#include "xparameters.h"

#define HEADER_STRING    "XROM for Virtex-II Pro\r\n"

MenuProc printMenu;
InputProc handleInput;
bool doPrintMenu;

typedef struct MenuStackEntry
{
  MenuProc menu;
  InputProc input;
} MenuStackEntry;

struct MenuStackEntry MenuStack[5];
int menuStackPtr = 0;

void PushMenu(MenuProc menu, InputProc input)
{
  /*  assert(menuStackPtr < 5); */
  MenuStack[menuStackPtr].menu = menu;
  MenuStack[menuStackPtr].input = input;
  menuStackPtr++;
}

void PopMenu(MenuProc *menu, InputProc *input)
{
  /*  assert(menuStackPtr > 0); */
  menuStackPtr--;
  *menu = MenuStack[menuStackPtr].menu;
  *input = MenuStack[menuStackPtr].input;
}

static void XromMenu(void)
{
#ifndef SPEED_UP
  XromLogLn();
  XromLogString(HR);
  XromLogString(HEADER_STRING);  XromLogString(VERSION_STRING);
  XromLogString(HR);
  XromLogLn();
#endif

  XromLogString("  1. Tests\r\n");
#ifndef V2PDK
  XromLogString("  2. Communication settings\r\n");
#endif
  XromLogString("  9. Cache settings\r\n");
}

static void XromInput(char ch)
{
  switch(ch)
  {
  case '1':
    printMenu = TestsMenu;
    handleInput = TestsInput;
    PushMenu(XromMenu, XromInput);
    doPrintMenu = true;
    break;
#ifndef V2PDK
  case '2':
    printMenu = XromLogMenu;
    handleInput = XromLogInput;
    PushMenu(XromMenu, XromInput);
    doPrintMenu = true;
    break;
#endif
  case '9':
    printMenu = XromCacheMenu;
    handleInput = XromCacheInput;
    PushMenu(XromMenu, XromInput);
    doPrintMenu = true;
    break;
  default:
    doPrintMenu = true;
    break;
  }
}


int main()
{
  XromLogInit();
  XromLedInit();
  XromCacheInit();
  XromPciInit();
  XromTftInit();
  XromPs2Init();
  XromTouchscreenInit();

  printMenu = XromMenu;
  handleInput = XromInput;
  doPrintMenu = true;

  for(;;)
  {
    if(doPrintMenu)
    {
      printMenu();
      doPrintMenu = false;
    }

    XromLedToggle();
    InputGetMenu();
  }

  return 0;
}
