/*-----------------------------------------------------------------------------
//     $Date: 2004/01/13 23:12:55 $
//     $RCSfile: xrompci.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/

#include <stdio.h>
#include "xuartns550_l.h"
#include "xpci_l.h"
#include "xromlog.h"
#include "xparameters.h"

#define PCI_BASEADDR  XPAR_OPB_PCI_REF_0_BASEADDR

/* from V2PDK */
#define PCI_CTRL_OFFSET        0x1E000004
#define PCI_CFG_VAL            0xffff1047
// redefine two constants in xpci_l.h so that XPci_mConfigIn works
#undef XPCI_CAR_OFFSET
#undef XPCI_CDR_OFFSET
#define XPCI_CAR_OFFSET        0x1C000000
#define XPCI_CDR_OFFSET        0x1C000004

#define MAX_NUM_BUSSES         1
#define MAX_NUM_DEVICES        32

/* local prototypes */
static void pci_dump_device(int busDevFunc);

static void pci_dump_device(int busDevFunc)
{
  unsigned int bus;
  unsigned int device;
  unsigned int value;
  int i;

  XromLogString("*********************************\r\n");
  XromLogString("Dumping raw data from PCI Config Space:\r\n");

  for (i = 0; i < 16; i++) {
    XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (i << 2), value);
    XromLogString("Address =  ");
    XromLogHex(i);
    XromLogString("  Data =  ");
    XromLogHex(value);
    XromLogLn();
  }
  XromLogString("*********************************\r\n");

  bus = (busDevFunc >> 16) & 0x1f;
  device = (busDevFunc >> 11) & 0x1f;

  XromLogString("Bus: ");
  XromLogInt(bus);
  XromLogString(", Device: ");
  XromLogInt(device);
  XromLogLn();
  XromLogString("*********************************\r\n");
  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | 0x0, value);
  XromLogString("Vendor ID    = ");
  XromLogHex(value & 0xffff);
  XromLogLn();

  XromLogString("Device ID    = ");
  XromLogHex((value >> 16) & 0xffff);
  XromLogLn();

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x2 << 2), value);
  XromLogString("Class code   = ");
  XromLogHex((value >> 8) & 0xffffff);
  XromLogLn();

  XromLogString("Rev ID   = ");
  XromLogHex(value & 0xff);
  XromLogLn();

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x3 << 2), value);
  XromLogString("Header Type = ");
  XromLogHex((value >> 16) & 0xff);
  XromLogLn();

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x4 << 2), value);
  XromLogString("Base Address 0 = ");
  XromLogHex(value);
  XromLogLn();

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x5 << 2), value);
  XromLogString("Base Address 1 = ");
  XromLogHex(value);
  XromLogLn();

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x6 << 2), value);
  XromLogString("Base Address 2 = ");
  XromLogHex(value);
  XromLogLn();

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x7 << 2), value);
  XromLogString("Base Address 3 = ");
  XromLogHex(value);
  XromLogLn();

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x8 << 2), value);
  XromLogString("Base Address 4 = ");
  XromLogHex(value);
  XromLogLn();

  XPci_mConfigIn(PCI_BASEADDR, busDevFunc | (0x9 << 2), value);
  XromLogString("Base Address 5 = ");
  XromLogHex(value);
  XromLogLn();
 }

void XromPciInit()
{
  /* Initialize PCI by self-configuration */
  XPci_mWriteReg(PCI_BASEADDR, PCI_CTRL_OFFSET, PCI_CFG_VAL);
}

void XromTestPci()
{
  int busDevFunc;
  int devices;
  int value;     // value read from PCI bus
  int i, j; 

  busDevFunc = 0;
  devices = 0;
  value = -1;

  // scan devices
  for (i=0; i<MAX_NUM_BUSSES; i++) {
    for (j=0; j<MAX_NUM_DEVICES; j++) {
      // bus number
      busDevFunc = (i << 16);
      // device number and set enable bit
      busDevFunc |= (j << 11) | 0x80000000;
      // scan bus for device
      XPci_mConfigIn(PCI_BASEADDR, busDevFunc, value);
      // if value == -1, then no device
      if (value != -1) {
	devices++; // increment total;
	pci_dump_device(busDevFunc);
      }
    }
  }

  XromLogString("\r\nNumber of PCI devices found while scanning: ");
  XromLogInt(devices);
  XromLogLn();
  XromLogLn();
  XromLogString("Press Esc key to continue\r\n");
  XromLogLn();

}


