/*-----------------------------------------------------------------------------
//     $Date: 2004/01/15 14:26:02 $
//     $RCSfile: xromps2.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/

#include "xromlog.h"
#include "xromps2.h"
#include "xps2_l.h"
#include "xps2.h"
#include "xromabort.h"
#include "xrominput.h"
#include "xparameters.h"

#define PS2_1_DEVICE_ID      XPAR_OPB_PS2_DUAL_REF_0_DEVICE_ID_0
#define PS2_1_INTERRUPT_ID   XPAR_DCR_INTC_0_OPB_PS2_DUAL_REF_0_SYS_INTR1_INTR

#define ESCAPE_SCAN_CODE    0x76    /* scan code for escape character */

static XPs2 Ps2_1;

static void XromPs2TestPort(unsigned int baseAdr)
{
  unsigned char c, status;

  XromLogString("  Resetting device and waiting for keys\r\n");
  XPs2_SendByte(baseAdr, 0xff);
  c = 0;
  do {
    // check if there's something to receive
    status = XPs2_mGetStatus(baseAdr);
    if (status & 0x1) {
      c = XPs2_RecvByte(baseAdr);
      XromLogHex(c);
      XromLogString("  ");
    }
  } while (!AbortByKeyboard() && (c != ESCAPE_SCAN_CODE));
  XromLogLn();
}


void XromPs2Init(void)
{
  XPs2_Initialize(&Ps2_1, PS2_1_DEVICE_ID);  // initialize port 1
}

void XromTestPs2(void)
{
  XromLogString("  Plug in Keyboard to PS/2 port nearest 10/100\r\n");
  XromLogString("    Press a key on terminal keyboard when ready (not PS/2)\r\n");
  for(;;)
  {
    if (InputAvailable())
    {
      InputGetChar();
      break;
    }
  }
  XromLogString("    Testing Port 1\r\n");
  XromLogString("      Press ESC on either PS/2 Keyboard or terminal keyboard to continue\r\n");
  XromPs2TestPort(Ps2_1.BaseAddress);
  XromLogLn();

  XromLogString("  Plug in Keyboard to PS/2 port nearest Serial DB-9\r\n");
  XromLogString("    Press a key on terminal keyboard when ready (not PS/2)\r\n");
  for(;;)
    {
      if (InputAvailable())
	{
	  InputGetChar();
	  break;
	}
    }
  
  XromLogString("    Testing Port 2\r\n");
  XromLogString("      Press ESC on either PS/2 Keyboard or terminal keyboard to continue\r\n");
  // in dual PS/2 core, this is the 2nd PS/2 port's base address
  XromPs2TestPort((Ps2_1.BaseAddress)+0x1000);
  XromLogLn();
  XromLogString("  PS/2 Tests Completed, Press ESC on terminal keyboard to continue\r\n");
}


