/*-----------------------------------------------------------------------------
//     $Date: 2004/03/10 23:35:36 $
//     $RCSfile: xromtestram.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/

#include "globaltypes.h"
#include "xromtestram.h"
#include "xromsmiley.h"
#include "xromlog.h"
#include "xromled.h"
#include "xromabort.h"
#include "xgpio_l.h"
/* No DCR bus error registers in Phase 5
   #include "bexr.h" */
#include "sleep.h"

#define memsize 128*1024*1024
//#define note   // prints general notes of where you are in code
//#define note2  // prints status during inner loop of 16 word inverse noise cycles

unsigned int code4[] =
{
  0x3863fffb, /* 0:   38 63 ff fb     addi    r3,r3,-5  */
  0x7c8400d0, /* 4:   7c 84 00 d0     neg     r4,r4     */
  0x7c632214, /* 8:   7c 63 22 14     add     r3,r3,r4  */
  0x4e800020  /* c:   4e 80 00 20     blr               */
};

static bool compError(unsigned int adr, unsigned int exp, unsigned int got,
                      unsigned int *errors, unsigned int maxerrors)
{
  (*errors)++;
  XromLogString("error; adr: ");
  XromLogHex(adr);
  XromLogString(", expected: ");
  XromLogHex(exp);
  XromLogString(", got: ");
  XromLogHex(got);
  XromLogLn();

  return (maxerrors && (*errors >= maxerrors));
}

/* No DCR bus error registers in Phase 5 design */
/*
  static bool TestRamBusError(void)
  {
  unsigned int val;
  
  val = bexr_Read(BESR_A_REG);
  if (val != 0)
  {
  //    XromLogBusError();
  XromLogString("error; bus error at Addr = ");
  XromLogHex(val);
  XromLogLn();
  }
  
  return false;
  }
*/


static bool TestRamWrite(
  unsigned int startAdr,
  unsigned int size,
  unsigned char startVal,
  unsigned char incr,
  unsigned char tsize,
  unsigned int *errors,
  unsigned int maxerrors)
{
  unsigned int i, j;
  unsigned int val;

  if (maxerrors && (*errors >= maxerrors))
    return false;

  /* write phase */
  i = 0;
  while (i < size)
  {
    val = 0;
    for(j=0; j < tsize; j++)
    {
      val = (val << 8) | startVal;
      startVal += incr;
    }
    switch(tsize)
    {
    case 1:
      *(volatile unsigned char *) startAdr = (unsigned char) val;
      startAdr++;
      break;
    case 2:
      *(volatile unsigned short *) startAdr = (unsigned short) val;
      startAdr += 2;
      break;
    case 4:
      *(volatile unsigned int *) startAdr = val;
      startAdr += 4;
      break;
    }
    i++;
    XromLedToggle();
    if (((i & 1023) == 0) && AbortByKeyboard())
      return true;
  }

  /* No DCR bus error registers in Phase 5
     TestRamBusError(); */
  return false;
}

static bool TestRamCompare(
  unsigned int startAdr,
  unsigned int size,
  unsigned char startVal,
  unsigned char incr,
  unsigned char tsize,
  unsigned int *errors,
  unsigned int maxerrors)
{
  unsigned int i, j;
  unsigned int val;
  unsigned int getVal=0;
  unsigned int curAdr;

  /* compare phase */
  i = 0;
  while (i < size)
  {
    val = 0;
    for(j=0; j < tsize; j++)
    {
      val = (val << 8) | startVal;
      startVal += incr;
    }

    curAdr = startAdr;
    switch(tsize)
    {
    case 1:
      getVal = (unsigned int) *(volatile unsigned char *) startAdr;
      startAdr++;
      break;
    case 2:
      getVal = (unsigned int) *(volatile unsigned short *) startAdr;
      startAdr += 2;
      break;
    case 4:
      getVal = *(volatile unsigned int *) startAdr;
      startAdr += 4;
      break;
    }

    if (getVal != val)
      if (compError(curAdr, val, getVal, errors, maxerrors))
        return false;

    i++;
    XromLedToggle();
    if (((i & 1023) == 0) && AbortByKeyboard())
      return true;
  }

  /* No DCR bus error registers in Phase 5
     TestRamBusError(); */
  return false;
}

static bool TestRamReadByte(
  unsigned int startAdr,
  unsigned int size,
  unsigned int *errors,
  unsigned int maxerrors)
{
  unsigned int i;
  unsigned char val;

  if (maxerrors && (*errors >= maxerrors))
    return false;

  /* read phase */
  i = 0;
  while (i < size)
  {
    val = *(volatile unsigned char *) startAdr;
    i++;
    startAdr++;
    XromLedToggle();
    if (((i & 1023) == 0) && AbortByKeyboard())
      return true;
  }

  /* No DCR bus error registers in Phase 5
     TestRamBusError(); */
  return false;
}

static bool TestRamWalkByte(
  unsigned int startAdr,
  unsigned int size,
  bool one,   /* true for walking ones, false for walking zeros */
  unsigned int *errors,
  unsigned int maxErrors)
{
  unsigned int i, j;
  unsigned char byte;
  unsigned char bit;
  unsigned char initVal;
  unsigned char bitVal;

  if (maxErrors && (*errors >= maxErrors))
    return false;

  if (one)
    initVal = 0x0;
  else
    initVal = 0xff;

  i = 0;
  while (i < size * 8)
  {
    if(TestRamWrite(startAdr, size, initVal, 0x0, 1, errors, maxErrors))
      return true;
    byte = i / 8;
    bit = i % 8;
    bitVal = 1 << bit;
    if (!one)
      bitVal = ~bitVal;

    if(TestRamWrite(startAdr+byte, 1, bitVal, 0x0, 1, errors, maxErrors))
      return true;

    /* compare */
    j = 0;
    while(j < byte)
    {
      if (TestRamCompare(startAdr+j, 1, initVal, 0, 1, errors, maxErrors))
        return true;
      j++;
    }
    if (TestRamCompare(startAdr+j, 1, bitVal, 0, 1, errors, maxErrors))
      return true;
    j++;
    while (j < size)
    {
      if (TestRamCompare(startAdr+j, 1, initVal, 0, 1, errors, maxErrors))
        return true;
      j++;
    }

    if (maxErrors && (*errors >= maxErrors))
      return false;

    i++;
    XromLedToggle();
    if (((i & 1023) == 0) && AbortByKeyboard())
      return true;
  }
  return false;
}

static unsigned int CalcWalkAddress(unsigned int startAdr, unsigned int size,
                                    unsigned int adr, bool one)
{

  if (one)
    return (startAdr + ((~adr) % size));
  else
    return (startAdr + adr);
}

static bool TestRamWalkAddress(
  unsigned int startAdr,
  unsigned int size,
  bool one,   /* true for walking ones, false for walking zeros */
  unsigned int *errors,
  unsigned int maxErrors)
{
  unsigned int adri, adrj;
  unsigned int adr;

  if (maxErrors && (*errors >= maxErrors))
    return false;

  adri = 1;
  while (adri < size)
  {
    if(TestRamWrite(startAdr, size, 0, 0x0, 1, errors, maxErrors))
      return true;

    adr = CalcWalkAddress(startAdr, size, adri, one);
    if(TestRamWrite(adr, 1, 0xff, 0x0, 1, errors, maxErrors))
      return true;

    /* compare */
    adrj = 1;
    while(adrj < adri)
    {
      adr = CalcWalkAddress(startAdr, size, adrj, one);
      if (TestRamCompare(adr, 1, 0, 0, 1, errors, maxErrors))
        return true;
      adrj *= 2;
    }
    adr = CalcWalkAddress(startAdr, size, adrj, one);
    if (TestRamCompare(adr, 1, 0xff, 0, 1, errors, maxErrors))
      return true;
    adrj *= 2;
    while (adrj < size)
    {
      adr = CalcWalkAddress(startAdr, size, adrj, one);
      if (TestRamCompare(adr, 1, 0, 0, 1, errors, maxErrors))
        return true;
      adrj *= 2;
    }

    if (maxErrors && (*errors >= maxErrors))
      return false;

    adri *= 2;
    XromLedToggle();

    if (AbortByKeyboard())
      return true;
  }
  return false;
}

/*
static bool ExecuteCode4(unsigned int startAdr,
                         unsigned int *errors,
                         unsigned int maxErrors)
{
  unsigned int i, j;
  unsigned int got;
  unsigned int exp;
  unsigned int (*f)(unsigned int, unsigned int);

  if (maxErrors && (*errors >= maxErrors))
    return false;

  for(i = 0; i < 4; i++)
  {
    for (j = 0; j < 4; j++)
      *(volatile unsigned int*) (startAdr+j*4) = code4[j];

    f = (void *) startAdr;
    got = f(startAdr, i);
    exp = (startAdr - 5) + (-i);
    if (exp != got)
      if (compError(startAdr, exp, got, errors, maxErrors))
        return false;

    startAdr += 9*4;
  }
  return false;
}
*/
void TestRam(
  unsigned int startAdr,
  unsigned int size,
  unsigned int maxErrors,
  unsigned int sleepTime)
{
  unsigned int errors;
  unsigned int i;

  errors = 0;

  /**************************************************************************/
  /* do single byte/half-word/word accesses                                 */
  /**************************************************************************/

  XromLogString("  \r\n* Single byte/half-word/word accesses\r\n");

  /* No DCR bus error registers in Phase 5
     TestRamBusError(); */

  /* read only test */
  XromLogString("  Testing for memory availability\r\n");
  if (TestRamReadByte(startAdr, size, &errors, maxErrors))
    return;

  /* test for stuck at 1 errors; write all zeros and compare */
  XromLogString("  Testing for data lines stuck at one\r\n");
  if (TestRamWrite(startAdr, 8, 0x0, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamCompare(startAdr, 8, 0x0, 0x0, 1, &errors, maxErrors))
    return;

  /* test for stuck at 0 errors; write all ones and compare */
  XromLogString("  Testing for data lines stuck at zero\r\n");
  if (TestRamWrite(startAdr, 8, 0xff, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamCompare(startAdr, 8, 0xff, 0x0, 1, &errors, maxErrors))
    return;

  /* test for shorts in adjacent data lines; do walking ones */
  XromLogString("  Testing for walking one data errors\r\n");
  if (TestRamWalkByte(startAdr, 8, true, &errors, maxErrors))
    return;

  /* test for shorts in adjacent data lines; do walking zeros */
  XromLogString("  Testing for walking zero data errors\r\n");
  if (TestRamWalkByte(startAdr, 8, false, &errors, maxErrors))
    return;

  /* test for address lines stuck at 1 errors */
  /* set memory to zero, write 0xff to zero, find change at other address */
  XromLogString("  Testing for address lines stuck at one\r\n");
  if (TestRamWrite(startAdr, size, 0x0, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamWrite(startAdr, 1, 0xff, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamCompare(startAdr, 1, 0xff, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamCompare(startAdr+1, size-1, 0x0, 0x0, 1, &errors, maxErrors))
    return;

  /* test for address lines stuck at 0 errors */
  /* set memory to zero, write 0xff to size-1, find change at other address */
  XromLogString("  Testing for address lines stuck at zero\r\n");
  if (TestRamWrite(startAdr, size, 0x0, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamWrite(startAdr+size-1, 1, 0xff, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamCompare(startAdr, size-1, 0x0, 0x0, 1, &errors, maxErrors))
    return;
  if (TestRamCompare(startAdr+size-1, 1, 0xff, 0x0, 1, &errors, maxErrors))
    return;

  /* test for shorts in adjacent address lines; do walking ones */
  XromLogString("  Testing for walking one address errors\r\n");
  if (TestRamWalkAddress(startAdr, size, true, &errors, maxErrors))
    return;

  /* test for shorts in adjacent address lines; do walking zeros */
  XromLogString("  Testing for walking zero address errors\r\n");
  if (TestRamWalkAddress(startAdr, size, false, &errors, maxErrors))
    return;

  /* test for pseudo-random data */
  XromLogString("  Testing for pseudo-random data errors\r\n");
  if (TestRamWrite(startAdr, size, 0x0, 0x19, 1, &errors, maxErrors))
    return;
  if (TestRamCompare(startAdr, size, 0x0, 0x19, 1, &errors, maxErrors))
    return;

  /* test for refresh errors */
#ifndef V2PDK
  if (sleepTime)
  {
    XromLogString("  Testing for refresh errors\r\n    sleeping...");
    sleep(sleepTime);
    XromLogString("comparing\r\n");
    if (TestRamCompare(startAdr, size, 0x0, 0x19, 1, &errors, maxErrors))
      return;
  }
#endif

  /* test for writing bytes and reading halfwords */
  XromLogString("  Testing for writing bytes / reading halfwords\r\n");
  if (TestRamWrite(startAdr, 9, 0x2, 0x27, 1, &errors, maxErrors))
    return;
  for(i=0; i < 2; i++)
  {
    XromLogString("    comparing with offset ");
    XromLogInt(i);
    XromLogLn();
    if (TestRamCompare(startAdr+i, 4, i*0x27 + 0x2, 0x27, 2, &errors, maxErrors))
      return;
  }

  /* test for writing bytes and reading words */
  XromLogString("  Testing for writing bytes / reading words\r\n");
  if (TestRamWrite(startAdr, 11, 0xb7, 0x5, 1, &errors, maxErrors))
    return;
  for(i=0; i < 4; i++)
  {
    XromLogString("    comparing with offset ");
    XromLogInt(i);
    XromLogLn();
    if (TestRamCompare(startAdr+i, 2, i*0x5 + 0xb7, 0x5, 4, &errors, maxErrors))
      return;
  }

  /* test for writing halfwords and reading bytes */
  XromLogString("  Testing for writing halfwords / reading bytes\r\n");
  for(i=0; i < 2; i++)
  {
    XromLogString("    comparing with offset ");
    XromLogInt(i);
    XromLogLn();
    if (TestRamWrite(startAdr+i, 4, i*0x53, 5*i+0x8b, 2, &errors, maxErrors))
      return;
    if (TestRamCompare(startAdr+i, 8, i*0x53, 5*i+0x8b, 1, &errors, maxErrors))
      return;
  }

  /* test for writing words and reading bytes */
  XromLogString("  Testing for writing words / reading bytes\r\n");
  for(i=0; i < 4; i++)
  {
    XromLogString("    comparing with offset ");
    XromLogInt(i);
    XromLogLn();
    if (TestRamWrite(startAdr+i, 2, i*0x2a, 0x65*i+0x44, 4, &errors, maxErrors))
      return;
    if (TestRamCompare(startAdr+i, 8, i*0x2a, 0x65*i+0x44, 1, &errors, maxErrors))
      return;
  }

  /**************************************************************************/
  /* do four-word transactions                                              */
  /**************************************************************************/

 // XromLogString("  \r\n* Four-word transactions\r\n");

  /* Execute code */
 // XromLogString("  Execute code\r\n");
 // if (ExecuteCode4(startAdr, &errors, maxErrors))
 //   return;


  XromLogString("\r\nRAM test completed; errors: ");
  XromLogInt(errors);
  XromLogLn();
  XromLogString("\r\nPress ESC key to continue \r\n");
}


/*********************************************************************
*
* FUNCTION : ddr_mem_test
*
*********************************************************************/
void ddr_mem_test(
  unsigned long startAdr,
  unsigned long size)
{
  unsigned long myAdr;
  unsigned long writeAdr;
  unsigned long data2store_f=0xFFFFFFFF;
//  unsigned long data2store_0=0x00000000;
  unsigned int  error=0;
  int           i=0, j=0, t=0, h=0;
  unsigned long ba, row, col;
  unsigned long dataRead[32];
  int           pass_count=1;

  XromLogString("  Warning:  These tests can take a really long time if caches are disabled\r\n");

  for (pass_count = 1; pass_count < 50000; pass_count++)
  {
    XromLogString("  Pass # ");
    XromLogInt(pass_count);
    XromLogString(", Errors = ");
    XromLogInt(error);
    XromLogString(", Memory Test Begins at Addr ");
    XromLogHex(startAdr);
    XromLogString(", Length = ");
    XromLogHex(size);
    XromLogLn();

  // ---------------------------------------
  //  TEST0 - Write all zeros and verify
  // ---------------------------------------
    XromLogString("    LOG: TEST0 - Write all memory to zero, using word writes and check\r\n");
    XromLogString("                 Writing\r\n");
  // clear range of memory
 
  for(myAdr = startAdr; myAdr < startAdr+size; myAdr=myAdr+4)
    {
        h++;
        if (( h & 100 ) == 0) {
          XromLedToggle(); }
       * (volatile unsigned long *) myAdr = 0x00000000;
    }

    XromLogString("                 Reading\r\n");
  // read it all back
  for(myAdr = startAdr; myAdr < startAdr+size; myAdr=myAdr+4)
    {
    h++;
    if (( h & 100 ) == 0) {
        XromLedToggle(); }
    if (((h & 1023) == 0) && AbortByKeyboard())
      {
        XromLogString("\r\n");
        XromLogString("\r\n");
        XromLogString("DDR SDRAM TEST SUMMARY:  PASSES = ");
        XromLogInt(pass_count);
        XromLogString(", TOTAL NUMBER OF ERRORS = ");
        XromLogInt(error);
        XromLogLn();
        return;
      }
       dataRead[0] = *(volatile unsigned long *) myAdr;

         if (dataRead[0] != 0x00000000)
            {
              error = error +1;
              XromLogString("  TEST0 - ERROR #");
              XromLogInt(error);
              XromLogString(": Address = ");
              XromLogHex(myAdr);
              XromLogString(", Data Expected was 0x00000000, Data Received was ");
              XromLogHex(dataRead[0]);
              XromLogLn();
            }
//       if (AbortByKeyboard()) return;
     }


  // ---------------------------------------
  //  TEST1 - Write all ones and verify
  // ---------------------------------------

    XromLogString("    LOG: TEST1 - Write all memory to Fs, using word writes and check\r\n");
    XromLogString("                 Writing\r\n");
  // clear range of memory to 0xFFFFFFFF
  for(myAdr = startAdr; myAdr < startAdr+size; myAdr=myAdr+4)
    {
        h++;
        if (( h & 100 ) == 0) {
          XromLedToggle(); }
       * (volatile unsigned long *) myAdr = 0xFFFFFFFF;
    }

    XromLogString("                 Reading\r\n");
  // read it all back
  for(myAdr = startAdr; myAdr < startAdr+size; myAdr=myAdr+4)
    {
    h++;
    if (( h & 100 ) == 0) {
      XromLedToggle(); }
    if (((h & 1023) == 0) && AbortByKeyboard())
      {
        XromLogString("\r\n");
        XromLogString("\r\n");
        XromLogString("DDR SDRAM TEST SUMMARY:  PASSES = ");
        XromLogInt(pass_count);
        XromLogString(", TOTAL NUMBER OF ERRORS = ");
        XromLogInt(error);
        XromLogLn();
        return;
      }
       dataRead[0] = *(volatile unsigned long *) myAdr;

         if (dataRead[0] != 0xFFFFFFFF)
            {
              error = error +1;
              XromLogString("  TEST1 - ERROR #");
              XromLogInt(error);
              XromLogString(": Address = ");
              XromLogHex(myAdr);
              XromLogString(", Data Expected was 0xFFFFFFFF, Data Received was ");
              XromLogHex(dataRead[0]);
              XromLogLn();
            }
//       if (AbortByKeyboard()) break;
     }


  // ---------------------------------------
  //  TEST2 - Test for stuck together row/col bits
  // ---------------------------------------

  XromLogString("    LOG: TEST2 - Testing for stuck together row/col bits \r\n");
    XromLogString("                 Clearing memory to zeros\r\n");
  // clear all of memory
  for(myAdr = startAdr; myAdr < startAdr+size; myAdr=myAdr+4)
    {
       h++;
       if (( h & 100 ) == 0) {
         XromLedToggle(); }
       * (volatile unsigned long *) myAdr = 0x00000000;
    }

  writeAdr = startAdr;

    XromLogString("                 Checking\r\n");

  for (j = 2; j < 24; j++) // Value needs to be update for mem size....
  {
    // write out the value we'll be checking for later
    #ifdef note
      XromLogString("    NOTE: New Write Address = ");
      XromLogHex(writeAdr);
      XromLogLn();
    #endif
    * (volatile unsigned long *) writeAdr = data2store_f;
    i = 1;
    myAdr = startAdr;
    while (myAdr < startAdr+size)
    {
    h++;
    if (( h & 100 ) == 0) {
       XromLedToggle(); }
    if (((h & 1023) == 0) && AbortByKeyboard())
      {
        XromLogString("\r\n");
        XromLogString("\r\n");
        XromLogString("DDR SDRAM TEST SUMMARY:  PASSES = ");
        XromLogInt(pass_count);
        XromLogString(", TOTAL NUMBER OF ERRORS = ");
        XromLogInt(error);
        XromLogLn();
        return;
      }
      // read back a word of data
      dataRead[0] = *(volatile unsigned long *) myAdr;
      #ifdef note
        XromLogString("    NOTE: New Data Read Address = ");
        XromLogHex(myAdr);
        XromLogLn();
      #endif
      i++;
      // check to make sure it is zero
      if (dataRead[0] != 0)
        {
          if (writeAdr == myAdr)
            {}  // this is where we wrote, is ok.
          else
            {
              error = error +1;
              XromLogString("  TEST2 - ERROR #");
              XromLogInt(error);
              XromLogString(": Address = ");
              XromLogHex(myAdr);
              XromLogString(", Data Expected was 0x00000000, Data Received was ");
              XromLogHex(dataRead[0]);
              XromLogLn();
            }

        }

      myAdr  = startAdr + (unsigned long) (1<<i);
    }
    // Clean out the old value before pointer update....
    * (volatile unsigned long *) writeAdr = 0x00000000;
    writeAdr = startAdr + (unsigned long) (1<<j);

//    if (AbortByKeyboard()) break;

  }


 // clear all of memory to 0
  XromLogString("    LOG: Set all memory to 0x00110055 for next test \r\n");
  for(myAdr = startAdr; myAdr < startAdr+size; myAdr=myAdr+4)
    {
       h++;
       if (( h & 100 ) == 0) {
         XromLedToggle(); }
       * (volatile unsigned long *) myAdr = 0x00110055;
       if (AbortByKeyboard()) return;
    }

  // ---------------------------------------
  //  TEST3 - MAX BA/ROW/COL NOISE
  // ---------------------------------------

  XromLogString("    LOG: TEST3 - Testing for maximum ba/row/col noise \r\n");
  XromLogString("      NOTE: This test performs 16 word writes followed by 16 word reads \r\n");
  XromLogString("      NOTE: Each 64 bytes inverts the ba/row/col address \r\n");

  // setup initial values
  writeAdr = startAdr;
  myAdr = writeAdr;
  ba  = writeAdr & 0x06000000;
  row = writeAdr & 0x01FFF000;
  col = writeAdr & 0x00000FFC;
  j = 0;
  
  for (i = 0; i < (size/64) ; i++) // Going to ping pong and meet in middle
  {
    #ifdef note2
      j++;
      if (j == 1024)
        {
          j = 0;
          XromLogString("     Another 1K passes of up and down\r\n");
        }
    #endif
    // write out the value we'll be checking for later
    #ifdef note
      XromLogString("      NOTE: Write 16 word Addresses starting at = ");
      XromLogHex(writeAdr);
      XromLogString(", then read them back\n");
      XromLogLn();
    #endif
    if (writeAdr > startAdr + size - 0x40)
      {
        XromLogString("        ERROR:Calc'd Low Address exceeds size\r\n");
        XromLogString("              writeAdr = ");
        XromLogHex(writeAdr);
        XromLogString(",  startAdr = ");
        XromLogHex(startAdr);
        XromLogString(",  size = ");
        XromLogHex(size);
        XromLogLn();
        return;
      }
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr = myAdr;
    dataRead[0]  = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[1]  = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[2]  = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[3]  = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[4]  = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[5]  = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[6]  = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[7]  = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[8]  = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[9]  = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[10] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[11] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[12] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[13] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[14] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[15] = *(volatile unsigned long *) writeAdr; writeAdr = myAdr;

    // lets compare the results to the expected values now....
    for (t=0; t < 16; t +=2)
    {
      if      (dataRead[t] != 0xFFFFFFFF)
        {
           error = error +1;
           XromLogString("  TEST3.1 - ERROR #");
           XromLogInt(error);
           XromLogString(": Address = ");
           XromLogHex(myAdr+4*t);
           XromLogString(", Data Expected was 0xFFFFFFFF, Data Received was ");
           XromLogHex(dataRead[t]);
           XromLogLn();
        }
      else if (dataRead[t+1] != 0x00000000)
        {
           error = error +1;
           XromLogString("  TEST3.2 - ERROR #");
           XromLogInt(error);
           XromLogString(": Address = ");
           XromLogHex(myAdr+4*t+4);
           XromLogString(", Data Expected was 0x00000000, Data Received was ");
           XromLogHex(dataRead[t+1]);
           XromLogLn();
        }
    }

    XromLedToggle();

    // Calculate the inverse address for DDR SDRAM
    row = ~row;
    col = ~col;
    ba = ~ba;
    ba  =  ba & 0x06000000;
    row = row & 0x01FFF000;
    col = col & 0x00000FFC;
    writeAdr = ba + row + col - 64;
    myAdr = writeAdr;
    //  write out 16 words, read them back
    if (writeAdr > startAdr + size - 0x40)
      {
        XromLogString("        ERROR:Calc'd High Address exceeds size\r\n");
        XromLogString("              writeAdr = ");
        XromLogHex(writeAdr);
        XromLogString(",  startAdr = ");
        XromLogHex(startAdr);
        XromLogString(",  size = ");
        XromLogHex(size);
        XromLogLn();
      }
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0xFFFFFFFF; writeAdr +=4;
    * (volatile unsigned long *) writeAdr = 0x00000000; writeAdr = myAdr;
    dataRead[16] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[17] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[18] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[19] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[20] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[21] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[22] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[23] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[24] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[25] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[26] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[27] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[28] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[29] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[30] = *(volatile unsigned long *) writeAdr; writeAdr +=4;
    dataRead[31] = *(volatile unsigned long *) writeAdr;

    // lets compare the results to the expected values now....
    for (t=16; t < 32; t +=2)
    {
      if      (dataRead[t] != 0xFFFFFFFF)
        {
           error = error +1;
           XromLogString("  TEST3.3 - ERROR #");
           XromLogInt(error);
           XromLogString(": Address = ");
           XromLogHex(myAdr+4*t);
           XromLogString(", Data Expected was 0xFFFFFFFF, Data Received was ");
           XromLogHex(dataRead[t]);
           XromLogLn();
//           printf("  ERROR #%d: Address = 0x%X, Data Expected was 0xFFFFFFFF, Data Received was 0x%X \n", error, (myAdr+4*t), dataRead[t]);
        }
      else if (dataRead[t+1] != 0x00000000)
        {
           error = error +1;
           XromLogString("  TEST3.4 - ERROR #");
           XromLogInt(error);
           XromLogString(": Address = ");
           XromLogHex(myAdr+4*t+4);
           XromLogString(", Data Expected was 0x00000000, Data Received was ");
           XromLogHex(dataRead[t+1]);
           XromLogLn();
//           printf("  ERROR #%d: Address = 0x%X, Data Expected was 0x00000000, Data Received was 0x%X \n", error, (myAdr+4*t+4), dataRead[t+1]);
        }
    }

    // Calculate the "normal" address for the next cycles
    row = ~row;
    col = ~col;
    ba = ~ba;
    ba  =  ba & 0x06000000;
    row = row & 0x01FFF000;
    col = col & 0x00000FFC;

    writeAdr = ba + row + col + 64;  // increase by one 64 byte quantum

    h++;
    XromLedToggle();
    if (((h & 1023) == 0) && AbortByKeyboard())
      {
        XromLogString("\r\n");
        XromLogString("\r\n");
        XromLogString("SDRAM TEST SUMMARY:  PASSES = ");
        XromLogInt(pass_count);
        XromLogString(", TOTAL NUMBER OF ERRORS = ");
        XromLogInt(error);
        XromLogLn();
        return;
      }

//    if (AbortByKeyboard()) return;
  }

  // ---------------------------------------
  //  TEST4 - DATA = !ADDR test
  // ---------------------------------------
  XromLogString("    LOG: TEST4 - Testing for Inverse Data at Address, write all then read all \r\n");
  for(myAdr = startAdr; myAdr < startAdr+size; myAdr=myAdr+4)
    {
       h++;
       if (( h &100) == 0) {
  	     XromLedToggle(); }
       * (volatile unsigned long *) myAdr = ~myAdr;
    }

  // read it all back
  for(myAdr = startAdr; myAdr < startAdr+size; myAdr=myAdr+4)
    {

    h++;
    if (( h &100) == 0) {
	  XromLedToggle(); }
    if (((h & 1023) == 0) && AbortByKeyboard())
      {
        XromLogString("\r\n");
        XromLogString("\r\n");
        XromLogString("DDR SDRAM TEST SUMMARY:  PASSES = ");
        XromLogInt(pass_count);
        XromLogString(", TOTAL NUMBER OF ERRORS = ");
        XromLogInt(error);
        XromLogLn();
        return;
      }

    dataRead[0] = *(volatile unsigned long *) myAdr;

    if (dataRead[0] == ~myAdr)
      {
      }
    else
      {
        error = error +1;
        XromLogString("  TEST4 - ERROR #");
        XromLogInt(error);
        XromLogString(": Address = ");
        XromLogHex(myAdr);
        XromLogString(", Data Expected was ");
        XromLogHex(!myAdr);
        XromLogString(", Data Received was ");
        XromLogHex(dataRead[0]);
        XromLogLn();
       }
    }

//    if (AbortByKeyboard()) break;
  }


  return;
}



