/*-----------------------------------------------------------------------------
//     $Date: 2004/03/10 23:35:36 $
//     $RCSfile: xromtests.c,v $
//-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/

#include "globaltypes.h"
#include "xrominput.h"
#include "xrom.h"
#include "xromlog.h"
#include "xromtestram.h"
#include "xromsmiley.h"
#include "xrompci.h"
#include "xromled.h"
#include "xromps2.h"
#include "xromtft.h"
#include "xromtouchscreen.h"
#include "xromtestbutton.h"


static void doLog(char *s, unsigned int memStart, unsigned int memLen)
{
  XromLogString(s);
  XromLogString(" [");
  XromLogHex(memStart);
  XromLogString(" - ");
  XromLogHex(memStart+memLen-1);
  XromLogString("]\r\n");
}

void TestsMenu()
{
  XromLogLn();
  XromLogString("XROM for ML300 Main Tests\r\n");
  XromLogString("  1. Test DDR SDRAM\r\n");
  XromLogString("  2. Test BRAM\r\n");
  XromLogString("  3. Test LEDs\r\n");
  XromLogString("  4. Test Push Button Switches\r\n");
  XromLogString("  5. Test TFTs\r\n");
  XromLogString("  6. Test PS2 ports\r\n");
  XromLogString("  7. Test Touchscreen\r\n");
  XromLogString("  9. Scan PCI Bus\r\n");
  XromLogString("  g. Play Mr. Smiley's Mover\r\n");

  XromLogString("  0. Return to the Main Menu\r\n");
}

void TestsInput(char ch)
{
  unsigned int memStart;
  unsigned int memLen;

  switch(ch)
  {
  case '1':
#ifdef V2PDK
    memStart = 0x0;
    memLen = 128;
#else
    memStart = 0x100000;
    memLen = 128*1024*1024 - memStart - 4;
#endif
    doLog("\r\nTesting DDR SDRAM", memStart, memLen);
    ddr_mem_test(memStart, memLen);
    break;
  case '2':
#ifdef V2PDK
    memStart = 0xffffff00;
    memLen = 128;
#else
    memStart = 0xffff8040;
    memLen = 32*1024 - 0x48;
#endif
    doLog("\r\nTesting BRAM", memStart, memLen);
    TestRam(memStart, memLen, 10, 0);
    break;
  case '3':
    XromLogString("\r\nTesting LEDs\r\n");
    XromTestLed();
    break;
  case '4':
    XromLogString("\r\nTesting Pushbutton Switches\r\n");
    XromTestPBs();
    break;
  case '5':
    XromLogString("\r\nTesting TFT\r\n");
    XromTestTft();
    break;
  case '6':
    XromLogString("\r\nTesting PS2\r\n");
    XromTestPs2();
    break;
  case '7':
    XromLogString("\r\nTesting Touchscreen\r\n");
    XromTestTouchscreen();
    break;
  case '9':
    XromLogString("\r\nInitializing PCI Controller\r\n");
    XromPciInit();
    XromLogString("\r\nScanning PCI Bus\r\n");
    XromTestPci();
    break;
  case 'g':
    playMrSmileyMover();
    break;
  case 0x1b:
  case '0':
    PopMenu(&printMenu, &handleInput);
    doPrintMenu = true;
    break;
  default:
    doPrintMenu = true;
    break;
  }
}
