/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <exports.h>

#define chip	0x6f

#define Y2K	7
#define DW	6
#define YR	5
#define MO	4
#define DT	3
#define HR	2
#define MN	1
#define SC	0

void read_time(uchar*);
void set_time(uchar*);
void print_time(uchar*);


/************************************************************************
 * Main program.  Use i2c_read to read out i2c bus device reading.
 */
int rtc_clock(int argc, char *argv[])
{
	uchar rtc[8];
	uchar yn;

	app_startup(argv);

	printf ("Example expects ABI version %d\n", XF_VERSION);
	printf ("Actual U-Boot ABI version %d\n\n", (int)get_version());

	printf("***** ");
	read_time(rtc);
	print_time(rtc);

	printf("\nDo you want to modify current clock setting [y/n]? ");
	yn = getc();
	printf("%c", yn);
	getc();

	/*
	printf("\nYou entered: %02x\n", yn);
	*/

	if (yn == 'y') {
		set_time(rtc);
		printf("\n\n***** New ");
		read_time(rtc);
		print_time(rtc);
	}

	printf("\n\n");
	return (0);
}


/************************************************************************
 * 
 */
void read_time(uchar *rtc)
{
  i2c_read(chip, 0x30, 2, rtc, 8);
}


/************************************************************************
 * 
 */
void set_time(uchar *rtc)
{
	uchar data[10];
	u8 i;

	printf("\n\nPlease enter date [mm/dd/yyyy]:      ");

	for (i=0; i<10; i++) {
		data[i] = getc();
		printf("%c", data[i]);
	}
	getc();

	rtc[MO] = (data[0] << 4) | (data[1] - 0x30);
	rtc[DT] = (data[3] << 4) | (data[4] - 0x30);
	rtc[Y2K]= (data[6] << 4) | (data[7] - 0x30);
	rtc[YR] = (data[8] << 4) | (data[9] - 0x30);
	/*
	printf("\tYou entered: %02x/%02x/%02x%02x ", rtc[MO], rtc[DT], 
		rtc[Y2K], rtc[YR]);
	*/

	printf("\nPlease enter day of week [0-6]:      ");
	data[0] = getc();
	printf("%c", data[0]);
	getc();

	rtc[DW] = data[0] - 0x30;

	printf("\nDo you want 24 hour time [y/n]:      ");

	data[9] = getc();
	printf("%c", data[9]);
	getc();

	if (data[9] == 'y') {
		printf("\nPlease enter time [hh:mm:ss]:      ");

		for (i=0; i<8; i++) {
			data[i] = getc();
			printf("%c", data[i]);
		}
		getc();

		rtc[HR] = (data[0] << 4) | (data[1] - 0x30) | 0x80;
		rtc[MN] = (data[3] << 4) | (data[4] - 0x30);
		rtc[SC] = (data[6] << 4) | (data[7] - 0x30);
	}
	else {
		printf("\nPlease enter time [hh:mm:ss(AM/PM)]: ");
		for (i=0; i<10; i++) {
			data[i] = getc();
			printf("%c", data[i]);
		}

		getc();

		rtc[HR] = (data[0] << 4) | (data[1] - 0x30);
		rtc[MN] = (data[3] << 4) | (data[4] - 0x30);
		rtc[SC] = (data[6] << 4) | (data[7] - 0x30);

		if (data[8] == 'P')
			rtc[HR] |= 0x20; 
	}

	data[0] = 0x02;
	i2c_write(chip, 0x3f, 2, data, 1);
	data[0] = 0x06;
	i2c_write(chip, 0x3f, 2, data, 1);

	i2c_write(chip, 0x30, 2, rtc, 8);

	data[0] = 0x00;
	i2c_write(chip, 0x3f, 2, data, 1);
}


/************************************************************************
 * 
 */
void print_time(uchar *rtc)
{
	printf("Clock Reading: %02x/%02x/%02x%02x ", rtc[MO], rtc[DT], 
		rtc[Y2K], rtc[YR]);
	printf("%02x:%02x:%02x", (rtc[HR] & 0x80) ? rtc[HR] & 0x7f : rtc[HR] & 0x1f, rtc[MN], rtc[SC]);

	if (!(rtc[HR] & 0x80)) {
		if(rtc[HR] & 0x20)
			printf("PM");
		else
			printf("AM");
	}

	switch (rtc[DW]) {
	case 0x00:
		printf(" Sunday\n");
		break;
	case 0x01:
		printf(" Monday\n");
		break;
	case 0x02:
		printf(" Tuesday\n");
		break;
	case 0x03:
		printf(" Wednesday\n");
		break;
	case 0x04:
		printf(" Thursday\n");
		break;
	case 0x05:
		printf(" Friday\n");
		break;
	case 0x06:
		printf(" Saturday\n");
		break;
	default:
		break;
	}
}

