/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <exports.h>

#define CPU255	0x14
#define CPU256	0x15
#define PIO2	0x16
#define PIO4	0x17
#define CPU251	0x18
#define CPU252	0x4b
#define CPU253	0x50
#define PIO3	0x56
#define PIO24	0x6f

static void pwmonitor_cpu255_pio2(uchar);
static void pwmonitor_cpu256_pio4(uchar);
static void onchip_temp(uchar);
static void ambient_temp(uchar);
/*void tft_brightness(uchar);*/
/*void rtc_clock(uchar);*/
static int convert_temp(uchar*, unsigned int*);
static unsigned int convert_volt(uchar, const unsigned int);


/************************************************************************
 * Main program.  Use i2c_read to read out i2c bus device reading.
 */
int system_monitor (int argc, char *argv[])
{
	/* Print the ABI version */
	app_startup(argv);

	printf ("Example expects ABI version %d\n", XF_VERSION);
	printf ("Actual U-Boot ABI version %d\n\n", (int)get_version());

	printf("CPU.U255 Power Monitor (1.8-5V) Reading...\n");
	pwmonitor_cpu255_pio2(CPU255);
	
	printf("CPU.U256 Power Monitor (2.5-12V) Reading...\n");
	pwmonitor_cpu256_pio4(CPU256);

	printf("PIO.U2 Power Monitor (1.8-5V) Reading...\n");
	pwmonitor_cpu255_pio2(PIO2);

	printf("PIO.U4 Power Monitor (2.5-12V) Reading...\n");
	pwmonitor_cpu256_pio4(PIO4);

	printf("On-Chip Temperature Sensor Reading...\n");
	onchip_temp(CPU251);

	printf("Ambient Temperature Sensor Reading...\n");
	ambient_temp(CPU252);

	return (0);
}


/************************************************************************
 * Reads out power monitor measurements.
 */
static void pwmonitor_cpu255_pio2(uchar chip)
{
	unsigned int v18, v25, v5, vcc;
	int temp;
	uchar buffer;

	buffer = 0x01;

	if (i2c_write(chip, 0x40, 1, &buffer, 1)) {
		printf("\t***** Device on address 0x%02x is not available\n\n", chip);
		return;
	}
	udelay(500000);

	i2c_read(chip, 0x20, 1, &buffer, 1);
	v18 = convert_volt(buffer, 9375);

	i2c_read(chip, 0x21, 1, &buffer, 1);
	v25 = convert_volt(buffer, 13020);

	i2c_read(chip, 0x22, 1, &buffer, 1);
	v5  = convert_volt(buffer, 26041);

	i2c_read(chip, 0x23, 1, &buffer, 1);
	vcc = convert_volt(buffer, 17187);

	i2c_read(chip, 0x27, 1, &buffer, 1);
	temp = (buffer >> 7) ? 0xffffff00 : 0;
	temp |= (int)(buffer);

	printf("\t1.8 Vin Measurement: %2i.%03iV\n", (int)v18/1000000, (int)(v18%1000000)/1000);
	printf("\t2.5 Vin Measurement: %2i.%03iV\n", (int)v25/1000000, (int)(v25%1000000)/1000);
	printf("\t5   Vin Measurement: %2i.%03iV\n", (int)v5/1000000, (int)(v5%1000000)/1000);
	printf("\t    VCC Measurement: %2i.%03iV\n", (int)vcc/1000000, (int)(vcc%1000000)/1000);
	printf("\t   Temp Measurement: % 6iC\n\n", temp);

	buffer = 0x80;
	i2c_write(chip, 0x40, 1, &buffer, 1);

	return;
}


/************************************************************************
 * Reads out power monitor measurements.
 */
static void pwmonitor_cpu256_pio4(uchar chip)
{
	unsigned int v25, v12, v33, vcc;
	int temp;
	uchar buffer;

	buffer = 0x01;

	if (i2c_write(chip, 0x40, 1, &buffer, 1)) {
		printf("\t***** Device on address 0x%02x is not available\n\n", chip);
		return;
	}
	udelay(500000);

	i2c_read(chip, 0x20, 1, &buffer, 1);
	v25 = convert_volt(buffer, 13020);

	i2c_read(chip, 0x21, 1, &buffer, 1);
	v12 = convert_volt(buffer, 62500);

	i2c_read(chip, 0x22, 1, &buffer, 1);
	v33 = convert_volt(buffer, 17187);

	i2c_read(chip, 0x23, 1, &buffer, 1);
	vcc = convert_volt(buffer, 26041);

	i2c_read(chip, 0x27, 1, &buffer, 1);
	temp = (buffer >> 7) ? 0xffffff00 : 0;
	temp |= (int)(buffer);

	printf("\t2.5 Vin Measurement: %2i.%03iV\n", (int)v25/1000000, (int)(v25%1000000)/1000);
	printf("\t12  Vin Measurement: %2i.%03iV\n", (int)v12/1000000, (int)(v12%1000000)/1000);
	printf("\t3.3 Vin Measurement: %2i.%03iV\n", (int)v33/1000000, (int)(v33%1000000)/1000);
	printf("\t    VCC Measurement: %2i.%03iV\n", (int)vcc/1000000, (int)(vcc%1000000)/1000);
	printf("\t   Temp Measurement: % 6iC\n\n", temp);

	buffer = 0x80;
	i2c_write(chip, 0x40, 1, &buffer, 1);

	return;
}


/************************************************************************
 * Reads out onchip_temperature measurements.
 */
static void onchip_temp(uchar chip)
{
	uchar buffer;
	int itemp, etemp, ih, il, eh, el;

	if (i2c_read(chip, 0, 1, &buffer, 1)) {
		printf("\t***** Device on address 0x%02x is not available\n\n", chip);
		return;
	}

	itemp = (buffer >> 7) ? 0xffffff00 : 0;
	itemp |= (int)(buffer);

	i2c_read(chip, 1, 1, &buffer, 1);
	etemp = (buffer >> 7) ? 0xffffff00 : 0;
	etemp |= (int)(buffer);

	i2c_read(chip, 5, 1, &buffer, 1);
	ih = (buffer >> 7) ? 0xffffff00 : 0;
	ih |= (int)(buffer);

	i2c_read(chip, 6, 1, &buffer, 1);
	il = (buffer >> 7) ? 0xffffff00 : 0;
	il |= (int)(buffer);

	i2c_read(chip, 7, 1, &buffer, 1);
	eh = (buffer >> 7) ? 0xffffff00 : 0;
	eh |= (int)(buffer);

	i2c_read(chip, 8, 1, &buffer, 1);
	el = (buffer >> 7) ? 0xffffff00 : 0;
	el |= (int)(buffer);

	printf("\tAmbient Temp:        % 4iC\n", itemp);
	printf("\tOn-die Temp:         % 4iC\n", etemp);
	/*
	printf("\tInternal High Limit: % 4ic\n", ih);
	printf("\tInternal Low Limit:  % 4ic\n", il);
	printf("\tExternal High Limit: % 4ic\n", eh);
	printf("\tExternal Low Limit:  % 4ic\n", el);
	*/
	printf("\n");

	return;
}


/************************************************************************
 * Reads out ambient_temp device measurements.
 */
static void ambient_temp(uchar chip)
{
	uchar buffer[2];
	int temp, hyst, crit, low, high;
	unsigned int dtemp, dhyst, dcrit, dlow, dhigh;

	if (i2c_read(chip, 0, 1, buffer, 2)) {
		printf("\t***** Device on address 0x%02x is not available\n\n", chip);
		return;
	}
	temp = convert_temp(buffer, &dtemp);

	i2c_read(chip, 2, 1, buffer, 2);
	hyst = convert_temp(buffer, &dhyst);

	i2c_read(chip, 3, 1, buffer, 2);
	crit = convert_temp(buffer, &dcrit);

	i2c_read(chip, 4, 1, buffer, 2);
	low = convert_temp(buffer, &dlow);

	i2c_read(chip, 5, 1, buffer, 2);
	high = convert_temp(buffer, &dhigh);


	printf("\tAmbient Temperature: % 3i.%04iC\n", temp, dtemp);
	/*
	printf("\tLow temp limit:      % 3i.%04ic\n", low, dlow);
	printf("\tHigh temp limit:     % 3i.%04ic\n", high, dhigh);
	printf("\tCritical temp point: % 3i.%04ic\n", crit, dcrit);
	printf("\tHysteresis:          % 3i.%04ic\n", hyst, dhyst);
	*/
	printf("\n");

	return;
}

/************************************************************************
 * Converts temperature reading.
 */
static int convert_temp (uchar *buffer, unsigned int *dec)
{
	int degree = 0;

	degree = (buffer[0] >> 7) ? 0xffffe000 : 0;
	degree |= ((int)(buffer[0])) << 1;
	degree |= ((int)(buffer[1])) >> 7;

	*dec = 0;
	*dec += (buffer[1] >> 6) & 0x01 ? 5000 : 0;
	*dec += (buffer[1] >> 5) & 0x01 ? 2500 : 0;
	*dec += (buffer[1] >> 4) & 0x01 ? 1250 : 0;
	*dec += (buffer[1] >> 3) & 0x01 ? 625  : 0;

	return degree;
}


/************************************************************************
 * Converts voltage reading.
 */
static unsigned int convert_volt (uchar buffer, const unsigned int rate)
{
	return ((unsigned int)(buffer) * rate);
}
